# -*- coding: utf-8 -*-
"""
    funcInter.py: function-Interface zu Maxima/pyGeo, W.S.,
    
    Vers. 0.02, 2007-08-08
    Vers. 0.03, 2007-09-19  => Korrektur von binomial (benutze ceiling)
    und floor statt der Lisp-Funktion ?truncate)
    Vers. 0.04, 2007-09-22  => Methode tangentialebene => neu
    Vers. 0.05, 2007-11-29  => Methode abstand => um den Abstand zwischen Punkten erweitert (muehsam)
    Vers. 0.06, 2007-12-10  => Methode findRoot
    Vers. 0.07, 2008-01-24  => Methode gueteFunktion
    Vers. 0.08, 2008-02-04  => Methode gueteFunktion ueberarbeitet
    Vers. 0.09, 2008-02-24  => Methode alternativtest

    Vers. 0.10, 2008-03-26  => Anpassung an Verteilungen von W. Urban (Modul stochastik.py)
    URL: http://www.hib-wien.at/leute/wurban/mathematik/distributions.html
    
    Vers. 0.11, 2008-03-30  => Methode schnittCG, Methode abstand an Kreis angepasst
    Vers. 0.12, 2008-03-31  => Methode schnittCE
    Vers. 0.13, 2008-07-04  => Methode pol (bestimmt Pol P)
    Vers. 0.14, 2008-07-05  => Methode schnittKE modifiziert => liefert Kreisobjekt
    Vers. 0.15, 2008-07-06  => Methode schnittKK modifiziert => liefert Kreisobjekt
    Vers. 0.16, 2008-09-01  => Methode kugelAusTangentialebene  => neu
                            => Methode tangentialebene => ueberarbeitet
    Vers. 0.17, 2010-04-07  => Funktion normalVerteilung angepasst an das draw-Modul
    Vers. 0.18, 2010-04-08  => Funktion gueteFunktion & Funktion binomial angepasst an das draw-Modul
    Vers. 0.19, 2010-04-08  => Funktion funkSchar2HTML: erzeugt eine HTML-Datei für die
                            Anzeige der Funktionenschar als animated gif
    Vers. 0.20, 2011-07-18  => Rechtschreibfehler (!) beim Hypothesentest korrigiert!
    Vers. 0.21, 2012-04-02  => Aufruf von descartes.vecLi2str zur Konvertierung vom Listen-Format ins pyGeo-Format
                            Das pyGeo-Format in der Darstellung wird benoetigt im Modul tkWidgets.py in der
                            Methode geoTable => siehe die Methoden formatReal, bearbeitePunkt und bearbeiteListe
    Vers. 0.22, 2012-11-16  => "Fehler" bei der Berechnung des alpha- bzw. beta-Fehlers im Alternativtest beseitigt
                            vgl. das Ende der Methode "alternativtest"

    Vers. 0.23, 2012-11-23  => Funktion "gueteFunktion" bei der Operationscharakteristik (OC) auf
                            zwei-, links- und rechtsseitig erweitert (TEST!)

    Vers. 0.24, 2012-11-24  => Funktion "gueteFunktion" bei der Guetefunktion g(P) auf
                            zwei-, links- und rechtsseitig erweitert (TEST!)

    Vers. 0.25, 2012-11-24  => Funktion "gueteFunktion" ueberarbeitet und bereinigt => experimentell!

    Vers. 0.26, 2013-12-01  => Funktion "wkBaum" => erstellt Wahrscheinlichkeitsbaum bzw. Vierfeldertafel

    Vers. 0.27, 2013-12-02  => Funktion "printItemListe" für die Funktion "wkBaum"

    Vers. 0.28, 2015-10-07  => Schrittweite _sw_ in "gueteFunktion" verändert (von 20 auf 100!)

    Vers. 0.29, 2016-06-16  => Da die Approximation durch die Normalverteilung fuer kleine Wk zu fehlerhaften Werten fuehrt, erfolgt die
                            Berechnung in "gueteFunktion" ueber die summierte Binomialverteilung; Bug in sumBin2 beseitigt (=> Skript
                            sumBin2 funktioniert nur fuer 0 < p < 1 !!!)            
           

"""

import string
import tkWidgets as tkw
import tkMatrix
import descartes, stochastik
import tkMessageBox
import pymaxUtil as util

##########################################################
# globals fuer wkBaum
wkBaum_kette = ""  # wg. HTML-Ausgabe! sammelt die Ausgabe in der rekursiven Funktion printWkBaum
wkBaum_liste = []

wkBaum_standardWert = 16
wkBaum_dist = wkBaum_standardWert


wkBaum_start = "Start"

##############################################

def stripStrichpunkt(arg):
        _arg_ = string.strip(arg)
        maxi = len(_arg_)
        if maxi > 0:
            if _arg_[maxi-1] == ';':
                return _arg_[:-1]
            elif _arg_[maxi-1] == ',':
                return _arg_[:-1]		
            else:
                return arg
        else:
            return arg

def addStrichpunkt(arg):
        _arg_ = string.strip(arg)
        maxi = len(_arg_)
        #print _arg_
        if maxi > 0:
            if (_arg_[maxi-1] != ';') and (_arg_[maxi-1] != '$'): # kein ';' am Ende UND kein '$' am Ende
                return _arg_ + ';'
            else:
                return arg
        else:
            return arg

##############################################


##############################################
    
def prelude(ereignisA, ereignisB):
    retListe = []
    retListe.append('"Bedingte Wahrscheinlichkeit/Vierfeldertafel"$' )       
    retListe.append('"Symbol für Ereignis 1: ' + str(ereignisA) + ' -> Gegenereignis: ~' + str(ereignisA) + '"$' )
    retListe.append('"Symbol für Ereignis 2: ' + str(ereignisB) + ' -> Gegenereignis: ~' + str(ereignisB)  + '"$')
    retListe.append('"Wahrscheinlichkeitsbaum"$' )               
    return retListe



def printBlank(dist):
    """
        Der >Fehler< ( >print ' ',< ) hier ist Absicht . . . 
    """
    retString = ""    
    for i in range(dist):
        retString = retString + "  "  # 2 Blanks!
    return retString        



def printItemListe(dist,liste):
    """
        Der >Fehler< ( >print ' ',< ) hier ist Absicht . . . 
    """
    _debug_ = False
    
    _kette_ = ""
        
    # ItemListe => BFS-Darstellung
    listIndex = 0
    li0 = []
    li1 = []
    li2 = []
    for element in liste:
        if (listIndex == 0):
            li0.append(element[0])
            li1.append(element[1])
            li1.append(element[2])            
        elif (listIndex == 1): 
            li2.append(element[1])
            li2.append(element[2])
        else:
            li2.append(element[1])
            li2.append(element[2])
            
        listIndex = listIndex + 1
    bfsTree = []
    bfsTree.append(li0)
    bfsTree.append(li1)
    bfsTree.append(li2)
    if _debug_:
        print "Transformation"    
        print str(bfsTree)
        print "printItemListe"
    
    # Transformation in BFS fertig: Jetzt ausgeben
    
    _kette_ = ''
    dist = 16
    retList = []
    
    for liste in bfsTree:
        _kette_ = ''    
        for element in liste:
            if len(liste) < 2: # Start!
                _kette_ = _kette_ + printBlank(dist-3)
            else:
                _kette_ = _kette_ + printBlank(dist)

            _kette_ = _kette_ + element
                
        dist = dist/2
                
        retList.append('"'+ _kette_ + '"$')
        
    return retList      


def linie(n):
    zeichen = ""
    for i in range(n):
        zeichen = zeichen + "-"
    return zeichen
        

def berechneWk(wkTreeList,A_B,A_Bquer,Aquer_B,Aquer_Bquer):
    """ berechnet nach den Werten für das Baumdiagramm die Wahrscheinlichkeiten
        In:  wkTreeList,A_B,A_Bquer,Aquer_B,Aquer_Bquer
        OUT: wkA, _bed_B_nach_A, _bed_B_nach_Aquer, A_B, A_Bquer, Aquer_B, Aquer_Bquer
    """
    wkA = -1.0        
    wkAquer = -1.0
    
    _bed_B_nach_A = _bed_Bquer_nach_A = _bed_B_nach_Aquer =  _bed_Bquer_nach_Aquer = -1.0
    
    
    if (len(wkTreeList) == 3):
        links = wkTreeList[1]
        rechts = wkTreeList[2]
        
        # Ast 1
        if A_B < 0: # A_B = -1.0 !
            A_B = float(links[0]) * float(links[1])
            _bed_B_nach_A = float(links[1])
            wkA = float(links[0])
        elif (float(links[0]) < 0): # wkA = -1.0               
            wkA = A_B / float(links[1])
            _bed_B_nach_A = float(links[1])                
        else:  # bedingte Wk = -1.0
            _bed_B_nach_A = A_B / float(links[0])
            wkA = float(links[0])               
            
        # Ast 2                
        if A_Bquer < 0:  # A_Bquer = -1.0 !  
            A_Bquer = float(links[0]) * float(links[2])
            _bed_Bquer_nach_A = float(links[2])
            wkA = float(links[0]) 
        elif (float(links[0]) < 0): # wkA = -1.0               
            wkA = A_B / float(links[2])
            _bed_Bquer_nach_A = float(links[2])                
        else:  # bedingte Wk = -1.0
            _bed_Bquer_nach_A = A_B / float(links[0])
            wkA = float(links[0])               

        # Ast 3    
        if Aquer_B < 0:  # Aquer_B = -1.0 !
            Aquer_B = float(rechts[0]) * float(rechts[1])
            _bed_B_nach_Aquer = float(rechts[1])
            wkAquer = float(rechts[0])
        elif (float(rechts[0]) < 0): # wkAquer = -1.0               
            wkAquer = Aquer_B / float(rechts[1])
            _bed_B_nach_Aquer = float(rechts[1])                
        else:  # bedingte Wk = -1.0
            _bed_B_nach_Aquer = Aquer_B / float(rechts[0])
            wkAquer = float(rechts[0])                

        # Ast 4    
        if Aquer_Bquer < 0:  # Aquer_Bquer = -1.0 !              
            Aquer_Bquer = float(rechts[0]) * float(rechts[2])
            _bed_Bquer_nach_Aquer = float(rechts[2])
            wkAquer = float(rechts[0])
        elif (float(rechts[0]) < 0):  # wkAquer = -1.0               
            wkAquer = Aquer_Bquer / float(rechts[2])
            _bed_Bquer_nach_Aquer = float(rechts[2])
        else:  # bedingte Wk = -1.0
            _bed_Bquer_nach_Aquer = Aquer_Bquer / float(rechts[0])
            wkAquer = float(rechts[0])
            
        return (wkA, _bed_B_nach_A, _bed_B_nach_Aquer, A_B, A_Bquer, Aquer_B, Aquer_Bquer)
    else:
        return (-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0)


def vierfeldertafel(wkTreeList,A_B,A_Bquer,Aquer_B,Aquer_Bquer,wkA,wkAquer,wkB,wkBquer,ereignisA,ereignisB):
    """
        erstellt eine Vierfeldertafel
    """
    retList=[]
    ereigA = ereignisA
    ereigB = ereignisB
    
    _format = "%0.4f"
    _dist_ = 4
    
    wkA, _bed_B_nach_A, _bed_B_nach_Aquer, A_B, A_Bquer, Aquer_B, Aquer_Bquer = berechneWk(wkTreeList,A_B,A_Bquer,Aquer_B,Aquer_Bquer)

    if (wkA >= 0):
        wkAquer = 1.0 - wkA
        # Formel der totalen Wk
        wkB = wkA * _bed_B_nach_A + wkAquer * _bed_B_nach_Aquer
        wkBquer = 1.0 - wkB
        
        _linie_ = ""        
        _linie_ = '"' + linie(40) + '"$'                       
        
        retList.append('"            Vierfeldertafel"$')
        retList.append(_linie_)
        
        retString = ""
        retList.append('"            ' + ereigB + '           ~' + ereigB + '"$')
        retList.append(_linie_)
        
        retList.append('"    '+ ereigA + '     ' + str(_format%A_B) + '      ' + str(_format%A_Bquer) + '    ' + str(_format%wkA) + '"$')        
        retList.append('"   ~'+ ereigA + '     ' + str(_format%Aquer_B) + '      ' + str(_format%Aquer_Bquer) + '    ' + str(_format%wkAquer) + '"$')
        retList.append(_linie_)

        retList.append('"          ' + str(_format%wkB) + '      ' + str(_format%wkBquer) + '    1.0000"$')         
            
        return retList
        

def blattWk(wkTreeList,A_B,A_Bquer,Aquer_B,Aquer_Bquer,ereignisA,ereignisB):
    """
        bestimmt die Wk der Blätter = Schnitt-Wk
    """
    retList=[]
    ereigA = ereignisA
    ereigB = ereignisB
    
    _format_ = "%0.2f"
    _dist_ = 4
    
    wkA = -1.0        
    wkAquer = -1.0
    
    wkB = -1.0        
    wkBquer = -1.0

    _bed_B_nach_A = _bed_Bquer_nach_A = _bed_B_nach_Aquer =  _bed_Bquer_nach_Aquer = -1.0

    wkA, _bed_B_nach_A, _bed_B_nach_Aquer, A_B, A_Bquer, Aquer_B, Aquer_Bquer = berechneWk(wkTreeList,A_B,A_Bquer,Aquer_B,Aquer_Bquer)

    
    if (wkA >= 0):
        wkAquer = 1.0 - wkA
                            
        _bed_Bquer_nach_A = 1.0 - _bed_B_nach_A

        _bed_Bquer_nach_Aquer = 1.0 - _bed_B_nach_Aquer

        anzahlBlank = _dist_ - len("Blätter")
        if anzahlBlank < 0:
            anzahlBlank = 0

        retString = ""
        
        retString = retString +  '"Blätter '
        retString = retString + printBlank(anzahlBlank )
        retString = retString +  _format_%A_B
        retString = retString + printBlank(_dist_ )
        retString = retString +  _format_%A_Bquer
        retString = retString + printBlank(_dist_ )
        retString = retString +  _format_%Aquer_B
        retString = retString + printBlank(_dist_)
        retString = retString + _format_%Aquer_Bquer
        retString = retString + '"$'
        retList.append(retString)
        
        retString = ""
        retString = '"Summe der Blätter: ' + str(A_B + A_Bquer + Aquer_B + Aquer_Bquer) + '"$'
        retList.append(retString)
            
 
        return retList
    
            
def berechneBlattWk(wkTreeList,A_B,A_Bquer,Aquer_B,Aquer_Bquer,ereignisA,ereignisB):
    """
        bestimmt bedingte Wk, die umgekehrten bedingten Wk, sowie die Wk der Blätter = Schnitt-Wk
        erstellt eine BaumListe für die umgekehrte Wk
        
    """
    global wkBaum_start

    retList = []
    
    ereigA = ereignisA
    ereigB = ereignisB

    _start_ = wkBaum_start 
    
    _format_ = "%0.2f"
    _dist_ = 4
    
    wkA = -1.0        
    wkAquer = -1.0
    
    wkB = -1.0        
    wkBquer = -1.0

    _bed_B_nach_A = _bed_Bquer_nach_A = _bed_B_nach_Aquer =  _bed_Bquer_nach_Aquer = -1.0

    wkA, _bed_B_nach_A, _bed_B_nach_Aquer, A_B, A_Bquer, Aquer_B, Aquer_Bquer = berechneWk(wkTreeList,A_B,A_Bquer,Aquer_B,Aquer_Bquer)

    
    if (wkA >= 0):
        wkAquer = 1.0 - wkA
                            
        _bed_Bquer_nach_A = 1.0 - _bed_B_nach_A

        _bed_Bquer_nach_Aquer = 1.0 - _bed_B_nach_Aquer

        anzahlBlank = _dist_ - len("Blätter")
        if anzahlBlank < 0:
            anzahlBlank = 0
            
        retString = ""
        

        retString = retString +  "Blätter "
        retString = retString + printBlank(anzahlBlank )
        retString = retString +  _format_%A_B
        retString = retString + printBlank(_dist_ )
        retString = retString +  _format_%A_Bquer
        retString = retString + printBlank(_dist_ )
        retString = retString +  _format_%Aquer_B
        retString = retString + printBlank(_dist_)
        retString = retString +  _format_%Aquer_Bquer
        retString = retString + "\nSumme der Blätter: " + str(A_B + A_Bquer + Aquer_B + Aquer_Bquer)
        retString = retString + "\n"
        
        # Formel der totalen Wk
        wkB = wkA * _bed_B_nach_A + wkAquer * _bed_B_nach_Aquer
        wkBquer = 1.0 - wkB

        retList.append('"P('+ereigA+') = %0.2f'%wkA + '"$')        
        retList.append('"P(~'+ereigA+') = %0.2f'%wkAquer + '"$')
                
        retList.append('"Bedingte Wahrscheinlichkeiten:"$')
        
        retList.append('"P('+ereigB+' nach '+ereigA+') = %0.6f'%_bed_B_nach_A + '"$') 
        retList.append('"P(~'+ereigB+' nach '+ereigA+') = %0.6f'%_bed_Bquer_nach_A + '"$') 
        retList.append('"P('+ereigB+' nach ~'+ereigA+') = %0.6f'%_bed_B_nach_Aquer + '"$') 
        retList.append('"P(~'+ereigB+' nach ~'+ereigA+') = %0.6f'%_bed_Bquer_nach_Aquer + '"$') 

        retList.append('"=> P('+ereigB+') = %0.6f'%wkB + '"$') 
        retList.append('"=> P(~'+ereigB+') = %0.6f'%wkBquer + '"$') 
        
        # Formel der bedingten Wk
        _bed_A_nach_B = A_B / wkB
        _bed_Aquer_nach_B = Aquer_B / wkB
        _bed_A_nach_Bquer = A_Bquer / wkBquer
        _bed_Aquer_nach_Bquer = Aquer_Bquer / wkBquer
        
        retList.append('"Umgekehrte bedingte Wahrscheinlichkeiten:"$')
        retList.append('"P('+ereigA+' nach '+ereigB+') = %0.6f'%_bed_A_nach_B + '"$') 
        retList.append('"P(~'+ereigA+' nach '+ereigB+') = %0.6f'%_bed_Aquer_nach_B + '"$') 
        retList.append('"Test  I: P('+ereigA+' nach '+ereigB+') + P(~'+ereigA+' nach '+ereigB+') = %0.6f'%(_bed_A_nach_B + _bed_Aquer_nach_B) + '"$') 
        retList.append('"P('+ereigA+' nach ~'+ereigB+') = %0.6f'%_bed_A_nach_Bquer + '"$') 
        retList.append('"P(~'+ereigA+' nach ~'+ereigB+') = %0.6f'%_bed_Aquer_nach_Bquer + '"$') 
        retList.append('"Test II: P('+ereigA+' nach ~'+ereigB+') + P(~'+ereigA+' nach ~'+ereigB+') = %0.6f'%(_bed_A_nach_Bquer + _bed_Aquer_nach_Bquer) + '"$') 
            
        
        # baue Liste für umgekehrte Wk auf
        teil1 = []
        teil2 = []
        teil3 = []
        teil1.append(_start_)
        teil1.append(str("%0.2f"%wkB) )
        teil1.append(str("%0.2f"%wkBquer) )
        teil2.append(str("%0.2f"%wkB) )
        teil2.append(str("%0.2f"%_bed_A_nach_B) )
        teil2.append(str("%0.2f"%_bed_Aquer_nach_B) )
        teil3.append(str("%0.2f"%wkBquer) )
        teil3.append(str("%0.2f"%_bed_A_nach_Bquer) )
        teil3.append(str("%0.2f"%_bed_Aquer_nach_Bquer) )
        retBaumListe = []
        retBaumListe.append(teil1)
        retBaumListe.append(teil2)
        retBaumListe.append(teil3)
        return retBaumListe,retList
    else:
        return [],[]                                   
                
def bearbeiteBaumListe(wkBaum):
    """
        Testet die Integrität der Baum-Liste und versucht, fehlende Wk in der Liste
        (entweder Wk/Gegen-Wk des Ereignisses oder bedingte Wk) zu bestimmen.
        
        Grenzen: fehlen die bedingten Wk in einem Zweig, da nur die Schnitt-Wk
        der Blätter bekannt sind, kann die Funktion die bedingten Wk nicht bestimmen!

        Beachte: Dieses Problem wird in der Eingabe-Funktion >leseWk()< abgefangen
    """
    _format = "%0.2f"
    magicValue = "-1.0"
    
    # falls P(A)/P(Aquer) fehlt
    changedValue = 0
    changedListe = 0
    changed = False
    
    if len(wkBaum) <> 3:
        return []
    else:
        retBaum = []
        i = 0  # zählt die Teilisten durch: i aus [1,2,3]
        for liste in wkBaum:
            i = i + 1
            if magicValue in liste:
                _liste_ = []                                    
                if (i==1):
                    _liste_.append(liste[0])
                else:  # i > 1 => Liste 2 bzw. 3 == 2. Ebene
                    if changed: # GegenWk in Kopf der 2./3. Liste nachtragen
                        if (changedListe == 1) and (i==2):
                            _liste_.append(changedValue)
                        elif (changedListe == 2) and (i==3):
                            _liste_.append(changedValue)
                        else:
                            _liste_.append(liste[0])
                    else:
                        _liste_.append(liste[0])
                
                if liste[1] == magicValue:
                    if liste[2] == magicValue: # dann fehlen beide Werte => schlecht . . .
                        _liste_.append(liste[1] )
                        _liste_.append(liste[2] )
                        
                    else:
                        wk = 1.0 - float(liste[2])
                        _liste_.append(str(_format%wk) )
                        _liste_.append(liste[2] )
                    
                    if (i == 1): # in Liste 1 fehlt die Wk/GegenWk
                        # changed-Variable setzen!
                        changedValue = str(_format%wk)
                        changedListe = 1
                        changed = True
                        
                elif liste[2] == magicValue:
                    if liste[1] == magicValue: # dann fehlen beide Werte => schlecht . . .
                        # dieser Fall kann eigentlich nicht eintreten . . . 
                        _liste_.append(liste[1] )
                        _liste_.append(liste[2] )
                    else:
                        wk = 1.0 - float(liste[1])
                        _liste_.append(liste[1] )
                        _liste_.append(str(_format%wk) )
                        
                    if (i == 1): # in Liste 1 fehlt die Wk/GegenWk
                        # changed-Variable setzen!
                        changedValue = str(_format%wk)
                        changedListe = 2
                        changed = True
                    
                retBaum.append(_liste_)
            else:    # da die Liste nicht verändert wurde, einfach anhängen            
                retBaum.append(liste)
                        
        return retBaum
############################################
def wkBaum(root,moduleDrawLoaded):            
    """bestimmt Wahrscheinlichkeitsbaum/Vierfeldertafel/bedingte Wk
       Parameter: viele
       URL: http://www.wspiegel.de/mmp/wkBaum_013.zip
    """
        
    global wkBaum_kette
    global wkBaum_dist
    global wkBaum_standardWert
    global wkBaum_start
   
    
    _default_ = "-1.0"

    
    _start_ = wkBaum_start

    _debug_ = True
    _debug_ = False
    _debug2_ = False

    _html_ = True
    
    ok = True
    default = _default_
    _format = "%0.2f"
    blattWk1 = blattWk2 = False    

    eingabe = tkw.wkBaum(root,"Wahrscheinlichkeitsbaum")     
    if eingabe.result <> None:
        # Fenster-Werte uebernehmen
        try:
            ereignisA = eingabe.a1
            ereignisB = eingabe.a2

            baum1 = float(eingabe.b1)
            baum2 = float(eingabe.b2)
            baum3 = float(eingabe.b3)
            baum4 = float(eingabe.b4)
            baum5 = float(eingabe.b5)
            baum6 = float(eingabe.b6)
            baum7 = float(eingabe.b7)
            baum8 = float(eingabe.b8)
            baum9 = float(eingabe.b9)
            baum10 = float(eingabe.b10)    
    
        except:
            tkMessageBox.showinfo("Parameter-Fehler!",  
                 "Fehler bei der Parametereingabe")
            return [] # 0 == not ok
        
        
        
        if _debug2_:
            tkMessageBox.showinfo("Eingabe gelesen . . .", "Starte Rechnung . . .")

        retLi = []
        
        # wkA/wkAquer -> Wk. des ersten Ereignisses
        wkA = str( baum1 )

        if (wkA == default):    
            wkAquer = str( baum2 )
            if (wkAquer == default):
                ok = False
            else:
                wkA = str(1.0 - float(wkAquer) )
        else:
            wkAquer = str(1.0 - float(wkA) )
            
        if _debug2_:    
            tkMessageBox.showinfo("Teil 1", wkA + "  " + wkAquer)
                                  
        # Eingabe der bedingten Wk
        if ok:
            wkB_nach_A = str( baum3 )
            if (wkB_nach_A == default):
                wkBquer_nach_A = str( baum4 )
                if (wkBquer_nach_A == default):                
                    blattWk1 = True
                else:
                    wkB_nach_A = str(1.0 - float(wkBquer_nach_A) )
            else:
                wkBquer_nach_A = str(1.0 - float(wkB_nach_A) )
                
            if _debug2_:    
                tkMessageBox.showinfo("Teil 2", wkB_nach_A + "  " + wkBquer_nach_A)
                
            if blattWk1: # Eingabe der Schnitt-Wk für 1. Zweig
                wkA_B  = str( baum7 )
                if (wkA_B == default):
                    wkA_Bquer = str( baum8 )
                    if (wkA_Bquer == default):
                        ok = False                
                    else:
                        if (wkA_Bquer > wkA): # kann nicht sein!
                            ok = False                
                        else:    
                            erg = float(wkA_Bquer) / float(wkA)
                            wkBquer_nach_A = str(_format%erg )
                            wkB_nach_A = str(1.0 - float(wkBquer_nach_A) )
                else:
                    if (wkA_B > wkA):  # kann nicht sein!
                        ok = False
                    else:
                        erg = float(wkA_B) / float(wkA)
                        wkB_nach_A = str(_format%erg )
                        wkBquer_nach_A = str(1.0 - float(wkB_nach_A) )
                        
                if _debug2_:    
                    tkMessageBox.showinfo("Teil 2b", wkB_nach_A + "  " + wkBquer_nach_A)
                    tkMessageBox.showinfo("Teil 3", wkA_B + "  " + wkA_Bquer)
                                
            if ok:            
                wkB_nach_Aquer = str( baum5 )
                if (wkB_nach_Aquer == default):
                    wkBquer_nach_Aquer = str( baum6 )
                    if (wkBquer_nach_Aquer == default):                    
                        blattWk2 = True
                    else:
                        wkB_nach_Aquer = str(1.0 - float(wkBquer_nach_Aquer) )
                else:
                    wkBquer_nach_Aquer = str(1.0 - float(wkB_nach_Aquer) )
                    
                if _debug2_:    
                    tkMessageBox.showinfo("Teil 4", wkB_nach_Aquer + "  " + wkBquer_nach_Aquer)
                    
                if blattWk2: # Eingabe der Schnitt-Wk für 2. Zweig
                    wkAquer_B  = str( baum9 )
                    if (wkAquer_B == default):
                        wkAquer_Bquer = str( baum10 )
                        if (wkAquer_Bquer == default):                        
                            ok = False                
                        else:
                            if (wkAquer_Bquer > wkAquer):   # kann nicht sein!                         
                                ok = False
                            else:
                                erg = float(wkAquer_Bquer) / float(wkAquer)
                                wkBquer_nach_Aquer = str(_format%erg )
                                wkB_nach_Aquer = str(1.0 - float(wkBquer_nach_Aquer) )
                    else:
                        if (wkAquer_B > wkAquer):    # kann nicht sein!                        
                            ok = False
                        else:
                            erg = float(wkAquer_B) / float(wkAquer)
                            wkB_nach_Aquer = str(_format%erg )
                            wkBquer_nach_Aquer = str(1.0 - float(wkB_nach_Aquer) )
                            
                    if _debug2_:    
                        tkMessageBox.showinfo("Teil 4b", wkB_nach_Aquer + "  " + wkBquer_nach_Aquer)
                        tkMessageBox.showinfo("Teil 5", wkAquer_B + "  " + wkAquer_Bquer)
                        
                wkA_B  = str( baum7 )
                wkA_Bquer = str( baum8 )
                wkAquer_B  = str( baum9 )
                wkAquer_Bquer = str( baum10 )
                                  
                if ok: # Schnittwk/Blaetter
                    if (wkA_B == default):
                        erg = float(wkA) * float(wkB_nach_A)
                        wkA_B = str( _format%erg )
                    if (wkA_Bquer == default):
                        erg = float(wkA) * float(wkBquer_nach_A)
                        wkA_Bquer = str( _format%erg )
                    if (wkAquer_B == default):
                        erg = float(wkAquer) * float(wkB_nach_Aquer)
                        wkAquer_B = str( _format%erg )
                    if (wkAquer_Bquer == default):
                        erg = float(wkAquer) * float(wkBquer_nach_Aquer)
                        wkAquer_Bquer = str( _format%erg )
                    
                                    
                if ok:
                    # baue die BaumListe auf
                    teil1 = []
                    teil2 = []
                    teil3 = []
                    teil1.append(_start_)
                    teil1.append( wkA )
                    teil1.append( wkAquer )
                    teil2.append( wkA )
                    teil2.append( wkB_nach_A )
                    teil2.append( wkBquer_nach_A )
                    teil3.append( wkAquer )
                    teil3.append( wkB_nach_Aquer )
                    teil3.append( wkBquer_nach_Aquer )
                    retBaumListe = []
                    retBaumListe.append(teil1)
                    retBaumListe.append(teil2)
                    retBaumListe.append(teil3)
                    if _debug_:    
                        tkMessageBox.showinfo("wk's ", wkA + "  " + wkAquer + "  " + wkB_nach_A + "  " + wkB_nach_Aquer + "  " +  wkBquer_nach_A + "  " + wkBquer_nach_Aquer)

        if ok:
            retLi = prelude(ereignisA,ereignisB)
                        
            wkBaum_dist = wkBaum_standardWert 
            retLi.extend(printItemListe(wkBaum_dist,retBaumListe))
                            
            retLi.extend( blattWk(retBaumListe,-1.0,-1.0,-1.0,-1.0,ereignisB,ereignisA) )

            umgekehrteBaumListe, _ret_ = berechneBlattWk(retBaumListe,-1.0,-1.0,-1.0,-1.0,ereignisA,ereignisB)

            retLi.extend( _ret_ )
            
            wkBaum_dist = wkBaum_standardWert
            retLi.append( '"Umgekehrter Wahrscheinlichkeitsbaum"$' )

            retLi.extend( printItemListe(wkBaum_dist,umgekehrteBaumListe) )
                            
            retLi.extend( blattWk(umgekehrteBaumListe,-1.0,-1.0,-1.0,-1.0,ereignisB,ereignisA) )
            
            retLi.extend( vierfeldertafel(retBaumListe,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,ereignisA,ereignisB) )
                                    
            return retLi
                        
            
##############################################

def matrixEingabe(root,dim):
    """Eingabe der Koeffizientenmatrix fuer Gauss """
    varListe = ['x','y','z','u','v','w','r','s','t']
    koeff_ = tkMatrix.Koeffizienten(root,dimension=dim)
    if koeff_.result <> None:
        #Matrix in koeff_.mtx
        # Beispiel: Dimension:  2
        # koeff_.mtx = [['1', '2', '3'], ['4', '5', '6']]
        variablen = '' # hier wird aufgebaut: [x,y,z]
        ret = 'linsolve(['  # hier wird aufgebaut: [3*x -y =1, 3*y +z =2, x + y - z =0]
        for i in range(dim): # Zeilen der Matrix
            #print str(koeff_.mtx[i])
            for j in range(dim+1): # Spalten der Matrix
                if (j == dim):
                    eintrag = '=' + koeff_.mtx[i][j]
                else:
                    if (j < dim-1):
                        eintrag = koeff_.mtx[i][j] + '*' + varListe[j] + '+'
                    else:
                        eintrag = koeff_.mtx[i][j] + '*' + varListe[j] # vor '=' kein '+'
                ret = ret + eintrag
            if (i < dim-1):    
                ret = ret + ','
                variablen = variablen + varListe[i] + ','
            else:
                variablen = variablen + varListe[i] 
        
        ret = ret + '],[' + variablen + ']);'
        value = ''
        return (ret,value)
    else:
        return ('','')        

##############################################
def mnMatrix(root,zeile,spalte):
    """Eingabe einer m x n Matrix """
    #print zeile,spalte
    title =  str(zeile) + " x " + str(spalte) + " - Matrix"
    objectName = "M1" # Matrix-Namen-Verwaltung?
    koeff_ = tkMatrix.Matrix(root,objectName,title,zeile,spalte)
    if koeff_.result <> None:
        # Matrix in koeff_.mtx
        # print koeff_.mtx        
        # Beispiel: 2 x 3 - Matrix
        # koeff_.mtx = [['1', '2', '3'], ['4', '5', '6']]
        # Ziel: matrix([1,2,3], [5,6,7]);

        objectName = koeff_.name
        #print objectName
        
        ret = 'matrix([' # Matrix aufbauen
        diff = int(zeile) - int(spalte)
        for i in range(int(zeile)): # Zeilen der Matrix
            for j in range(int(spalte)): # Spalten der Matrix
                ret = ret + str(koeff_.mtx[i][j])
                if (j < spalte-1):    
                    ret = ret + ','
                else:
                    ret = ret + ']'
                
            if (i < zeile-1):    
                ret = ret + ',['
            else:
                ret = ret + ');'
        
        value = ''
        #print ret
        ret = objectName + ": " + ret
        return (ret,value)
    else:
        return ('','')        

##############################################
def funkSchar2HTML(root,workingDir,ausdruck,value,parameter,von,bis):
        """generate html-file for animated gif"""
        import html_tools as hts
        import os
        import webbrowser
        plotDateiname = 'gifanim.gif'
        sname = workingDir + 'gifanim.html'    

        data = "Funktionenschar: f_" + parameter + "(" + value + ") = " + ausdruck  + "  (" + str(von) + " <= " + parameter + " <= " + str(bis) + ")"
        dname = "pyMaxima: Funktionenschar"
        zeit =  hts.ucodefilter_( util._DatumZeit_() )
        kopf1 = hts.h_(dname,3,align='center') + '\n'
        kopf2 =  hts.h_(zeit,3,align='center') + '\n'
        arialKopf = hts.font_(kopf1 +  kopf2) + '\n'
        kopf = hts.hr_(80) + '\n' + arialKopf +  hts.hr_(80) + '\n'

        plot = hts.img_(plotDateiname ,'Funktionenschar')
        plotc = hts.center_ ( plot )

        rest = hts.center_ ( hts.b_( hts.pre_(data) ))  + plotc

        textfarbe = '#000000'
        bg = '#efefef'
        html_session = hts.html_('pyMaxima: Funktionenschar',kopf + rest,textfarbe,bg,generated_by="pyMaxima")
        ofp = open(sname,"w") 
        ofp.write(html_session) 
        ofp.flush() 
        ofp.close()
        # jetzt im Browser anzeigen
        try: 
                if os.name == 'posix':
                    try:
                        konqueror = webbrowser.get('kfm')
                        konqueror.open( sname )
                    except:
                        webbrowser.open( sname )
                else:    
                    webbrowser.open( sname )

        except:
                pass
                            
        #self.workingDir = os.path.abspath(os.path.dirname(sname)) + os.sep



def summe(root,expr,varLi,produkt=0):
    if 'i' not in varLi:
        varLi.append('i')
    if 'j' not in varLi:
        varLi.append('j')            
    if 'k' not in varLi:
        varLi.append('k')          
    mydici = {}
    mydici[1] = ('lab1',"von:",'1','0','ein1',"1",'1','1')
    mydici[2] = ('lab2',"bis:",'1','2','ein2',"n",'1','3')
    if produkt:
        eingabe = tkw.eingabeListEx(root,"Produkt",expr,"Term:",varLi,mydici)
    else:
        eingabe = tkw.eingabeListEx(root,"Summe",expr,"Term:",varLi,mydici)
    if eingabe.result <> None:
        ausdruck = string.strip(str(eingabe.result))
        if len(ausdruck) > 0:
            ausdruck = stripStrichpunkt(ausdruck)
            value = str(eingabe.value)
            von = str(eingabe.ret1)
            bis = str(eingabe.ret2)
            ret = 'sum('+  ausdruck  + ',' + value + ',' + von + ',' + bis + '), simpsum;'
            if produkt:
                ret = 'product('+  ausdruck  + ',' + value + ',' + von + ',' + bis + '), simpproduct;'
            else:
                ret = 'sum('+  ausdruck  + ',' + value + ',' + von + ',' + bis + '), simpsum;'
            return (ret,value)
        else:
            return ('','')        
    else:
        return ('','')
            

def limit(root,expr,varLi):
    richtung = None
    mydici = {}
    mydici[1] = ('lab1',"Stelle:",'ein1',"0")
    helpLi = ['inf','minf','%pi','%e']
    eingabe = tkw.eingabe2OptMenu(root,"Grenzwert",expr,varLi,helpLi)
    if eingabe.result <> None:
        ausdruck = string.strip(str(eingabe.result))
        if len(ausdruck) > 0:
            ausdruck = stripStrichpunkt(ausdruck)
            value = str(eingabe.value)
            stelle = str(eingabe.a5)
            ret = 'limit(' +  ausdruck  +',' + value + ',' + stelle + ');'
            return (ret,value)
        else:
            return ('','')        
    else:
        return ('','')
            
            


def taylor(root,expr,varLi):
    """Taylor-Entwicklung um die Stelle x"""
    mydici = {}
    mydici[1] = ('lab1',"Stelle:",'1','0','ein1',"0",'1','1')
    mydici[2] = ('lab2',"Ordnung:",'1','2','ein2',"3",'1','3')
    eingabe = tkw.eingabeListEx(root,"Taylor",expr,"Funktion:",varLi,mydici)
    if eingabe.result <> None:
        ausdruck = string.strip(str(eingabe.result))
        if len(ausdruck) > 0:
            ausdruck = stripStrichpunkt(ausdruck)
            value = str(eingabe.value)
            stelle = str(eingabe.ret1)
            order = str(eingabe.ret2)
            ret = 'taylor('+  ausdruck  + ',' + value + ',' + stelle + ',' + order + ');'
            return (ret,value)
        else:
            return ('','')        
    else:
        return ('','')


def newton(root,expr,varLi):
    """Newton-Verfahren zur Nullstellenbestimmung"""
    mydici = {}
    mydici[1] = ('lab1',"Startwert:",'ein1',"0")
    #mydici[2] = ('lab2',"Epsilon:",'ein2',"0.0001")
    eingabe = tkw.eingabeList(root,"Newton",expr,"Funktion:","Variable:",varLi,mydici)
    if eingabe.result <> None:
        ausdruck = string.strip(str(eingabe.result))
        if len(ausdruck) > 0:
            ausdruck = stripStrichpunkt(ausdruck)
            value = str(eingabe.value)
            startWert = str(eingabe.ret1)
            #epsilon = str(eingabe.ret2)
            ret = 'mnewton([' + ausdruck + '],[' + value + '],[' + startWert + ']);' # ' + epsilon + ');'
            return (ret,value)
        else:
            return ('','')
    else:
        return ('','')
            


def polyDiv(root,expr,varLi):
    """Polynomdivision"""    
    if len(expr) < 1: # expr == ''
        expr = '1'
    mydici = {}
    mydici[1] = ('lab1',"Zählerpolynom:",'0','0','ein1',expr,'0','1')
    mydici[2] = ('lab2',"Nennerpolynom:",'1','0','ein2',"1",'1','1')
    eingabe = tkw.eingabeListEx2(root,"Polynomdivision",expr,varLi,mydici)
    if eingabe.value <> None:
        value = str(eingabe.value)
        dividend = string.strip(str(eingabe.ret1))
        divisor = string.strip(str(eingabe.ret2))            
        if (len(dividend) > 0) and (len(divisor) > 0):
            dividend = stripStrichpunkt(dividend)
            divisor = stripStrichpunkt(divisor)                
            ret = 'divide('  + dividend + ',' + divisor + ',' + value + ');'
            return (ret,value)
        else:
            return ('','')
    else:
        return ('','')


def solveDGL(root,expr,varLi):
    """DGL loesen"""        
    mydici = {}
    if not ('y' in varLi):
        varLi.append('y')
    eingabe = tkw.eingabeDGL(root,"DGL",expr,varLi)
    if eingabe.result <> None:
        ausdruck = string.strip(str(eingabe.result))
        if len(ausdruck) > 0:
            ausdruck = stripStrichpunkt(ausdruck)
            var_x = str(eingabe.var_x)
            var_y = str(eingabe.var_y)
            ret = 'ode2(' + ausdruck +  ',' + var_y +  ',' + var_x + ');'
                        
            return (ret)
        else:
            return ('')
    else:
        return ('')


def dglAWP1(root,expr,varLi):
    """AWP einer DGL 1. Ordnung"""        
    mydici = {}
    if not ('y' in varLi):
        varLi.append('y')
    expr = '%'
    eingabe = tkw.eingabeDglAwp1(root,"DGL-AWP",expr,varLi)
    if eingabe.result <> None:
        ausdruck = string.strip(str(eingabe.result))
        if len(ausdruck) > 0:
            ausdruck = stripStrichpunkt(ausdruck)
            var_x = str(eingabe.var_x)
            val_x = str(eingabe.val_x)
            var_y = str(eingabe.var_y)            
            val_y = str(eingabe.val_y)
            ret = 'ic1(' + ausdruck +  ',' + var_x+'='+ val_x + ',' + var_y +'='+ val_y + ');'                        
            return (ret)
        else:
            return ('')
    else:
        return ('')


def dglAWP2(root,expr,varLi):
    """AWP einer DGL 2. Ordnung"""        
    mydici = {}
    if not ('y' in varLi):
        varLi.append('y')
    expr = '%'
    eingabe = tkw.eingabeDglAwp2(root,"DGL-AWP",expr,varLi)
    if eingabe.result <> None:
        ausdruck = string.strip(str(eingabe.result))
        if len(ausdruck) > 0:
            ausdruck = stripStrichpunkt(ausdruck)
            var_x = str(eingabe.var_x)
            val_x = str(eingabe.val_x)
            var_y = str(eingabe.var_y)            
            val_y = str(eingabe.val_y)
            var_diff = str(eingabe.var_diff)
            val_diff = str(eingabe.val_diff)            
            ret = 'ic2(' + ausdruck +  ',' + var_x+'='+ val_x + ',' + var_y +'='+ val_y + ',' + var_diff+'='+ val_diff + ');'                        
            return (ret)
        else:
            return ('')
    else:
        return ('')


def dglRWP(root,expr,varLi):
    """RWP einer DGL 2. Ordnung"""        
    mydici = {}
    if not ('y' in varLi):
        varLi.append('y')
    expr = '%'
    eingabe = tkw.eingabeDglRwp(root,"DGL-RWP",expr,varLi)
    if eingabe.result <> None:
        ausdruck = string.strip(str(eingabe.result))
        if len(ausdruck) > 0:
            ausdruck = stripStrichpunkt(ausdruck)
            var_x = str(eingabe.var_x)
            val_x = str(eingabe.val_x)
            var_y = str(eingabe.var_y)            
            val_y = str(eingabe.val_y)
            val_x2 = str(eingabe.val_x2)
            val_y2 = str(eingabe.val_y2)            
            ret = 'bc2(' + ausdruck +  ',' + var_x+'='+ val_x + ',' + var_y +'='+ val_y
            ret = ret + ',' + var_x+'='+ val_x2 + ',' + var_y +'='+ val_y2 + ');'                     
            return (ret)
        else:
            return ('')
    else:
        return ('')



def solve(root,expr,varLi):
    """Gleichungen loesen"""        
    mydici = {}
    eingabe = tkw.eingabeList(root,"Gleichungen lösen",expr,"Gleichung:","Variable:",varLi,mydici)
    if eingabe.result <> None:
        ausdruck = string.strip(str(eingabe.result))
        if len(ausdruck) > 0:
            ausdruck = stripStrichpunkt(ausdruck)
            value = str(eingabe.value)
            
            if (string.find(ausdruck,'=') != -1):
                ret = 'solve('+ ausdruck + ',' + value + ');' 
            else:
                ret = 'solve('+ ausdruck +' = 0,' + value + ');' 
            return (ret,value)
        else:
            return ('','')
    else:
        return ('','')





def getMatrixName(root,varLi):
    """holt Matrix-Namen"""
    eingabe = tkw.simpleEingabe(root,"Matrix => Auswahl",varLi)
    if eingabe.result <> None:
        matName = string.strip(str(eingabe.result))
        if len(matName) > 0:            
            value = str(eingabe.value)                
            ret = matName
            return (ret,value)
        else:
            return ('','')
    else:
        return ('','')



def diff(root,expr,varLi):
    """alle anderen Funktionen analog anpassen!"""
    eingabe = tkw.eingabeOptMenu(root,"f'(x)",expr,varLi)
    if eingabe.result <> None:
        ausdruck = string.strip(str(eingabe.result))
        if len(ausdruck) > 0:
            ausdruck = stripStrichpunkt(ausdruck)
            
            value = str(eingabe.value)                
            ret = 'diff('+ ausdruck +',' + value + ');'
            return (ret,value)
        else:
            return ('','')
    else:
        return ('','')


def stamm(root,expr,varLi):
    eingabe = tkw.eingabeOptMenu(root,"Stammfunktion",expr,varLi)
    if eingabe.result <> None:
        ausdruck = string.strip(str(eingabe.result))
        if len(ausdruck) > 0:
            ausdruck = stripStrichpunkt(ausdruck)
            value = str(eingabe.value)
            ret = 'integrate(' + ausdruck +',' + value + ');'
            return (ret,value)
        else:
            return ('','')
    else:
        return ('','')





def rotintegral(root,expr,varLi):
    mydici = {}
    mydici[1] = ('lab1',"von a=",'1','0','ein1',"0",'1','1')
    mydici[2] = ('lab2',"bis b=",'1','2','ein2',"1",'1','3')
    eingabe = tkw.eingabeListEx(root,"Rotationsintegral von a bis b",expr,"Funktion:",varLi,mydici)                
    if eingabe.result <> None:
        ausdruck = string.strip(str(eingabe.result))
        if len(ausdruck) > 0:
            ausdruck = stripStrichpunkt(ausdruck)                
            ausdruck = '%pi * (' + ausdruck +')^2'
            #print ausdruck
            value = str(eingabe.value)
            von = str(eingabe.ret1)
            bis = str(eingabe.ret2)
            ret = 'integrate( %s,%s,%s,%s);' % (ausdruck, value, von, bis)
            return (ret,value)
        else:
            return ('','')
    else:
        return ('','')


def integral(root,expr,varLi):    
    mydici = {}
    mydici[1] = ('lab1',"von a=",'1','0','ein1',"0",'1','1')
    mydici[2] = ('lab2',"bis b=",'1','2','ein2',"1",'1','3')
    eingabe = tkw.eingabeListEx(root,"Integral von a bis b",expr,"Funktion:",varLi,mydici)                
    if eingabe.result <> None:
        ausdruck = string.strip(str(eingabe.result))
        if len(ausdruck) > 0:
            ausdruck = stripStrichpunkt(ausdruck)
            value = str(eingabe.value)
            von = str(eingabe.ret1)
            bis = str(eingabe.ret2)
            ret = 'integrate( %s,%s,%s,%s);' % (ausdruck, value, von, bis)
            return (ret,value)
        else:
            return ('','')
    else:
        return ('','')


def findRoot(root,expr,varLi):    
    mydici = {}
    mydici[1] = ('lab1',"untere Grenze",'1','0','ein1',"-10",'1','1')
    mydici[2] = ('lab2',"obere Grenze",'1','2','ein2',"10",'1','3')
    eingabe = tkw.eingabeListEx(root,"Numerische Lösung",expr, "Gleichung:     " ,varLi,mydici)                
    if eingabe.result <> None:
        ausdruck = string.strip(str(eingabe.result))
        if len(ausdruck) > 0:
            ausdruck = stripStrichpunkt(ausdruck)
            value = str(eingabe.value)
            von = str(eingabe.ret1)
            bis = str(eingabe.ret2)
            ret = 'find_root( %s,%s,%s,%s);' % (ausdruck, value, von, bis)
            return (ret,value)
        else:
            return ('','')
    else:
        return ('','')


def kettenbruch(root):
    retLi = []
    pp = 0
    eingabe = tkw.kettenbruch(root,"Kettenbruch")
    if eingabe.result <> None:
        wert = string.strip(str(eingabe.wert))
        if len(wert) > 0:
            wert = stripStrichpunkt(wert)            
            retLi.append('cf('+ wert + ');')
            pp = str(eingabe.pp )            
            if (pp == "1"):
                retLi.append('cfdisrep(%);')
            return retLi
        else:
            return []
    else:
        return []




def partialBruchZerlegung(root,expr,varLi):
    eingabe = tkw.eingabeOptMenuEx(root,"Partialbruchzerlegung",'%',varLi,'Ausdruck')
    if eingabe.result <> None:
        ausdruck = string.strip(str(eingabe.result))
        if len(ausdruck) > 0:
            ausdruck = stripStrichpunkt(ausdruck)            
            value = str(eingabe.value)                
            ret = 'partfrac('+ ausdruck +',' + value + ');'
            return (ret,value)
        else:
            return ('','')
    else:
        return ('','')


def partialBruch(root,expr,varLi):
    if len(expr) < 1: # expr == ''
        expr = '1'
    mydici = {}
    mydici[1] = ('lab1',"Zählerpolynom:",'0','0','ein1',expr,'0','1')
    mydici[2] = ('lab2',"Nennerpolynom:",'1','0','ein2',"1",'1','1')
    eingabe = tkw.eingabeListEx2(root,"Partialbruchzerlegung",expr,varLi,mydici)
    if eingabe.value <> None:
        #ausdruck = str(eingabe.result)
        #ausdruck = self.stripStrichpunkt(ausdruck)
        value = str(eingabe.value)
        dividend = string.strip(str(eingabe.ret1))
        divisor = string.strip(str(eingabe.ret2))            
        if (len(dividend) > 0) and (len(divisor) > 0):
            dividend = stripStrichpunkt(dividend)
            divisor = stripStrichpunkt(divisor)                
            ret = 'partfrac( ('  + dividend + ')/(' + divisor + '),' + value + ');'
            return (ret,value)
        else:
            return ('','')
    else:
        return ('','')

##############################################
def mehrzeilig(root):
    _debug_ = 0
    from string import join
    myButton = "OK"        
    myTitle = "mehrzeiliger Ausdruck"
    eingabe = tkw.exprFenster(root,'',myTitle)
    if eingabe.result <> None:
        ausdruck = addStrichpunkt( eingabe.result  ) # neu, 2007-01-14
        maxLen = len(ausdruck)
        neu = []
        i = 0
        # Leerzeichen entfernen, leider notwendig, sonst verhaspelt sich maxima!
        while i < (maxLen-1):                
            if (ausdruck[i]==' ') and (ausdruck[i+1]==' '):
                i = i + 1
            elif (ausdruck[i]==' ') and (ausdruck[i+1]!=' '):
                neu.append(ausdruck[i])
                i = i + 1
            else:
                neu.append(ausdruck[i])
                i = i + 1
        #print neu
        ret = "".join(neu)
        return (ret,ausdruck)
    else:
        return ('','')

####################################################
def abstand(root,geoOops,geoObjekt="Gerade"):
    """Abstand Punkt-Gerade/Ebene/Kugel/Kreis/Punkt (je nach Argument geoObjekt)"""                
    if len(geoOops.geoDir) > 0:
        oListe = []
        liste = []
        selectP = []                

        pListe = geoOops.getObjectList("Punkt")
        if len(pListe) > 0:
            
            for element in pListe:
                liste.append(geoOops.getGeoElement(element) )
            if string.find(geoObjekt, "Punkt" ) == -1: # geoObjekt ist kein Punkt!!!
                oListe = geoOops.getObjectList( geoObjekt )
                for element in oListe:
                    liste.append(geoOops.getGeoElement(element) )
                pListe.extend(oListe)    
            # in liste stehen jetzt alle Punkte und Objekte
            headers = ['Select','Name','Typ','Beschreibung']	
            ergebnis = tkw.geoTable(root,liste,headers)  # Objekte anzeigen

            for j in range(len(ergebnis)):  # Punkt und Objekt auswaehlen
                if ergebnis[j]==1:
                    selectP.append(pListe[j])
            if len(selectP) != 2:
                return
            else: # Punkt und Objekt aus {Gerade, Ebene Kugel,Punkt} ausgewaehlt => schneiden?
                #print selectP
                tester = [0,0]  # Test: 1 Objekt und 1 Punkt gewaehlt?
                if string.find(geoObjekt, "Punkt" ) == -1: # geoObjekt ist kein Punkt!!!
                    for element in selectP:
                        test = geoOops.getGeoType(element)
                        #print test, geoObjekt
                        if string.find(test, geoObjekt ) != -1:
                            tester[0] = 1
                        elif string.find(test,'Punkt') != -1:
                            tester[1] = 1
                    erg = tester[0] and tester[1]
                else: # Test: 2 Punkte gewaehlt?
                    for element in selectP:
                        test = geoOops.getGeoType(element)
                        #print test, geoObjekt
                        if string.find(test, geoObjekt ) != -1:
                            tester[0] = 1
                        if string.find(test,'Punkt') != -1:
                            tester[1] = 1
                    erg = tester[0] and tester[1]
                    
                
                if erg != 1:                        
                    return
                else: # =>  1 Objekt und 1 Punkt gewaehlt, es geht los
                    if string.find(geoObjekt, "Punkt" ) == -1: # nicht zwei Punkte gewaehlt
                        for element in selectP:
                            pScript = []
                            pRep = []
                            pScript = geoOops.getScriptEintrag(element)
                            test = geoOops.getGeoType(element)
                            if string.find(test, geoObjekt ) != -1:
                                if string.find(geoObjekt, "Gerade" ) != -1:                                
                                    #print 'gerade',test
                                    pRep = geoOops.script2Intern(pScript,2)                                
                                    aufpunkt = pRep[0]
                                    uvec = descartes.punkt2Gerade(pRep)[1]
                                elif string.find(geoObjekt, "Ebene" ) != -1:
                                    pRep = geoOops.script2Intern(pScript,3)
                                    eIntern = descartes.ebeneNF(pRep)
                                    nvec = eIntern[0]
                                    _d_ = eIntern[1]
                                    #print nvec,_d_
                                elif string.find(geoObjekt, "Kugel" ) != -1:
                                    pRep = geoOops.script2Intern(pScript,2)
                                    mittelpunkt = pRep[0]
                                    rad = descartes.getRadius(mittelpunkt,pRep[1])
                                    #print mittelpunkt,rad
                                elif string.find(geoObjekt, "Kreis" ) != -1:
                                    k_name = geoOops.getGeoName(element)                                    
                                    pRep = geoOops.script2Intern(pScript,3)
                                    eIntern = descartes.ebeneNF(pRep)
                                    nvec = eIntern[0]
                                    _d_ = eIntern[1]
                                    erg = descartes.kreis(k_name,pRep[0],pRep[1],pRep[2])
                                    mittelpunkt = erg[0]
                                    rad = erg[1]
                                else:
                                    pass
                                
                            else: #if string.find(test,'Punkt') == -1:
                                pRep = geoOops.script2Intern(pScript,1)
                                punkt = pRep[0]
                                
                    else: # zwei Punkte gewaehlt!!!
                        punktLi = []
                        for element in selectP:
                            pScript = []
                            pRep = []
                            pScript = geoOops.getScriptEintrag(element)
                            pRep = geoOops.script2Intern(pScript,1)
                            punktLi.append( pRep[0] )
                            
                            
                    ret = ''  # '"Punkt ' + '"$'
                    #return ret
                    if string.find(geoObjekt, "Gerade" ) != -1:                                
                        abstand = descartes.abstandGP(aufpunkt,uvec,punkt)
                        if descartes.nullTest( abstand ):
                            ret = '"Punkt ' + selectP[0] + ' liegt auf der Gerade ' + selectP[1]  + '"$'
                        else:
                            ret = '"Abstand Punkt ' + selectP[0] + ' von Gerade ' + selectP[1]  + ': d = ' + abstand +  '"$'
                        return ret
                        #print abstand
                    elif string.find(geoObjekt, "Ebene" ) != -1:
                        abstand = descartes.abstandEP(nvec,_d_,punkt)
                        if descartes.nullTest( abstand ):
                            ret = '"Punkt ' + selectP[0] + ' liegt in der Ebene ' + selectP[1]  + '"$'
                        else:
                            ret = '"Abstand Punkt ' + selectP[0] + ' von Ebene ' + selectP[1]  + ': d = ' + abstand +  '"$'
                        return ret                        
                    elif string.find(geoObjekt, "Kugel" ) != -1:                                
                        (quadAbstand,abstand) = descartes.abstandKP(mittelpunkt,rad,punkt)
                        if descartes.nullTest( quadAbstand ):
                            ret = '"Punkt ' + selectP[0] + ' liegt auf der Kugel ' + selectP[1]  + '"$'
                        elif ( quadAbstand > 0):
                            ret = '"Punkt ' + selectP[0] + ' liegt außerhalb der Kugel ' + selectP[1]  + ': d = ' + abstand +  '"$'
                        else: # (quadAbstand < 0):
                            ret = '"Punkt ' + selectP[0] + ' liegt innerhalb der Kugel ' + selectP[1]  + ': d = ' + abstand +  '"$'
                        return ret
                    elif string.find(geoObjekt, "Kreis" ) != -1:
                        abstand = descartes.abstandEP(nvec,_d_,punkt)
                        if descartes.nullTest( abstand ):
                            (quadAbstand,abstand) = descartes.abstandKP(mittelpunkt,rad,punkt)                            
                            epsilon=1.0e-5 # leider notwendig!!
                            if descartes.nullTest( quadAbstand, eps = epsilon ):
                                ret = '"Punkt ' + selectP[0] + ' liegt auf dem Kreis ' + selectP[1]  + '"$'
                            elif ( quadAbstand > 0):
                                ret = '"Punkt ' + selectP[0] + ' liegt außerhalb des Kreises ' + selectP[1]  + ': d = ' + abstand +  '"$'
                            else: # (quadAbstand < 0):
                                ret = '"Punkt ' + selectP[0] + ' liegt innerhalb des Kreises ' + selectP[1]  + ': d = ' + abstand +  '"$'
                        else:
                            ret = '"Punkt ' + selectP[0] + ' liegt nicht in der Kreis-Ebene von '  + selectP[1]  +  '!"$'
                        
                        return ret                        
                    
                    elif string.find(geoObjekt, "Punkt" ) != -1:                        
                        abstand = descartes.abstandPP(punktLi[0],punktLi[1])
                        if descartes.nullTest( abstand ):
                            ret = '"Punkt ' + selectP[0] + ' und Punkt ' + selectP[1]  + ' sind identisch!"$'                            
                        else:
                            ret = '"Abstand Punkt ' + selectP[1] + ' von Punkt ' + selectP[0]  + ': d = ' + abstand +  '"$'
                        return ret

####################################################
def kugelAusTangentialebene(root,geoOops,m2g):
    """konstruiert Tangentialebene aus Kugelgleichung und Punkt auf Oberflaeche"""                
    if len(geoOops.geoDir) > 0:
        oListe = []
        liste = []
        selectP = []
        
        geoObjekt="Ebene"
        
        pListe = geoOops.getObjectList("Punkt")
        if len(pListe) > 0:
            
            for element in pListe:
                liste.append(geoOops.getGeoElement(element) )
            
            oListe = geoOops.getObjectList( geoObjekt )
            for element in oListe:
                liste.append(geoOops.getGeoElement(element) )
            pListe.extend(oListe)    
            # in liste stehen jetzt alle Punkte und Ebenen
            headers = ['Select','Name','Typ','Beschreibung']	
            ergebnis = tkw.geoTable(root,liste,headers)  # Objekte anzeigen

            for j in range(len(ergebnis)):  # Punkt und Objekt auswaehlen
                if ergebnis[j]==1:
                    selectP.append(pListe[j])
            if len(selectP) != 2:
                return []
            else: # Punkt und Objekt aus {Gerade, Ebene Kugel} ausgewaehlt => schneiden?
                #print selectP
                            
                tester = [0,0]  # Test: 1 Objekt und 1 Punkt gewaehlt?
                for element in selectP:
                    test = geoOops.getGeoType(element)
                    if string.find(test, geoObjekt ) != -1:
                        tester[0] = 1
                    elif string.find(test,'Punkt') != -1:
                        tester[1] = 1
                erg = tester[0] and tester[1]
                if erg != 1:                        
                    return []
                else: # =>  1 Ebene und 1 Punkt gewaehlt, es geht los
                    #print selectP
                    
                    for element in selectP:
                        pScript = []
                        pRep = []
                        pScript = geoOops.getScriptEintrag(element)
                        test = geoOops.getGeoType(element)
                        if string.find(test, geoObjekt ) != -1:
                            eRep = geoOops.script2Intern(pScript,3)
                            eIntern = descartes.ebeneNF(eRep)
                            nvec = eIntern[0]
                            _d_ = eIntern[1]
                            
                        else: #if string.find(test,'Punkt') == -1:
                            pRep = geoOops.script2Intern(pScript,1)
                            punkt = pRep[0]
                            
                    ret = ''
                    retLi = []
                    abstand = descartes.abstandEP(nvec,_d_,punkt)
                    if descartes.nullTest( abstand ):
                        retLi.append('"Punkt ' + selectP[0] + ' liegt in der Ebene ' + selectP[1]  + '!"$')
                        
                    else:  # Punkt liegt NICHT in der Ebene => es geht los!
                        
                        k_name = 'K_' + selectP[1] + selectP[0]
                        retLi.append('"Kugel '  + k_name + ' mit Mittelpunkt ' + selectP[0]  + ' zur Tangentialebene ' + selectP[1]   + '"$')
                        
                        # >>>>  jetzt Kugel bestimmen!!!  <<<<
                        radius = abstand

                        x_vect = ['x','y','z']
                        xvec_ = descartes.punkt2maxima( x_vect )

                        m_geo = punkt
                        #print m_geo
                        m_punkt = descartes.punkt2maxima( punkt )
                        #print m_punkt

                        o_geo = descartes.genOberflaechenpunkt(m_geo,radius)
                        o_punkt = descartes.punkt2maxima(o_geo)
                        
                        mp_ = 'transpose(' + m_punkt +')'

                        #retLi.append('"Kugel mit Mittelpunkt M und Radius r"$')
                        retLi.append('M : ' + m_punkt +  ';')
                        retLi.append('xvec : ' +  xvec_ + ';')              
                        retLi.append('"Kugelgleichung"$')                               
                        retLi.append('(xvec - M).(xvec - M) = ' + str(radius)  +  '^2;')

                        _colorIndex_ = 0
                        x_vect_ = "[x,y,z]"
                        maxima_rep = '(' + xvec_ + ' - ' + m_punkt + ').('+  x_vect_ + ' - ' + m_punkt + ')  = ' + radius + '^2'
                        #print maxima_rep
                        script = []
                        script.append( m2g.punkt(m_geo,m_punkt,k_name+"m") )
                        script.append( m2g.punkt(o_geo,o_punkt,k_name+"o") )
                        script.append( m2g.kugel(k_name+"m",k_name+"o",maxima_rep,k_name,_colorIndex_) )
                        
                        _objectRep_ = [("rkm", m_geo, float(radius) ),("colorIndex",_colorIndex_)] # rkm: radius Kugel Mittelpunkt
                        
                        maximaObjectRep = '(' + x_vect_ + ' - ' + descartes.vecLi2str(m_geo) + ')^2 = ' + radius + '^2'
                        #print maximaObjectRep
                        _liste_ = geoOops.buildListEntry(k_name,"Kugel",maximaObjectRep, script,_objectRep_ )
                        geoOops.addGeoDirEntry(k_name,_liste_)
                        ########################
                    return retLi                        

####################################################
def tangentialebene(root,geoOops,m2g):
    """konstruiert Tangentialebene aus Kugelgleichung und Punkt auf Oberflaeche"""                
    if len(geoOops.geoDir) > 0:
        oListe = []
        liste = []
        selectP = []
        
        geoObjekt="Kugel"
        
        pListe = geoOops.getObjectList("Punkt")
        if len(pListe) > 0:
            
            for element in pListe:
                liste.append(geoOops.getGeoElement(element) )
            
            oListe = geoOops.getObjectList( geoObjekt )
            for element in oListe:
                liste.append(geoOops.getGeoElement(element) )
            pListe.extend(oListe)    
            # in liste stehen jetzt alle Punkte und Kugel
            headers = ['Select','Name','Typ','Beschreibung']	
            ergebnis = tkw.geoTable(root,liste,headers)  # Objekte anzeigen

            for j in range(len(ergebnis)):  # Punkt und Objekt auswaehlen
                if ergebnis[j]==1:
                    selectP.append(pListe[j])
            if len(selectP) != 2:
                return []
            else: # Punkt und Objekt aus {Gerade, Ebene Kugel} ausgewaehlt => schneiden?
                #print selectP
                            
                tester = [0,0]  # Test: 1 Objekt und 1 Punkt gewaehlt?
                for element in selectP:
                    test = geoOops.getGeoType(element)
                    if string.find(test, geoObjekt ) != -1:
                        tester[0] = 1
                    elif string.find(test,'Punkt') != -1:
                        tester[1] = 1
                erg = tester[0] and tester[1]
                if erg != 1:                        
                    return []
                else: # =>  1 Kugel und 1 Punkt gewaehlt, es geht los
                    for element in selectP:
                        pScript = []
                        pRep = []
                        pScript = geoOops.getScriptEintrag(element)
                        test = geoOops.getGeoType(element)
                        if string.find(test, geoObjekt ) != -1:
                            pRep = geoOops.script2Intern(pScript,2)
                            mittelpunkt = pRep[0]
                            rad = descartes.getRadius(mittelpunkt,pRep[1])
                            
                        else: #if string.find(test,'Punkt') == -1:
                            pRep = geoOops.script2Intern(pScript,1)
                            punkt = pRep[0]
                            
                    ret = ''
                    retLi = []
                    (quadAbstand,abstand) = descartes.abstandKP(mittelpunkt,rad,punkt)
                    if descartes.nullTest( quadAbstand ):
                        e_name = 'T' + selectP[1] + selectP[0]
                        retLi.append('"Tangentialebene ' + e_name + ' an die Kugel ' + selectP[1]  + ' im Punkt ' + selectP[0] + '"$')                                                
                        # >>>>  jetzt Tangentialebene bestimmen!!!  <<<<
                        (n_geo,_d_) = descartes.tangentialebene(mittelpunkt,rad,punkt)
                        
                        d_geo = str(_d_)
                        n_vect_ = descartes.punkt2maxima( n_geo )
                        x_vect = ['x','y','z']
                        xvec_ = descartes.punkt2maxima( x_vect )

                        d_ = "d" + e_name + " : " + d_geo + "$"
                        retLi.append('"x-Vektor xvec"$')
                        retLi.append('xvec : ' +  xvec_ + ';')
                        retLi.append('"Normalenvektor"$')
                        retLi.append('nvec' + e_name  + ' : ' + n_vect_ +  ';')
                        retLi.append('"Abstand d' + e_name  + '"$')
                        retLi.append(d_)
                        retLi.append('koordinatenform(x,n,d) := (print( x . n  = d) )$')
                        retLi.append('"Koordinatenform"$')                
                        retLi.append('koordinatenform(xvec,nvec'+e_name  + ',d'+e_name +')$')

                        # Uebernahme in geoObject-Dir
        		maxima_rep = xvec_ + " . nvec" + e_name + " - d" + e_name + " = 0"
        		
                        s_x = descartes.spurpunkt(n_geo,d_geo,1) # Spurpunkt auf x-Achse
                        s_y = descartes.spurpunkt(n_geo,d_geo,2) # Spurpunkt auf y-Achse
                        s_z = descartes.spurpunkt(n_geo,d_geo,3) # Spurpunkt auf z-Achse
                        uvec_geo = descartes.rvec1(n_geo)
                        vvec_geo = descartes.rvec2(n_geo)
                        auf_geo = descartes.generateAufpunkt(s_x,s_y,s_z)
                        
        		#e_name = 'TE' + selectP[1] + selectP[0]
                        _colorIndex_ = 0
                        x_vect_ = "[x,y,z]"
                        script = []
                        script.append( m2g.richtVec(n_geo,n_vect_,"nvec"+e_name) )		
                        script.append( m2g.normalformEbene(uvec_geo,vvec_geo,auf_geo,d_geo,maxima_rep,e_name,_colorIndex_) )


                        _objectRep_ = [("nebene",n_geo,d_geo),("colorIndex",_colorIndex_)]

                        maximaObjectRep = "( " + x_vect_ + " . " + str(n_geo) + ") - " + d_geo + " = 0"                                                
                        _liste_ = geoOops.buildListEntry(e_name,"Ebene",maximaObjectRep, script,_objectRep_ )
                        geoOops.addGeoDirEntry(e_name,_liste_)
                        
                    else:  # Punkt liegt NICHT auf Kugeloberflaeche!!!
                        retLi.append('"Punkt ' + selectP[0] + ' liegt NICHT auf der Oberfläche der Kugel ' + selectP[1]  + '!"$')
                    return retLi                        
#######################################################
def pol(root,geoOops,m2g):
    """konstruiert zu einer Polarebene p und einer Kugelgleichung den Pol P"""                
    if len(geoOops.geoDir) > 0:
        oListe = []
        liste = []
        selectE = []
        
        geoObjekt="Kugel"
        
        eListe = geoOops.getObjectList("Ebene")
        if len(eListe) > 0:
            
            for element in eListe:
                liste.append(geoOops.getGeoElement(element) )
            
            oListe = geoOops.getObjectList( geoObjekt )
            for element in oListe:
                liste.append(geoOops.getGeoElement(element) )
            eListe.extend(oListe)    
            # in liste stehen jetzt alle Ebenen und Kugeln
            headers = ['Select','Name','Typ','Beschreibung']	
            ergebnis = tkw.geoTable(root,liste,headers)  # Objekte anzeigen

            for j in range(len(ergebnis)):  # Ebene und Kugel auswaehlen
                if ergebnis[j]==1:
                    selectE.append(eListe[j])
            if len(selectE) != 2:
                return []
            else: # Ebene und Kugel ausgewaehlt => bestimme Pol P
                #print selectE
                            
                tester = [0,0]  # Test: 1 Objekt und 1 Ebene gewaehlt?
                for element in selectE:
                    test = geoOops.getGeoType(element)
                    if string.find(test, geoObjekt ) != -1:
                        tester[0] = 1
                    elif string.find(test,'Ebene') != -1:
                        tester[1] = 1
                erg = tester[0] and tester[1]
                if erg != 1:                        
                    return []
                else: # =>  1 Kugel und 1 Ebene gewaehlt, es geht los
                    for element in selectE:
                        #print element
                    
                    
                        eScript = []
                        eRep = []
                        eScript = geoOops.getScriptEintrag(element)
                        test = geoOops.getGeoType(element)
                        if string.find(test, geoObjekt ) != -1:
                            eRep = geoOops.script2Intern(eScript,2)
                            mittelpunkt = eRep[0]
                            rad = descartes.getRadius(mittelpunkt,eRep[1])
                            
                        else: #if string.find(test,'Ebene') == -1:
                            eRep = geoOops.script2Intern(eScript,3)
                            eIntern = descartes.ebeneNF(eRep)
                            nvec = eIntern[0]
                            _d_ = eIntern[1]

                            #print nvec, _d_
                     
                    ret = ''
                    retLi = []
                    polP = descartes.pol(mittelpunkt,rad,nvec,_d_)
                    if polP:
                        #print polP
                        pol_P_ = descartes.punkt2maxima( polP )
                        #print pol_P_
                        p_name = 'P' + selectE[1] + selectE[0]
                        retLi.append('"Pol ' + p_name + ' der Polarebene ' + selectE[0]  + ' zur Kugel ' + selectE[1] + '"$')
                        retLi.append(p_name + ' : ' +  pol_P_ + ';')
                        # Uebernahme in geoObject-Dir
                        script = []
                        script.append( m2g.punkt(polP,pol_P_,p_name,'Pol') )
                        _liste_ = geoOops.buildListEntry(p_name,"Punkt",descartes.vecLi2str(polP), script )
                        geoOops.addGeoDirEntry(p_name,_liste_)                                                                        
                    else:
                        retLi.append('"Ebene ' + selectE[0] + ' ist keine Polarebene der Kugel ' + selectE[1]  + '!"$')                                                    
                            
                    return retLi 

#################################################
def schnittEE(root,geoOops,m2g):
    """Lagebeziehungen zwischen Ebenen"""                     
    if len(geoOops.geoDir) > 0:
        ebenenListe = []
        liste = []
        selectEbene = []
        
        ebenenListe = geoOops.getObjectList("Ebene")
        for element in ebenenListe:
            liste.append(geoOops.getGeoElement(element) )
        # in liste stehen jetzt alle Ebenen
        headers = ['Select','Name','Typ','Beschreibung']	
        ergebnis = tkw.geoTable(root,liste,headers)  # Objekte anzeigen

        for j in range(len(ergebnis)):  # Ebenen auswaehlen
            if ergebnis[j]==1:
                selectEbene.append(ebenenListe[j])
        if len(selectEbene) != 2:
            return
        else: # zwei Ebenen ausgewaehlt => schneiden?
            nfEbene = []

            for ebene in selectEbene:
                ebScript = []
                ebeneRep = []
                ebScript = geoOops.getScriptEintrag(ebene)
                ebeneRep = geoOops.script2Intern(ebScript,3)
                #print ebeneRep # Koordinaten restauriert => 3 Punkte der Ebene
                eIntern = descartes.ebeneNF(ebeneRep) 
                nvec = eIntern[0]
                _d_ = eIntern[1]                
                nfEbene.append([nvec,_d_] )
                aufpunkt = ebeneRep[0]
                
            #print nfEbene[0][0],nfEbene[0][1],nfEbene[1][0],nfEbene[1][1]
                
            schnittLi = descartes.schnittEE(nfEbene[0][0],nfEbene[0][1],nfEbene[1][0],nfEbene[1][1])
            #print schnittLi
            retLi = []
            if schnittLi[0] == 'id': # identisch
                retLi.append('"Die beiden Ebenen ' + selectEbene[0] + ' und ' + selectEbene[1]  + ' sind identisch."$')
            elif schnittLi[0] == 'sg': # Schnittgerade
                aufpunkt_ = descartes.punkt2maxima(schnittLi[1])
                uvec_ =  descartes.punkt2maxima(schnittLi[2])
                retLi.append('"Schnittgerade der Ebenen ' + selectEbene[0] + ' und ' + selectEbene[1]  + '"$')
                retLi.append('display(' +  aufpunkt_  + ' +  r * ' +  uvec_  +')$')
                schnittwinkel = descartes.winkel(nfEbene[0][0],nfEbene[1][0])
                retLi.append('"Schnittwinkel alpha = ' + str(schnittwinkel) + '"$')
                # Uebernahme in geoObject-Dir                        
                uvec_geo = ''  # wird nur als Dummy-Argument benoetigt                        
                script = []
                g_name = 'g' + selectEbene[0] + selectEbene[1]
                script.append( m2g.punkt(schnittLi[1],aufpunkt_,"stuetz"+g_name) )
                script.append( m2g.richtVec(uvec_geo,uvec_,"uvec"+g_name) )                        
                maxima_rep = "stuetz"+g_name+ " + r * ("  + uvec_ + ")"                        
                script.append( m2g.parameterformGerade(schnittLi[2],schnittLi[1],maxima_rep,g_name) )
                maximaObjectRep = descartes.vecLi2str(schnittLi[1]) + " + r * (" + descartes.vecLi2str(schnittLi[2]) + ")" 
                _liste_ = geoOops.buildListEntry(g_name,"Gerade",maximaObjectRep, script )
                geoOops.addGeoDirEntry(g_name,_liste_)

            else: # parallel
                retLi.append('"Die beiden Ebenen ' + selectEbene[0] + ' und ' + selectEbene[1]  + ' sind parallel."$')
                abstand = descartes.abstandEE(nfEbene[0][0],nfEbene[0][1],aufpunkt)
                retLi.append('"Abstand der beiden Ebenen d = ' + str(abstand) + '"$')                        
            return retLi
                    

#################################################
def schnittGE(root,geoOops,m2g):
    """Lagebeziehungen zwischen Ebene und Gerade"""                    
    if len(geoOops.geoDir) > 0:
        egListe = []
        liste = []
        selectEG = []
        

        egListe = geoOops.getObjectList("Ebene")
        for element in egListe:
            liste.append(geoOops.getGeoElement(element) )
        gListe = geoOops.getObjectList("Gerade")
        for element in gListe:
            liste.append(geoOops.getGeoElement(element) )
        egListe.extend(gListe)    
        # in liste stehen jetzt alle Ebenen und Geraden
        headers = ['Select','Name','Typ','Beschreibung']	
        ergebnis = tkw.geoTable(root,liste,headers)  # Objekte anzeigen

        for j in range(len(ergebnis)):  # Geraden auswaehlen
            if ergebnis[j]==1:
                selectEG.append(egListe[j])
        if len(selectEG) != 2:
            return
        else: # zwei Geraden ausgewaehlt => schneiden?
            #print selectEG
            tester = [0,0]  # Test: Gerade und Ebene gewaehlt
            for element in selectEG:
                test = geoOops.getGeoType(element)
                if string.find(test,'Gerade') != -1:
                    tester[0] = 1
                elif string.find(test,'Ebene') != -1:
                    tester[1] = 1
            erg = tester[0] and tester[1]
            if erg != 1:                        
                return
            else: # =>  Gerade und Ebene gewaehlt, es geht los
                #print tester
                
                egIntern = []

                for element in selectEG:
                    egScript = []
                    egRep = []
                    egScript = geoOops.getScriptEintrag(element)
                    #print egScript
                    test = geoOops.getGeoType(element)
                    if string.find(test,'Gerade') != -1:
                        #print 'gerade',test
                        egRep = geoOops.script2Intern(egScript,2)
                        egIntern.append(descartes.punkt2Gerade(egRep) )
                        aufpunkt = egRep[0]     
                    else: #if string.find(test,'Ebene') == -1:
                        #print 'ebene',test
                        egRep = geoOops.script2Intern(egScript,3)
                        egIntern.append(descartes.ebeneNF(egRep) )
                                           
                    #print egRep # Koordinaten restauriert => 2 Punkte der Gerade


                #print egIntern

                schnittLi = descartes.schnittGE(egIntern[1][0],egIntern[1][1],egIntern[0][0],egIntern[0][1])
                #print schnittLi
                #return
                retLi = []
                if schnittLi[0] == 'id': # identisch
                    retLi.append('"Die Gerade ' + selectEG[1] + ' liegt in der Ebene ' + selectEG[0]  + '."$')
                elif schnittLi[0] == 'sc': # Schnittpunkt
                    aufpunkt_ = descartes.punkt2maxima(schnittLi[1])
                    #uvec_ =  descartes.punkt2maxima(schnittLi[2])
                    retLi.append('"Schnittpunkt der Gerade ' + selectEG[1] + ' und der Ebene ' + selectEG[0]  + '"$')
                    retLi.append('sp : ' +  aufpunkt_ + ';')
                    schnittwinkel = descartes.winkelSin(egIntern[0][0],egIntern[1][1])
                    retLi.append('"Schnittwinkel alpha = ' + str(schnittwinkel) + '"$')
                    # Uebernahme in geoObject-Dir
                    script = []
                    a_name = 'SP' + selectEG[0] + selectEG[1] 
                    script.append( m2g.punkt(schnittLi[1],aufpunkt_,a_name,'SP') )
                    _liste_ = geoOops.buildListEntry(a_name,"Punkt",descartes.vecLi2str(schnittLi[1]), script )
                    geoOops.addGeoDirEntry(a_name,_liste_)
                    
                else:  # schnittLi[0] == 'pa': # parallel
                    retLi.append('"Die Gerade ' + selectEG[1] + ' liegt in einer zur Ebene ' + selectEG[0]  + ' parallelen Ebene."$')
                    abstand = descartes.abstandEG(egIntern[0][0],egIntern[0][1],egIntern[1][0],egIntern[1][1])
                    retLi.append('"Abstand Gerade-Ebene d = ' + str(abstand) + '"$')
                return retLi

#################################################
def schnittGG(root,geoOops,m2g):
    """Lagebeziehungen zwischen Geraden"""                
    if len(geoOops.geoDir) > 0:
        geradenListe = []
        liste = []
        selectGerade = []
        
        geradenListe = geoOops.getObjectList("Gerade")
        for element in geradenListe:
            liste.append(geoOops.getGeoElement(element) )
        # in liste stehen jetzt alle Geraden
        headers = ['Select','Name','Typ','Beschreibung']	
        ergebnis = tkw.geoTable(root,liste,headers)  # Objekte anzeigen

        for j in range(len(ergebnis)):  # Geraden auswaehlen
            if ergebnis[j]==1:
                selectGerade.append(geradenListe[j])
        if len(selectGerade) != 2:
            return
        else: # zwei Geraden ausgewaehlt => schneiden?
            geradeIntern = []

            for gerade in selectGerade:
                geScript = []
                geradenRep = []
                geScript = geoOops.getScriptEintrag(gerade)
                #print geScript
                geradenRep = geoOops.script2Intern(geScript,2)
                #print geradenRep # Koordinaten restauriert => 2 Punkte der Gerade
                geradeIntern.append(descartes.punkt2Gerade(geradenRep) )
##                        aufpunkt = ebeneRep[0]
            #print geradeIntern
            schnittLi = descartes.schnittGG(geradeIntern[0][0],geradeIntern[0][1],geradeIntern[1][0],geradeIntern[1][1])
            #print schnittLi
            retLi = []
            if schnittLi[0] == 'id': # identisch
                retLi.append('"Die beiden Geraden ' + selectGerade[0] + ' und ' + selectGerade[1]  + ' sind identisch."$')
            elif schnittLi[0] == 'sc': # Schnittpunkt
                aufpunkt_ = descartes.punkt2maxima(schnittLi[1])
                #uvec_ =  descartes.punkt2maxima(schnittLi[2])
                retLi.append('"Schnittpunkt der Geraden ' + selectGerade[0] + ' und ' + selectGerade[1]  + '"$')
                retLi.append('sp : ' +  aufpunkt_ + ';')
                schnittwinkel = descartes.winkel(geradeIntern[0][1],geradeIntern[1][1])
                retLi.append('"Schnittwinkel alpha = ' + str(schnittwinkel) + '"$')
                # Uebernahme in geoObject-Dir
                script = []
                a_name = 'SP' + selectGerade[0] + selectGerade[1] 
                script.append( m2g.punkt(schnittLi[1],aufpunkt_,a_name,'SP') )
                _liste_ = geoOops.buildListEntry(a_name,"Punkt",descartes.vecLi2str(schnittLi[1]), script )
                geoOops.addGeoDirEntry(a_name,_liste_)
                
            elif schnittLi[0] == 'pa': # parallel
                retLi.append('"Die beiden Geraden ' + selectGerade[0] + ' und ' + selectGerade[1] + ' sind parallel."$')
                abstand = descartes.abstandGG(geradeIntern[0][0],geradeIntern[0][1],geradeIntern[1][0],geradeIntern[1][1])
                retLi.append('"Abstand der beiden Geraden d = ' + str(abstand) + '"$')
            else:  # windschief
                retLi.append('"Die beiden Geraden ' + selectGerade[0] + ' und ' + selectGerade[1] + ' sind windschief."$')
                abstand = descartes.abstandGG(geradeIntern[0][0],geradeIntern[0][1],geradeIntern[1][0],geradeIntern[1][1])
                retLi.append('"Abstand der beiden Geraden d = ' + str(abstand) + '"$')
            return retLi

#################################################
def schnittKG(root,geoOops,m2g):
    """Schnitt Kugel-Gerade"""

    if len(geoOops.geoDir) > 0:
        kgListe = []
        liste = []
        selectKG = []
        

        kgListe = geoOops.getObjectList("Kugel")
        for element in kgListe:
            liste.append(geoOops.getGeoElement(element) )
        gListe = geoOops.getObjectList("Gerade")
        for element in gListe:
            liste.append(geoOops.getGeoElement(element) )
        kgListe.extend(gListe)    
        # in liste stehen jetzt alle Ebenen und Geraden
        headers = ['Select','Name','Typ','Beschreibung']	
        ergebnis = tkw.geoTable(root,liste,headers)  # Objekte anzeigen

        for j in range(len(ergebnis)):  # Geraden auswaehlen
            if ergebnis[j]==1:
                selectKG.append(kgListe[j])
        if len(selectKG) != 2:
            return
        else: # zwei Geraden ausgewaehlt => schneiden?
            #print selectKG
            tester = [0,0]  # Test: Gerade und Ebene gewaehlt
            for element in selectKG:
                test = geoOops.getGeoType(element)
                if string.find(test,'Gerade') != -1:
                    tester[0] = 1
                elif string.find(test,'Kugel') != -1:
                    tester[1] = 1
            erg = tester[0] and tester[1]
            if erg != 1:                        
                return
            else: # =>  Gerade und Kugel gewaehlt, es geht los
                #print tester
                
                kgIntern = []

                for element in selectKG:
                    kgScript = []
                    kgRep = []
                    kgScript = geoOops.getScriptEintrag(element)
                    #print kgScript
                    test = geoOops.getGeoType(element)
                    if string.find(test,'Gerade') != -1:
                        #print 'gerade',test
                        kgRep = geoOops.script2Intern(kgScript,2)
                        kgIntern.append(descartes.punkt2Gerade(kgRep) )
                        aufpunkt = kgRep[0]     
                    else: #if string.find(test,'Kugel') == -1:
                        kgRep = geoOops.script2Intern(kgScript,2)
                        mittelpunkt = kgRep[0]
                        rad = descartes.getRadius(mittelpunkt,kgRep[1])
                                                                                                           
                    #print kgRep # Koordinaten restauriert => 2 Punkte der Gerade

                retLi = []
                schnittLi = descartes.schnittKG(mittelpunkt,rad,aufpunkt,kgIntern[0][1])
                if len(schnittLi) == 0: # Passante
                    retLi.append('"Es gibt keinen Schnittpunkt der Kugel ' + selectKG[0] + ' mit der Geraden ' + selectKG[1]  + '"$')
                    retLi.append('"Die Gerade ' + selectKG[1]  + ' ist eine Passante."$')
                elif len(schnittLi) == 1: # Tangente
                    beruehrpunkt = descartes.punkt2maxima(schnittLi[0])
                    retLi.append('"Berührpunkt der Kugel ' + selectKG[0] + ' mit der Geraden ' + selectKG[1]  + ' "$')
                    retLi.append('bp : ' +  beruehrpunkt + ';')
                    # Uebernahme in geoObject-Dir
                    script = []
                    s_name = 'BP' + selectKG[0] + selectKG[1] 
                    script.append( m2g.punkt(schnittLi[0],beruehrpunkt,s_name,'BP') )
                    _liste_ = geoOops.buildListEntry(s_name,"Punkt",descartes.vecLi2str(schnittLi[0]), script )
                    geoOops.addGeoDirEntry(s_name,_liste_)
                else: # Sekante!
                    schnittpunkt0 = descartes.punkt2maxima(schnittLi[0])
                    schnittpunkt1 = descartes.punkt2maxima(schnittLi[1])                            
                    retLi.append('"Schnittpunkte der Kugel ' + selectKG[0] + ' mit der Geraden ' + selectKG[1]  + ' "$')
                    retLi.append('sp1 : ' +  schnittpunkt0 + ';')
                    retLi.append('sp2 : ' +  schnittpunkt1 + ';')                            
                    # Uebernahme in geoObject-Dir
                    script = []
                    s_name = 'SP1' + selectKG[0] + selectKG[1] 
                    script.append( m2g.punkt(schnittLi[0],schnittpunkt0,s_name,'SP1') )
                    _liste_ = geoOops.buildListEntry(s_name,"Punkt",descartes.vecLi2str(schnittLi[0]), script )
                    geoOops.addGeoDirEntry(s_name,_liste_)
                    script = []
                    s_name = 'SP2' + selectKG[0] + selectKG[1] 
                    script.append( m2g.punkt(schnittLi[1],schnittpunkt1,s_name,'SP2') )
                    _liste_ = geoOops.buildListEntry(s_name,"Punkt",descartes.vecLi2str(schnittLi[1]), script )
                    geoOops.addGeoDirEntry(s_name,_liste_)
                return retLi
                    
#################################################
def schnittKE(root,geoOops,m2g):
    """Schnitt Kugel-Ebene"""

    if len(geoOops.geoDir) > 0:
        keListe = []
        liste = []
        selectKE = []
        

        keListe = geoOops.getObjectList("Kugel")
        for element in keListe:
            liste.append(geoOops.getGeoElement(element) )
        eListe = geoOops.getObjectList("Ebene")
        for element in eListe:
            liste.append(geoOops.getGeoElement(element) )
        keListe.extend(eListe)    
        # in liste stehen jetzt alle Ebenen und Kugeln
        headers = ['Select','Name','Typ','Beschreibung']	
        ergebnis = tkw.geoTable(root,liste,headers)  # Objekte anzeigen

        for j in range(len(ergebnis)):  # Ebene/Kugel auswaehlen
            if ergebnis[j]==1:
                selectKE.append(keListe[j])
        if len(selectKE) != 2:
            return
        else: # Ebene und Kugel gewaehlt
            #print selectKE
            tester = [0,0]  # Test: 1 Kugel und  1 Ebene gewaehlt
            for element in selectKE:
                test = geoOops.getGeoType(element)
                if string.find(test,'Ebene') != -1:
                    tester[0] = 1
                elif string.find(test,'Kugel') != -1:
                    tester[1] = 1
            erg = tester[0] and tester[1]
            if erg != 1:                        
                return
            else: # =>  Ebene und Kugel gewaehlt, es geht los
                
                keIntern = []
                for element in selectKE:
                    keScript = []
                    keRep = []
                    keScript = geoOops.getScriptEintrag(element)
                    #print keScript
                    #print
                    test = geoOops.getGeoType(element)
                    if string.find(test,'Ebene') != -1:
                        keRep = geoOops.script2Intern(keScript,3)
                        #print "keRep => Ebene"                        
                        #print keRep
                        keIntern = descartes.ebeneNF(keRep) 
                        nvec = keIntern[0]
                        _d_ = keIntern[1]

                        #print "nvec, _d_"
                        #print nvec
                        #print _d_
                        
                    else: #if string.find(test,'Kugel') == -1:
                        keRep = geoOops.script2Intern(keScript,2)
                        mittelpunkt = keRep[0]
                        rad = descartes.getRadius(mittelpunkt,keRep[1])
                        #print mittelpunkt
                        #print rad                                                                                                           

                schnittLi = descartes.schnittKE(mittelpunkt,rad,nvec,_d_)
                #print schnittLi
                #return []
                
                retLi = []
                if len(schnittLi) == 0: # Passante
                    retLi.append('"Die Ebene ' + selectKE[1] + ' berührt die Kugel ' + selectKE[0] + ' nicht."$')
                elif len(schnittLi) == 1: # Tangentialebene
                    beruehrpunkt = descartes.punkt2maxima(schnittLi[0])
                    retLi.append('"Die Ebene ' + selectKE[1] + ' ist Tangentialebene der Kugel ' + selectKE[0] + ', Berührpunkt bp"$')
                    retLi.append('bp : ' +  beruehrpunkt + ';')
                    # Uebernahme in geoObject-Dir
                    script = []
                    s_name = 'BP' + selectKE[0] + selectKE[1] 
                    script.append( m2g.punkt(schnittLi[0],beruehrpunkt,s_name,'BP') )
                    _liste_ = geoOops.buildListEntry(s_name,"Punkt",descartes.vecLi2str(schnittLi[0]), script )
                    geoOops.addGeoDirEntry(s_name,_liste_)
                else: # Schnittkreis!
                    cPunktLi = schnittLi[2] # drei Punkte auf Kreislinie
                    #print len(cPunktLi)
                    #print str(cPunktLi)                    
                    xvec = descartes.punkt2maxima( ['x','y','z'] )
                    m_punkt = descartes.punkt2maxima(schnittLi[0])                            
                    rad = "%16.12f" % schnittLi[1]
                    retLi.append('"Die Ebene ' + selectKE[1] + ' schneidet die Kugel ' + selectKE[0] + '"$')
                    retLi.append('mp : ' +  m_punkt + ';')
                    retLi.append('xvec : ' +  xvec + ';')
                    retLi.append('"Schnittkreis-Gleichung sk:"$')                            
                    retLi.append('sk : (xvec - mp) . (xvec - mp) = ' + rad + '^2;')

                    # Uebernahme in geoObject-Dir
                    script = []
                    s_name = 'MP' + selectKE[0] + selectKE[1] 
                    script.append( m2g.punkt(schnittLi[0],m_punkt,s_name,'MP') )
                    _liste_ = geoOops.buildListEntry(s_name,"Punkt",descartes.vecLi2str(schnittLi[0]), script )
                    geoOops.addGeoDirEntry(s_name,_liste_)
                    script = []
                    s_name = 'SK' + selectKE[0] + selectKE[1]
                    maxima_rep = '(' + xvec  + ' - ' + m_punkt + ') . (' + xvec  + ' - ' + m_punkt + ') = ' + rad + '^2'                        
                    maxima = '[x - ' + descartes.vecLi2str(schnittLi[0]) + ']^2 = ' + rad + '^2'
                    #print maxima
##                    ==>> ALT!                    
##                    script.append( m2g.schnittkreisE(selectKE[0],selectKE[1],maxima_rep,s_name) )
##                    _liste_ = geoOops.buildListEntry(s_name,"Kreis",maxima, script )
##                    geoOops.addGeoDirEntry(s_name,_liste_)
                    
                    # neu: Kreis durch drei Punkte                    
                    _colorIndex_ = 0                                          
                    _objectRep_ = [("k3p",cPunktLi[0],cPunktLi[1],cPunktLi[2] ),("colorIndex",_colorIndex_)] #
                
                    script = []
                    cP0_ = descartes.punkt2maxima(cPunktLi[0])
                    #print cP0_
                    cP1_ = descartes.punkt2maxima(cPunktLi[1])
                    cP2_ = descartes.punkt2maxima(cPunktLi[2])
                    script.append( m2g.punkt(cPunktLi[0],cP0_,s_name+"p1") )
                    script.append( m2g.punkt(cPunktLi[1],cP1_,s_name+"p2") )
                    script.append( m2g.punkt(cPunktLi[2],cP2_,s_name+"p3") )                    
                    script.append( m2g.kreis(s_name+"p1",s_name+"p2",s_name+"p3",maxima_rep,s_name,_colorIndex_)  )                        

                    _liste_ = geoOops.buildListEntry(s_name,"Kreis",maxima, script, _objectRep_ )
                    
                    geoOops.addGeoDirEntry(s_name,_liste_)

                    
                return retLi
                        



############################################
def schnittKK(root,geoOops,m2g):            
    """Schnitt Kugel-Kugel"""
    if len(geoOops.geoDir) > 0:
        kkListe = []
        liste = []
        selectKK = []
        
        kkListe = geoOops.getObjectList("Kugel")
        for element in kkListe:
            liste.append(geoOops.getGeoElement(element) )
        # in liste stehen jetzt alle Kugeln
        headers = ['Select','Name','Typ','Beschreibung']	
        ergebnis = tkw.geoTable(root,liste,headers)  # Objekte anzeigen

        for j in range(len(ergebnis)):  # Kugeln auswaehlen
            if ergebnis[j]==1:
                selectKK.append(kkListe[j])
        if len(selectKK) != 2:
            return
        else: # zwei Kugeln ausgewaehlt => schneiden?
            #print selectKK
                
            kkIntern = []
            for element in selectKK:
                kkScript = []
                kkRep = []
                kkScript = geoOops.getScriptEintrag(element)
                test = geoOops.getGeoType(element)
                kkRep = geoOops.script2Intern(kkScript,2)
                mittelpunkt = kkRep[0]
                rad = descartes.getRadius(mittelpunkt,kkRep[1])
                kkIntern.append([mittelpunkt,rad])

            schnittLi = descartes.schnittKK(kkIntern[0][0],kkIntern[0][1],kkIntern[1][0],kkIntern[1][1])
            #print schnittLi
            retLi = []
            if len(schnittLi) == 0: # Schnittmenge leer
                retLi.append('"Die Kugel ' + selectKK[0] + ' schneidet/berührt die Kugel ' + selectKK[1] + ' nicht!"$')
                
            elif len(schnittLi) == 1: # Beruehrpunkt
                beruehrpunkt = descartes.punkt2maxima(schnittLi[0])
                retLi.append('"Die Kugel ' + selectKK[0] + ' berührt die Kugel ' + selectKK[1] + ', Berührpunkt bp"$')
                retLi.append('bp : ' +  beruehrpunkt + ';')
                # Uebernahme in geoObject-Dir
                script = []
                s_name = 'BP' + selectKK[0] + selectKK[1] 
                script.append( m2g.punkt(schnittLi[0],beruehrpunkt,s_name,'BP') )
                _liste_ = geoOops.buildListEntry(s_name,"Punkt",descartes.vecLi2str(schnittLi[0]), script )
                geoOops.addGeoDirEntry(s_name,_liste_)
                
            elif len(schnittLi) == 2: # Kugeln identisch!
                retLi.append('"Die beiden Kugeln ' + selectKK[0] + ' und ' + selectKK[1] + ' sind identisch!"$')
                
            else: # Schnittkreis! ==> len(schnittLi) == 3 !!!
                cPunktLi = schnittLi[2] # drei Punkte auf Kreislinie
                
                xvec = descartes.punkt2maxima( ['x','y','z'] )
                m_punkt = descartes.punkt2maxima(schnittLi[0])                            
                rad = "%16.12f" % schnittLi[1]
                retLi.append('"Die Kugel ' + selectKK[0] + ' schneidet die Kugel ' + selectKK[1] + '"$')
                retLi.append('mp : ' +  m_punkt + ';')
                retLi.append('xvec : ' +  xvec + ';')
                retLi.append('"Schnittkreis-Gleichung sk:"$')                            
                retLi.append('sk : (xvec - mp) . (xvec - mp) = ' + rad + '^2;')

                # Uebernahme in geoObject-Dir
                script = []
                s_name = 'MP' + selectKK[0] + selectKK[1] 
                script.append( m2g.punkt(schnittLi[0],m_punkt,s_name,'MP') )
                _liste_ = geoOops.buildListEntry(s_name,"Punkt",descartes.vecLi2str(schnittLi[0]), script )
                geoOops.addGeoDirEntry(s_name,_liste_)
                script = []
                s_name = 'SK' + selectKK[0] + selectKK[1]
                maxima_rep = '(' + xvec  + ' - ' + m_punkt + ') . (' + xvec  + ' - ' + m_punkt + ') = ' + rad + '^2'                        
                maxima = '[x - ' + descartes.vecLi2str(schnittLi[0]) + ']^2 = ' + rad + '^2'
                #print maxima
##                ==>> ALT!                  
##                script.append( m2g.schnittkreis(selectKK[0],selectKK[1],maxima_rep,s_name) )
##                _liste_ = geoOops.buildListEntry(s_name,"Kreis",maxima, script )
##                geoOops.addGeoDirEntry(s_name,_liste_)                    
                  
                # neu: Kreis durch drei Punkte                    
                _colorIndex_ = 0                                          
                _objectRep_ = [("k3p",cPunktLi[0],cPunktLi[1],cPunktLi[2]),("colorIndex",_colorIndex_)] #
            
                script = []
                cP0_ = descartes.punkt2maxima(cPunktLi[0])
                #print cP0_
                cP1_ = descartes.punkt2maxima(cPunktLi[1])
                cP2_ = descartes.punkt2maxima(cPunktLi[2])
                script.append( m2g.punkt(cPunktLi[0],cP0_,s_name+"p1") )
                script.append( m2g.punkt(cPunktLi[1],cP1_,s_name+"p2") )
                script.append( m2g.punkt(cPunktLi[2],cP2_,s_name+"p3") )                    
                script.append( m2g.kreis(s_name+"p1",s_name+"p2",s_name+"p3",maxima_rep,s_name,_colorIndex_)  )                        

                _liste_ = geoOops.buildListEntry(s_name,"Kreis",maxima, script, _objectRep_ )
                geoOops.addGeoDirEntry(s_name,_liste_)
                
            return retLi


#################################################
def schnittCG(root,geoOops,m2g):
    """Schnitt Kreis-Gerade (C: Circle)"""

    if len(geoOops.geoDir) > 0:
        kgListe = []
        liste = []
        selectKG = []
        

        kgListe = geoOops.getObjectList("Kreis")
        for element in kgListe:
            liste.append(geoOops.getGeoElement(element) )
        gListe = geoOops.getObjectList("Gerade")
        for element in gListe:
            liste.append(geoOops.getGeoElement(element) )
        kgListe.extend(gListe)    
        # in liste stehen jetzt alle Ebenen und Geraden
        headers = ['Select','Name','Typ','Beschreibung']	
        ergebnis = tkw.geoTable(root,liste,headers)  # Objekte anzeigen

        for j in range(len(ergebnis)):  # Geraden auswaehlen
            if ergebnis[j]==1:
                selectKG.append(kgListe[j])
        if len(selectKG) != 2:
            return
        else: # 
            #print selectKG
            tester = [0,0]  # Test: Gerade und Kreis gewaehlt
            for element in selectKG:
                test = geoOops.getGeoType(element)
                if string.find(test,'Gerade') != -1:
                    tester[0] = 1
                elif string.find(test,'Kreis') != -1:
                    tester[1] = 1
            erg = tester[0] and tester[1]
            if erg != 1:                        
                return
            else: # =>  Gerade und Kreis gewaehlt, es geht los
                #print tester
                
                kgIntern = []

                for element in selectKG:
                    kgScript = []
                    kgRep = []
                    kgScript = geoOops.getScriptEintrag(element)
                    #print kgScript
                    test = geoOops.getGeoType(element)
                    if string.find(test,'Gerade') != -1:
                        #print 'gerade',test
                        kgRep = geoOops.script2Intern(kgScript,2)
                        kgIntern.append(descartes.punkt2Gerade(kgRep) )
                        aufpunkt = kgRep[0]     
                    else: #if string.find(test,'Kreis') == -1:
                        k_name = geoOops.getGeoName(element)
                        #print kgScript
                        pRep = geoOops.script2Intern(kgScript,3)
                        eIntern = descartes.ebeneNF(pRep)
                        nvec = eIntern[0]
                        _d_ = eIntern[1]
                        erg = descartes.kreis(k_name,pRep[0],pRep[1],pRep[2])
                        mittelpunkt = erg[0]
                        rad = erg[1]

                retLi = []
                schnittLi = descartes.schnittGE(aufpunkt,kgIntern[0][1],nvec,_d_)
                #print str(schnittLi)                
                if schnittLi[0] == 'id': # identisch
                    retLi.append('"Die Gerade ' + selectKG[1] + ' liegt in der Kreis-Ebene ' + selectKG[0]  + '."$')
                    schnittLi = []
                    schnittLi = descartes.schnittKG(mittelpunkt,rad,aufpunkt,kgIntern[0][1])
                    if len(schnittLi) == 0: # Passante
                        retLi.append('"Es gibt keinen Schnittpunkt des Kreises ' + selectKG[0] + ' mit der Geraden ' + selectKG[1]  + '"$')
                        retLi.append('"Die Gerade ' + selectKG[1]  + ' ist eine Passante."$')
                    elif len(schnittLi) == 1: # Tangente
                        beruehrpunkt = descartes.punkt2maxima(schnittLi[0])
                        retLi.append('"Berührpunkt des Kreises ' + selectKG[0] + ' mit der Geraden ' + selectKG[1]  + ' "$')
                        retLi.append('bp : ' +  beruehrpunkt + ';')
                        # Uebernahme in geoObject-Dir
                        script = []
                        s_name = 'BP' + selectKG[0] + selectKG[1] 
                        script.append( m2g.punkt(schnittLi[0],beruehrpunkt,s_name,'BP') )
                        _liste_ = geoOops.buildListEntry(s_name,"Punkt",descartes.vecLi2str(schnittLi[0]), script )
                        geoOops.addGeoDirEntry(s_name,_liste_)
                    else: # Sekante!
                        schnittpunkt0 = descartes.punkt2maxima(schnittLi[0])
                        schnittpunkt1 = descartes.punkt2maxima(schnittLi[1])                            
                        retLi.append('"Schnittpunkte des Kreises ' + selectKG[0] + ' mit der Geraden ' + selectKG[1]  + ' "$')
                        retLi.append('sp1 : ' +  schnittpunkt0 + ';')
                        retLi.append('sp2 : ' +  schnittpunkt1 + ';')                            
                        # Uebernahme in geoObject-Dir
                        script = []
                        s_name = 'SP1' + selectKG[0] + selectKG[1] 
                        script.append( m2g.punkt(schnittLi[0],schnittpunkt0,s_name,'SP1') )
                        _liste_ = geoOops.buildListEntry(s_name,"Punkt",descartes.vecLi2str(schnittLi[0]), script )
                        geoOops.addGeoDirEntry(s_name,_liste_)
                        script = []
                        s_name = 'SP2' + selectKG[0] + selectKG[1] 
                        script.append( m2g.punkt(schnittLi[1],schnittpunkt1,s_name,'SP2') )
                        _liste_ = geoOops.buildListEntry(s_name,"Punkt",descartes.vecLi2str(schnittLi[1]), script )
                        geoOops.addGeoDirEntry(s_name,_liste_)
                    
                    #schnittLi = descartes.schnittCG(mittelpunkt,rad,aufpunkt,kgIntern[0][1])
                elif schnittLi[0] == 'sc': # Schnittpunkt
                    schnittpunkt = schnittLi[1]
                    aufpunkt_ = descartes.punkt2maxima(schnittpunkt)
                    (quadAbstand,abstand) = descartes.abstandKP(mittelpunkt,rad,schnittpunkt)                            
                    epsilon=1.0e-4 # leider notwendig!!
                    #print quadAbstand
                    if descartes.nullTest( quadAbstand, eps = epsilon ):
                        retLi.append('"Die Gerade '  + selectKG[1] + ' schneidet den Kreis ' + selectKG[0]  + ' im Punkt ' + str(schnittpunkt) + '"$')
                        # Uebernahme in geoObject-Dir
                        script = []
                        a_name = 'SP' + selectKG[0] + selectKG[1] 
                        script.append( m2g.punkt(schnittpunkt,aufpunkt_,a_name,'SP') )
                        _liste_ = geoOops.buildListEntry(a_name,"Punkt",descartes.vecLi2str(schnittpunkt), script )
                        geoOops.addGeoDirEntry(a_name,_liste_)
                        
                    else: # (quadAbstand < 0):
                        retLi.append('"Die Gerade '  + selectKG[1] + ' schneidet den Kreis ' + selectKG[0]  + ' nicht!"$')
                    
                else:  # schnittLi[0] == 'pa': # parallel
                    retLi.append('"Die Gerade ' + selectKG[1] + ' liegt in einer zur Kreis-Ebene ' + selectKG[0]  + ' parallelen Ebene: Kein Schnittpunkt!"$')                

                return retLi
                    
#################################################
def schnittCE(root,geoOops,m2g):
    """Schnitt Kreis-Ebene (C: Circle)"""

    if len(geoOops.geoDir) > 0:
        keListe = []
        liste = []
        selectKE = []
        

        keListe = geoOops.getObjectList("Kreis")
        for element in keListe:
            liste.append(geoOops.getGeoElement(element) )
        eListe = geoOops.getObjectList("Ebene")
        for element in eListe:
            liste.append(geoOops.getGeoElement(element) )
        keListe.extend(eListe)    
        # in liste stehen jetzt alle Ebenen und Kugeln
        headers = ['Select','Name','Typ','Beschreibung']	
        ergebnis = tkw.geoTable(root,liste,headers)  # Objekte anzeigen

        for j in range(len(ergebnis)):  # Ebene/Kugel auswaehlen
            if ergebnis[j]==1:
                selectKE.append(keListe[j])
        if len(selectKE) != 2:
            return
        else: # Ebene und Kugel gewaehlt
            #print selectKE
            tester = [0,0]  # Test: 1 Kugel und  1 Ebene gewaehlt
            for element in selectKE:
                test = geoOops.getGeoType(element)
                if string.find(test,'Ebene') != -1:
                    tester[0] = 1
                elif string.find(test,'Kreis') != -1:
                    tester[1] = 1
            erg = tester[0] and tester[1]
            if erg != 1:                        
                return
            else: # =>  Ebene und Kugel gewaehlt, es geht los
                #print selectKE
                keIntern = []
                for element in selectKE:
                    keScript = []
                    keRep = []
                    keScript = geoOops.getScriptEintrag(element)
                    #print keScript
                    #print
                    test = geoOops.getGeoType(element)
                    if string.find(test,'Ebene') != -1:
                        keRep = geoOops.script2Intern(keScript,3)
                        #print "keRep => Ebene"                        
                        #print keRep
                        keIntern = descartes.ebeneNF(keRep) 
                        nvec = keIntern[0]
                        _d_ = keIntern[1]

                        #print "nvec, _d_"
                        #print nvec
                        #print _d_
                        
                    else: #if string.find(test,'Kreis') == -1:
                        k_name = geoOops.getGeoName(element)                                    
                        pRep = geoOops.script2Intern(keScript,3)
                        eIntern = descartes.ebeneNF(pRep)
                        nvecKr = eIntern[0]
                        _d_Kr = eIntern[1]
                        erg = descartes.kreis(k_name,pRep[0],pRep[1],pRep[2])
                        mittelpunkt = erg[0]
                        rad = erg[1]


                schnittLi = descartes.schnittEE(nvec,_d_,nvecKr,_d_Kr)                
                #print str(schnittLi)
                #return ['"Schnitt Kreis <==> Ebene $"']

            
                retLi = []
                if schnittLi[0] == 'id': # identisch
                    retLi.append('"Der Kreis ' + selectKE[0] + ' liegt in der Ebene ' + selectKE[1]  + '."$')
                elif schnittLi[0] == 'sg': # Schnittgerade
                    aufpunkt_ = schnittLi[1]
                    uvec_ =  schnittLi[2]
                    schnittLi = []
                    schnittLi = descartes.schnittKG(mittelpunkt,rad,aufpunkt_,uvec_)
                    if len(schnittLi) == 0: # Passante
                        retLi.append('"Es gibt keine gemeinsamen Punkte zwischen dem Kreis ' + selectKE[0] + ' und der Ebene ' + selectKE[1]  + '"$')
                    elif len(schnittLi) == 1: # Tangente
                        beruehrpunkt = descartes.punkt2maxima(schnittLi[0])
                        retLi.append('"Der Kreis ' + selectKE[0] + ' berührt die Ebene ' + selectKE[1]  + ' im Punkt bp"$')
                        retLi.append('bp : ' +  beruehrpunkt + ';')
                        # Uebernahme in geoObject-Dir
                        script = []
                        s_name = 'BP' + selectKE[0] + selectKE[1] 
                        script.append( m2g.punkt(schnittLi[0],beruehrpunkt,s_name,'BP') )
                        _liste_ = geoOops.buildListEntry(s_name,"Punkt",descartes.vecLi2str(schnittLi[0]), script )
                        geoOops.addGeoDirEntry(s_name,_liste_)
                    else: # Sekante!
                        schnittpunkt0 = descartes.punkt2maxima(schnittLi[0])
                        schnittpunkt1 = descartes.punkt2maxima(schnittLi[1])                            
                        retLi.append('"Schnittpunkte des Kreises ' + selectKE[0] + ' mit der Ebene ' + selectKE[1]  + ' "$')
                        retLi.append('sp1 : ' +  schnittpunkt0 + ';')
                        retLi.append('sp2 : ' +  schnittpunkt1 + ';')                            
                        # Uebernahme in geoObject-Dir
                        script = []
                        s_name = 'SP1' + selectKE[0] + selectKE[1] 
                        script.append( m2g.punkt(schnittLi[0],schnittpunkt0,s_name,'SP1') )
                        _liste_ = geoOops.buildListEntry(s_name,"Punkt",descartes.vecLi2str(schnittLi[0]), script )
                        geoOops.addGeoDirEntry(s_name,_liste_)
                        script = []
                        s_name = 'SP2' + selectKE[0] + selectKE[1] 
                        script.append( m2g.punkt(schnittLi[1],schnittpunkt1,s_name,'SP2') )
                        _liste_ = geoOops.buildListEntry(s_name,"Punkt",descartes.vecLi2str(schnittLi[1]), script )
                        geoOops.addGeoDirEntry(s_name,_liste_)

                else: # parallel
                    retLi.append('"Der Kreis ' + selectKE[0] + ' liegt in einer zur Ebene ' + selectKE[1]  + ' parallelen Ebene."$')
                return retLi                


                
############################################
def binomial(root,moduleDrawLoaded):            
    """bestimmt Binomialverteilung
       Parameter: n, p 
    """
    ok = 1

    eingabe = tkw.binomial(root,"Binomialverteilung")     
    if eingabe.result <> None:
        # Fenster-Werte uebernehmen
        try:
            n = int(eingabe.a1)
                            
            p = float(eingabe.a2)
            if not util.intervall(p,0,1):
                ok = 0
                tkMessageBox.showinfo("Eingabe-Fehler!",  
                     "Beachte: 0 <= p <= 1")
                return []           
            
        except:
            tkMessageBox.showinfo("Parameter-Fehler!",  
                 "Fehler bei der Parametereingabe")
            return [] # 0 == not ok
        
        retLi = []
        if ok:
            #print n,p
            retLi.append( '"Parameter der Binomialverteilung"$' )
            retLi.append( 'bin_n : ' + str(n) + '$' )
            retLi.append( 'bin_p : ' + str(p) + '$' )
            retLi.append( 'bin_q : 1-bin_p;' )
            retLi.append( '"Erwartungswert ew"$' )                
            retLi.append( 'bin_ew : bin_n * bin_p;' )
            retLi.append( '"Standardabweichung sigma"$' )                
            retLi.append( 'bin_sigma : ev(sqrt(bin_n * bin_p * bin_q),numer);' )
            
            sumBin = 'sumBin(n,links,rechts,p) := block( sum_ : 0, q_ : 1 - p,'
            sumBin = sumBin + '    if (links < 0)  then (links : 0),'
            sumBin = sumBin + '    if (rechts > n) then (rechts : n),'            
            sumBin = sumBin + '    for j_: links thru rechts do ('
            sumBin = sumBin + '       sum_ :  sum_ + p^j_ * q_^(n-j_) * binomial(n,j_) ),'
            sumBin = sumBin + '    print("P(",links, "<= X <=", rechts,") = ",sum_) );'
            retLi.append( sumBin )
            

            retLi.append( '"P(ew-sigma <= X <= ew+sigma) (1-Sigma-Umgebung)"$' )
            ret = "sumBin(bin_n,ceiling(bin_ew-bin_sigma), floor(bin_ew+bin_sigma), bin_p)$"
            retLi.append( ret )
            
            retLi.append( '"P(ew-2*sigma <= X <= ew+2*sigma) (2-Sigma-Umgebung)"$' )                
            ret = "sumBin(bin_n,ceiling(bin_ew-2*bin_sigma), floor(bin_ew+2*bin_sigma), bin_p)$"
            retLi.append( ret )
            
            retLi.append( '"P(ew-3*sigma <= X <= ew+3*sigma) (3-Sigma-Umgebung)"$' )                
            ret = "sumBin(bin_n,ceiling(bin_ew-3*bin_sigma), floor(bin_ew+3*bin_sigma), bin_p)$"
            retLi.append( ret )

                
            
            #makelist = 'xy:makelist([x,binomial(' + str(n) + ', x)*' + str(p) + '^x*bin_q^(' + str(n) + '-x)],x,0,' + str(n) + ');'
            makelist = 'xy:makelist([x,binomial(bin_n, x)*bin_p^x*bin_q^(bin_n-x)],x,0,bin_n);'
            retLi.append( makelist )
            if moduleDrawLoaded: # draw2d
                    
                    #plot = 'draw2d(points_joined = true,line_width = 2,color = red,points(xy) )$'
                    plot = 'draw2d(points_joined = impulses,line_width = 2,color = red,title="Binomialverteilung (n='
                    plot = plot + str(n) + ', p=' + str(p) + ')" ,points(xy) )$'
            else:
                    plot = 'plot2d([discrete,xy],[gnuplot_curve_titles,"title '
                    plot = plot + "'Binomialverteilung (n=" + str(n) + ", p=" + str(p) + ")'"
                    plot = plot + '"],[gnuplot_curve_styles,["with boxes fill solid 0.3"]])$'
            retLi.append( plot )
            
            return retLi


######################################
def sumBinomial(root):            
    """bestimmt summierte Binomialverteilung
       Parameter: n, p, von li=k1 bis re=k2 summiert
    """
    ok = 1

    eingabe = tkw.verteilung(root,"Summierte Binomialverteilung")     
    if eingabe.result <> None:
        # Fenster-Werte uebernehmen
        try:
            n = int(eingabe.a1)
                            
            li = int(eingabe.a2)
            if not util.intervall(li,0,n):
                ok = 0
                tkMessageBox.showinfo("Eingabe-Fehler!",  
                     "Beachte: 0 <= li <= " + str(n) )
                return []          


            re = int(eingabe.a3)
            if not util.intervall(re,li,n):
                ok = 0
                tkMessageBox.showinfo("Eingabe-Fehler!",  
                     "Beachte: " + str(li) + "<= re <= " + str(n) )
                return []           

            
            p = float(eingabe.a4)
            if not util.intervall(p,0,1):
                ok = 0
                tkMessageBox.showinfo("Eingabe-Fehler!",  
                     "Beachte: 0 <= p <= 1")
                return []           
            
        except:
            tkMessageBox.showinfo("Parameter-Fehler!",  
                 "Fehler bei der Parametereingabe")
            return [] # 0 == not ok
        
        retLi = []
        if ok:
            #print n,li,re,p
            sumBin = 'sumBin(n,links,rechts,p) := block( sum_ : 0, q_ : 1 - p,'
            sumBin = sumBin + '    if (links < 0)  then (links : 0),'
            sumBin = sumBin + '    if (rechts > n) then (rechts : n),'                        
            sumBin = sumBin + '    for j_: links thru rechts do ('
            sumBin = sumBin + '       sum_ :  sum_ + p^j_ * q_^(n-j_) * binomial(n,j_) ),'
            sumBin = sumBin + '    print(sum_) );'
            retLi.append( sumBin )

            retLi.append( '"P(k1 <= X <= k2) = P(' + str(li) + ' <= X <= '+ str(re) + ' )"$' )
            ret = "sumBin(" + str(n) + "," + str(li) + "," + str(re) +"," + str(p) + ")$"
            retLi.append( ret )
            return retLi
    
######################################
def normalVerteilung(root,moduleDrawLoaded):            
    """zeichnet & integriert Normalverteilung
       Parameter: li=x1, re=x2 
    """
    #print moduleDrawLoaded
    eingabe = tkw.nVerteilung(root,"Gauss'sche Normalverteilung")     
    if eingabe.result <> None:
        # Fenster-Werte uebernehmen
        try:
                            
            li = float(eingabe.a1)

            re = float(eingabe.a2)
            
        except:
            tkMessageBox.showinfo("Parameter-Fehler!",  
                 "Fehler bei der Parametereingabe")
            return [] # 0 == not ok
        
        #print li,re
        
        retLi = []
        
        gauss = "phi(x) := (1/sqrt(2*%pi)) * %e^(-0.5*x^2)$"
        retLi.append( gauss )
        dichte = "dichte(x) := 0.5 * erf(x/sqrt(2) ) $"
        retLi.append( dichte )
        
        retLi.append( '"Integral phi(x) von ' + str(li) + ' bis ' + str(re) + '"$' )
        summe = 'ev( dichte(' + str(re) + ') - dichte(' + str(li) +'), numer);'
        retLi.append( summe )

        
#            # alternativ
#            summe = "integrate( normalverteilung(x),x," + str(li) + "," + str(re) +")$"
#            self.bearbeiteAusdruck( summe )
#            self.bearbeiteAusdruck( "ev(%,numer);" )
        if moduleDrawLoaded: # draw2d
                plot = """draw2d(key = "Gauss''sche Summenfunktion",explicit(phi(x),x,""" + str(li) + "," + str(re)
                plot = plot + '), xaxis = true,yaxis = true,grid = true)$'
                retLi.append( plot )
                plot = 'draw2d(key = "Dichtefunktion",explicit(dichte(x),x,-3,3),xaxis = true,yaxis = true,grid = true)$'
                retLi.append( plot )                
        else:  # plot
                plot = 'plot2d([phi(x)],[x,' + str(li) + "," + str(re) + '],[y,-1,1],'
                plot = plot + '[gnuplot_preamble,"set grid; set zeroaxis linetype -1;"])$'                
                retLi.append( plot )
                plot = 'plot2d([dichte(x)],[x,-3,3],[y,-1,1],'
                plot = plot + '[gnuplot_preamble,"set grid; set zeroaxis linetype -1;"])$'                
                retLi.append( plot )
        # erf(x) := D(2*exp(-x^2)/sqrt(%pi) ) /dx
        return retLi
                

######################################
def konfIntervall(root):
    """bestimmt das Vertrauens/Konfidenzintervall
       Parameter: n, gamma, Stichprobenwert X (mit 0 <= X <= n)
    """

    ok = 1

    eingabe = tkw.konfidenz(root,"Konfidenzintervall")     
    if eingabe.result <> None:
        # Fenster-Werte übernehmen
        try:
            n = int(eingabe.a1)
                            
            gamma = float(eingabe.a2)
            if not util.intervall(gamma,0,1):
                ok = 0
                tkMessageBox.showinfo("Eingabe-Fehler!",  
                     "Beachte: 0 <= gamma <= 1")
                return []           


            x_wert = int(eingabe.a3)
            if not util.intervall(x_wert,0,n):
                ok = 0
                tkMessageBox.showinfo("Eingabe-Fehler!",  
                     "Beachte: 0 <= Stichprobenwert X <= " + str(n) )
                return []           

                            
        except:
            tkMessageBox.showinfo("Parameter-Fehler!",  
                 "Fehler bei der Parametereingabe")
            return None # 0 == not ok
        if ok:
            #print n,gamma, x_wert
            retLi = []
            konfidenz=stochastik.intervall()

            grenzen = konfidenz.konfidenzintervall(n,gamma,x_wert) # grenzen == [h,h1,h2,p1,p2]
            
            retLi.append( '"Konfidenzintervall für p"$' )
            retLi.append( '"Stichprobenumfang n = ' + str(n) + '"$' )
            retLi.append( '"Stichprobenwert X = ' + str(x_wert) + '"$' )                
            retLi.append( '"gamma = ' + str(gamma) + '"$' )                

            #konfidenz.self.h == grenzen[0] == h
            
            ausgabe = "rel. Häufigkeit h=" + str(grenzen[0])
            retLi.append( '"' + ausgabe + '"$' )
            ausgabe = "p in [%4.3f,%4.3f] mit " % (grenzen[1],grenzen[2])
            ausgabe = ausgabe +  str(int(gamma*100)) + "%-iger Wahrscheinlichkeit"
            retLi.append( '"' + ausgabe + '"$' )
            ausgabe = "exakt: "
            ausgabe = ausgabe + "p in [%4.3f,%4.3f] mit " % (grenzen[3],grenzen[4])
            ausgabe = ausgabe +  str(int(gamma*100)) + "%-iger Wahrscheinlichkeit"        
            retLi.append( '"' + ausgabe + '"$' )
            
            return retLi


######################################
def stichprobenUmfang(root):
    """bestimmt den StichprobenUmfang n
    Parameter: gamma, Intervalllänge d (0<= d <=1)
    """
    ok = 1

    eingabe = tkw.stichprobenUmfang(root,"Stichprobenumfang n")     
    if eingabe.result <> None:
        # Fenster-Werte uebernehmen
        try:
                            
            gamma = float(eingabe.a1)
            if not util.intervall(gamma,0,1):
                ok = 0
                tkMessageBox.showinfo("Eingabe-Fehler!",  
                     "Beachte: 0 <= gamma <= 1")
                return []           

            laenge_d = float(eingabe.a2)
            if not util.intervall(laenge_d,0,1):
                ok = 0
                tkMessageBox.showinfo("Eingabe-Fehler!",  
                     "Beachte: 0 <= laenge_d <= 1")
                return []           

        except:
            tkMessageBox.showinfo("Parameter-Fehler!",  
                 "Fehler bei der Parametereingabe")
            return [] # 0 == not ok
        if ok:
            retLi = [] 
            konfidenz=stochastik.intervall()
            umfang_n = konfidenz.stichprobenumfang(gamma,laenge_d) # umfang n 
            retLi.append( '"Bestimmung des Stichprobenumfangs n"$' )

            retLi.append( '"Intervalllänge d = ' + str(laenge_d) + '"$' )                
            retLi.append( '"gamma = ' + str(gamma) + '"$' )
            retLi.append( '"Stichprobenumfang n = ' + str(umfang_n) + '"$' )
            return retLi


######################################
def kombinatorik(root,arg):
    """bestimmt Anzahlen
       Parameter: Anzahl der Kugeln in der Urne n, gezogen werden: k
    """
    ok = 1
    #print arg
    kombiListe = ["Ziehen mit Zurücklegen, geordnet","Ziehen ohne Zurücklegen, geordnet","Ziehen ohne Zurücklegen, ungeordnet","Ziehen mit Zurücklegen, ungeordnet"]
    eingabe = tkw.kombinatorik(root,kombiListe[arg])     
    if eingabe.result <> None:
        # Fenster-Werte übernehmen
        try:
            n = int(eingabe.a1)
                            
            k = int(eingabe.a2)
            if not util.intervall(k,0,n):
                ok = 0
                tkMessageBox.showinfo("Eingabe-Fehler!",  
                     "Beachte: 0 <= k <= " + str(n) )
                return []                           
        except:
            tkMessageBox.showinfo("Parameter-Fehler!",  
                 "Fehler bei der Parametereingabe")
            return [] # 0 == not ok
        if ok:
            #print n,k
            retLi = []             
            retLi.append( '"' + kombiListe[arg] + '"$' )
            if (arg == 0):  #   "Ziehen mit Zurücklegen, geordnet" => n^k
                ret = str(n) + "^" + str(k) + ";"
                retLi.append( ret )                    
            elif (arg == 1):    #   "Ziehen ohne Zurücklegen, geordnet" => n!/(n-k)! 
                ret = str(n) + "!/((" + str(n) + " - " +str(k) +")!);"
                retLi.append( ret )
            elif (arg == 2):    # "Ziehen ohne Zurücklegen, ungeordnet" => Menge aller Teilmengen, n über k
                ret = "binomial(" + str(n) + "," + str(k) + ");"
                retLi.append( ret )
            else:   # arg == 3: "Ziehen mit Zurücklegen, ungeordnet" => (n+k-1) über k
                ret = "binomial( (" + str(n) + " + " + str(k) + " - 1)," + str(k) + ");"
                retLi.append( ret )
            return retLi
######################################
def gueteFunktion(root,gp_arg,moduleDrawLoaded):
    """Bestimmt die Gütefunktion g(p) bzw. die
       Operationscharakteristik O(p) zu einem
       zweisetigen Signifikanztest einer binomial-
       verteilten Zufallsvariablen.
       Parameter: n, alpha, Nullhypothese h0         
    """

    ok = 1
    
    art = ["zweiseitiger","linksseitiger","rechtsseitiger"]
    gleich = ["= ",">= ","<= "]
    
    if (gp_arg == "gp0"):  # Gütefunktion, zweiseitig
        """     arg == "gp0" <==> zweiseitig
                arg == "gp1" <==> linksseitig
                arg == "gp2" <==> rechtsseitig
        """
        arg = 0    # wichtig!
        eingabe = tkw.hypothesentest(root, "Guetefunktion g(p)",arg,_poisson_=0)
    elif (gp_arg == "gp1"): #linksseitig
        arg = 1    # wichtig!
        eingabe = tkw.hypothesentest(root, "Guetefunktion g(p)",arg,_poisson_=0)
        
    elif (gp_arg == "gp2"): # rechtsseitig
        arg = 2    # wichtig!
        eingabe = tkw.hypothesentest(root, "Guetefunktion g(p)",arg,_poisson_=0)
        
    elif (gp_arg == "op0"):  # Operationscharakteristik, zweiseitig
        """     arg == "op0" <==> zweiseitig
                arg == "op1" <==> linksseitig
                arg == "op2" <==> rechtsseitig
        """        
        arg = 0    # wichtig!
        eingabe = tkw.hypothesentest(root, "Operationscharakteristik O(p)",arg,_poisson_=0)
    elif (gp_arg == "op1"): #linksseitig
        arg = 1    # wichtig!
        eingabe = tkw.hypothesentest(root, "Operationscharakteristik O(p)",arg,_poisson_=0)
    else: # rechtsseitig  => (gp_arg == "op2")
        arg = 2    # wichtig!
        eingabe = tkw.hypothesentest(root, "Operationscharakteristik O(p)",arg,_poisson_=0)                        
                
    if eingabe.result <> None:
        # Fenster-Werte übernehmen
        try:
            _n_ = int(eingabe.a1)

            _alpha_ = float(eingabe.a2)
            if not util.intervall(_alpha_,0,1):
                ok = 0
                tkMessageBox.showinfo("Eingabe-Fehler!", "Beachte: 0 <= alpha <= 1")
                return []           
            
            h_null = float(eingabe.a3) # h_null
            if not util.intervall(h_null,0,1):
                ok = 0
                tkMessageBox.showinfo("Eingabe-Fehler!", "Beachte: 0 <= h0 <= 1")
                return []                               
            
        except:
            tkMessageBox.showinfo("Parameter-Fehler!",  
                 "Fehler bei der Parametereingabe")
            return [] # 0 == not ok
        if ok:
            #_sw_ = '20'  # _sw_ = schrittweite, wird benutzt im Feld xy
            _sw_ = '100'  # _sw_ = schrittweite, wird benutzt im Feld xy    
            
            _approx_offset_ = 6 # wird bei Approximation durch die Normalverteilung addiert!
            
            retLi = [] 
            #print _n_,_alpha_,h_null
            ewx = _n_ * h_null
            alpha_2 = _alpha_ / 2
            retLi.append( '"' + art[arg] + ' Hypothesentest (Binomial verteilte Zufallsvaribale X)"$' )
            retLi.append( '"Stichprobenumfang n = ' + str(_n_) + '"$' )
            retLi.append( '"Irrtumswahrscheinlichkeit alpha = ' + str(_alpha_) + '"$' )                
            retLi.append( '"Nullhypthese H0: p ' + gleich[arg] + str(h_null) + '"$' )                    
            pruef = "Prüfvariable X ist B(%d;%3.2f)-verteilt" % (_n_,h_null)
            retLi.append( '"' + pruef + '"$' )


            if ewx * (1-h_null) > (9 + _approx_offset_): # n*p*q > 9 => Testen mit Hilfe der Normalverteilung
                normTest = stochastik.normalTest(h0=h_null,n=_n_,alpha=_alpha_,stichp=37,korrektur=1)
                ausgabe = ""
                if arg == 0:    # zweiseitig
                    retLi.append( '"Konstruktion des Ablehnungsbereiches K = {0;...;gl} u {gr;...;' + str(_n_) + '}"$' )
                    gl =  normTest.berechneLinks(_n_,h_null,ewx,alpha_2)
                    gr =  normTest.berechneRechts(_n_,h_null,ewx,alpha_2)
                    if (gr>=_n_) and (gl<=0):            
                        ausgabe = "K = {0} u {%d}" % (_n_)
                    elif gl<=0:
                        ausgabe = "K = {0} u {%d;...;%d}" % (gr,_n_)
                    elif gr>=_n_:
                        ausgabe = "K = {0;...;%d} u {%d}" % (gl,_n_)
                    else:    
                        ausgabe = "K = {0;...;%d} u {%d;...;%d}" % (gl,gr,_n_)
                elif (arg == 1): # linksseitig
                    retLi.append( '"Konstruktion des Ablehnungsbereiches K = {0;...;gl}"$' )                    
                    gl =  normTest.berechneLinks(_n_,h_null,ewx,_alpha_)
                    if gl<=0:
                        ausgabe ="K = {0}"
                    else:
                        ausgabe ="K = {0;...;%d}" % gl
                    
                else:   # rechtsseitig
                    retLi.append( '"Konstruktion des Ablehnungsbereiches K = {gr;...;' + str(_n_) + '}"$' )                                        
                    gr =  normTest.berechneRechts(_n_,h_null,ewx,_alpha_)
                    if (gr>=_n_):
                        ausgabe = "K = {%d}" % (_n_)
                    else:
                        ausgabe = "K = {%d;...;%d}" % (gr,_n_)                            

            else: # Test mit Hilfe der Binomialverteilung
                binTest = stochastik.binomialTest(h0=h_null,n=_n_,alpha=_alpha_,stichp=37)
                ausgabe = ""
                if arg == 0:    # zweiseitig
                    retLi.append( '"Konstruktion des Ablehnungsbereiches K = {0;...;gl} u {gr;...;' + str(_n_) + '}"$' )
                    gl =  binTest.berechneLinks(_n_,h_null,ewx,alpha_2)
                    gr =  binTest.berechneRechts(_n_,h_null,ewx,alpha_2)
                    if (gr>=_n_) and (gl<=0):            
                        ausgabe = "K = {0} u {%d}" % (_n_)
                    elif gl<=0:
                        ausgabe = "K = {0} u {%d;...;%d}" % (gr,_n_)
                    elif gr>=_n_:
                        ausgabe = "K = {0;...;%d} u {%d}" % (gl,_n_)
                    else:    
                        ausgabe = "K = {0;...;%d} u {%d;...;%d}" % (gl,gr,_n_)
                elif (arg == 1): # linksseitig
                    retLi.append( '"Konstruktion des Ablehnungsbereiches K = {0;...;gl}"$' )                    
                    gl =  binTest.berechneLinks(_n_,h_null,ewx,_alpha_)
                    if gl<=0:
                        ausgabe ="K = {0}"
                    else:
                        ausgabe ="K = {0;...;%d}" % gl
                
                else:   # rechtsseitig
                    retLi.append( '"Konstruktion des Ablehnungsbereiches K = {gr;...;' + str(_n_) + '}"$' )                                        
                    gr =  binTest.berechneRechts(_n_,h_null,ewx,_alpha_)
                    if (gr>=_n_):
                        ausgabe = "K = {%d}" % (_n_)
                    else:
                        ausgabe = "K = {%d;...;%d}" % (gr,_n_)       
                
                
            retLi.append( '"' + ausgabe + '"$' )
            retLi.append( 'gp_bin_n : ' + str(_n_) + '$')
            if arg == 0:    # zweiseitig
                retLi.append( 'gp_k1 : ' + str(gl) + '+1$')
                retLi.append( 'gp_k2 : ' + str(gr) + '-1$')
                #retLi.append( 'gp_p : ' + str(h_null) + '$')
            elif (arg == 1): # linksseitig
                retLi.append( 'gp_k1 : ' + str(gl) + '+1$')
            else: # rechtsseitig
                retLi.append( 'gp_k2 : ' + str(gr) + '-1$')

            ## Skript funktioniert nur fuer 0 < p < 1 !!!
            ## epsilon=1.0e-5
            bin_sum_skript = """sumBin2(n,links,rechts,p) :=   block( sum_ : 0, null_ : 1.0e-5,
            eins_ : 1 - null_, if (links < 0)  then (links : 0), 
            if (rechts > n) then (rechts : n), if (p < null_) then (p : null_),
            if (p > eins_) then (p : eins_), q_ : 1 - p, for j_: links thru rechts do (
            sum_ :  sum_ + p^j_  * q_^(n-j_) * binomial(n,j_) ), print(sum_) )$"""
            
            retLi.append( bin_sum_skript )
            
##  Die Approximation durch die Normalverteilung fuehrt fuer kleine Wk zu fehlerhften Werten => benutze sumBin2         
##            if ewx * (1-h_null) > (9 + _approx_offset_): # n*p*q > 9 => Testen mit Hilfe der Normalverteilung
##                approx_bin_skript = """approxBin2(n,links,rechts,p_) := block(   erg_ : 0, q_ : 1 - p_,
##                ew_ : n * p_, sigma_ : sqrt( ew_ * q_),
##                if (links < 0)  then (links : 0), if (rechts > n) then (rechts : n),
##                app_links  : (links - ew_) / sigma_, app_rechts : (rechts - ew_) / sigma_,
##                erg_ : ev( 0.5 * erf(app_rechts/sqrt(2) ) - 0.5 * erf(app_links/sqrt(2) ), numer),
##                print(erg_) )$"""
##                retLi.append( approx_bin_skript )
##            else:
##                bin_sum_skript = """sumBin2(n,links,rechts,p) :=   block( sum_ : 0, q_ : 1 - p, if (links < 0)  then (links : 0),
##                if (rechts > n) then (rechts : n), for j_: links thru rechts do (                        
##                sum_ :  sum_ + p^j_  * q_^(n-j_) * binomial(n,j_) ), print(sum_) )$"""
##                retLi.append( bin_sum_skript )

            

            if ( gp_arg.startswith("gp") ):
                retLi.append('"Gütefunktion g(p)"$')
                if (arg == 1): # linksseitig
                    retLi.append( 'gp_k2 : ' + str(_n_)+ '$')
                elif (arg == 2): # rechtsseitig
                    retLi.append( 'gp_k1 : ' + str(0) + '$')

##  Die Approximation durch die Normalverteilung fuehrt fuer kleine Wk zu fehlerhften Werten => benutze sumBin2
##                if ewx * (1-h_null) > (9 + _approx_offset_): # n*p*q > 9 => Testen mit Hilfe der Normalverteilung
##                    retLi.append('xy:ev(makelist([p/' + _sw_ + ',1 - approxBin2(gp_bin_n,gp_k1,gp_k2,p/' + _sw_ + ')],p,0+1,' + _sw_ + '-1) , numer)$' )
##                else:
##                    retLi.append('xy:ev(makelist([p/' + _sw_ + ',1 - sumBin2(gp_bin_n,gp_k1,gp_k2,p/' + _sw_ + ')],p,0+1,' + _sw_ + ') , numer)$' )
                    
                retLi.append('xy:ev(makelist([p/' + _sw_ + ',1 - sumBin2(gp_bin_n,gp_k1,gp_k2,p/' + _sw_  + ')],p,0,' + _sw_  + ') , numer)$' )

                if moduleDrawLoaded: # draw2d                    
                        plot_gp = 'draw2d(points_joined = false,color = red,title="Guetefunktion g(p)",points(xy),user_preamble="set encoding iso_8859_1" )$'
                else:    
                        plot_gp = """plot2d([discrete,xy],[gnuplot_curve_titles,["title 'Guetefunktion g(p)'"]],[gnuplot_curve_styles,["with points"]],[gnuplot_preamble,"set encoding iso_8859_1; set grid; set xtics 1; set size ratio -1; set xrange [0:1]; set yrange [0:1.2]"])$"""
                plot_gp_uni = plot_gp.decode('utf-8')
                retLi.append( plot_gp_uni.encode('iso-8859-1') )
                #retLi.append( plot_gp.decode('utf-8') )
            else:# (gp_arg == "op")
                retLi.append('"Operationscharakteristik O(p)"$')
                if (arg == 1): # linksseitig
                    retLi.append( 'gp_k2 : ' + str(_n_)+ '$')
                elif (arg == 2): # rechtsseitig
                    retLi.append( 'gp_k1 : ' + str(0) + '$')

##  Die Approximation durch die Normalverteilung fuehrt fuer kleine Wk zu fehlerhften Werten => benutze sumBin2                                
##                if ewx * (1-h_null) > (9 + _approx_offset_): # n*p*q > 9 => Testen mit Hilfe der Normalverteilung
##                    retLi.append('xy:ev(makelist([p/' + _sw_ + ',approxBin2(gp_bin_n,gp_k1,gp_k2,p/' + _sw_ + ')],p,0+1,' + _sw_ + '-1) , numer)$' )
##                else:                
##                    retLi.append('xy:ev(makelist([p/' + _sw_ + ',sumBin2(gp_bin_n,gp_k1,gp_k2,p/' + _sw_ + ')],p,0+1,' + _sw_ + ') , numer)$' )
                    
                retLi.append('xy:ev(makelist([p/' + _sw_ + ',sumBin2(gp_bin_n,gp_k1,gp_k2,p/' + _sw_ + ')],p,0,' + _sw_  + ') , numer)$' )
                    
                if moduleDrawLoaded: # draw2d                    
                        plot_op = 'draw2d(points_joined = false,color = red,title="Operationscharakteristik O(p)",points(xy))$'
                else:  
                        plot_op = """plot2d([discrete,xy],[gnuplot_curve_titles,["title 'Operationscharakteristik O(p)'"]],[gnuplot_curve_styles,["with points"]],[gnuplot_preamble,"set encoding iso_8859_1; set grid; set xtics 1; set size ratio -1; set xrange [0:1]; set yrange [0:1.2]"])$"""
                retLi.append( plot_op )

            return retLi

        
######################################
def alternativtest(root, arg):
    """Alternativtest mit Hilfe der (summierten) Binomialverteilung
       Parameter: n, alpha, Nullhypothese H0, Alternativhypothese H1
       arg == 0  <==> Binomialverteilung
       arg == 1  <==> Normalverteilung                 
    """

    ok = 1
    korrekturglied=0 # 
    art = ["zweiseitiger","linksseitiger","rechtsseitiger"]
    gleich = ["= ",">= ","<= "]
    if (arg == 0):
        eingabe = tkw.alternativtest(root,"Alternativtest")
    else:  # arg == 1
        eingabe = tkw.nAlternativtest(root,"Alternativtest")
    if eingabe.result <> None:
        # Fenster-Werte übernehmen
        try:
            _n_ = int(eingabe.a1)
##            if (arg == 0): # n <= 2500 nur bei Binomialverteilung!
##                if not util.intervall(_n_,1,2500):
##                    ok = 0
##                    tkMessageBox.showinfo("Eingabe-Fehler!", "Beachte: 1 <= n <= 2500")
##                    return []           

            _alpha_ = float(eingabe.a2)
            if not util.intervall(_alpha_,0,1):
                ok = 0
                tkMessageBox.showinfo("Eingabe-Fehler!", "Beachte: 0 <= alpha <= 1")
                return []           
            
            h_null = float(eingabe.a3) # h_null => H0
            if not util.intervall(h_null,0,1):
                ok = 0
                tkMessageBox.showinfo("Eingabe-Fehler!", "Beachte: 0 <= h0 <= 1")
                return []                               
            

            h_eins = float(eingabe.a4) # h_eins => H1
            if not util.intervall(h_eins,0,1):
                ok = 0
                tkMessageBox.showinfo("Eingabe-Fehler!", "Beachte: 0 <= h1 <= 1")
                return []                               

            if (arg == 1):
                korrekturglied=eingabe.a5
                
        except:
            tkMessageBox.showinfo("Parameter-Fehler!",  
                 "Fehler bei der Parametereingabe")
            return [] # 0 == not ok
        if ok:
            retLi = [] 
            #print _n_,_alpha_,h_null
            ewx = _n_ * h_null
            alpha_2 = _alpha_ / 2
            retLi.append( '"Alternativtest (Binomial verteilte Zufallsvaribale X)"$' )

            if (arg == 1):
                if korrekturglied:
                    retLi.append( '"Approximation durch die Normalverteilung (mit Stetigkeitskorrektur)"$' )                    
                else:
                    retLi.append( '"Approximation durch die Normalverteilung"$' )

            retLi.append( '"Stichprobenumfang n = ' + str(_n_) + '"$' )
            retLi.append( '"Irrtumswahrscheinlichkeit alpha = ' + str(_alpha_) + '"$' )                
            retLi.append( '"Nullhypothese H0: p = ' + str(h_null) + '"$' )
            retLi.append( '"Alternativhypothese H1: p = ' + str(h_eins) + '"$' )                                
            pruef = "Prüfvariable X ist B(%d;%3.2f)-verteilt" % (_n_,h_null)
            retLi.append( '"' + pruef + '"$' )
            
            if (arg == 0):
                binTest = stochastik.binomialTest(h0=h_null,n=_n_,alpha=_alpha_,stichp=37)
            else:                
                neunerTest = _n_ * h_null * (1-h_null)    
                normBedingung = "n * p * (1-p) = n * p * q = %3.2f" % neunerTest
                retLi.append( '"' + normBedingung + '"$' )
                                
                binTest = stochastik.normalTest(h0=h_null,n=_n_,alpha=_alpha_,stichp=37,korrektur=korrekturglied)
                #gl =  normTest.berechneLinks(_n_,h_null,ewx,alpha_2)
                #gr =  normTest.berechneRechts(_n_,h_null,ewx,alpha_2)

                
            ausgabe = ""
            if h_null <= h_eins:
                retLi.append( '"Konstruktion des Ablehnungsbereiches K = {gr;...;' + str(_n_) + '}"$' )                                        
                gr =  binTest.berechneRechts(_n_,h_null,ewx,_alpha_)
                if (gr>=_n_):
                    ausgabe = "K = {%d}" % (_n_)
                else:
                    ausgabe = "K = {%d;...;%d}" % (gr,_n_)
                #alpha_fehler = binTest.bin_sum(_n_,h_null,gr,_n_)
                alpha_fehler = 1 - binTest.bin_sum(_n_,h_null,0,gr-1)    
                beta_fehler = binTest.bin_sum(_n_,h_eins,0,gr-1) # summiere ueber Annahmebereich                
            else: #
                retLi.append( '"Konstruktion des Ablehnungsbereiches K = {0;...;gl}"$' )
                gl =  binTest.berechneLinks(_n_,h_null,ewx,_alpha_)
                if gl<=0:
                    ausgabe ="K = {0}"
                else:
                    ausgabe ="K = {0;...;%d}" % gl
                alpha_fehler = binTest.bin_sum(_n_,h_null,0,gl)
                #beta_fehler = binTest.bin_sum(_n_,h_eins,gl+1,_n_) # summiere ueber Annahmebereich
                beta_fehler = 1 - binTest.bin_sum(_n_,h_eins,0,gl) # summiere ueber Annahmebereich
                #print _n_,h_eins,gl+1,_n_
                
            retLi.append( '"' + ausgabe + '"$' )
            retLi.append( '"Fehler erster Art: p = ' + str( alpha_fehler ) + '"$' )
            retLi.append( '"Fehler zweiter Art: beta = ' + str( beta_fehler ) + '"$' )                                        
            
            return retLi
######################################
        
def hypothesentest(root,arg):
    """Hypothesentest mit Hilfe der (summierten) Binomialverteilung
       Parameter: n, alpha, Nullhypothese h0
       arg == 0 <==> zweiseitig
       arg == 1 <==> linksseitig
       arg == 2 <==> rechtsseitig           
    """

    ok = 1
    art = ["zweiseitiger","linksseitiger","rechtsseitiger"]
    gleich = ["= ",">= ","<= "]
    eingabe = tkw.hypothesentest(root, art[arg] + " Hypothesentest",arg,_poisson_=0)     
    if eingabe.result <> None:
        # Fenster-Werte übernehmen
        try:
            _n_ = int(eingabe.a1)
##            if not util.intervall(_n_,1,2500):
##                ok = 0
##                tkMessageBox.showinfo("Eingabe-Fehler!", "Beachte: 1 <= n <= 2500")
##                return []           

            _alpha_ = float(eingabe.a2)
            if not util.intervall(_alpha_,0,1):
                ok = 0
                tkMessageBox.showinfo("Eingabe-Fehler!", "Beachte: 0 <= alpha <= 1")
                return []           
            
            h_null = float(eingabe.a3) # h_null
            if not util.intervall(h_null,0,1):
                ok = 0
                tkMessageBox.showinfo("Eingabe-Fehler!", "Beachte: 0 <= h0 <= 1")
                return []                               
            
        except:
            tkMessageBox.showinfo("Parameter-Fehler!",  
                 "Fehler bei der Parametereingabe")
            return [] # 0 == not ok
        if ok:
            retLi = [] 
            #print _n_,_alpha_,h_null
            ewx = _n_ * h_null
            alpha_2 = _alpha_ / 2
            retLi.append( '"' + art[arg] + ' Hypothesentest (Binomial verteilte Zufallsvaribale X)"$' )
            retLi.append( '"Stichprobenumfang n = ' + str(_n_) + '"$' )
            retLi.append( '"Irrtumswahrscheinlichkeit alpha = ' + str(_alpha_) + '"$' )                
            retLi.append( '"Nullhypothese H0: p ' + gleich[arg] + str(h_null) + '"$' )                    
            pruef = "Prüfvariable X ist B(%d;%3.2f)-verteilt" % (_n_,h_null)
            retLi.append( '"' + pruef + '"$' )

            binTest = stochastik.binomialTest(h0=h_null,n=_n_,alpha=_alpha_,stichp=37)
            ausgabe = ""
            if arg == 0:    # zweiseitig
                retLi.append( '"Konstruktion des Ablehnungsbereiches K = {0;...;gl} u {gr;...;' + str(_n_) + '}"$' )
                gl =  binTest.berechneLinks(_n_,h_null,ewx,alpha_2)
                gr =  binTest.berechneRechts(_n_,h_null,ewx,alpha_2)
                if (gr>=_n_) and (gl<=0):            
                    ausgabe = "K = {0} u {%d}" % (_n_)
                elif gl<=0:
                    ausgabe = "K = {0} u {%d;...;%d}" % (gr,_n_)
                elif gr>=_n_:
                    ausgabe = "K = {0;...;%d} u {%d}" % (gl,_n_)
                else:    
                    ausgabe = "K = {0;...;%d} u {%d;...;%d}" % (gl,gr,_n_)
                
            elif (arg == 1): # linksseitig
                retLi.append( '"Konstruktion des Ablehnungsbereiches K = {0;...;gl}"$' )                    
                gl =  binTest.berechneLinks(_n_,h_null,ewx,_alpha_)
                if gl<=0:
                    ausgabe ="K = {0}"
                else:
                    ausgabe ="K = {0;...;%d}" % gl
                
            else:   # rechtsseitig
                retLi.append( '"Konstruktion des Ablehnungsbereiches K = {gr;...;' + str(_n_) + '}"$' )                                        
                gr =  binTest.berechneRechts(_n_,h_null,ewx,_alpha_)
                if (gr>=_n_):
                    ausgabe = "K = {%d}" % (_n_)
                else:
                    ausgabe = "K = {%d;...;%d}" % (gr,_n_)                    
                
            retLi.append( '"' + ausgabe + '"$' )
            return retLi


######################################
def pHypothesentest(root,arg):
    """Hypothesentest mit Hilfe der Poissonverteilung
       Parameter: n, alpha, Nullhypothese h0
       arg == 0 <==> zweiseitig
       arg == 1 <==> linksseitig
       arg == 2 <==> rechtsseitig           
    """
    ok = 1
    art = ["zweiseitiger","linksseitiger","rechtsseitiger"]
    gleich = ["= ",">= ","<= "]
    eingabe = tkw.hypothesentest(root, art[arg] + " Hypothesentest",arg,_poisson_=1)     
    if eingabe.result <> None:
        # Fenster-Werte übernehmen
        try:
            _n_ = int(eingabe.a1)
##            if not util.intervall(_n_,1,5000):
##                ok = 0
##                tkMessageBox.showinfo("Eingabe-Fehler!", "Beachte: 1 <= n <= 5000")
##                return []           

            _alpha_ = float(eingabe.a2)
            if not util.intervall(_alpha_,0,1):
                ok = 0
                tkMessageBox.showinfo("Eingabe-Fehler!", "Beachte: 0 <= alpha <= 1")
                return []
            
            h_null = float(eingabe.a3) # h_null
            if not util.intervall(h_null,0,1):
                ok = 0
                tkMessageBox.showinfo("Eingabe-Fehler!", "Beachte: 0 <= h0 <= 1")
                return []                               
            
        except:
            tkMessageBox.showinfo("Parameter-Fehler!",  
                 "Fehler bei der Parametereingabe")
            return [] # 0 == not ok
        if ok:
            retLi = []
            #print _n_,_alpha_,h_null
            ewx = _n_ * h_null
            alpha_2 = _alpha_ / 2
            retLi.append( '"' + art[arg] + ' Hypothesentest"$' )
            retLi.append( '"Näherung durch die Poissonverteilung für kleine Wahrscheinlichkeiten"$' )
            retLi.append( '"Stichprobenumfang n = ' + str(_n_) + '"$' )
            retLi.append( '"Irrtumswahrscheinlichkeit alpha = ' + str(_alpha_) + '"$' )                
            retLi.append( '"Nullhypothese H0: p ' + gleich[arg] + str(h_null) + '"$' )                    
            #pruef = "Prüfvariable X ist B%d,%3.2f-verteilt" % (_n_,h_null)
            #retLi.append( '"' + pruef + '"$' )

            poissonTest = stochastik.poissonTest(h0=h_null,n=_n_,alpha=_alpha_,stichp=37)
            ausgabe = ""
            if arg == 0:    # zweiseitig
                retLi.append( '"Konstruktion des Ablehnungsbereiches K = {0;...;gl} u {gr;...;' + str(_n_) + '}"$' )
                gl =  poissonTest.berechneLinks(_n_,h_null,ewx,alpha_2)
                gr =  poissonTest.berechneRechts(_n_,h_null,ewx,alpha_2)
                if (gr>=_n_) and (gl<=0):            
                    ausgabe = "K = {0} u {%d}" % (_n_)
                elif gl<=0:
                    ausgabe = "K = {0} u {%d;...;%d}" % (gr,_n_)
                elif gr>=_n_:
                    ausgabe = "K = {0;...;%d} u {%d}" % (gl,_n_)
                else:    
                    ausgabe = "K = {0;...;%d} u {%d;...;%d}" % (gl,gr,_n_)
                
            elif (arg == 1): # linksseitig
                retLi.append( '"Konstruktion des Ablehnungsbereiches K = {0;...;gl}"$' )                    
                gl =  poissonTest.berechneLinks(_n_,h_null,ewx,_alpha_)
                if gl<=0:
                    ausgabe ="K = {0}"
                else:
                    ausgabe ="K = {0;...;%d}" % gl
                
            else:   # rechtsseitig
                retLi.append( '"Konstruktion des Ablehnungsbereiches K = {gr;...;' + str(_n_) + '}"$' )                                        
                gr =  poissonTest.berechneRechts(_n_,h_null,ewx,_alpha_)
                if (gr>=_n_):
                    ausgabe = "K = {%d}" % (_n_)
                else:
                    ausgabe = "K = {%d;...;%d}" % (gr,_n_)                    
                
            retLi.append(  '"' + ausgabe + '"$' )
            return retLi


######################################
def nHypothesentest(root,arg):
    """Hypothesentest mit Hilfe der Normalverteilung
       Parameter: n, alpha, Nullhypothese h0, Korrekturglied (0 oder 1)
       arg == 0 <==> zweiseitig
       arg == 1 <==> linksseitig
       arg == 2 <==> rechtsseitig           
    """
    ok = 1
    art = ["zweiseitiger","linksseitiger","rechtsseitiger"]
    gleich = ["= ",">= ","<= "]
    eingabe = tkw.nHypothesentest(root, art[arg] + " Hypothesentest",arg)     
    if eingabe.result <> None:
        # Fenster-Werte übernehmen
        try:
            _n_ = int(eingabe.a1)

            _alpha_ = float(eingabe.a2)
            if not util.intervall(_alpha_,0,1):
                ok = 0
                tkMessageBox.showinfo("Eingabe-Fehler!", "Beachte: 0 <= alpha <= 1")
                return []          
            
            h_null = float(eingabe.a3) # h_null
            if not util.intervall(h_null,0,1):
                ok = 0
                tkMessageBox.showinfo("Eingabe-Fehler!", "Beachte: 0 <= h0 <= 1")
                return []                               

            korrekturglied=eingabe.a4                
            
        except:
            tkMessageBox.showinfo("Parameter-Fehler!",  
                 "Fehler bei der Parametereingabe")
            return [] # 0 == not ok
        if ok:
            retLi = []
            #print _n_,_alpha_,h_null
            ewx = _n_ * h_null
            alpha_2 = _alpha_ / 2
            if korrekturglied:
                retLi.append( '"' + art[arg] + ' Hypothesentest mit Stetigkeitskorrektur (Approximation durch die Normalverteilung)"$' )                    
            else:
                retLi.append( '"' + art[arg] + ' Hypothesentest (Approximation durch die Normalverteilung)"$' )
            retLi.append( '"Stichprobenumfang n = ' + str(_n_) + '"$' )
            retLi.append( '"Irrtumswahrscheinlichkeit alpha = ' + str(_alpha_) + '"$' )                
            retLi.append( '"Nullhypothese H0: p ' + gleich[arg] + str(h_null) + '"$' )
            
            pruef = "Prüfvariable X ist B(%d;%3.2f)-verteilt" % (_n_,h_null)
            retLi.append( '"' + pruef + '"$' )
            
            neunerTest = _n_ * h_null * (1-h_null)    
            normBedingung = "n * p * (1-p) = n * p * q = %3.2f" % neunerTest
            retLi.append( '"' + normBedingung + '"$' )
            
            normTest = stochastik.normalTest(h0=h_null,n=_n_,alpha=_alpha_,stichp=37,korrektur=korrekturglied)
            ausgabe = ""
            if arg == 0:    # zweiseitig
                retLi.append( '"Konstruktion des Ablehnungsbereiches K = {0;...;gl} u {gr;...;' + str(_n_) + '}"$' )
                gl =  normTest.berechneLinks(_n_,h_null,ewx,alpha_2)
                gr =  normTest.berechneRechts(_n_,h_null,ewx,alpha_2)
                if (gr>=_n_) and (gl<=0):            
                    ausgabe = "K = {0} u {%d}" % (_n_)
                elif gl<=0:
                    ausgabe = "K = {0} u {%d;...;%d}" % (gr,_n_)
                elif gr>=_n_:
                    ausgabe = "K = {0;...;%d} u {%d}" % (gl,_n_)
                else:    
                    ausgabe = "K = {0;...;%d} u {%d;...;%d}" % (gl,gr,_n_)
                    
                # \u222A steht fuer Vereinigung in Python
                
            elif (arg == 1): # linksseitig
                retLi.append( '"Konstruktion des Ablehnungsbereiches K = {0;...;gl}"$' )                    
                gl =  normTest.berechneLinks(_n_,h_null,ewx,_alpha_)
                if gl<=0:
                    ausgabe ="K = {0}"
                else:
                    ausgabe ="K = {0;...;%d}" % gl
                
            else:   # rechtsseitig
                retLi.append( '"Konstruktion des Ablehnungsbereiches K = {gr;...;' + str(_n_) + '}"$' )                                        
                gr =  normTest.berechneRechts(_n_,h_null,ewx,_alpha_)
                if (gr>=_n_):
                    ausgabe = "K = {%d}" % (_n_)
                else:
                    ausgabe = "K = {%d;...;%d}" % (gr,_n_)                    
                
            retLi.append( '"' + ausgabe + '"$' )
            return retLi

###################################






































