#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
   maxinter.py: stellt Klasse MaximaInterface bereit
   Schnittstelle zu Maxima via subprocess/Popen-Modul

   2006-11-08: funktioniert fast

   2006-11-10: aktualisierte Version (handleOut!)

   2006-11-28: aktualisierte Version (buildOutput, cleanOut)

   2007-05-11: ctrlcHandler unterbricht die Verbindung zu Maxima!

   2007-10-06: arbeitet unter Win NT mit maxima.bat
               => self.maximaWinCall in der Klasse MaximaInterface

   2008-01-15: neue Methode setInOutChar(maximaObj)
               Aufruf: maxinter.setInOutChar(maxima)
               wurde notwendig in Maxima, Vers. 5.14.0

   2008-02-10: wg. des Maxima Befehls read die Methode handleOut
               angepasst, s. den Kommentar in der Methode

   2008-10-25: neue Methode getLispImplementationType wg. unterschiedlicher
               Behandlung von Sonderzeichen in GCL bzw. CLISP
               neue Eigenschaft self.lispImplementationType in der Klasse MaximaInterface      

   2008-11-02: neue Methode getMaximaVersion, neue Eigenschaft self.MaximaVersion in der Klasse MaximaInterface

   2010-04-01: Modul-Test "main" ueberarbeitet;
               
               angepasst fuer das Maxima-Modul "draw", http://www.telefonica.net/web2/biomates/maxima/gpdraw/index.html
               vgl. die Methode waitTest

               
   2012-09-15: erfordert die Dateien "maxima.bat" sowie "interface.lisp"
               => Die Datei "maxima.bat" muss im gleichem Verzeichnis stehen!!
               => Die Datei "interface.lisp" muss von diesem Verzeichnis aus im Unterverzeichnis "./init" stehen
               Neu ist der Befehl "build_info();" im Test

   2012-11-17: Methode "getMaximaVersion" der Klasse "MaximaInterface" an Maxima Version 5.27 angepasst,
               da "build_info()$" eine leere Liste retour gibt . . .
               => viele Debug-Befehle in der Methode "startMaxima" ergaenzt!
"""



import os
import Popen # from Josiah Carlson, URL: http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/440554
import time
import subprocess
import string
import sys
import pymaxUtil as util
from Tkinter import * 


import os
# die folgende Variable wird offensichtlich nicht genutzt!!!  <<===
pyMaximaHome = os.getcwd() + os.sep

# pfad_zu_maxima = 'G:\\Programme\\Maxima-5.10.0\\bin\\' # anpassen!!
# D:\mathe\Maxima-5.15.0

pfad_zu_maxima = 'D:\\programme\\Maxima-5.26.0\\bin\\' # anpassen!!

if os.name == 'posix':
    pfad_zu_maximaDoc = '/home/walter/py_prg/pyMaxima/maxima_doc/' # anpassen!!    
else:
    pfad_zu_maximaDoc = 'G:\\Programme\\Maxima-5.10.0\\bin\\maxima_doc\\' # anpassen!!

maximaDocHome =  os.path.join(pfad_zu_maximaDoc, 'maxima.html')    


handle = None


#######################################    
def setInOutChar(maximaObj):
    "setzt in maxima: inchar, outchar, linechar"
    maximaObj.call(":lisp (defmvar $inchar '$%i \"The alphabetic prefix of the names of expressions typed by the user.\")")
    maximaObj.call(":lisp (defmvar $outchar '$%o \"The alphabetic prefix of the names of expressions returned by the system.\")")
    maximaObj.call(":lisp (defmvar $linechar '$%t \"The alphabetic prefix of the names of intermediate displayed expressions.\")")



class MaximaInterface:
    def __init__(self,delay_time=0.3,_pfad_zu_maxima_=None):


        if delay_time:
            self._delay_time_ = delay_time
        else:
            self._delay_time_ = 0.3	# Standardwert

        self.lispImplementationType = ""	
        self.MaximaVersion = ""
        
        self.displayPrefix = '_INTERFACE_GENERAL_DISPLAY_PREFIX_'
        self.promptPrefix = '_INTERFACE_PROMPT_PREFIX_'
        self.promptSuffix = '_INTERFACE_PROMPT_SUFFIX_'
        
        
        self.dispPrefix = len(self.displayPrefix)
        self.proPrefix = len(self.promptPrefix)
        self.proSuffix = len(self.promptSuffix)
        
        self.handle = None
        
        # alt self.lispSkript = 'interface.lisp'
        self.lispSkript = 'init/interface.lisp'

        self.maximaWinCall = 'maxima.bat --preload-lisp='
        self.maximaCall = 'maxima --preload-lisp='
        self.maximaDisableReadline = 'maxima --disable-readline --preload-lisp='
    
        if _pfad_zu_maxima_:
            self.pfad_zu_maxima = _pfad_zu_maxima_
        else:
            self.pfad_zu_maxima = ''
            
        if os.name == 'posix':
            self.tail = "\n"
        else:        
            self.tail = "\r\n"

        self.offset = 0.5       # wird benutzt in startMaxima() => time.sleep: self._delay_time_ + self.offset 
        self.maxTry = 7         # wird ueberschrieben in buildOutput()    
        self.win32Factor = 6    # help-Funktion dauert unter win32 lange


    def getMaximaVersion(self):
        "gibt (hoffentlich) die Maxima-Version"
        """ Anpassung an Version 5.27, da "build_info()$" eine leere Liste retour gibt . . ."""
        _debug_ = 0
        _ret_ = "5.xx"
        #maximaVers,nextIn,gefunden = self.call("build_info()$")
        maximaVers,nextIn,gefunden = self.call("build_info();")
        if _debug_:
             print maximaVers,nextIn,gefunden    
        maximaList = string.split(maximaVers)
        for element in maximaList:
            if _debug_:
                print element
            if (string.find(element,'"5.',0,3) != -1) or (string.find(element,'5.',0,2) != -1):
                _ret_ = element

        return _ret_


    def getLispImplementationType(self):
        "findet (hoffentlich) die Lisp-Implementation von Maxima"
        _ret_ = ""
        lispImplementation,nextIn,gefunden = self.call(":lisp (lisp-implementation-type)")
        if (string.find(lispImplementation,"GCL") != -1) or (string.find(lispImplementation,"(GCL)") != -1):
            # POSIX: "GCL"
            # WIN NT: "(GCL)"
            _ret_ = "GCL"
        elif (string.find(lispImplementation,"CLISP") != -1):
            _ret_ =  "CLISP"
        else:
            _ret_ = "UNKNOWN"
        
        return _ret_

    def handleOut(self,line,maximaOut):
        # gelesen werden muss immer bis gilt: promptSuffix in 'line'
        # ausgabe (Rueckwert):  zw. displayPrefix und promptPrefix
        # naechste Eingabe (z. B. '(%i2)' )  zw. promptPrefix und promptSuffix
        # BEACHTE: Bei Fehlern ('Incorrect syntax') oder Aufruf des
        #          Lisp-Interpreters (':lisp')  wird kein displayPrefix ausgegeben!!!
        
        mitPrefix = 0
        promptPrefix = 0
        
        _debug_ = 0
        if _debug_:
            print line
            print "\n\n"

        # Test auf Ausgabe mit '_INTERFACE_GENERAL_DISPLAY_PREFIX_' am Anfang
        if line.startswith(self.displayPrefix):
            mitPrefix = 1

        # Test auf Ausgabe mit '_INTERFACE_PROMPT_PREFIX_' am Anfang    
        if line.startswith(self.promptPrefix):
            promptPrefix = 1


        if line.endswith(self.promptSuffix): # '_INTERFACE_PROMPT_SUFFIX_' am Ende !!
            pos0 = line.find(self.promptPrefix)
            pos1 = pos0 + self.proPrefix
            if promptPrefix:
                _pos0_  = line.find(self.promptPrefix,pos1)
                pos0 = _pos0_
                pos1 = pos0 + self.proPrefix                            
            pos2 = line.find(self.promptSuffix)
            if promptPrefix:
                pos4 = pos2 + self.proSuffix
                _pos2_  = line.find(self.promptSuffix,pos4)
                pos2 = _pos2_
            neuerPrompt = string.strip(line[pos1:pos2])
            
            
            if promptPrefix:
                if (line.find(self.displayPrefix) != -1): # mit _INTERFACE_GENERAL_DISPLAY_PREFIX_
                    pos5 = line.find(self.displayPrefix)                         
                    maximaOut = maximaOut + line[pos5:pos0]
                    if _debug_:
                        print 'Test-Ausgabe: #### => promptPrefix !!! MIT _INTERFACE_GENERAL_DISPLAY_PREFIX_ !!!'                     
                else:
                    maximaOut = maximaOut + line[pos4:pos0]
                    if _debug_:
                        print 'Test-Ausgabe: #### => promptPrefix (ohne _INTERFACE_GENERAL_DISPLAY_PREFIX_)'
                        
            elif mitPrefix:
                maximaOut = maximaOut + line[self.dispPrefix:pos0]
                if len(line[self.dispPrefix:pos0]) < 1:  # neu in Vers. 1.04 wg. Maxima-read-Befehl
                    if _debug_:
                        print "read-Treffer . . ., ",
                        print "line: ",line[self.dispPrefix:pos2]
                    maximaOut = maximaOut + line[self.dispPrefix:pos2]
                    return (maximaOut , '' , 0) # Wichtig!!!!
                if _debug_:
                    print 'Test-Ausgabe: #### => mitPrefix'
                    
            else:                
                maximaOut = maximaOut + line[:pos0]
                if _debug_:
                    print 'Test-Ausgabe: #### => REST: letzter Fall'                 
            return (maximaOut , neuerPrompt , 1) 
        else: # kein '_INTERFACE_PROMPT_SUFFIX_' am Ende !!
            if (line.find(self.displayPrefix) != -1): # mit _INTERFACE_GENERAL_DISPLAY_PREFIX_
                pos6 = line.find(self.displayPrefix)
                if _debug_:
                    print 'Test-Ausgabe: #### => kein >_INTERFACE_PROMPT_SUFFIX_< am Ende, mit self.displayPrefix !!'                 

            else:
                pos6 = 0
                if _debug_:
                    print 'Test-Ausgabe: #### => kein >_INTERFACE_PROMPT_SUFFIX_< am Ende ==>> pos6 == 0!!'                 
                
            #print 'Test-Ausgabe: #### ' + line[pos6:]
            maximaOut = maximaOut + line[pos6:]
            
            return (maximaOut,'',0) 



    def cleanOutput(self,arg):
        _debug_ = 0
        if _debug_:
             print arg        
        pos1 = arg.find(self.promptPrefix)
        pos2 = arg.find(self.promptSuffix)
        if (pos1 != -1):
            liste = arg.split(self.promptPrefix)
            ret1 = string.join(liste)
            arg = ret1
            if _debug_:
                print arg
        if (pos2 != -1):
            liste = arg.split(self.promptSuffix)
            ret1 = string.join(liste)
            arg = ret1
            if _debug_:
                print arg
        return arg
        
                     
    def buildOutput(self,wait=0,debug=0):
        # gefunden == 0: war wohl nix
        # gefunden == 1: alles ok
        # gefunden == 2: maxima will einen Wert/stellt eine Frage => Antwort-Fenster
        gefunden = 0
        retWert = None
        maximaOut = ''
        i = 0
        #print wait
        if (wait == 1): # plot/draw
            self.maxTry = 100
        elif (wait == 2): # ? help-Funktion
            if os.name == 'posix':
                self.maxTry = 6
            else: # win32/nt, dauert lange!
                self.maxTry = self.win32Factor * 6
        else:
            self.maxTry = 2
        while (gefunden == 0): #
            try: 
                zeile = Popen.recv_some(self.handle)#
            except:
                return ('',gefunden)
            if debug:
                print 'Retour: >>>>>:  ' + zeile
            (retWert,nextIn,gefunden) = self.handleOut(zeile,maximaOut)
            maximaOut = retWert
            i = i + 1
            time.sleep( self._delay_time_ )
            if len(zeile) < 1:
                if i > self.maxTry:
                    #print "gefunden == 2", retWert
                    retWert = self.cleanOutput(retWert)
                    #print "gefunden == 2", retWert
                    gefunden = 2
                    if (wait == 1) or (wait == 2): # ?help/plot => return-Wert auf ok setzen
                        gefunden = 1
                    if debug:
                        print "Abgebrochen!"
                    break
            #print maximaOut
        if debug:        
            print ">>>>> " + retWert + ", gefunden: " + str(gefunden)
        if retWert.startswith(self.displayPrefix):
            retWert = retWert[self.dispPrefix:]
        return (retWert,nextIn,gefunden)


    def waitTest(self,expr):
        """ Test auf Plotten!!"""
        _debug_ = 0
        trefferListe = ["plot"]
        if (string.find(expr,"plot") != -1): # Plot!
            if _debug_:
                print 'TREFFER: found "plot" <<<<<<<<<<<'
            return 1
        elif (string.find(expr,"draw") != -1): # Modul draw
            if _debug_:
                print 'TREFFER: found "draw" <<<<<<<<<<<'
            return 1        
        elif (string.find(expr,"? ") != -1): # Maxima-Hilfe
            return 2
        else:
            return 0




    def call(self,expr,_debug_=0):
        _debug_ = 0
        wait = False

        gefunden = 0
        if _debug_:
            print  expr 

        ## es geht los!
        wait = self.waitTest(expr)
        try:
            if (self.lispImplementationType == "GCL"):
                expr = util.ucodefilter_(expr)      # experimentell: Vers. 1.46
            else: # clisp
                expr = util.ucodefilter_(expr, org = 0)      # experimentell: Vers. 1.46
            Popen.send_all(self.handle, expr + self.tail)
            #time.sleep(abs(self._delay_time_ - 0.1))
            (text,nextIn,gefunden) = self.buildOutput(wait)
            if _debug_:                
                print ">>>>> " + text + ", gefunden: " + str(gefunden)
                print nextIn
            return (text,nextIn,gefunden)
        except:
            return ('','',gefunden)


    def startMaxima(self):
        _debug_ = 0
        magic = "_INTERFACE_MAXIMA_PROLOG_"
        if os.name == 'posix':
            #expr = 'maxima  --preload-lisp=' + lispSkript
            #expr = self.maximaDisableReadline + self.lispSkript
            expr = self.maximaCall + self.lispSkript            
            shell = 'sh'
        else:     # win nt   
            expr = self.pfad_zu_maxima + self.maximaWinCall + self.lispSkript # 
            shell = 'cmd'        
        if _debug_:
            print expr 
        try:
            self.handle = Popen.Popen(shell, stdin=subprocess.PIPE, stdout=subprocess.PIPE)

            time.sleep(self._delay_time_)
            Popen.send_all(self.handle, expr + self.tail)
            time.sleep(self._delay_time_)
            ret = Popen.recv_some(self.handle)
            if _debug_:                
                print ret
            time.sleep(self._delay_time_ + self.offset)
            if _debug_:                
                print 'Vor IF-Test auf "_INTERFACE_MAXIMA_PROLOG_" '
            
            if (string.find(ret,magic) == -1): # kein "_INTERFACE_MAXIMA_PROLOG_" gefunden => MIST!
                if _debug_:
                    print "First Try: Theres a CONNECT-Problem with maxima, Sorry!"
                raise 
            else:
                if _debug_:
                    print "else-Zweig . . ."
                
                self.lispImplementationType = self.getLispImplementationType()
                if _debug_:
                    print "else-Zweig:  self.getLispImplementationType()"
                    print "else-Zweig:  ", self.lispImplementationType
                    
                self.MaximaVersion = self.getMaximaVersion() # geht ab Version 5.27 schief . . .
                if _debug_:
                    print "else-Zweig;  self.getMaximaVersion()"
                    print "else-Zweig:  ", self.MaximaVersion
                    
                print "\nStarting maxima version %s, lisp implementation type: %s \n" %(self.MaximaVersion, self.lispImplementationType)
                return 1
        except:
            try:  # 2. Versuch . . .
                ret = Popen.recv_some(self.handle)
                time.sleep(self._delay_time_ + self.offset)
                if (string.find(ret,magic) == -1): # kein "_INTERFACE_MAXIMA_PROLOG_" gefunden => MIST!
                    if _debug_:
                        print "SECOND Try: Theres a CONNECT-Problem with maxima, Sorry . . ."
                    raise
                else:
                    self.lispImplementationType = self.getLispImplementationType()
                    self.MaximaVersion = self.getMaximaVersion()
                    print "\nStarting maxima version %s, lisp implementation type: %s \n" %(self.MaximaVersion, self.lispImplementationType)                
                    return 1
            except:
                return 0
	    
    def ctrlcHandler(self):
        try:
            for i in range(3):
                Popen.send_all(self.handle,'chr(3);' + self.tail) #            
                time.sleep(0.1)
            Popen.send_all(self.handle, "quit();" + self.tail)
            time.sleep(0.2)            
            #print Popen.recv_some(handle)
            Popen.send_all(self.handle, 'exit' + self.tail)
            #print "<ctrlcHandler> hat die Verbindung zu Maxima beendet!"
            return 1
        except:
            #print "<ctrlcHandler>: Mist!"            
            return 0
        
        

    def endMaxima(self):    
        try:
            Popen.send_all(self.handle, "quit();" + self.tail)
            time.sleep(0.2)
            #print Popen.recv_some(handle)
            Popen.send_all(self.handle, 'exit' + self.tail)
        except:
            pass

    def endConnection(self):    
        try:
            Popen.send_all(self.handle, 'exit' + self.tail)
        except:
            pass


#################################################

    
if __name__ == "__main__":
    # Die Datei "maxima.bat" muss im gleichem Verzeichnis stehen!!
    print 'Die Datei "maxima.bat" muss im gleichem Verzeichnis stehen!!'
    # Die Datei "interface.lisp" muss von diesem Verzeichnis aus im Unterverzeichnis "./init" stehen
    print 'Die Datei "interface.lisp" muss von diesem Verzeichnis aus'
    print 'im Unterverzeichnis "./init" stehen'
    
    print "\nStarte maxima-Interface Test!\n"
    nr = 1
    maxima = MaximaInterface( delay_time=0.4 )
    erg = maxima.startMaxima()
    print "startMaxima gibt retour: ", erg
    print '(%i2)',
    max_in = '2+3;'
    print max_in
    ergebnis = maxima.call(max_in)
    #print ergebnis
    print ergebnis[0]
    print ergebnis[1],
    max_in = '2*3;'
    print max_in    
    ergebnis = maxima.call(max_in)
    #print ergebnis
    print ergebnis[0]
    print ergebnis[1],
    max_in = 'build_info();'
    print max_in    
    ergebnis = maxima.call(max_in)    
    print ergebnis[0]
    print ergebnis[1],    
    max_in = 'diff(x^2-x,x);'
    print max_in    
    ergebnis = maxima.call(max_in)    
    print ergebnis[0]
    print ergebnis[1],
    max_in = 'diff(1/x^2,x);'
    print max_in    
    ergebnis = maxima.call(max_in)        
    print ergebnis[0]
    print ergebnis[1],
    max_in = ':lisp (+ 2 3);'
    print max_in    
    ergebnis = maxima.call(max_in)            
    print ergebnis[0]
    print ergebnis[1],
    max_in = '?? integ;'
    print max_in    
    ergebnis = maxima.call(max_in)                
    #ergebnis = maxima.call('?? integ;')
    print ergebnis[0]
    print ergebnis[1]
    
    maxima.endMaxima()
    print "\n>>> Fertig! <<<\n"
    eingabe = raw_input("Enter or Return")



############################################
# Testlauf
#
##Starte maxima-Interface Test!
##
##(%i1) 2+3;
##(%o1) 				       5
##
##(%i2) 2*3;
##(%o2) 				       6
##
##(%i3) diff(x^2-X(;
##Incorrect syntax: Premature termination of input at ;.
##diff(x^2-X(;
##          ^
##
##(%i3) diff(1/x^2,x);
##				               2
##(%o3) 				     - --
##					        3
##				               x
##
##(%i4) :lisp (+ 2 3);
##
##5
##
##(%i4) ? integ;
##
## 0: (maxima.info)Introduction to Elliptic Functions and Integrals.
## 1: Definitions for Elliptic Integrals.
## 2: Integration.
## 3: Introduction to Integration.
## 4: Definitions for Integration.
## 5: askinteger :Definitions for Simplification.
## 6: integer_partitions :Definitions for Sets.
## 7: integerp :Definitions for Miscellaneous Options.
## 8: integrate :Definitions for Integration.
## 9: integrate_use_rootsof :Definitions for Integration.
## 10: integration_constant_counter :Definitions for Integration.
## 11: nonnegintegerp :Definitions for linearalgebra.
##
##Enter space-separated numbers, `all' or `none':
##
##>>> Fertig! <<<

