# -*- coding: utf-8 -*-
"""
    pymax2geo.py: Schnittstelle zu pygeo, W.S.,
    Vers. 0.11, 2008-02-17 (Methode dreieck, viereck, pyramide, tetraeder ==>> Klasse max2geo)
    Vers. 0.12, 2008-03-29 (Methode kreis ==>> Klasse max2geo)
    Vers. 0.13, 2008-04-02 (Farbunterstuetzung in der Klasse max2geo; s. parameterformEbene, ebene, normalformEbene )
    Vers. 0.14, 2008-04-04 (Farbunterstuetzung in der Klasse max2geo; s. parameterformGerade, gerade, punkt)
    Vers. 0.15, 2008-05-04 (Farbunterstuetzung in der Klasse max2geo; s. kugel, kreis );
                           ACHTUNG: Methode _punkt_ in der Klasse max2geo modifiziert:
                           erg = string.lstrip(erg,' ') # neu: fuehrende Leerzeichen entfernen
    Vers. 0.16, 2008-05-20 (Farbunterstuetzung in der Klasse max2geo; s. dreieck );
    Vers. 0.17, 2008-05-22 (Farbunterstuetzung in der Klasse max2geo; s. viereck, tetraeder, pyramide );
    Vers. 0.18, 2008-07-27 neue Methoden in der Klasse geoObjects: saveAppendGeoDir (sicheres Anhaengen!),
                           _setGeoName_ (interne Methode!!)
    Vers. 0.19, 2011-04-03 Methoden _ende_ modifiziert, neuer Parameter dragPointing=0
    Vers. 0.20, 2012-02-26 neue Methode quader(self,p1,p2,p3,p4,p5,p6,p7,p8,maxima,name="Q1",color=0)
    Vers. 0.21, 2015-03-29 neue Methode strecke(self,p1,p2,maxima,name="S1",color=0)
    
"""

import string
import time




def _start_():
    "erzeugt den Kopf des pyGeo-Scripts"
    import time    
    zeit = time.strftime("%A %B %d, %Y %I:%M %p",time.localtime(time.time()))
    
    # Punktgroesse fuer pyGeo
    pointsize = "pointsize=0.4"
    
    scriptHead="#\n#\t pyGeo-Script, generated by pyMaxima\n#\t on " + zeit + "\n#\n"
    scriptStart='from pygeo import *\ndisplay=display(axis=True, panel = False, title="pyGeo-Sitzung")\nPoint(' + pointsize + ')\n'          
    return scriptHead + scriptStart    


def _ende_ ():
    "erzeugt das Ende des pyGeo-Scripts"
    #scriptEnd="""display.pickloop(dragPointing=0)\n\n"""
    scriptEnd="""try:\n"""
    scriptEnd=scriptEnd + """   display.pickloop(dragPointing=0)\n"""
    scriptEnd=scriptEnd + """except:\n"""
    scriptEnd=scriptEnd + """   display.pickloop()\n\n"""    
    return scriptEnd



def generateScript(scriptList=[]):
    "erzeugt das pyGeo-Scripts"    
    type_Liste = "<type 'list'>"
    script = ""
    #print scriptList
    if len(scriptList) > 0:
        for element in scriptList:
	    erg = str(type(element) )
	    if (erg == type_Liste):
		for el in element:
			script = script + el + '\n'			
	    else:
		script = script + element + '\n'
    return _start_() + script + _ende_()


def script2maxima(scriptList):
    "erzeugt aus einem pyGeo-Script die maxima-Befehle"
    maximaList = []
    for element in scriptList:
        if len(element) > 1:
            if (element[0] == '#') and (element[1] == '#'):  #  "##" => so beginnt eine maxima-Zeile
                maximaList.append(string.strip(element[2:]) )
    return maximaList



class max2geo:
    """Umsetzung maxima => pyGeo
       Generiere die entsprechenden pyGeo-Befehle 
       Beachte: Bei jedem Befehl wird -als python-Kommentar
       mit ## eingeleitet- zugleich der
       korresponiderende Maxima-Befehl erzeugt.
       Die Methode 'script2maxima' erzeugt
       dann aus einer Script-Liste eine Liste von
       Maxima-Befehlen!
       color == 0 ==>> Default-Color!
    """
    def __init__(self):
        self.pointsize = "pointsize=0.3"
        self.colList = ["Default  ","Black    ", "Red      ", "Green    ", "Yellow   ", "Blue     ", "Magenta  ", "Cyan     ", "Darkgray ", "Lightgray", "Brown    ", "Orange   ", "Pink     ", "Purple   ","Verylight", "White    "]
        self.pyGeoColor = ["BLACK", "RED", "GREEN", "YELLOW", "BLUE", "MAGENTA", "CYAN", "DARKGRAY", "LIGHTGRAY", "BROWN", "ORANGE", "PINK", "PURPLE","VERYLIGHT", "WHITE"]        

    def _punkt_(self,koordinaten,name="p1",label=None,color=0):
        """Punkt im Koordinatensystem"""
        dim = len(koordinaten)
        ret = name + " = Point(" 
        for j in range(len(koordinaten)):
            if len(koordinaten[j]) > 1:  # fuehrende Nullen rauswerfen => mag pyGeo nicht
                erg = string.lstrip(koordinaten[j],'0') 
            else:
                erg = koordinaten[j]
                
            erg = string.lstrip(erg,' ') # neu: fuehrende Leerzeichen rauswerfen, 2008-05-04
            #print '"' + erg + '"' # neu
            
            if j < dim - 1:
                ret = ret + erg + ','
            else:
                if label:
                    if color == 0:
                        ret = ret + erg + ',' + self.pointsize + ", label='" + str(label) + "')"
                    else:
                        index = color - 1 # da self.pyGeoColor ohne Default-Eintrag
                        farbe = self.pyGeoColor[index]
                        ret = ret + erg + ',' + self.pointsize + ", label='" + str(label) + "',color=" + farbe + ")"
                            
                else:
                    if color == 0:
                        ret = ret + erg + ',' + self.pointsize + ')'
                    else:
                        index = color - 1 # da self.pyGeoColor ohne Default-Eintrag
                        farbe = self.pyGeoColor[index]
                        ret = ret + erg + ',' + self.pointsize + ",color=" + farbe + ")"
                            
                        
        return (ret)

    def _generatePoint_(self,gerade,punkt):
	retP = []  # Punkt generieren
	for j in range(len(gerade)):
		retP.append(str( float(punkt[j]) + float(gerade[j]) )    )
	return retP


    def punkt(self,koordinaten,maxima,name="p1",label=None,color=0):
        """Punkt im Koordinatensystem"""
        geoPunkt = '## "Punkt ' + name + '"$\n'  # Maxima-Befehl
        geoPunkt = geoPunkt + "## " + name + " : " + maxima + ";\n"  # Maxima-Befehl
        
        geoPunkt = geoPunkt + self._punkt_(koordinaten,name,label,color)
        return (geoPunkt)

        
    def richtVec(self,koordinaten,maxima,name="uvec1",label=None):
        """Richtungs- oder Normalenvektor"""
        geoVec = '## "Vektor ' + name + '"$\n'  # Maxima-Befehl
        geoVec = geoVec + "## " + name + " : " + maxima + ";"  # Maxima-Befehl
        return (geoVec)

    def strecke(self,p1,p2,maxima,name="S1",color=0):
        """Strecke von A nach B"""        
        geoStrecke = '## "Strecke ' + name + '"$\n'  # Maxima-Befehl        
    
        if color == 0:
            geoStrecke = geoStrecke + name + "AB = Line(" + name + "p1 ," + name +  "p2 , seg=True )"
        else:
            index = color - 1 # da self.pyGeoColor ohne Default-Eintrag
            farbe = self.pyGeoColor[index]
            geoStrecke = geoStrecke + name + "AB = Line(" + name + "p1 ," + name +  "p2 ,color=" + farbe + ", seg=True )"
            
        return (geoStrecke)



    def dreieck(self,p1,p2,p3,maxima,name="D1",color=0):
        """Dreieck durch drei Punkte p1,p2,p3"""        
        geoDreieck = '## "Dreieck ' + name + '"$\n'  # Maxima-Befehl        
        #geoDreieck = geoDreieck + "## " + name + " : " + maxima + ";\n"  # Maxima-Befehl
        if color == 0:
            geoDreieck = geoDreieck + name + "a = Line(" + name + "p2 ," + name +  "p3 , seg=True )\n"
            geoDreieck = geoDreieck + name + "b = Line(" + name + "p1 ," + name +  "p3 , seg=True )\n"
            geoDreieck = geoDreieck + name + "c = Line(" + name + "p1 ," + name +  "p2 , seg=True )"
        else:
            index = color - 1 # da self.pyGeoColor ohne Default-Eintrag
            farbe = self.pyGeoColor[index]
            geoDreieck = geoDreieck + name + "a = Line(" + name + "p2 ," + name +  "p3 ,color=" + farbe + ", seg=True )\n"
            geoDreieck = geoDreieck + name + "b = Line(" + name + "p1 ," + name +  "p3 ,color=" + farbe + ", seg=True )\n"
            geoDreieck = geoDreieck + name + "c = Line(" + name + "p1 ," + name +  "p2 ,color=" + farbe + ", seg=True )"
            
        return (geoDreieck)
        
    def tetraeder(self,p1,p2,p3,p4,maxima,name="T1",color=0):
        """Tetraeder durch vier Punkte p1,p2,p3,p4"""        
        geoTetraeder = '## "Tetraeder ' + name + '"$\n'  # Maxima-Befehl        
        #geoTetraeder = geoTetraeder + "## " + name + " : " + maxima + ";\n"  # Maxima-Befehl
        if color == 0:
            geoTetraeder = geoTetraeder + name + "a = Line(" + name + "p2 ," + name +  "p3 , seg=True )\n"
            geoTetraeder = geoTetraeder + name + "b = Line(" + name + "p1 ," + name +  "p3 , seg=True )\n"
            geoTetraeder = geoTetraeder + name + "c = Line(" + name + "p1 ," + name +  "p2 , seg=True )\n"
            geoTetraeder = geoTetraeder + name + "d = Line(" + name + "p1 ," + name +  "p4 , seg=True )\n"
            geoTetraeder = geoTetraeder + name + "e = Line(" + name + "p2 ," + name +  "p4 , seg=True )\n"
            geoTetraeder = geoTetraeder + name + "f = Line(" + name + "p3 ," + name +  "p4 , seg=True )\n"        
        else:        
            index = color - 1 # da self.pyGeoColor ohne Default-Eintrag
            farbe = self.pyGeoColor[index]        
            geoTetraeder = geoTetraeder + name + "a = Line(" + name + "p2 ," + name +  "p3 ,color=" + farbe + ", seg=True )\n"
            geoTetraeder = geoTetraeder + name + "b = Line(" + name + "p1 ," + name +  "p3 ,color=" + farbe + ", seg=True )\n"
            geoTetraeder = geoTetraeder + name + "c = Line(" + name + "p1 ," + name +  "p2 ,color=" + farbe + ", seg=True )\n"
            geoTetraeder = geoTetraeder + name + "d = Line(" + name + "p1 ," + name +  "p4 ,color=" + farbe + ", seg=True )\n"
            geoTetraeder = geoTetraeder + name + "e = Line(" + name + "p2 ," + name +  "p4 ,color=" + farbe + ", seg=True )\n"
            geoTetraeder = geoTetraeder + name + "f = Line(" + name + "p3 ," + name +  "p4 ,color=" + farbe + ", seg=True )\n"        
        return (geoTetraeder)

    def viereck(self,p1,p2,p3,p4,maxima,name="V1",color=0):
        """Viereck durch vier Punkte p1,p2,p3,p4"""        
        geoViereck = '## "Viereck ' + name + '"$\n'  # Maxima-Befehl        
        #geoViereck = geoViereck + "## " + name + " : " + maxima + ";\n"  # Maxima-Befehl
        if color == 0:        
            geoViereck = geoViereck + name + "a = Line(" + name + "p1 ," + name +  "p2 , seg=True )\n"
            geoViereck = geoViereck + name + "b = Line(" + name + "p2 ," + name +  "p3 , seg=True )\n"
            geoViereck = geoViereck + name + "c = Line(" + name + "p3 ," + name +  "p4 , seg=True )\n"
            geoViereck = geoViereck + name + "d = Line(" + name + "p4 ," + name +  "p1 , seg=True )\n"
        else:        
            index = color - 1 # da self.pyGeoColor ohne Default-Eintrag
            farbe = self.pyGeoColor[index]        
            geoViereck = geoViereck + name + "a = Line(" + name + "p1 ," + name +  "p2 ,color=" + farbe + ", seg=True )\n"
            geoViereck = geoViereck + name + "b = Line(" + name + "p2 ," + name +  "p3 ,color=" + farbe + ", seg=True )\n"
            geoViereck = geoViereck + name + "c = Line(" + name + "p3 ," + name +  "p4 ,color=" + farbe + ", seg=True )\n"
            geoViereck = geoViereck + name + "d = Line(" + name + "p4 ," + name +  "p1 ,color=" + farbe + ", seg=True )\n"
            
        return ( geoViereck )

######################  to do!!!!   
    def quader(self,p1,p2,p3,p4,p5,p6,p7,p8,maxima,name="Q1",color=0):
        """Quader durch acht Punkte p1,p2,p3,p4,p5,p6,p7,p8"""        
        geoQuader = '## "Quader ' + name + '"$\n'  # Maxima-Befehl        
        #geoQuader = geoQuader + "## " + name + " : " + maxima + ";\n"  # Maxima-Befehl
        if color == 0:        
            geoQuader = geoQuader + name + "a = Line(" + name + "p1 ," + name +  "p2 , seg=True )\n"
            geoQuader = geoQuader + name + "b = Line(" + name + "p2 ," + name +  "p3 , seg=True )\n"
            geoQuader = geoQuader + name + "c = Line(" + name + "p3 ," + name +  "p4 , seg=True )\n"
            geoQuader = geoQuader + name + "d = Line(" + name + "p4 ," + name +  "p1 , seg=True )\n"
            geoQuader = geoQuader + name + "e = Line(" + name + "p1 ," + name +  "p5 , seg=True )\n"
            geoQuader = geoQuader + name + "f = Line(" + name + "p2 ," + name +  "p6 , seg=True )\n"
            geoQuader = geoQuader + name + "g = Line(" + name + "p3 ," + name +  "p7 , seg=True )\n"
            geoQuader = geoQuader + name + "h = Line(" + name + "p4 ," + name +  "p8 , seg=True )\n"
            geoQuader = geoQuader + name + "j = Line(" + name + "p5 ," + name +  "p6 , seg=True )\n"
            geoQuader = geoQuader + name + "k = Line(" + name + "p6 ," + name +  "p7 , seg=True )\n"
            geoQuader = geoQuader + name + "l = Line(" + name + "p7 ," + name +  "p8 , seg=True )\n"
            geoQuader = geoQuader + name + "m = Line(" + name + "p8 ," + name +  "p5 , seg=True )\n"            
            
        else:        
            index = color - 1 # da self.pyGeoColor ohne Default-Eintrag
            farbe = self.pyGeoColor[index]        
            geoQuader = geoQuader + name + "a = Line(" + name + "p1 ," + name +  "p2 ,color=" + farbe + ", seg=True )\n"
            geoQuader = geoQuader + name + "b = Line(" + name + "p2 ," + name +  "p3 ,color=" + farbe + ", seg=True )\n"
            geoQuader = geoQuader + name + "c = Line(" + name + "p3 ," + name +  "p4 ,color=" + farbe + ", seg=True )\n"
            geoQuader = geoQuader + name + "d = Line(" + name + "p4 ," + name +  "p1 ,color=" + farbe + ", seg=True )\n"
            geoQuader = geoQuader + name + "e = Line(" + name + "p1 ," + name +  "p5 ,color=" + farbe + ",  seg=True )\n"
            geoQuader = geoQuader + name + "f = Line(" + name + "p2 ," + name +  "p6 ,color=" + farbe + ",  seg=True )\n"
            geoQuader = geoQuader + name + "g = Line(" + name + "p3 ," + name +  "p7 ,color=" + farbe + ",  seg=True )\n"
            geoQuader = geoQuader + name + "h = Line(" + name + "p4 ," + name +  "p8 ,color=" + farbe + ",  seg=True )\n"
            geoQuader = geoQuader + name + "j = Line(" + name + "p5 ," + name +  "p6 ,color=" + farbe + ",  seg=True )\n"
            geoQuader = geoQuader + name + "k = Line(" + name + "p6 ," + name +  "p7 ,color=" + farbe + ",  seg=True )\n"
            geoQuader = geoQuader + name + "l = Line(" + name + "p7 ," + name +  "p8 ,color=" + farbe + ",  seg=True )\n"
            geoQuader = geoQuader + name + "m = Line(" + name + "p8 ," + name +  "p5 ,color=" + farbe + ",  seg=True )\n"            
            
        return ( geoQuader )
######################  to do!!!!

    def pyramide(self,p1,p2,p3,p4,s1,maxima,name="Py1",color=0):
        """Pyramide durch fuenf Punkte p1,p2,p3,p4, s1"""        
        geoPyramide = '## "Pyramide ' + name + '"$\n'  # Maxima-Befehl        
        #geoPyramide = geoPyramide + "## " + name + " : " + maxima + ";\n"  # Maxima-Befehl
        if color == 0:                
            geoPyramide = geoPyramide + name + "a = Line(" + name + "p1 ," + name +  "p2 , seg=True )\n"
            geoPyramide = geoPyramide + name + "b = Line(" + name + "p2 ," + name +  "p3 , seg=True )\n"
            geoPyramide = geoPyramide + name + "c = Line(" + name + "p3 ," + name +  "p4 , seg=True )\n"
            geoPyramide = geoPyramide + name + "d = Line(" + name + "p4 ," + name +  "p1 , seg=True )\n"
            geoPyramide = geoPyramide + name + "e = Line(" + name + "p1 ," + name +  "s1 , seg=True )\n"
            geoPyramide = geoPyramide + name + "f = Line(" + name + "p2 ," + name +  "s1 , seg=True )\n"
            geoPyramide = geoPyramide + name + "g = Line(" + name + "p3 ," + name +  "s1 , seg=True )\n"
            geoPyramide = geoPyramide + name + "h = Line(" + name + "p4 ," + name +  "s1 , seg=True )\n"
        else:        
            index = color - 1 # da self.pyGeoColor ohne Default-Eintrag
            farbe = self.pyGeoColor[index]        
            geoPyramide = geoPyramide + name + "a = Line(" + name + "p1 ," + name +  "p2 ,color=" + farbe + ", seg=True )\n"
            geoPyramide = geoPyramide + name + "b = Line(" + name + "p2 ," + name +  "p3 ,color=" + farbe + ", seg=True )\n"
            geoPyramide = geoPyramide + name + "c = Line(" + name + "p3 ," + name +  "p4 ,color=" + farbe + ", seg=True )\n"
            geoPyramide = geoPyramide + name + "d = Line(" + name + "p4 ," + name +  "p1 ,color=" + farbe + ", seg=True )\n"
            geoPyramide = geoPyramide + name + "e = Line(" + name + "p1 ," + name +  "s1 ,color=" + farbe + ", seg=True )\n"
            geoPyramide = geoPyramide + name + "f = Line(" + name + "p2 ," + name +  "s1 ,color=" + farbe + ", seg=True )\n"
            geoPyramide = geoPyramide + name + "g = Line(" + name + "p3 ," + name +  "s1 ,color=" + farbe + ", seg=True )\n"
            geoPyramide = geoPyramide + name + "h = Line(" + name + "p4 ," + name +  "s1 ,color=" + farbe + ", seg=True )\n"            
        return (geoPyramide)



    def gerade(self,p1,p2,maxima,name="g1",color=0):
        """Gerade durch zwei Punkte"""
        geoGerade = '## "Gerade ' + name + '"$\n'  # Maxima-Befehl        
        geoGerade = geoGerade + "## " + name + " : " + maxima + ";\n"  # Maxima-Befehl
        if color == 0:               
            geoGerade = geoGerade + name + " = Line(" + p1 + "," + p2 +  ", label='" + name + "' )"
        else:
            index = color - 1 # da self.pyGeoColor ohne Default-Eintrag
            farbe = self.pyGeoColor[index]
            geoGerade = geoGerade + name + " = Line(" + p1 + "," + p2 +  ", label='" + name + "',color=" + farbe + ")"                                
            
        return (geoGerade)


    def parameterformGerade(self,g1,p1,maxima,name="g1",color=0):
        """Gerade in Parameterform"""
        geoGerade = '## "Gerade ' + name + '"$\n'  # Maxima-Befehl                
        geoGerade = geoGerade + "## " + name + " : " + maxima + ";\n"  # Maxima-Befehl
        
	p2 = self._generatePoint_(g1,p1)
	geoGerade = geoGerade + self._punkt_(p2, name + "p2")  + "\n"

        if color == 0:               	
            geoGerade = geoGerade + name + " = Line(stuetz" + name + ","  + name + "p2, label='" + name + "' )"
        else:
            index = color - 1 # da self.pyGeoColor ohne Default-Eintrag
            farbe = self.pyGeoColor[index]
            geoGerade = geoGerade + name + " = Line(stuetz" + name + ","  + name + "p2, label='" + name + "',color=" + farbe + ")"                                
        return (geoGerade)


    def schnittkreis(self,k1,k2,maxima,name="SK1",color=0):
        """Schnittkreis durch zwei Kugeln"""
        geoKreis = '## "Kreis ' + name + '"$\n'  # Maxima-Befehl   
        geoKreis = geoKreis + "## " + name + " : " + maxima + ";\n"  # Maxima-Befehl
        if color == 0:                       
            geoKreis = geoKreis + name + " = Circle(" + k1 + "," + k2 + ")"
        return (geoKreis)

    def schnittkreisE(self,k1,e1,maxima,name="SK1",color=0):
        """Schnittkreis Kugel, Ebene"""
        geoKreis = '## "Kreis ' + name + '"$\n'  # Maxima-Befehl   
        geoKreis = geoKreis + "## " + name + " : " + maxima + ";\n"  # Maxima-Befehl
        if color == 0:                       
            geoKreis = geoKreis + name + " = Circle(" + k1 + "," + e1 + ")"
        return (geoKreis)


    def kreis(self,p1,p2,p3,maxima,name="SK1",color=0):
        """Kreis durch drei Punkte (Umkreis = circumscribed circle)"""
        geoKreis = '## "Kreis ' + name + '"$\n'  # Maxima-Befehl   
        geoKreis = geoKreis + "## " + name + " : " + maxima + ";\n"  # Maxima-Befehl
        if color == 0:               
            geoKreis = geoKreis + name + " = Circle(" + p1 + "," + p2 + "," + p3 + ",CIRCUM)"
        else:
            index = color - 1 # da self.pyGeoColor ohne Default-Eintrag
            farbe = self.pyGeoColor[index]            
            geoKreis = geoKreis + name + " = Circle(" + p1 + "," + p2 + "," + p3 + ",CIRCUM,color=" + farbe + ")"
        return (geoKreis)
    

    def ebene(self,p1,p2,p3,maxima,name="E1",color=0):
        """Ebene durch drei Punkte"""
        geoEbene = '## "Ebene ' + name + '"$\n'  # Maxima-Befehl   
        geoEbene = geoEbene + "## " + name + " : " + maxima + ";\n"  # Maxima-Befehl
        if color == 0:                       
            geoEbene = geoEbene + name + " = Plane(" + p1 + "," + p2 + "," + p3 + ")"
        else:
            index = color - 1 # da self.pyGeoColor ohne Default-Eintrag
            farbe = self.pyGeoColor[index]
            geoEbene = geoEbene + name + " = Plane(" + p1 + "," + p2 + "," + p3 + ",color=" + farbe + ")"
        return (geoEbene)


    def parameterformEbene(self,g1,g2,p1,maxima,name="E1",color=0):
        """Ebene in Parameterform"""
        geoEbene = '## "Ebene ' + name + '"$\n'  # Maxima-Befehl           
        geoEbene = geoEbene + "## " + name + " : " + maxima + ";\n"  # Maxima-Befehl

        p2 = self._generatePoint_(g1,p1)
        geoEbene = geoEbene + self._punkt_(p2,name + "p2")  + "\n"
        
        p3 = self._generatePoint_(g2,p1)
        geoEbene = geoEbene + self._punkt_(p3, name + "p3")  + "\n"

        if color == 0:               
            geoEbene = geoEbene + name + " = Plane(stuetz" + name + "," + name + "p2," + name + "p3 )"
        else:
            index = color - 1 # da self.pyGeoColor ohne Default-Eintrag
            farbe = self.pyGeoColor[index]            
            geoEbene = geoEbene + name + " = Plane(stuetz" + name + "," + name + "p2," + name + "p3,color=" + farbe + ")"
        return (geoEbene)


    def normalformEbene(self,g1,g2,p1,d,maxima,name="E1",color=0):
        """Ebene in Normalenform"""
        geoEbene = '## "Ebene ' + name + '"$\n'  # Maxima-Befehl
        geoEbene = geoEbene + "## d" + name + " : " + d + ";\n"  # Maxima-Befehl
        geoEbene = geoEbene + "## " + name + " : " + maxima + ";\n"  # Maxima-Befehl
        
        geoEbene = geoEbene + self._punkt_(p1,name + "p")  + "\n" # Aufpunkt
        
        p2 = self._generatePoint_(g1,p1)
        geoEbene = geoEbene + self._punkt_(p2,name + "p2")  + "\n"
        
        p3 = self._generatePoint_(g2,p1)
        geoEbene = geoEbene + self._punkt_(p3, name + "p3")  + "\n"

        if color == 0:        
            geoEbene = geoEbene + name + " = Plane(" + name + "p," + name + "p2," + name + "p3 )"
        else:
            index = color - 1 # da self.pyGeoColor ohne Default-Eintrag
            farbe = self.pyGeoColor[index]
            geoEbene = geoEbene + name + " = Plane(" + name + "p," + name + "p2," + name + "p3,color=" + farbe + ")"
        return (geoEbene)



    def kugel(self,m,p,maxima,name="K1",color=0):
        """Kugel mit Mittelpunkt m und Punkt p auf Oberfläche"""
        geoKugel = '## "Kugel ' + name + '"$\n'  # Maxima-Befehl   
        geoKugel = geoKugel + "## " + name + " : " + maxima + ";\n"  # Maxima-Befehl
        if color == 0:               
            geoKugel = geoKugel + name + " = Sphere(" + m + "," + p + ")"
        else:
            index = color - 1 # da self.pyGeoColor ohne Default-Eintrag
            farbe = self.pyGeoColor[index]            
            geoKugel = geoKugel + name + " = Sphere(" + m + "," + p + ",color=" + farbe + ")"
        # k1 = Sphere(m,p)
        return (geoKugel)




######################################################

class geoObjects:
    """ speichert Geometrie-Objekte in einem Dicitonary
        Benutzung:
        geoOops = geoObjects()  # global
        geoOops.getGeoDir()     # alle Geometrie-Objekte anzeigen
        # in den Methoden punkt,gerade,ebene
        liste = geoOops.buildListEntry(name,beschreibung,geoScript,maxima_rep)
        geoOops.addGeoDirEntry(name,liste)
        geoOops.getObject(name)
        
        ==> Problem: was ist mit abhängigen Objekten?
        Beispiel: Gerade g durch zwei Punkte p1 und P2?
    """
    def __init__(self):
        self.geoDir = {}
        self._maxPosten_ = 5 # für buildListEntry

        
    def getObject(self,name):
        print str(self.geoDir[name])
        
    def addGeoDirEntry(self,name,wert):
        self.geoDir[name] = wert

    def delGeoDirEntry(self,name):
        del(self.geoDir[name])

    def clearGeoDir(self):
        self.geoDir.clear()        


    def buildListEntry(self,name,beschreibung,maxima_rep,geoScript,_more_=None):
        # ohne _more_: self._maxPosten_ = 5
        # mit _more_: self._maxPosten_ + 1
        flag = 0
        valueList = []
        valueList.append(flag)          # pos 0    
        valueList.append(name)          # pos 1
        valueList.append(beschreibung)  # pos 2
        valueList.append(maxima_rep)    # pos 3 
        valueList.append(geoScript)     # pos 4
        if _more_:
            valueList.append(_more_)    # pos 5

        return valueList                

    def getGeoElement(self,name):
        eintrag = self.geoDir[name]
        if len(eintrag) > self._maxPosten_:
            eintrag = eintrag[:-2] # eintrag ohne Script/_more_
        else:
            eintrag = eintrag[:-1] # eintrag ohne Script
        zeile = []
        for element in eintrag:
            zeile.append( str(element).replace("matrix ","")  ) # Darstellung ohne matrix	    	    
	return zeile  # gibt zeile retour

    def getGeoType(self,name):
        eintrag = self.geoDir[name]        
        return eintrag[2] # eintrag->beschreibung/Type

    def getGeoName(self,name):
        eintrag = self.geoDir[name]        
        return eintrag[1] # eintrag->Name

    def _setGeoName_(self,neuerName):
        # falls ein Eintrag >neuerName< in geoDir exitiert, so wird
        # der Name des Objekts (=eintrag[1]) auf >neuerName< gesetzt
        # Methode wird benutzt in saveAppendGeoDir
        try:
            eintrag = self.geoDir[neuerName]
            eintrag[1] = neuerName
            # eintrag[1] => eintrag->Name
        except:
            pass



    def getGeoDir(self):
        bereich = self.geoDir.keys()
        bereich.sort()
	ret = []
	scripts = []
        for name in bereich:
            skript = self.getScript(name)
            scripts.append( skript )	 # Script-Liste
            zeile = self.getGeoElement(name)
	    ret.append(zeile) # Eintrag fürs Fenster
	return ret,scripts  # gibt zwei Listen zurück!


    def saveAppendGeoDir(self,appendDici):
        """tiefe Kopie eines Dicis mit sicherem Anhaengen (=append)
        """
        maxFromDici = len(appendDici)
        maxToDici = len(self.geoDir)
        if (maxToDici == 0):
            self.geoDir.clear()
        if maxFromDici > 0:        
            bereich = appendDici.iterkeys()
            for j in bereich:
                if not ( self.geoDir.has_key(j) ): # new entry => append                     
                    self.geoDir[j] = appendDici[j]
                else:   # same name
                    index = 1
                    neuerName = str(j) + "_" + str(index)   # Object with same name => new name: name + "_1"                    
                    while self.geoDir.has_key(neuerName): # try new name
                        index = index + 1
                        neuerName = str(j) + "_" + str(index)
                        
                    self.geoDir[neuerName] = appendDici[j]
                    self._setGeoName_(neuerName)


    def getObjectList(self,objectString):
        """Aufruf: getObjectList("Punkt")
            getObjectList("Gerade")
            getObjectList("Ebene")
            bzw. in pyMaxima:
            punktListe = self.geoOops.getObjectList("Punkt") 
            ebenenListe = self.geoOops.getObjectList("Ebene") 
            geradenListe = self.geoOops.getObjectList("Gerade")
            kugelListe = self.geoOops.getObjectList("Kugel")
            kreisListe = self.geoOops.getObjectList("Kreis")             
            vektorListe = self.geoOops.getObjectList("Vektor") 
        """
        bereich = self.geoDir.keys()
        bereich.sort()
	ret = []
	for name in bereich:
            eintrag = self.geoDir[name]
            if eintrag[2].find(objectString,0,6) != -1:
                ret.append(name)
	return ret  # gibt namens-Liste zurück!


    def getAllObjectNameList(self):
        """Aufruf: liste = getAllObjectNameList()
        """
        bereich = self.geoDir.keys()
        bereich.sort()
	ret = []
	for name in bereich:
            ret.append(name)
	return ret  # gibt namens-Liste zurück!


    def generateObjectName(self,basis):
        """Aufruf: name = generateObjectName("P")
        """
        geoListe = self.getAllObjectNameList()                    
        geoName = basis + "1"
        for i in range(2,99):
            if geoName in geoListe: # Name schon vorhanden
                geoName = basis + str(i) # naechster Versuch
            else:   # Name ok
                break
        return geoName          


    def getSimpleDir(self):
        bereich = self.geoDir.keys()
        bereich.sort()
	ret = []
	keyList = []
        for name in bereich:
            eintrag = self.geoDir[name]
            # print self._maxPosten_,len(eintrag)            
            if len(eintrag) > self._maxPosten_:
                eintrag = eintrag[:-2] # eintrag ohne Script/_more_
            else:
                eintrag = eintrag[:-1] # eintrag ohne Script

            zeile = []
            for element in eintrag:
                zeile.append( str(element).replace("matrix ","")  ) # Darstellung ohne matrix	    
	    ret.append(zeile) # Eintrag fürs Fenster
	    keyList.append(name)
	return ret,keyList  # gibt Liste zurück!

    def getScript(self,name):
        """gibt das Skript zu einem Objekt mit <name> retour"""
        #print "getScript . . .\n"
        eintrag = self.geoDir[name]
        if len(eintrag) > self._maxPosten_:            
            return eintrag[self._maxPosten_ - 1]  # eintrag mit _more_
        else:
            return eintrag[self._maxPosten_ - 1]  # eintrag ohne _more_
        
    def printEintrag(self,name):
        eintrag = self.geoDir[name]
        for element in eintrag:
            print element

    def getScriptEintrag(self,name):
        import string
        ret = []        
        skript = self.getScript(name)
        for teil in skript:
            teilLi = teil.split('\n')
            for element in teilLi:
                if string.find(element,'##') == -1:
                    ret.append(element) # liefert pyGeo-Repraesentation
        return ret

        
    def printScriptEintrag(self,name):
        import string
        skript = self.getScript(name)
        for teil in skript:
            teilli = teil.split('\n')
            for element in teilli:
                if string.find(element,'##') == -1:
                    print element # liefert pyGeo-Repraesentation

    def printMaximaEintrag(self,name):
        import string
        skript = self.getScript(name)
        for teil in skript:
            teilli = teil.split('\n')
            for element in teilli:
                if string.find(element,'##') != -1:
                    print element # liefert Maxima-Repraesentation

    def getMaximaEintrag(self,name):
        import string
        ret = []
        skript = self.getScript(name)
        #print skript
        for teil in skript:
            teilLi = teil.split('\n')
            for element in teilLi:
                #print element
                if string.find(element,'##') != -1:
                    maxBefehl = element[3:]
                    ret.append(maxBefehl) # liefert Maxima-Repraesentation
        return ret
                    
            
    def script2Intern(self,scriptListe,maximum = None):
        import string
        _debug_ = 0
        if maximum:
            maxi = maximum
        else:
            maxi = len(scriptListe)
        ret = []
        for i in range(maxi): # nur Punkte holen
            element = scriptListe[i]
            if _debug_:
                print element
            teilLi = element.split('(')
            koordLi = teilLi[1].split(',')
            punkt = []
            if _debug_:            
                print koordLi
            for k in range(3):
                ki = koordLi[k]
                if k < 2:
                    punkt.append(ki)
                else:
                    if string.find(ki,')') == -1:
                        punkt.append(ki)
                    else:
                        ki = ki[:-1]
                        punkt.append(ki)
            if _debug_:            
                print punkt
            ret.append(punkt)
        return ret
            
######################################################

def saveGeoOops(datei,geoOopsDici):    
    import cPickle,tkMessageBox
    try:
        dici_datei = open(datei,'wb')       
        cPickle.dump(geoOopsDici,dici_datei)
        dici_datei.close()
    except:        
        tkMessageBox.showerror('ERROR', 'There occurred a problem . . .' )

def getGeoOops(datei):
    import cPickle,tkMessageBox    
    geoOopsDici = {}
    try:
        dici_datei = open(datei,'rb')
        geoOopsDici = cPickle.load(dici_datei)
        dici_datei.close()
        return geoOopsDici    
    except:
        tkMessageBox.showerror('ERROR', 'There occurred a problem . . .' )
        return geoOopsDici



























