# -*- coding: utf-8 -*-
"""#-*- coding: cp1252 -*- """
"""
pymaxUtil.py: utility functions for pyMaxima


-   Zeit- und Datumsfunktion
    Date: 2008-10-12
    Version: 0.19
    
-   renaming cause of import problems: from "util.py" to "pymaxUtil.py"
    Date: 2011-08-04
    Version: 0.20
    
-   Funktion "build_maxima_bat" angepasst an Maxima-Versionen > 5.22
    wg. Pfad zu gnuplot (unter Maxima)
    Date: 2011-08-20
    Version: 0.21
    
-   Funktion "build_maxima_bat" angepasst an Maxima-Versionen > 5.27
    wg. Pfad zu Maxima Vers. 5.28.0-2 => erfordert als Versionsangabe 5.28.0-2 statt 5.28.0 (!!)
    Date: 2012-11-17
    Version: 0.22
    
-   Funktion "build_maxima_bat" angepasst an Maxima-Versionen > 5.30
    da sich der Pfad zu gnuplot unter Windows veraendert hat (!): aus "\gnuplot\gnuplot.exe" wurde "\gnuplot\bin\gnuplot.exe"  (!!)
    Date: 2016-07-13
    Version: 0.23
    
ws
"""

import os
from os.path import abspath,exists
import time
import string
import codecs

pyMaximaHome = os.getcwd() + os.sep
_initdir_ = pyMaximaHome  + os.sep  + 'init' + os.sep

def datei_schreiben (home,datei,text):
    """Schreibt 'text' in eine Datei
       ACHTUNG:  Die Datei wird ohne Rückfrage  überschrieben!
    """
    import tkMessageBox

    try:
        out_file = open( home+ os.sep + datei,"w")  # Datei öffnen
    except IOError: # EA-Fehler
        tkMessageBox.showerror("Datei-Fehler!", 'Konnte Datei \n<' + home+ os.sep + datei + '> \nnicht öffnen!')
        return 0
    else:
        out_file.write(text) # text in Datei schreiben
        out_file.close()     # Datei schliessen
        return 1


def druckeDici(myDici):
    id_liste = myDici.keys()
    max_dici = len(myDici)
    if (max_dici) > 0:
        max = 0
        arg1 = '%s:   '
        arg2 = '    %s'	
        for element in id_liste:
            if len(element)  > max:
                max = len(element)	    
        diff = max - len('Key')
        arg = arg1 + ' '*diff + arg2
        print '\n'
        print arg % ('Key' , 'Value' )

        for element in id_liste:	
            diff = max - len(element)
            arg = arg1 + ' '*diff + arg2
            print arg % (str(element) , str( myDici.get(element) ) )
    else:
        print "Dictionary ist leer!!!"
            
def kopiereDici(fromDici,toDici):
    """tiefe Kopie eines Dicis"""
    maxDici = len(fromDici)
    if maxDici > 0:
        toDici.clear()
        bereich = fromDici.iterkeys()
        for j in bereich:
            toDici[j] = fromDici[j]


def saveAppendDici(fromDici,toDici):
    """tiefe Kopie eines Dicis mit sicherem Anhaengen (=append)
    vgl. auch die analoge Methode im Modul pymax2geo.py: saveAppendGeoDir
    """
    maxFromDici = len(fromDici)
    maxToDici = len(toDici)
    if (maxToDici == 0):
        toDici.clear()
    if maxFromDici > 0:        
        bereich = fromDici.iterkeys()
        for j in bereich:
            if not ( toDici.has_key(j) ):   # new entry => append             
                toDici[j] = fromDici[j]
            else: # same name
                index = 1
                neuerName = str(j) + "_" + str(index)   # Object with same name => new name: name + "_1" 
                while toDici.has_key(neuerName): # try new name
                    index = index + 1
                    neuerName = str(j) + "_" + str(index)
                    
                toDici[neuerName] = fromDici[j]
                

def linuxFilter(text):
    textliste = text.split('\n')
    neu = []
    for zeile in textliste: # text zeilenweise bearbeiten
        if zeile[-1:] == '\r':
            zeile = zeile[:-1] #+ '\n'
            neu.append(zeile)
    return neu


def hpCopyHelp(helpdatei):
    '''display help screen'''
    import os
    import os.path
    debug = 0	 
    pfad = os.getcwd()
    pfad = os.path.abspath (os.getcwd()) + os.sep
    if debug: 
        print pfad 
    return sicheres_datei_lesen(pfad+helpdatei)      



def sicheres_datei_lesen (datei,zeilen=0): 
    """Liest eine Datei; gibt 'text' zurck, falls erfolgreich"""  
    try:      
        in_file = open(datei,"r")  
    # Datei zum Lesen (r: read) ?fnen  
    except IOError:  
        feedback = '=> Konnte '+ datei + ' nicht ?fnen!'
        ausgabe(feedback)        
        return ''  
    else:
        if (zeilen == 1):
            text = in_file.readlines() # Datei lesen  
        else:    
            text = in_file.read() # Datei lesen  
        in_file.close() # Datei schliessen  
        return text              


def leseConfi(home,confiDatei,_iniDici_):
    from string import strip
    _debug_ = 0
    if _debug_:
        druckeDici(_iniDici_)
        
    liste = []
    liste = dateiLesen(home,confiDatei)
    if _debug_:
        print str(liste)
    if (liste != []):
        for element in liste:
            if (element.find('#') == -1):                    
                if len( element ) > 0:
                    if _debug_:
                        print "verarbeite", element
                    if (element.find('delayTime') != -1):
                        j = liste.index(element)
                        _iniDici_['delay_time'] = strip(liste[j+1])
                        if _debug_:
                            print strip(liste[j+1])
                        
                    if (element.find('maximaPath"') != -1):
                        j = liste.index(element)
                        if os.name != 'posix': # win nt
                            _iniDici_['maxima_path'] = abspath(strip(liste[j+1]))                            
                            if _debug_:
                                print strip(liste[j+1])
                        else: # posix
                            _iniDici_['maxima_path'] = 'maxima'
        
                    if (element.find('docuPath') != -1):
                        j = liste.index(element)

                        _iniDici_['docu_path'] = abspath(strip(liste[j+1]))
                        
                        if _debug_:
                            print strip(liste[j+1])
                            
                    if (element.find('pyPath') != -1):
                        j = liste.index(element)
                        if  os.name != 'posix': # win nt   
                            _iniDici_['py_path'] = abspath(strip(liste[j+1]))
                        else: # posix
                            _iniDici_['py_path'] = strip(liste[j+1])
                        
                        if _debug_:
                            print strip(liste[j+1])
                        

                    if (element.find('fontSizePara') != -1):
                        j = liste.index(element)
                        _iniDici_['font_size_para'] = strip(liste[j+1])
                        
                        if _debug_:
                            print strip(liste[j+1])

                    if (element.find('nrLinesMain') != -1):
                        j = liste.index(element)
                        _iniDici_['nr_lines_main'] = strip(liste[j+1])
                        
                        if _debug_:
                            print strip(liste[j+1])


                        
                        
    else:
        print "Lesen der Config-Datei '" + confiDatei + "' fehlgeschlagen!\n" 
        print "Benutze Default-Einstellungen!\n"        

    if _debug_:
        druckeDici(_iniDici_)
                
def dateiLesen (home,datei): 
    """liest eine Datei, gibt Inhalt als Liste zurueck, falls erfolgreich, sonst [] """ 
    try:        
        in_file = open(home + os.sep + datei,"r") 
        # Datei zum Lesen (r: read) oeffnen 
    except IOError: 
        #in_file.close() # Datei schliessen
        return []
    else: 
        liste = in_file.readlines() # Datei lesen 
        in_file.close() # Datei schliessen 
        return liste 


def intervall(wert,li,re):
    if (wert >= li) and (wert <= re):
        return 1
    else:
        return 0



def iniDici2Text(myDici):
    kopf = """#
# Konfigurations-Datei fuer pyMaxima.py
#
# Verzoegerungszeit"""
    delay = '\n"delayTime"\n'
    
    maximaHead = """\n\n# 
#
# Maxima path
#"""
    maxima = '\n"maximaPath"\n'
    
    docuHead ="""\n\n# 
#
# Maxima-Manual path
#"""
    pyHead ="""\n\n# 
#
# Python path
#"""
    paraHead ="""\n\n# 
#
# font size in parameter window
#"""    
    nrLinesHead ="""\n\n# 
#
# Number of Lines in Main Window
#"""        
    docu = '\n"docuPath"\n'
    pyPfad = '\n"pyPath"\n'
    fsPara = '\n"fontSizePara"\n'
    nrLines = '\n"nrLinesMain"\n'
    
    text = kopf + delay + myDici['delay_time']
    text = text + maximaHead + maxima + myDici['maxima_path']
    text = text + docuHead + docu + myDici['docu_path'] + '\n'
    text = text + pyHead + pyPfad + myDici['py_path'] + '\n'
    text = text + paraHead + fsPara + myDici['font_size_para'] + '\n'
    text = text + nrLinesHead + nrLines + myDici['nr_lines_main'] + '\n'        
    #print text
    return text
    

def getFuncListe(home,datei):
    liste = dateiLesen(home,datei)
    return liste


def test_extension(Dateiname,Erweiterung):
    Dateiname = string.lower(Dateiname)
    if ( string.rfind(Dateiname,Erweiterung) == -1 ):
        return 0
    else:  ## Erweiterung in Dateiname gefunden
        return 1

def ergebnisTest(liste=None):
    zaehler = 0
    ort = 0
    if liste:
        if len(liste) > 0:
            for j in range(len(liste)):
                if liste[j]==1:
                    zaehler = zaehler + 1
                    ort = j
            return (zaehler,ort)
        else:
            return None
    else:
        return None

def saveList2txt(pfad,liste):
    """Sichern einer Liste als *.txt-datei
       Aufruf: saveList2txt(self.home,liste)
    """
    import tkFileDialog 
    pfad_ = os.path.abspath( pfad )
    os.chdir(pfad_)
    oname = tkFileDialog.asksaveasfilename(filetypes=[("Texte", "*.txt"),("Alle Dateien","*.*")],initialdir=pfad_)
    if oname:        
        if (not test_extension(oname,".txt")):
            oname = oname + ".txt"
                
        kopf = "#\n# generated by pyMaxima\n"
        zeit = "# @ " +  time.strftime("%A %B %d, %Y %I:%M %p",time.localtime(time.time())) + "\n#\n"		    
        try:
            listDatei = open(oname,'w')
            seq = []
            seq.append(kopf)
            seq.append(zeit)
            for element in liste:
                seq.append(element+'\n')
            listDatei.writelines(seq)
            listDatei.close()
        except IOError:
            print 'EA-Fehler: Sichern fehlgeschlagen!'

########################

def ucodefilter_(zeile,dec=1,org=1,debug=0):
    "kodiert Unicode-Sonderzeichen in HTML"
    # die Sonderzeichen werden im Dictionary 'codes' festgelegt
    import umlautCodec
    if org:
        codes = umlautCodec.unicode2org
    else:
        codes = umlautCodec.unicodeCodes
    
    text = ''
    try:
        if dec:
            zeile = zeile.decode('iso-8859-1')            
            #print zeile
    except:
        pass
    try:
        zeile = zeile.encode('latin-1')
    except:
        pass
    for i in range( len(zeile) ):
        #if debug:
        #    print zeile[i]
        try:    
            if codes.has_key(zeile[i]):
                #print zeile[i]
                text = text + codes[zeile[i]]
            else:
                text = text + zeile[i]
        except:
            text = text + zeile[i]
            
        if debug:
            print text,    
    return text
##########################
def build_maxima_bat(_maxima_ver_=None,_maxima_path_=None):
    _debug_ = 0
    import time,os.path
    if _maxima_ver_:
        maxima_ver = _maxima_ver_
        if _debug_:
            print maxima_ver
    else:
        return
    if _maxima_path_:
        maxima_path = _maxima_path_
        if _debug_:
            print maxima_path                
    else:
        return
    
    _maxima_name_ = os.path.basename(maxima_path)
    # Der folgende Befehl macht aus "Maxima-5.27.0" => "5.27.0"
    new_maxima_ver = string.replace(_maxima_name_,"Maxima-","")
    if _debug_:
        print new_maxima_ver

    sub_version = new_maxima_ver.split(".")
    if _debug_:
        print sub_version[1] # 5.31.xxx => gibt hier 31 retour
    

    if (not maxima_ver.endswith(new_maxima_ver) ) :
        try:
            _test_ = int(new_maxima_ver[-1:])  # Ende von "5.27.0"/"5.28.0-2"
            maxima_ver = string.strip(new_maxima_ver)
        except:  #  "Maxima-5.25.0-gcl" => "5.25.0-gcl" => maxima_ver NICHT ersetzen!
            pass
        
    

    zeit = time.strftime("%A %B %d, %Y %I:%M %p",time.localtime(time.time()))
    part1 =r"""@echo off

set arg0=%0
set arg1=%1
set arg2=%2
set arg3=%3
set arg4=%4
set arg5=%5
set arg6=%6
set arg7=%7
set arg8=%8
set arg9=%9

rem maxima.bat; generated by pyMaxima.py at """
    part2 = """
    
set lisp=gcl
"""
    part3 = "set version=" + maxima_ver + "\n"
    part4 = r"set prefix=c:/maxima"
    part5 = "\nset maxima_prefix=" + maxima_path + "\n"
    if (int(sub_version[1]) > 30):  # Maxima-Version > 5.30 !
        part6 = r"""set package=maxima
set verbose=false
set mingw_gccver=3.3.1
rem set path=%maxima_prefix%\bin;%path%
set path=%maxima_prefix%\gnuplot\bin;%maxima_prefix%\bin;%maxima_prefix%\lib\gcc-lib\mingw32\3.3.1;%path%"""
    else:  # Maxima-Version <= 5.30
        part6 = r"""set package=maxima
set verbose=false
set mingw_gccver=3.3.1
rem set path=%maxima_prefix%\bin;%path%
set path=%maxima_prefix%\gnuplot;%maxima_prefix%\bin;%maxima_prefix%\lib\gcc-lib\mingw32\3.3.1;%path%"""
        
    part7 = r"""
if "%USERPROFILE%" == "" goto win9x
if "%MAXIMA_USERDIR%" == "" set MAXIMA_USERDIR=%USERPROFILE%\maxima
if "%MAXIMA_TEMPDIR%" == "" set MAXIMA_TEMPDIR=%USERPROFILE%
goto startparseargs
:win9x
if "%MAXIMA_USERDIR%" == "" set MAXIMA_USERDIR=%maxima_prefix%\user
if "%MAXIMA_TEMPDIR%" == "" set MAXIMA_TEMPDIR=%maxima_prefix%

:startparseargs
if "%1" == "-l" goto foundlisp
if "%1" == "--lisp" goto foundlisp
if "%1" == "-u" goto foundversion
if "%1" == "--use-version" goto foundversion
if "%1" == "-v" goto foundverbose
if "%1" == "--verbose" goto foundverbose

:continueparseargs
shift
if not "%1" == "" goto startparseargs
goto endparseargs

:foundlisp
set lisp=%2
shift
goto continueparseargs

:foundversion
set version=%2
shift
goto continueparseargs

:foundverbose
set verbose=true
goto continueparseargs

:endparseargs

if "%MAXIMA_LAYOUT_AUTOTOOLS%" == "" goto defaultlayout
set layout_autotools=true
goto endlayout

:defaultlayout
set layout_autotools=true

:endlayout

if "%MAXIMA_PREFIX%" == "" goto defaultvars
if "%layout_autotools%" == "true" goto maxim_autotools
set maxima_imagesdir=%MAXIMA_PREFIX%\src
goto endsetupvars

:maxim_autotools
set maxima_imagesdir=%MAXIMA_PREFIX%\lib\%package%\%version%
goto endsetupvars

:defaultvars
if "%layout_autotools%" == "true" goto defmaxim_autotools
set maxima_imagesdir=%prefix%\src
goto endsetupvars

:defmaxim_autotools
set maxima_imagesdir=%prefix%\lib\%package%\%version%
goto endsetupvars

:endsetupvars

set maxima_image_base=%maxima_imagesdir%\binary-%lisp%\maxima

if "%verbose%" == "true" @echo on
if "%lisp%" == "gcl" goto dogcl
if "%lisp%" == "clisp" goto doclisp

@echo "maxima error: lisp %lisp% not known."
goto end

:dogcl
%maxima_imagesdir%\binary-gcl\maxima.exe -eval "(cl-user::run)" -f -- "%arg1%" "%arg2%" "%arg3%" "%arg4%" "%arg5%" "%arg6%" "%arg7%" "%arg8%" "%arg9%"
goto end

:doclisp
if "%layout_autotools%" == "true" goto clisp_autotools
clisp -q -M %maxima_image_base%.mem "" -- "%arg1%" "%arg2%" "%arg3%" "%arg4%" "%arg5%" "%arg6%" "%arg7%" "%arg8%" "%arg9%"
goto end

:clisp_autotools
%maxima_imagesdir%\binary-clisp\lisp.exe -q -M %maxima_image_base%.mem "" -- "%arg1%" "%arg2%" "%arg3%" "%arg4%" "%arg5%" "%arg6%" "%arg7%" "%arg8%" "%arg9%"
goto end

:end



"""
    daten = part1 + zeit + part2 + part3 + part4 + part5 + part6 + part7
    datei_schreiben(_initdir_,"maxima.bat",daten)

def protectBlank(name):
    from string import find
    if find(name,'\040') != -1:
        return '"' + name + '"'
    else:
        return name

def changeLwArg(_lw_arg_,pfad):
    if os.name <> 'posix':
        return _lw_arg_ + pfad[2:]


##########################################################
def nullTest(zahl,eps=1.0e-7):
    return _nullTest_( float(zahl), _eps_ = eps )
    
def _nullTest_(zahl,_eps_=1.0e-7):
    #epsilon = 1.0/(10**7)  = 0.0000001
    #epsilon = 1.0/(10**10) 
    epsilon = _eps_
    if abs(zahl) < epsilon:
        return 1
    else:
        return 0
    
def _ggt_(i,j):
    if  nullTest(j):
        return i
    else:
        return _ggt_(j, i%j)    

def ggt(i,j):
    i = float(i) # int(i)
    j = float(j)  # int(j)
    return _ggt_(i,j) # 

def GGT(i,j,k):
    tmp = ggt(j,k)
    return ggt(i,tmp)
##########################################################    
# Rationale Schreibweise: 1/3 => 0.33333333
import string
def rat2float(r):
    # modifiziert mit ggt . . .; 2009-03-01
    rep = str(r)
    if rep.find("/") != -1:
        (z,n) = rep.split("/")
        #print z + " ~/~ " + n
        # Vorzeichenbehandlung
        if z.find("-") != -1:
            neg = True
            (z1,z2) = z.split("-")
            z = z2
        else:            
            neg = False
        #Bruch
        a = b = ""
        laenge = len(z)
        if laenge > 2: # mixed rep?
            zahl = True
            for i in range( laenge ):
                if (z[i] != " ") and zahl:
                    a = a + z[i]
                else:
                    zahl = False
                    b = b + z[i]
            b = string.strip(b, " ")
        if len(a) > 0: # mixed rep
            fl_b = float( b )
            fl_n = float( n )
            
            ggt_b_n = ggt( fl_b , fl_n )
            if not nullTest( (ggt_b_n - 1.0) ):
                fl_b = fl_b / ggt_b_n
                fl_n = fl_n / ggt_b_n
                # print fl_b,"/",fl_n            
            if neg:
                return (-1.0) *  (float(a) + ( fl_b / fl_n ) )
            else:
                return float(a) + ( fl_b / fl_n )
        else:
            fl_z = float( z )
            fl_n = float( n )
            ggt_z_n = ggt( fl_z , fl_n )
            if not nullTest( (ggt_z_n - 1.0) ):
                fl_z = fl_z / ggt_z_n
                fl_n = fl_n / ggt_z_n
                # print fl_z,"/",fl_n
            if neg:
                return (-1.0) * fl_z/fl_n                
            else:
                return fl_z/fl_n
    else: # kein Bruch
        try:
            return float( r )
        except:
            return r
        
def _old_rat2float_(r):
    # alte Version
    rep = str(r)
    if rep.find("/") != -1:
        (z,n) = rep.split("/")
        #print z + " ~/~ " + n
        # Vorzeichenbehandlung
        if z.find("-") != -1:
            neg = True
            (z1,z2) = z.split("-")
            z = z2
        else:            
            neg = False
        #Bruch
        a = b = ""
        laenge = len(z)
        if laenge > 2: # mixed rep?
            zahl = True
            for i in range( laenge ):
                if (z[i] != " ") and zahl:
                    a = a + z[i]
                else:
                    zahl = False
                    b = b + z[i]
            b = string.strip(b, " ")
        if len(a) > 0: # mixed rep
            if neg:
                return (-1.0) *  (float(a) + ( float( b )/float( n ) ) )
            else:
                return float(a) + ( float( b )/float( n ) )
        else:
            if neg:
                return (-1.0) * float(z)/float(n)                
            else:
                return float(z)/float(n)
    else: # kein Bruch
        try:
            return float( r )
        except:
            return r


def r2f( arg ):
    return str( rat2float( arg ) )
   

def expr2TEX( arg ):
    ausnahmen = ["plot2d","plot3d"]
    to_tex_flag = 1
    for element in ausnahmen:
        if not (arg.find(element) == -1):
            to_tex_flag = 0
    if to_tex_flag:
        arg = arg[:-1]
        tex_expr = "tex(" + arg + ")$"        
        return tex_expr
    else: # Befehl bleibt unveraendert
        return arg

def tex_verbatim( arg ):
    begin_verbatim = r"\begin{verbatim}" + "\n"
    end_verbatim = r"\end{verbatim}" + "\n"
    ret = begin_verbatim + arg + end_verbatim
    return ret

def tex_subsection( arg ):
    # \subsection*{test}
    ret = r"\subsection*{" + arg + "}" + "\n"
    return ret


def tex_lstlisting( arg ):    
    ret = r"\begin{lstlisting} [breaklines=false]" + "\n" + arg + "\n" + r"""\end{lstlisting}""" + "\n"
    return ret

def _heute_():
    """gibt aktuelles Datum als String retour
    Beispiel: 12. Oktober 2008"""
    import time
    monate_de = {}
    monate_de[1] = "Januar"
    monate_de[2] = "Februar"
    monate_de[3] = "März"
    monate_de[4] = "April"
    monate_de[5] = "Mai"
    monate_de[6] = "Juni"
    monate_de[7] = "Juli"
    monate_de[8] = "August"
    monate_de[9] = "September"
    monate_de[10] = "Oktober"
    monate_de[11] = "November"
    monate_de[12] = "Dezember"
    zeit = time.localtime()
    heute = "%i. %s %i" % (zeit[2],monate_de[zeit[1]],zeit[0])
    return heute

def _zeit_():
    """gibt aktuelle Zeit als String retour
    Beispiel: 13:09"""
    import time
    zeit = time.localtime()
    std_min_sek = "%i:%i:%i" % (zeit[3],zeit[4],zeit[5])
    
    if (zeit[3] < 10):
        if (zeit[4] < 10):
            std_min = "0%i:0%i" % (zeit[3],zeit[4])
        else:
            std_min = "0%i:%i" % (zeit[3],zeit[4])
    else:
        if (zeit[4] < 10):
            std_min = "%i:0%i" % (zeit[3],zeit[4])
        else:
            std_min = "%i:%i" % (zeit[3],zeit[4])
        
    return std_min

def _DatumZeit_():
    """gibt aktuelles Datum + Zeit als String retour
    Beispiel: 12. Oktober 2008, 13:09"""    
    return "%s, %s" % (_heute_(),_zeit_())
    
##########################
def debugAusdruckAusgeben(_debug_,expr,gefunden,text,nextIn,tauscheAusdruck, oldExpr,counter):
    if _debug_:
        try:
            print "Methode ausdruckAusgeben: counter => ", counter
            print "self.nextIn: ", nextIn
            print "text: ",text
            print
            print "self.tauscheAusdruck: ",
            print tauscheAusdruck
            print "alter Ausdruck: ",oldExpr
            print "Ausdruck: ",expr
            print
        except:
            print "FEHLER in debugAusdruckAusgeben: Vermutlich Zeichen nicht in ASCII(128) . . ."

def test():
    print 'This is "util.py"'
    
#########################################   
# main   
if __name__ == "__main__":
    print 'This is "util.py"'
