#!/usr/local/bin/python
# -*- coding: utf-8 -*-
"""# -*- coding: Latin1 -*- """
"""
  stochastik.py: Test-Klasse
  2007-07-27, Vers. 0.01, WS
  2008-03-26, Vers. 0.02, WS
  with a little help from Wolfgang Urban,
  URL: http://www.hib-wien.at/leute/wurban/mathematik/distributions.html
"""

import math


###########################################        
class normalverteilung:
    """Integration der gaussschen Summenfunktion (Normalverteilung)
    Benutzung:
    self.normalVert = normalverteilung()
    ergebnis = self.normalVert.integration(self.normalVert.func, -10, x)
    oder
    ergebnis = self.normalVert.integration(self.normalVert.func, -20, x)
    """
    
    def __init__(self):
        """Normalverteilung"""
        pass
    
    def gauss(self,x):
        """Standardnormalverteilung, X ist N(0,1)-verteilt """
        wert = (1/math.sqrt(2*math.pi))* math.exp(-0.5*x*x)
        return wert


    def func(self,x):
        """wird benötigt in closedpoints bzw. in openpoints"""
        return self.gauss(x)

    def summenfunktion(x1,x2):
        pass
        
    ##################################################################
        
    def simpson(self,func, a, b, n):
        """Quelle: Feldmann, Dietrich: Turbopascal-Quelltexte
        zur Ingenieurmathematik, Parameter:
        func: Funktion die integriert werden soll
        a : linke Grenze
        b : rechte Grenze
        n : Anzahl der Teilintervalle
        """
        if (n % 2) == 1: # odd(n) !!
            n = n + 1
        h = (b-a) / n
        s1 = func(b-h)
        s2 = 0
        i = 1
        while (i < n-1):
            s1 = s1 + func(a + i*h)
            s2 = s2 + func(a + h + i*h)
            i = i + 2
        wert = (4*s1 + 2*s2 + func(a) + func(b) ) * h / 3.0
        #print wert, n
        return wert
        
    def integration(self,func, a, b, n=20):
        """Parameter:
        func: Funktion die integriert werden soll
        a : linke Grenze
        b : rechte Grenze
        n : Anzahl der Teilintervalle
        """
        return self.simpson(func, a, b, n)      
             
######################################################################

class binomialTest:
    ## Hypothesentest: zweiseitig, links- und rechtsseitig
    ## mit Hilfe der Binomialverteilung    
    ## Aufruf: myTest=test(h0=0.5,n=100,alpha=0.05,stichp=37)
    ##         myTest.zweiseitig()
    ##
    
    def __init__(self,h0=0.5,n=100,alpha=0.05,stichp=37):
        """Testparmeter"""
        self.h0=h0 
        self.n=n
        self.alpha=alpha
        self.stichp=stichp
        self.p = h0
        self.ewx = self.n * self.p
        self.alpha2 = alpha / 2.0
    
    def intervall(self,wert,li,re):
        if (wert >= li) and (wert <= re):
            return 1
        else:
            return 0

    # cumulative probability
    # URL: http://www.hib-wien.at/leute/wurban/mathematik/distributions.html
    def binomialc(self,n,p,k1,k2):
        ## special cases
        if p==0.0:
            return 1.0
        if p==1.0:
            if k==n: return 1.0
            else: return 0.0

        q = 1.0-p
        m = float(n+1)
        r = math.log(p/q)
        prob = n*math.log(q)
        cum = math.exp(prob)
        #print k1, k2
        if k1 == 0: # wichtig! 
            k1 = k1 + 1        
        for i in range( int(k1),int(k2+1) ):
            prob += r + math.log(m/i-1.0)
            cum += math.exp(prob)
        return cum


    def bin_sum(self,n,p,k1,k2):
        """berechnet die summierte Binmialverteilung
           Aufruf: print bin_sum(25,0.75,15,23)
           Argumente: Parameter n,p sowie
           die Grenzen k1 und k2
        """
        return self.binomialc(n,p,k1,k2)        


    def berechneLinks(self,n,p,ewx,alpha2):
        # >>> vorläufig!!!!!!!!!!
        # TESTEN!!!!!!
        li = 0
        re = ewx
        summe = self.bin_sum(n,p,0,re)
        altmitte = re
        while (abs(summe - alpha2) > 0.0001 ):
            neumitte = int( (li + re)/2 )
            summe = self.bin_sum(n,p,0,neumitte)
            if summe < alpha2:
                li = neumitte
            elif summe > alpha2:
                re = neumitte
            if altmitte == neumitte:
                if summe > alpha2:
                    neumitte = neumitte - 1
                if neumitte < 0:
                    neumitte = 0
                break #return neumitte
            altmitte = neumitte
        if neumitte > 0: #
            for i in range(1,-2,-1):
                gl = neumitte+i
                summe = self.bin_sum(n,p,0,gl)
                if summe <= alpha2:
                    return gl                               
        else: # neumitte == 0!!
            return 0 # ==neumitte                   

    #############################
    def berechneRechts(self,n,p,ewx,alpha2):
        # >>> vorläufig!!!!!!!!!!
        # TESTEN!!!!!!
        li = ewx
        re = n
        summe = self.bin_sum(n,p,0,re)
        altmitte = li
        while (abs(summe - alpha2) > 0.0001 ):
            neumitte = int( (li + re)/2 )
            summe = 1 - self.bin_sum(n,p,0,neumitte-1)
            if summe < alpha2:
                re = neumitte
            elif summe > alpha2:
                li = neumitte
            if altmitte == neumitte:
                if summe > alpha2:
                    neumitte = neumitte + 1
                if neumitte > n:
                    neumitte = n
                break #return neumitte
            altmitte = neumitte
        if neumitte < n: #
            for i in range(-1,2):
                gr = neumitte+i
                summe = 1 - self.bin_sum(n,p,0,gr)
                if summe <= alpha2:
                    return gr + 1                 
        else: # neumitte == n!!
            return n  # ==neumitte

###############################################
class normalTest:
    ## Hypothesentest: zweiseitig, links- und rechtsseitig
    ## mit Hilfe der Normalverteilung
    ## Aufruf: myTest=test(h0=0.5,n=100,alpha=0.05,stichp=37)
    ##         myTest.zweiseitig()
    ## 
    
    def __init__(self,h0=0.5,n=100,alpha=0.05,stichp=37,korrektur=0):
        """Testparmeter"""
        self.h0=h0 
        self.n=n
        self.alpha=alpha
        self.stichp=stichp
        self.p = h0
        self.ewx = self.n * self.p
        self.var = self.n * self.p * (1- self.p)        
        self.sigma = math.sqrt(self.var)
        self.alpha2 = alpha / 2.0
        self.korrekturglied=korrektur
        self.normalVert = normalverteilung()
        #x1 = (k1-ew)/sigma
        #x2 = (k2-ew)/sigma        

    def berechneLinks(self,n,p,ewx,alpha):
        # >>> vorläufig!!!!!!!!!!
        # TESTEN!!!!!!
        # ohne print-Befehle
        # IN: n,p,ewx,sigma,alpha
        # OUT: linke Grenze gl
        # P(X <= k ) = phi( [k + 0.5 - ex] / sigma ) = phi(x) <= alpha # mit Korrekturglied
        # P(X <= k ) = phi( [k - ex]/sigma ) = phi(x) <= alpha
        # phi(x) <= alpha -> x = [k - ex]/sigma -> x*sigma = k - ex
        # oder gl = k = x*sigma + ex
        # bzw. mit Korrekturglied: gl = k = x*sigma + ex - 0.5
        # Problem: bestimme aus phi(x) <= alpha den Wert x!
        _debug_ = 0
        var = ewx * (1 - p)        
        sigma = math.sqrt(var)        
        x_alt = -90
        for x in range(-90,100):
            x = x /10.0
            ergebnis = self.normalVert.integration(self.normalVert.func, -10, x)
            if  ergebnis > alpha:
                x_g = x_alt                
                break
            x_alt = x
        x_alt = x_g
        x = x_g
        for i in range(0,1000):
            i = i / 1000.0
            x = x + i
            ergebnis = self.normalVert.integration(self.normalVert.func, -10, x)
            if  ergebnis > alpha:
                x_g = x_alt
                break
            x_alt = x
        gl =  int( math.floor(x_g * sigma + ewx) )
        gl_korrektur = int( math.floor(x_g * sigma + ewx - 0.5) )
        if _debug_:
            print
            #print x_g * sigma + ewx
            if var < 9:
                print "Approximation durch Normalverteilung kritisch, da n*p*q < 9 (",
                print "n*p*q = " + str(var) + ")!!!"
            print "Ew(X)=%4.3f, sigma=%4.3f, alpha=%4.3f" % (ewx, sigma, alpha)
            print "x=%4.3f, linke Grenze gl=%d " % ( x_g , gl )        
            print "Mit Korrekturglied: linke Grenze gl=%d " % ( gl_korrektur )
        #if gl_korrektur < gl:            
        if self.korrekturglied:    
            return gl_korrektur
        else:
            return gl
        
    #############################
    def berechneRechts(self,n,p,ewx,alpha):
        # >>> vorläufig!!!!!!!!!!
        # TESTEN!!!!!!
        # IN: n,p,ewx,sigma,alpha
        # OUT: rechte Grenze gr
        # P(X >= k ) <= alpha
        # P(X <= k-1) >= 1 - alpha
        # P(X <= k-1) = phi( [k - 1 + 0.5 - ex] / sigma ) = phi(x) >= 1 - alpha # mit Korrekturglied
        # P(X <= k-1 ) = phi( [k - 1 - ex]/sigma ) = phi(x) >= 1 - alpha
        # phi(x) >= 1 - alpha -> x = [k - 1 - ex]/sigma -> x*sigma = k - 1 - ex
        # oder gl = k = x*sigma + ex + 1
        # bzw. mit Korrekturglied: gl = k = x*sigma + ex + 1 - 0.5
        # Problem: bestimme aus phi(x) >= 1 - alpha den Wert x!
        _debug_ = 0
        var = ewx * (1 - p)        
        sigma = math.sqrt(var)        
        x_alt = -90
        for x in range(-90,100):
            x = x /10.0
            ergebnis = self.normalVert.integration(self.normalVert.func, -10, x)
            if  ergebnis > (1-alpha):
                x_g = x_alt                
                break
            x_alt = x
        x_alt = x_g
        x = x_g
        for i in range(0,1000):
            i = i / 1000.0
            x = x + i
            ergebnis = self.normalVert.integration(self.normalVert.func, -10, x)
            if  ergebnis > (1-alpha):
                x_g = x # !! nicht x_alt
                break

        gr =  int( math.ceil(x_g * sigma + ewx + 1) )
        gr_korrektur = int( math.ceil(x_g * sigma + ewx + 1 - 0.5) )
        if _debug_:
            print
            #print x_g * sigma + ewx + 1
            if var < 9:
                print "Approximation durch Normalverteilung kritisch, da n*p*q < 9 (",
                print "n*p*q = " + str(var) + ")!!!"
            print "Ew(X)=%4.3f, sigma=%4.3f, alpha=%4.3f" % (ewx, sigma, alpha)
            print "x=%4.3f, rechte Grenze gr=%d " % ( x_g , gr )        
            print "Mit Korrekturglied: rechte  Grenze gr=%d " % ( gr_korrektur )
        #if gr_korrektur <> gr:    
        if self.korrekturglied:                      
            return gr_korrektur
        else:
            return gr
    #############################
    def bin_sum(self,n,p,k1,k2):
        """berechnet die summierte Binmialverteilung
           mit Hilfe der Approximation durch die Normalverteilung
           Aufruf: print bin_sum(25,0.75,15,23)
           Argumente: Parameter n,p sowie
           die Grenzen k1 und k2
        """
        #print n,p,k1,k2
        ewx = n * p
        var = ewx * (1 - p)
        sigma = math.sqrt(var)
        #print ewx,var,sigma
        
        # Grenzen umrechnen
        x1 = (k1 - 0.5 - ewx) / sigma
        x2 = (k2 + 0.5 - ewx) / sigma
        #print
        #print x1,x2
        
        ergebnis = self.normalVert.integration(self.normalVert.func, x1, x2, n=2000)
        # wichtig: n >= 1000 (d. h. mind. 1000 Teilintervalle bei der numerischen Integration!)
        #print ergebnis
        
        return ergebnis

 
    #############################

#############################
class poissonTest:
    ## Hypothesentest: zweiseitig, links- und rechtsseitig
    ## mit Hilfe der Normalverteilung    
    ## Aufruf: myTest=test(h0=0.5,n=100,alpha=0.05,stichp=37)
    ##         myTest.zweiseitig()
    ## 
    
    def __init__(self,h0=0.5,n=100,alpha=0.05,stichp=37):
        """Testparmeter"""
        self.h0=h0 
        self.n=n
        self.alpha=alpha
        self.stichp=stichp
        self.p = h0
        self.ewx = self.n * self.p
        self.alpha2 = alpha / 2.0

    # cumulative probability
    # URL: http://www.hib-wien.at/leute/wurban/mathematik/distributions.html
    def binomialc(self,n,p,k1,k2):
        ## special cases
        if p==0.0:
            return 1.0
        if p==1.0:
            if k==n: return 1.0
            else: return 0.0

        q = 1.0-p
        m = float(n+1)
        r = math.log(p/q)
        prob = n*math.log(q)
        cum = math.exp(prob)
        if k1 == 0: # wichtig! 
            k1 = k1 + 1        
        for i in range( int(k1),int(k2+1) ):
            prob += r + math.log(m/i-1.0)
            cum += math.exp(prob)
        return cum


    def bin_sum(self,n,p,k1,k2):
        """berechnet die summierte Binmialverteilung
           Aufruf: print bin_sum(25,0.75,15,23)
           Argumente: Parameter n,p sowie
           die Grenzen k1 und k2
        """
        return self.binomialc(n,p,k1,k2)        



    # cumulative probability
    # from http://www.hib-wien.at/leute/wurban/mathematik/distributions.html
    def poisson_sum(self,m,k1,k2):
        prob = math.exp(-m)     
        cum = prob
        if k1 == 0: # wichtig! 
            k1 = k1 + 1
        for i in range(k1,k2+1):
            prob = prob*m/i
            cum += prob
        return cum    




    def berechneLinks(self,n,p,ewx,alpha2):
        # >>> vorläufig!!!!!!!!!!
        # TESTEN!!!!!!
        # ohne print-Befehle
        li = 0
        re = ewx
        summe = self.bin_sum(n,p,0,re)
        altmitte = re
        while (abs(summe - alpha2) > 0.0001 ):
            neumitte = int( (li + re)/2 )
            summe = self.poisson_sum(self.ewx,0,neumitte)
            if summe < alpha2:
                li = neumitte
            elif summe > alpha2:
                re = neumitte
            if altmitte == neumitte:
                if summe > alpha2:
                    neumitte = neumitte - 1
                if neumitte < 0:
                    neumitte = 0
                break #return neumitte
            altmitte = neumitte
        if neumitte > 0: #
            for i in range(1,-2,-1):
                gl = neumitte+i
                summe = self.poisson_sum(self.ewx,0,gl)
                if summe <= alpha2:
                    return gl                               
        else: # neumitte == 0!!
            return 0 # ==neumitte                   

    #############################
    def berechneRechts(self,n,p,ewx,alpha2):
        # >>> vorläufig!!!!!!!!!!
        # TESTEN!!!!!!
        # ohne print-Befehle
        li = ewx
        re = n
        summe = self.bin_sum(n,p,0,re)
        altmitte = li
        while (abs(summe - alpha2) > 0.0001 ):
            neumitte = int( (li + re)/2 )
            summe = 1 - self.poisson_sum(self.ewx,0,neumitte-1)
            if summe < alpha2:
                re = neumitte
            elif summe > alpha2:
                li = neumitte
            if altmitte == neumitte:
                if summe > alpha2:
                    neumitte = neumitte + 1
                if neumitte > n:
                    neumitte = n
                break #return neumitte
            altmitte = neumitte
        if neumitte < n: #
            for i in range(-1,2):
                gr = neumitte+i
                summe = 1 - self.poisson_sum(self.ewx,0,gr)
                if summe <= alpha2:
                    return gr + 1                 
        else: # neumitte == n!!
            return n  # ==neumitte     
        
 

        
#############################
class intervall:
    ## bestimmt Konfidenzintervall bzw. Stichprobenumfang 
    ## Aufruf: konfidenz=intervall()
    ##         ausgabe = konfidenz.konfidenzintervall(n,gamma,stichp)
    ## 
    
    def __init__(self):
        """Testparmeter"""
        self.normalVert = normalverteilung()

    def berechneC(self,gamma=0.95):
        # >>> vorläufig!!!!!!!!!!
        # TESTEN!!!!!!
        # ohne print-Befehle
        # IN: gamma
        # OUT: zugehöriges Argument c der Normalverteilung
        # phi(c) = 1+gamma / 2
        # Problem: bestimme aus phi(c) = 1+gamma / 2 das Argument c!
        _debug_ = 0
        phi = (1+gamma) / 2
        x_alt = -90
        for x in range(-90,100):
            x = x /10.0
            ergebnis = self.normalVert.integration(self.normalVert.func, -20, x)
            if  ergebnis > phi:
                x_g = x_alt                
                break
            x_alt = x
        x_alt = x_g
        x = x_g
        for i in range(0,10000):
            i = i / 10000.0
            x = x + i
            ergebnis = self.normalVert.integration(self.normalVert.func, -20, x)
            if  ergebnis > phi:
                x_g = x_alt                
                break
            x_alt = x         

        if _debug_:
            print
            print "gamma=%4.3f" % (gamma)
            print "phi=%4.3f" % (phi)
            print "x=%5.4f " % ( x_g  )        

        return x_g
    #############################
    def stichprobenumfang(self,gamma,intervallLaenge):
        phi_c = self.berechneC(gamma)
        n = int( math.ceil(  (phi_c*phi_c)/(intervallLaenge*intervallLaenge) ))
        return n
    #############################            

            
    def konfidenzintervall(self,n,gamma,stichp):
        # Testparmeter
        # Das muss berechnet werden!!!!!
        h = stichp*1.0/n
        phi_c = self.berechneC(gamma)
        wurzel = phi_c * math.sqrt( abs(( h*(1-h) )/ n ))
        h1 = h - wurzel
        h2 = h + wurzel
        
        a_ = (phi_c * phi_c) + n
        b_ = (2 * h * n) + (phi_c * phi_c)
        c_ = (h * h) * n
        p_ = b_/a_
        q_ = c_/a_

        det = ((p_/2) * (p_/2)) - q_

        p1 = (p_/2) - math.sqrt(det)
        p2 = (p_/2) + math.sqrt(det)
        
        return [h,h1,h2,p1,p2]
        
#################################

if __name__ == "__main__":
    print "\nKonfidenzintervall"    
    konfidenz=intervall()
    print konfidenz.konfidenzintervall(100,0.8,30)
    konfidenz=intervall()                    
    print str( konfidenz.stichprobenumfang(0.8,0.1) )
    
    print "\nTest mit der Normalverteilung => Ablehnungsbereich"
    print "n=1000,h0=0.2,alpha=0.05"    
    normTest = normalTest(h0=0.2,n=1000,alpha=0.05,stichp=37,korrektur=0)
    gl =  normTest.berechneLinks(1000,0.2,0.2*1000,0.05)
    print "K = {0;...;%d}" % gl
    gr =  normTest.berechneRechts(1000,0.2,0.2*1000,0.05)
    print "K = {%d;...;%d}" % (gr,1000)    
    gl =  normTest.berechneLinks(1000,0.2,0.2*1000,0.05/2)
    gr =  normTest.berechneRechts(1000,0.2,0.2*1000,0.05/2)
    print "K = {0;...;%d} u {%d;...;%d}" % (gl,gr,1000)

    print "\nTest mit der Normalverteilung => mit Korrekturglied"
    print "n=1000,h0=0.2,alpha=0.05"        
    normTest = normalTest(h0=0.2,n=1000,alpha=0.05,stichp=37,korrektur=1)
    gl =  normTest.berechneLinks(1000,0.2,0.2*1000,0.05)
    print "K = {0;...;%d}" % gl
    gr =  normTest.berechneRechts(1000,0.2,0.2*1000,0.05)
    print "K = {%d;...;%d}" % (gr,1000)    
    gl =  normTest.berechneLinks(1000,0.2,0.2*1000,0.05/2)
    gr =  normTest.berechneRechts(1000,0.2,0.2*1000,0.05/2)
    print "K = {0;...;%d} u {%d;...;%d}" % (gl,gr,1000)    
    
    print "\nTest mit der Binomialverteilung => Ablehnungsbereich"
    print "n=100,h0=0.2,alpha=0.05"
    binTest = binomialTest(h0=0.2,n=100,alpha=0.05,stichp=37)
    gl =  binTest.berechneLinks(100,0.2,0.2*100,0.05)
    print "K = {0;...;%d}" % gl
    gr =  binTest.berechneRechts(100,0.2,0.2*100,0.05)
    print "K = {%d;...;%d}" % (gr,100)    
    gl =  binTest.berechneLinks(100,0.2,0.2*100,0.05/2)
    gr =  binTest.berechneRechts(100,0.2,0.2*100,0.05/2)
    print "K = {0;...;%d} u {%d;...;%d}" % (gl,gr,100)
    

    print "\nTest mit der Poissonverteilung => Ablehnungsbereich"
    print "n=100,h0=0.02,alpha=0.05"    
    pTest = poissonTest(h0=0.02,n=100,alpha=0.05,stichp=37)
    gl =  pTest.berechneLinks(100,0.02,0.02*100,0.05)
    print "K = {0;...;%d}" % gl
    gr =  pTest.berechneRechts(100,0.02,0.02*100,0.05)
    print "K = {%d;...;%d}" % (gr,100)    
    gl =  pTest.berechneLinks(100,0.02,0.02*100,0.05/2)
    gr =  pTest.berechneRechts(100,0.02,0.02*100,0.05/2)
    print "K = {0;...;%d} u {%d;...;%d}" % (gl,gr,100)
