# -*- coding: utf-8 -*-
"""
tkWidgets.py: Eingabe-Widgets für pyMaxima.py
ws

Vers. 0.02: get_Max und geoTable sowie die Klasse simpleTable

Vers. 0.03: Mit Tracing, 2007-05-30/2007-05-31
Beachte die Modifikationen in den Klassen
- eingabeList
- eingabeListEx
- eingabeListEx2
- eingabeOptMenuEx
- eingabeOptMenu
- eingabe2OptMenu
- eingabeOptMenuScharList
- eingabeOptMenuSchar

Vers. 0.04: Stochastik-Fenster, 2007-07-27
- hypothesentest
- nHypothesentest
- verteilung
- binomial
- konfidenz
- stichprobenUmfang
- kombinatorik
- nVerteilung

Vers. 0.05: DGL-Fenster, 2007-08-10
- eingabeDGL
- eingabeDglAwp1
- eingabeDglAwp2
- eingabeDglRwp

Vers. 0.06: 2007-09-09
- eingabeOptMenuScharList angepasst wg. Funktionenschar-Plot speichern


Vers. 0.07: 2007-09-23
- Dialog mit Option ohne Cancel-Button
- hilfetext mit 

Vers. 0.08: 2007-12-10
- eingabeListEx mit Parameter _label_

Vers. 0.09: 2007-12-19
- eingabeCrossProduct

Vers. 0.10: 2008-03-01
- alternativtest, nAlternativtest
- kettenbruch

Vers. 0.11: 2008-03-23
- simpleTable mit Scrollbars
  aus Pmw => PmwBase.py und PmwScrolledFrame.py
  URL: http://pmw.sourceforge.net/

Vers. 0.12: 2008-03-31
- rat2float bzw. r2f integriert (erlaubt Bruchschreibweise)

Vers. 0.13: 2008-10-08
- eingabeDreiDPlot

Vers. 0.14: 2008-10-15
- eingabeZweiDPlot
- Code bereinigt


Vers. 0.15: 2012-03-03
- Code fuer simpleTable ueberarbeitet

Vers. 0.16: 2012-03-31
- formatierte Ausgabe von Real (Kommazahlen) in geoTable
- neue Methoden: formatReal, bearbeitePunkt und bearbeiteListe

Vers. 0.17: 2012-04-03
- Die Breite der Tabelle wird jetzt in der Klasse simpleTable berechnet

Vers. 0.18: 2013-11-30
- class Dialog um Hilfe-Button erweitert
- in der abgeleiteten Klasse muss die Methode >hilfeText(self)< ueberschrieben werden
- neue Methode: wkBaum (benutzt den Hilfe-Button!)

"""
#######################################
import os
import Tkinter
if os.name == 'posix':
    Tkinter.wantobjects = 0
    # tkMessageBox.askyesno wrong result under Linux!
    # see http://mail.python.org/pipermail/python-bugs-list/2004-September/025172.html

from Tkinter import *  
import tkMessageBox 
from tkFileDialog import *
# try: help(tkFileDialog)
from ScrolledText import *

import fileinput
import pymaxUtil as util

# URL: http://pmw.sourceforge.net/
import PmwBase as Pmw
import PmwScrolledFrame




################################
class Dialog(Toplevel): 
    # Quelle: F. Lundh: An Introduction to Tkinter, ch. 9: Dialog Windows   
    # http://www.pythonware.com/library/tkinter/introduction/index.htm
    # Modifikation 1: der Cancel-Button kann ausgeblendet werden => _cancelButton_ = 1
    # Modifikation 2: der Help-Button kann ausgeblendet werden => _helpButton_ = 1
    def __init__(self, parent, title = None,_cancelButton_ = 1,_helpButton_ = 0): 
 
        Toplevel.__init__(self, parent) 
        self.transient(parent) 
 
        if title: 
            self.title(title) 
 
        self.parent = parent 
        
        self.result = None
        
        if (_cancelButton_ == 1):
            self.cancelButton = 1
        else:
            self.cancelButton = 0

        if (_helpButton_ == 1):
            self.helpButton = 1
        else:
            self.helpButton = 0
 
        body = Frame(self) 
        self.initial_focus = self.body(body) 
        body.pack(padx=5, pady=5)         
        
        self.buttonbox() 
 
        self.grab_set() 
 
        if not self.initial_focus: 
            self.initial_focus = self 
 
        self.protocol("WM_DELETE_WINDOW", self.cancel) 
 
        self.geometry("+%d+%d" % (parent.winfo_rootx()+50, 
                                  parent.winfo_rooty()+50)) 
 
        self.initial_focus.focus_set() 
 
        self.wait_window(self)

        
 
    # 
    # construction hooks 
 
    def body(self, master): 
        # create dialog body.  return widget that should have 
        # initial focus.  this method should be overridden 
 
        pass 
 
    def buttonbox(self): 
        # add standard button box. override if you don't want the 
        # standard buttons 
         
        box = Frame(self) 
 
        w = Button(box, text="OK", width=10, command=self.ok, default=ACTIVE) 
        w.pack(side=LEFT, padx=5, pady=5)
        if self.cancelButton:
            w = Button(box, text="Cancel", width=10, command=self.cancel) 
            w.pack(side=LEFT, padx=5, pady=5) 

        if self.helpButton:
            w = Button(box, text="Help", width=10, command=self.hilfe) 
            w.pack(side=LEFT, padx=5, pady=5) 
 
        self.bind("<Return>", self.ok)
        if self.cancelButton:        
            self.bind("<Escape>", self.cancel)                   
 
        box.pack() 
 
    # 
    # standard button semantics 
 
    def ok(self, event=None): 
 
        if not self.validate(): 
            self.initial_focus.focus_set() # put focus back 
            return 
 
        self.withdraw() 
        self.update_idletasks() 
 
        self.apply() 
         
        self.cancel() 
 
    def cancel(self, event=None): 
 
        # put focus back to the parent window 
        self.parent.focus_set() 
        self.destroy() 

    def hilfe(self, event=None):  
        # put focus back to the parent window 
        self.hilfeText()        

 
    # 
    # command hooks

    def hilfeText(self):
        pass # override 
 
    def validate(self): 
 
        return 1 # override 
 
    def apply(self): 
 
        pass # override 

################################
    
class Einzeiler(Dialog):   
    # wird benutzt in loescheFunktionenVar, floatPointPrecision
    def __init__(self,parent,_title_ = None, _label_ = 0, _start_ = None):
        self._title_ = _title_
        self.label = _label_
        self.start = _start_
        Dialog.__init__(self,parent,_title_)

    
    def body(self, master):      
        self.title( self._title_ )
        self.resizable('0','0')  # window should not be resizable!
        if os.name == 'posix': 
            self.font_size = 12
            self.breite = 25
            self.font_type = 'fixed'
        else:     
            self.font_size = 10
            self.breite = 25
            self.font_type = 'ansifixed'
        
        
        Label(master, text=self.label,width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=0)   
                
        self.e1 = Entry(master,width=28,font=(self.font_type,self.font_size,'normal') )   
        
        self.e1.insert(END, str( self.start ) )         
 
        self.e1.grid(row=0, column=1)   

        return self.e1 # initial focus   
                               
         
    def apply(self):   
        self.a1 = util.r2f( self.e1.get() )
        self.result = 1
         
#################################
class Kommentar(Dialog):   

    def __init__(self,parent,_title_ = None, _label_ = 0, _start_ = None):
        self._title_ = _title_
        self.label = _label_
        self.start = _start_
        Dialog.__init__(self,parent,_title_)

    
    def body(self, master):      
        self.title( self._title_ )
        self.resizable('0','0')  # window should not be resizable!
        if os.name == 'posix': 
            self.font_size = 12
            self.breite = 17
            self.font_type = 'fixed'
        else:     
            self.font_size = 10
            self.breite = 17
            self.font_type = 'ansifixed'
        
        
        Label(master, text=self.label,width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=0)   
                
        self.e1 = Entry(master,width=68,font=(self.font_type,self.font_size,'normal') )   
        
        self.e1.insert(END, str( self.start ) )         
 
        self.e1.grid(row=0, column=1)   

        return self.e1 # initial focus   
                               
    def sonderzeichen(self,eing):
        # vgl. auch class exprFenster
        a = repr(eing)
        a = a.replace("\\xe4","ä")
        a = a.replace("\\xfc","ü")
        a = a.replace("\\xf6","ö")
        a = a.replace("\\xd6","Ö")
        a = a.replace("\\x84","Ä")
        a = a.replace("\\xc4","Ä")
        a = a.replace("\\x9c","Ü")
        a = a.replace("\\xdc","Ü")
        a = a.replace("\\xdf","ß")
        a = a.replace("\\t","")
        a = a.replace("\\n","  ")        
        a = a[1:]
        a = a.replace("\'","")     
        a = str(a)
        return a


         
    def apply(self):
        myString =  self.sonderzeichen( self.e1.get() ).decode('iso-8859-1')
        self.a1 = myString.encode('iso-8859-1')        
        # self.a1 = str( self.e1.get() )
        self.result = 1
        
#################################
class wkBaum(Dialog):      
    # Fenster fuer Wk-Baum
    def __init__(self,parent,_title_ = None):
         self.symbol01='G'
         self.symbol02='H'
         self.p=0.5
         self.baumWidth = 7
         self._start_ = "Start"
         self._default_ = "-1.0"
         self._title_ = _title_
                  
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_,_helpButton_ = 1) 
             
    
    def body(self, master):      
        self.title( self._title_ )
        _test_ = True
        _test_ = False

        if os.name == 'posix': 
            self.font_size = 12
            self.breite = 20
            self.font_type = 'fixed'
        else:     
            self.font_size = 10
            self.breite = 16
            self.font_type = 'ansifixed'
        
        
        Label(master, text=u"Symbol für Ereignis 1: ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=0)   
        Label(master, text=u"Symbol für Ereignis 2: ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=1)
                
        self.e1 = Entry(master,width=20,font=(self.font_type,self.font_size,'normal') )   
        self.e2 = Entry(master,width=20,font=(self.font_type,self.font_size,'normal') )        
        
        self.e1.insert(END, str(self.symbol01) )         
        self.e2.insert(END, str(self.symbol02) )         
 
        self.e1.grid(row=0, column=1)   
        self.e2.grid(row=1, column=1)

        lab3=Label(master,text=u">>> Eingabe der Wahrscheinlichkeiten <<<",font=(self.font_type,self.font_size,'normal') )
        lab3.grid(row=2,column=0,columnspan=4)

        fmitte = Frame(master)
        fmitte.grid(row=3, column=0, columnspan=4, sticky=NSEW)
        ##fmitte.pack(expand=YES, fill=BOTH, padx=110)

        self.baum0 = Entry(fmitte,width=self.baumWidth)             ## Eingabezeile erzeugen
        self.baum0.grid(row=0,column=4)
        self.baum0.insert(END,"   "+self._start_)
        
        self.baum1 = Entry(fmitte,width=self.baumWidth)             ## Eingabezeile erzeugen
        self.baum1.grid(row=1,column=2)        
        if _test_:
            self.baum1.insert(END,"0.8")
        else:
            self.baum1.insert(END,self._default_)

        self.baum2 = Entry(fmitte,width=self.baumWidth)             ## Eingabezeile erzeugen
        self.baum2.grid(row=1,column=6)
        self.baum2.insert(END,self._default_)

        self.baum3 = Entry(fmitte,width=self.baumWidth)             ## Eingabezeile erzeugen
        self.baum3.grid(row=2,column=1)        
        if _test_:
            self.baum3.insert(END,"0.1")
        else:
            self.baum3.insert(END,self._default_)

        self.baum4 = Entry(fmitte,width=self.baumWidth)             ## Eingabezeile erzeugen
        self.baum4.grid(row=2,column=3)
        self.baum4.insert(END,self._default_)

        self.baum5 = Entry(fmitte,width=self.baumWidth)             ## Eingabezeile erzeugen
        self.baum5.grid(row=2,column=5)        
        if _test_:
            self.baum5.insert(END,"0.2")
        else:
            self.baum5.insert(END,self._default_)

        self.baum6 = Entry(fmitte,width=self.baumWidth)             ## Eingabezeile erzeugen
        self.baum6.grid(row=2,column=7)
        self.baum6.insert(END,self._default_)

        lab4=Label(fmitte,text=u" ")
        lab4.grid(row=3,column=0)

        lab5=Label(fmitte,text=u"Blätter   ")
        lab5.grid(row=5,column=0)
        
        self.baum7 = Entry(fmitte,width=self.baumWidth)             ## Eingabezeile erzeugen
        self.baum7.grid(row=5,column=1)
        self.baum7.insert(END,self._default_)

        self.baum8 = Entry(fmitte,width=self.baumWidth)             ## Eingabezeile erzeugen
        self.baum8.grid(row=5,column=3)
        self.baum8.insert(END,self._default_)

        self.baum9 = Entry(fmitte,width=self.baumWidth)             ## Eingabezeile erzeugen
        self.baum9.grid(row=5,column=5)
        self.baum9.insert(END,self._default_)

        self.baum10 = Entry(fmitte,width=self.baumWidth)             ## Eingabezeile erzeugen
        self.baum10.grid(row=5,column=7)
        self.baum10.insert(END,self._default_)

        return self.e1 # initial focus   
                               
    def hilfeText(self):
        # overwrite Help-Method from class Dialog!
        helpString = u"Notwendige Eingaben:\n\n- Symbol für Ereignis 1\n\n- Symbol für Ereignis 2\n\n- Wk/Gegen-Wk für Ereignis 1\n\n- 1 x Wk/Gegen-Wk aus linkem Ast\n\n- 1 x Wk/Gegen-Wk aus rechtem Ast\n\n=> Wk=bedingte Wk oder Blatt-Wk\n\n=> Wk unbekannt: >-1.0<"
        tkMessageBox.showinfo("Help . . .", helpString)        
        
        
    def apply(self):   
        self.a1 = self.e1.get() # Symbol 1
        self.a2 = self.e2.get() # Symbol 2
        self.b1 = self.baum1.get()
        self.b2 = self.baum2.get()
        self.b3 = self.baum3.get()
        self.b4 = self.baum4.get()
        self.b5 = self.baum5.get()
        self.b6 = self.baum6.get()
        self.b7 = self.baum7.get()
        self.b8 = self.baum8.get()
        self.b9 = self.baum9.get()
        self.b10 = self.baum10.get()                
        
        self.result = 1
         
#################################
class hypothesentest(Dialog):   
    def __init__(self,parent,_title_ = None, _arg_ = 0, _poisson_ = 0):
         self.h0=0.5
         self.n=100
         self.alpha=0.05
         self._title_ = _title_
         self.art = _arg_
         self.poisson = _poisson_
         if self.poisson:
             self.h0=0.01
##           arg == 0 <==> zweiseitig
##           arg == 1 <==> linksseitig
##           arg == 2 <==> rechtsseitig           
         
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_) 
             
    
    def body(self, master):      
        self.title( self._title_ + ": Testparameter")
        ##hypothesentest=test(h0=1/6.0,n=50,alpha=0.05,stichp=17)

        if os.name == 'posix': 
            self.font_size = 12
            self.breite = 22
            self.font_type = 'fixed'
        else:     
            self.font_size = 10
            self.breite = 18
            self.font_type = 'ansifixed'
        
        
        Label(master, text="n:                ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=0)   
        Label(master, text="alpha:            ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=1)
        if self.art == 0:   #  zweiseitig
            Label(master, text="Nullhypothese H0: p = ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=2)
        elif self.art == 1: # linksseitig    
            Label(master, text="Nullhypothese H0: p>= ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=2)
        else:    # art = 2 , rechtsseitig 
            Label(master, text="Nullhypothese H0: p<= ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=2)


                
        self.e1 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )   
        self.e2 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )
        self.e3 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )


        
        self.e1.insert(END, str(self.n) )         
        self.e2.insert(END, str(self.alpha) )         
        self.e3.insert(END, str(self.h0) )         


 
        self.e1.grid(row=0, column=1)   
        self.e2.grid(row=1, column=1)
        self.e3.grid(row=2, column=1)


        return self.e1 # initial focus   
                               
         
    def apply(self):   
        self.a1 = self.e1.get() # n        
        self.a2 = util.r2f( self.e2.get() ) # alpha
        self.a3 = util.r2f( self.e3.get() ) # h0
        self.result = 1


class nHypothesentest(Dialog):      

    def __init__(self,parent,_title_ = None, _arg_ = 0, _korrektur_=0):
         self.h0=0.5
         self.n=1000
         self.alpha=0.05
         self.korrektur_default = _korrektur_
         self._title_ = _title_
         self.art = _arg_
##           arg == 0 <==> zweiseitig
##           arg == 1 <==> linksseitig
##           arg == 2 <==> rechtsseitig           
         
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_) 
             
    
    def body(self, master):      
        self.title( self._title_ + ": Testparameter")
        ##hypothesentest=test(h0=1/6.0,n=50,alpha=0.05,stichp=17)

        if os.name == 'posix': 
            self.font_size = 12
            self.breite = 24
            self.font_type = 'fixed'
        else:     
            self.font_size = 10
            self.breite = 20
            self.font_type = 'ansifixed'
            
        self.korrektur = IntVar(master)
        
        Label(master, text= "n:                        ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=0)   
        Label(master, text= "alpha:                    ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=1)
        if self.art == 0:   #  zweiseitig
            Label(master, text="Nullhypothese H0:    p = ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=2)
        elif self.art == 1: # linksseitig    
            Label(master, text="Nullhypothese H0:    p>= ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=2)
        else:    # art = 2 , rechtsseitig 
            Label(master, text="Nullhypothese H0:    p<= ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=2)
        Label(master, text= "Mit Stetigkeitskorrektur: ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=3)            


                
        self.e1 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )   
        self.e2 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )
        self.e3 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )
        self.e4 = Checkbutton(master,variable=self.korrektur)
        


        
        self.e1.insert(END, str(self.n) )         
        self.e2.insert(END, str(self.alpha) )         
        self.e3.insert(END, str(self.h0) )
        self.korrektur.set(self.korrektur_default)        


 
        self.e1.grid(row=0, column=1)   
        self.e2.grid(row=1, column=1)
        self.e3.grid(row=2, column=1)
        self.e4.grid(row=3, column=1, sticky=W )        


        return self.e1 # initial focus   
                               
         
    def apply(self):   
        self.a1 = self.e1.get() # n        
        self.a2 = util.r2f( self.e2.get() )# alpha
        self.a3 = util.r2f( self.e3.get() ) # h0
        self.a4 = self.korrektur.get()
        self.result = 1

###############################################
class alternativtest(Dialog):   
   

    def __init__(self,parent,_title_ = None,):
         self.h0=0.2
         self.h1=0.7         
         self.n=100
         self.alpha=0.05        
         self._title_ = _title_
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_) 
             
    
    def body(self, master):      
        self.title( self._title_ )
        ##hypothesentest=test(h0=1/6.0,n=50,alpha=0.05,stichp=17)

        if os.name == 'posix': 
            self.font_size = 12
            self.breite = 28
            self.font_type = 'fixed'
        else:     
            self.font_size = 10
            self.breite = 24
            self.font_type = 'ansifixed'
        
        Label(master, text="n:                ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=0)   
        Label(master, text="alpha:            ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=1)
        Label(master, text="Nullhypothese H0:       p = ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=2)
        Label(master, text="Alternativhypothese H1: p = ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=3)


                
        self.e1 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )   
        self.e2 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )
        self.e3 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )
        self.e4 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )
        
        
        self.e1.insert(END, str(self.n) )         
        self.e2.insert(END, str(self.alpha) )         
        self.e3.insert(END, str(self.h0) )         
        self.e4.insert(END, str(self.h1) )         

 
        self.e1.grid(row=0, column=1)   
        self.e2.grid(row=1, column=1)
        self.e3.grid(row=2, column=1)
        self.e4.grid(row=3, column=1)


        return self.e1 # initial focus   
                               
         
    def apply(self):   
        self.a1 = self.e1.get() # n        
        self.a2 = util.r2f( self.e2.get() )# alpha
        self.a3 = util.r2f( self.e3.get() )# h0        
        self.a4 = util.r2f( self.e4.get() ) # h1
        self.result = 1



class nAlternativtest(Dialog):      

    def __init__(self,parent,_title_ = None, _korrektur_=0):
         self.h0=0.2
         self.h1=0.7
         self.n=1000
         self.alpha=0.05
         self.korrektur_default = _korrektur_
         self._title_ = _title_
         
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_) 
             
    
    def body(self, master):      
        self.title( self._title_ )
        ##hypothesentest=test(h0=1/6.0,n=50,alpha=0.05,stichp=17)

        if os.name == 'posix': 
            self.font_size = 12
            self.breite = 28
            self.font_type = 'fixed'
        else:     
            self.font_size = 10
            self.breite = 24
            self.font_type = 'ansifixed'
            
        self.korrektur = IntVar(master)
        
        Label(master, text="n:                          ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=0)   
        Label(master, text="alpha:                      ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=1)
        Label(master, text="Nullhypothese H0:       p = ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=2)        
        Label(master, text="Alternativhypothese H1: p = ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=3)        
        Label(master, text="Mit Stetigkeitskorrektur:   ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=4)            

                
        self.e1 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )   
        self.e2 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )
        self.e3 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )
        self.e4 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )        
        self.e5 = Checkbutton(master,variable=self.korrektur)
        


        
        self.e1.insert(END, str(self.n) )         
        self.e2.insert(END, str(self.alpha) )         
        self.e3.insert(END, str(self.h0) )
        self.e4.insert(END, str(self.h1) )
        self.korrektur.set(self.korrektur_default)        


 
        self.e1.grid(row=0, column=1)   
        self.e2.grid(row=1, column=1)
        self.e3.grid(row=2, column=1)
        self.e4.grid(row=3, column=1)        
        self.e5.grid(row=4, column=1, sticky=W )        


        return self.e1 # initial focus   
                               
         
    def apply(self):   
        self.a1 = self.e1.get() # n        
        self.a2 = util.r2f( self.e2.get() )# alpha
        self.a3 = util.r2f( self.e3.get() )# h0
        self.a4 = util.r2f( self.e4.get() ) # h1        
        self.a5 = self.korrektur.get()
        self.result = 1


###############################################
class verteilung(Dialog):   
    # wird benutzt in ???   

    def __init__(self,parent,_title_ = None,):
         self.p=0.5
         self.n=100
         self.li=0
         self.re=self.n/2
         self._title_ = _title_
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_) 
             
    
    def body(self, master):      
        self.title( self._title_ )
        ##hypothesentest=test(h0=1/6.0,n=50,alpha=0.05,stichp=17)

        if os.name == 'posix': 
            self.font_size = 12
            self.breite = 22
            self.font_type = 'fixed'
        else:     
            self.font_size = 10
            self.breite = 18
            self.font_type = 'ansifixed'
        
        
        Label(master, text="Länge n:          ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=0)   
        Label(master, text="linke Grenze k1:  ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=1)
        Label(master, text="rechte Grenze k2: ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=2)        
        Label(master, text="Treffer-wk p:     ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=3)


                
        self.e1 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )   
        self.e2 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )
        self.e3 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )
        self.e4 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )
        
        
        self.e1.insert(END, str(self.n) )         
        self.e2.insert(END, str(self.li) )         
        self.e3.insert(END, str(self.re) )         
        self.e4.insert(END, str(self.p) )         

 
        self.e1.grid(row=0, column=1)   
        self.e2.grid(row=1, column=1)
        self.e3.grid(row=2, column=1)
        self.e4.grid(row=3, column=1)


        return self.e1 # initial focus   
                               
         
    def apply(self):   
        self.a1 = self.e1.get() # n        
        self.a2 = self.e2.get() # k1
        self.a3 = self.e3.get() # k2        
        self.a4 = util.r2f( self.e4.get() ) # p
        self.result = 1

class binomial(Dialog):      
    # Binomialverteilung-Fenster
    def __init__(self,parent,_title_ = None,):
         self.n=25
         self.p=0.5
         self._title_ = _title_
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_) 
             
    
    def body(self, master):      
        self.title( self._title_ )

        if os.name == 'posix': 
            self.font_size = 12
            self.breite = 28
            self.font_type = 'fixed'
        else:     
            self.font_size = 10
            self.breite = 24
            self.font_type = 'ansifixed'
        
        
        Label(master, text="Länge der Bernoullikette n:  ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=0)   
        Label(master, text="Trefferwahrscheinlichkeit p: ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=1)
                
        self.e1 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )   
        self.e2 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )
        
        
        self.e1.insert(END, str(self.n) )         
        self.e2.insert(END, str(self.p) )         

 
        self.e1.grid(row=0, column=1)   
        self.e2.grid(row=1, column=1)


        return self.e1 # initial focus   
                               
         
    def apply(self):   
        self.a1 = self.e1.get() # n        
        self.a2 = util.r2f( self.e2.get() ) # p
        self.result = 1


#######################
class nVerteilung(Dialog):      
    # Normalverteilung-Fenster
    def __init__(self,parent,_title_ = None,):
         self.li=-3
         self.re=3
         self._title_ = _title_
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_) 
             
    
    def body(self, master):      
        self.title( self._title_ )

        if os.name == 'posix': 
            self.font_size = 12
            self.breite = 28
            self.font_type = 'fixed'
        else:     
            self.font_size = 10
            self.breite = 24
            self.font_type = 'ansifixed'
        
        
        Label(master, text="Linke Intervallgrenze x1:  ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=0)   
        Label(master, text="Rechte Intervallgrenze x2: ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=1)
                
        self.e1 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )   
        self.e2 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )
        
        
        self.e1.insert(END, str(self.li) )         
        self.e2.insert(END, str(self.re) )         

 
        self.e1.grid(row=0, column=1)   
        self.e2.grid(row=1, column=1)


        return self.e1 # initial focus   
                               
         
    def apply(self):   
        self.a1 = self.e1.get() # x1        
        self.a2 = self.e2.get() # x2
        self.result = 1

###############################################
class kombinatorik(Dialog):      
    # Kombinatorik-Fenster
    def __init__(self,parent,_title_ = None,):
         self.n=25
         self.k=int(self.n/2)
         self._title_ = _title_
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_) 
             
    
    def body(self, master):      
        self.title( self._title_ )

        if os.name == 'posix': 
            self.font_size = 12
            self.breite = 28
            self.font_type = 'fixed'
        else:     
            self.font_size = 10
            self.breite = 24
            self.font_type = 'ansifixed'
        
        
        Label(master, text="Anzahl der Kugeln n:      ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=0)   
        Label(master, text="gezogen werden k Kugeln:  ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=1)
                
        self.e1 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )   
        self.e2 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )
        
        
        self.e1.insert(END, str(self.n) )         
        self.e2.insert(END, str(self.k) )         

 
        self.e1.grid(row=0, column=1)   
        self.e2.grid(row=1, column=1)


        return self.e1 # initial focus   
                               
         
    def apply(self):   
        self.a1 = self.e1.get() # n        
        self.a2 = self.e2.get() # k
        self.result = 1

#################################
class konfidenz(Dialog):   
    # Quelle: F. Lundh: An Introduction to Tkinter, ch. 9: Dialog Windows   
    # http://www.pythonware.com/library/tkinter/introduction/index.htm   
   
    def __init__(self,parent,_title_ = None,):
         self.n=100
         self.stichp=int(self.n/5)
         self.gamma = 0.95
         self._title_ = _title_
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_) 
                

    
    def body(self, master):      
        self.title( self._title_ + ": Schätzparameter")


        if os.name == 'posix': 
            self.font_size = 12
            self.breite = 22
            self.font_type = 'fixed'
        else:     
            self.font_size = 10
            self.breite = 18
            self.font_type = 'ansifixed'
        
        
        Label(master, text="n:                 ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=0)   
        Label(master, text="gamma:             ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=1)
        Label(master, text="Stichprobenwert X: ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=2)
                
        self.e1 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )   
        self.e2 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )
        self.e3 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )

        
        self.e1.insert(END, str(self.n) )         
        self.e2.insert(END, str(self.gamma) )         
        self.e3.insert(END, str(self.stichp) )
        
 
        self.e1.grid(row=0, column=1)   
        self.e2.grid(row=1, column=1)
        self.e3.grid(row=2, column=1)

        return self.e1 # initial focus   
                               
         
    def apply(self):   
        self.a1 = self.e1.get()         # n
        self.a2 = util.r2f( self.e2.get() )  # gamma
        self.a3 = self.e3.get()
        self.result = 1

#################################    
class stichprobenUmfang(Dialog):   
    # Quelle: F. Lundh: An Introduction to Tkinter, ch. 9: Dialog Windows   
    # http://www.pythonware.com/library/tkinter/introduction/index.htm   
   
    def __init__(self,parent,_title_ = None,):
         self.n=25
         self.intervallLaenge=0.2
         self.gamma = 0.95
         self._title_ = _title_
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_) 
                

    
    def body(self, master):      
        self.title( self._title_ + ": Schätzparameter")


        if os.name == 'posix': 
            self.font_size = 12
            self.breite = 22
            self.font_type = 'fixed'
        else:     
            self.font_size = 10
            self.breite = 18
            self.font_type = 'ansifixed'
               

        Label(master, text="gamma:            ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=0)
        Label(master, text="Intervalllänge d: ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=1)
                
        self.e1 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )   
        self.e2 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )


        
        
        self.e1.insert(END, str(self.gamma) )         
        self.e2.insert(END, str(self.intervallLaenge) )
        
 
        self.e1.grid(row=0, column=1)   
        self.e2.grid(row=1, column=1)


        return self.e1 # initial focus   
                               
         
    def apply(self):   
        self.a1 = util.r2f( self.e1.get() )  # gamma
        self.a2 = util.r2f( self.e2.get() )  # Intervalllänge
        self.result = 1

##########################################
    
class simpleTable(Dialog):
    """
         class simpleTable erzeugt eine Tabelle mit Checkbuttons und Text;
         
         Scrollbars, aus Pmw => PmwBase.py und PmwScrolledFrame.py
         URL: http://pmw.sourceforge.net/
    """ 
    
    def __init__(self,root,headerList,tableList,cbFlag=1,_cancelButton_=1):
        
        self.max_height = root.winfo_height()
        #self.max_height = self.max_height - 100 # max Hoehe fuer ScrolledFrame
        self.max_height = self.max_height - 50 # max Hoehe fuer ScrolledFrame
        self.index = 0
        self.border = 1
        self.relief = "sunken"
        #self.relief = "groove" # ridge, sunken ,groove, raised
        self.ergebnis = []
	self.cb = None
	self.withCB = cbFlag
	

        self.headerList = headerList
	self.tableList = tableList
        #self.root = root
        _title_ = None
	
        Dialog.__init__(self,root,_title_,_cancelButton_)
       

    def header(self,scFrame,headers):
        
        if self.withCB:
            i = 0
        else:
            i = 1
        for col,w in headers:		
                #Label(master, width=w,text=col,relief=self.relief, bd=self.border).grid(row=self.index, column=i, sticky=W+E)
                Label(scFrame, width=w,text=col,relief=self.relief, bd=self.border).grid(row=self.index, column=i, sticky=W+E)
                i = i + 1
        self.index = self.index + 1

                    
    def insert(self,master,scFrame,elements):
        i = 0				
        for col in elements:
            if (i==0) and self.withCB:
                self.retFlag = IntVar(master)
                self.retFlag.set(int(col))
                cb = Checkbutton(scFrame,variable=self.retFlag,relief=self.relief, bd=self.border).grid(row=self.index, column=i, sticky=W+E)
                self.intVarList.append(self.retFlag)	
                if self.index == 1:
                    self.cb = cb	# für self.initial_focus.focus_set()
            elif (i==0):
                pass
            else:
                Label(scFrame,text=col, bg="white", relief=self.relief, bd=self.border).grid(row=self.index, column=i, sticky=W+E)
            i = i + 1
        self.index = self.index + 1


    def body(self, master):                    
        self.default = False
        
        #self.resizable('0','0')  # window should not be resizable!
        
        self.title("pyGeo: Geometrie-Objekte")
        
        self.intVarList = []

        # Weite des ScrolledFrames
        # mit_CB_linux, ohne_CB_linux, mit_CB_win_nt, ohne_CB_win_nt
        # alt:
        #_CB_Width_lin, _Width_lin, _CB_Width_win_nt, _Width_win_nt =  800,660,750,510
        _CB_Width_lin, _Width_lin, _CB_Width_win_nt, _Width_win_nt =  1000,800,1000,750

        #####################################################
        # Berechnung der Breite => neu: Vers. 1.69, 2012-04-03 <=
        sf_weite = 0
        anzahl_spalten = 0
        w_offset = -3 # Offset fuer Anzahl der Spalten
        if os.name == 'posix':
            w_faktor = 6 # empirisch ermittelt!
        else: # win nt
            w_faktor = 5 # empirisch ermittelt!            
        
        for element,w in self.headerList:                    
            sf_weite = sf_weite + len(element)
        sf_weite = sf_weite * 2
        #print sf_weite
            
        for element in self.tableList:
            sf_weite_i = 0
            for daten in element:
                sf_weite_i = sf_weite_i + len(daten)
            if sf_weite_i > sf_weite:
                sf_weite = sf_weite_i
        
        # weite = max_weite * faktor + offset * anzahl_Spalten
        sf_weite = sf_weite * w_faktor + w_offset * anzahl_spalten
        #print sf_weite
        #####################################################        
        
        # Hoehe/Weite des ScrolledFrames festlegen        
        if os.name == 'posix':
            faktor = 31
            sf_Height = len(self.tableList) * faktor # Faktor 30/31 == Zeilenhoehe unter suse 10.1            
            if sf_Height < 220:
                sf_Height = 220
            elif (sf_Height > self.max_height):
                sf_Height = self.max_height
            
            if self.withCB:
                #sf_Width = 1000
                #sf_Width = _CB_Width_lin
                sf_Width = sf_weite + 200
            else:
                #sf_Width = 800
                #sf_Width = _Width_lin
                sf_Width = sf_weite
        else: # win nt
            faktor = 25
            sf_Height = len(self.tableList) * faktor # Faktor 24/25 == Zeilenhoehe unter win xp
            if sf_Height < 180:  # eventuell: 170
                sf_Height = 180
            elif (sf_Height > self.max_height):
                sf_Height = self.max_height
                
            
            if self.withCB:
                #sf_Width = 1000
                #sf_Width = _CB_Width_win_nt
                sf_Width = sf_weite + 250
                
            else:
                #sf_Width = 750
                #sf_Width = _Width_win_nt
                sf_Width = sf_weite

        ##############################################
        #print sf_Height, sf_Width
                
	# Create the ScrolledFrame.
        self.sf = PmwScrolledFrame.ScrolledFrame(master,		
                usehullsize = 1,
                hull_width = sf_Width,
                hull_height = sf_Height  
        )
	
	self.sf.configure(hscrollmode = 'dynamic')
        # items = ['none', 'static', 'dynamic']

	self.sf.configure(vscrollmode = 'dynamic')
        # items = ['none', 'static', 'dynamic']

	self.sf.configure(horizflex = 'expand')        
        # items = ['fixed', 'expand', 'shrink', 'elastic']

	self.sf.configure(vertflex = 'expand')
        # items = ['fixed', 'expand', 'shrink', 'elastic']

        #self.sf.configure(hull_width=cb.component('entryfield').winfo_reqwidth())

	self.sf.pack(padx = 3, pady = 3, fill = 'both', expand = 1)
    

        self.frame = self.sf.interior()
        

        self.header(self.frame,self.headerList) # Kopf der Tabelle
	for element in self.tableList:      # Tabelle aufbauen
		self.insert(master,self.frame,element)
		#print element

				                	

        # URL: http://sebsauvage.net/python/gui/#tkinter_resize
        #self.resizable(True,True)
        # horizontal, vertikal = True,False
        self.resizable(True,False)
        self.update()
               
        #self.geometry(self.geometry())
        
        return self.cb # initial focus   
                               
         
    def apply(self):       
        self.result = 1
        self.ergebnis = [] # wichtig!!! globale Matrix self.ergebnis auf NULL [] setzen!
        
	if self.withCB and len(self.intVarList) > 0:
	    for element in self.intVarList:			
		    self.ergebnis.append(element.get())
	    #print self.ergebnis
	else:
	    self.result = 0
		
         
 
################################# 
class Text(Toplevel): 
 
    def __init__(self, parent, title = None): 
 
        Toplevel.__init__(self, parent) 
        ## Mit dem folgenden Befehl liegt das Fenster
        ## immer ber dem parent-Fenster	
        ##self.transient(parent) 
 
        if title: 
            self.title(title) 
 
        self.parent = parent 
 
        #self.result = None
        

        body = Frame(self) 
        self.initial_focus = self.body(body) 
        body.pack(padx=5, pady=5) 
 
        self.buttonbox() 
 
        #self.grab_set() 
 
        if not self.initial_focus: 
            self.initial_focus = self 
 
        self.protocol("WM_DELETE_WINDOW", self.die) 
 
        self.geometry("+%d+%d" % (parent.winfo_rootx()+50, 
                                  parent.winfo_rooty()+50)) 
 
        self.initial_focus.focus_set() 
	
	self.resizable('0','0')  # window should NOT be resizable!
 
        self.wait_window(self)

        #self.state = DISABLED
 
    # 
    # construction hooks 
 
    def body(self): 
        # create dialog body.  return widget that should have 
        # initial focus.  this method should be overridden 
        pass 
 
    def buttonbox(self): 
        # add standard button box. override if you don't want the 
        # standard buttons 
         
        box = Frame(self) 
 
        w = Button(box, text="OK", width=10, command=self.ok, default=ACTIVE) 
        w.pack(side=LEFT, padx=5, pady=5) 
 
        self.bind("&lt;Return>", self.ok) 
 
        box.pack() 
 
    # 
    # standard button semantics 
 
    def ok(self, event=None): 
 
        if not self.validate(): 
            self.initial_focus.focus_set() # put focus back 
            return 
 
        self.withdraw() 
        self.update_idletasks() 
 
        #self.apply() 
         
        self.die() 
         
 
    def die(self, event=None): 
        # put focus back to the parent window 
        self.parent.focus_set() 
        self.destroy() 
         
    # 
    # command hooks 
     
 
    def validate(self): 
 
        return 1 # override 
 
    def apply(self): 
 
        pass # override 
     
################################# 
         
class hilfefenster(Text): 
    # abgeleitet aus tkSimpleText   

    def __init__(self,parent,_text_ = None,_title_ = None, _font_size_ = 10): 
        self.text = _text_
        self._title_ = _title_
        
        ## font-type and font-size
        ## see also tkSimpleText.py
        if os.name == 'posix':
            gf_font_art='bold'
            gf_font_type='courier'
            gf_font_size=_font_size_
            gf_font_type2=gf_font_type
        else:
            gf_font_art='normal'
            gf_font_type2='courier'
            gf_font_type='courier'
            gf_font_size=_font_size_
        self.gf_font=(gf_font_type,gf_font_size,gf_font_art)
        
        Text.__init__(self,parent)

    
    def body(self, master):   
        self.title(self._title_)    
        if os.name == 'posix': 
            self.t = ScrolledText(master,width=100,height=20,relief=RAISED,bd=2,font=self.gf_font,background="white",wrap=WORD)          
        else: 
            #self.t = ScrolledText(master,width=100,height=20,relief=RAISED,font="ansifixed",bd=2,background="white",wrap=WORD)
            self.t = ScrolledText(master,width=100,height=20,relief=RAISED,font=self.gf_font,bd=2,background="white",wrap=WORD)
            
        self.t.insert(END,self.text)        
        self.t.pack(fill=BOTH, expand=1)
        self.t.config(state=DISABLED)        
##        self.popupMenu = Menu(master,tearoff=0)  
##        self.popupMenu.add_command(label='copy', command=(lambda arg='t2' : self.gf_copy(arg) )  )
##        self.t.bind("<Button-3>",self.popup)
        
   
    def apply(self):     
        pass   

    #
    # construction hooks
##    def popup(self, event): 
##        self.popupMenu.post(event.x_root, event.y_root) 
##   
##    def gf_copy(self,fenster):
##        win = self.t
##        try:
##            marked_text = win.get(SEL_FIRST, SEL_LAST)
##            win.clipboard_clear()
##            win.clipboard_append(marked_text)
##        except TclError:
##            pass
##    
#####################################
class exprFenster(Text): 
    # abgeleitet aus tkSimpleText   

    def __init__(self,parent,_text_ = None,_title_ = None): 
         self.text = _text_
         self._title_ = _title_
         self.result = ''
         Text.__init__(self,parent)

    
    def body(self, master):   
        self.title(self._title_)    
        if os.name == 'posix': 
            self.t = ScrolledText(master,width=100,height=20,relief=RAISED,bd=2,background="white",wrap=WORD)          
        else: 
            self.t = ScrolledText(master,width=100,height=20,relief=RAISED,font="ansifixed",bd=2,background="white",wrap=WORD)
            
        #self.t.insert(END,self.text)        
        self.t.pack(fill=BOTH, expand=1)
        #self.t.config(state=DISABLED)        
        self.popupMenu = Menu(master,tearoff=0)  
        self.popupMenu.add_command(label='copy', command=(lambda arg='t2' : self.copy(arg) )  )
        self.popupMenu.add_command(label='cut', command=(lambda arg='t2' : self.cut(arg) )  )
        self.popupMenu.add_command(label='paste', command=(lambda arg='t2' : self.paste(arg) )  )        
        self.t.bind("<Button-3>",self.popup)
        self.resizable('0','0')  # window should not be resizable!
        
        
        
    def ok(self, event=None): 
 
        if not self.validate(): 
            self.initial_focus.focus_set() # put focus back 
            return 
 
        self.withdraw() 
        self.update_idletasks() 
 
        self.apply() 
         
        self.die() 

    def sonderzeichen(self,eing):
        a = repr(eing)
        a = a.replace("\\xe4","ä")
        a = a.replace("\\xfc","ü")
        a = a.replace("\\xf6","ö")
        a = a.replace("\\xd6","Ö")
        a = a.replace("\\xc4","Ä")
        a = a.replace("\\xdc","Ü")
        a = a.replace("\\xdf","ß")
####    geht in Zusammenhang mit:
####    myString =  self.sonderzeichen( self.t.get(1.0, END) ).decode('iso-8859-1')
####    self.result = myString.encode('iso-8859-1')
###########################################   
##        a = a.replace("ä","\\xe4")
##        a = a.replace("ü","\\xfc")
##        a = a.replace("ö","\\xf6")
##        a = a.replace("Ö","\\xd6")
##        a = a.replace("Ä","\\xc4")
##        a = a.replace("Ü","\\xdc")
##        a = a.replace("ß","\\xdf")
##        a = a.replace("ä","ae")
##        a = a.replace("ü","ue")
##        a = a.replace("ö","oe")
##        a = a.replace("Ö","Oe")
##        a = a.replace("Ä","Ae")
##        a = a.replace("Ü","Ue")
##        a = a.replace("ß","ss")                
##        a = a.replace("\\xe4","ae")
##        a = a.replace("\\xfc","ue")
##        a = a.replace("\\xf6","oe")
##        a = a.replace("\\xd6","Oe")
##        a = a.replace("\\xc4","Ae")
##        a = a.replace("\\xdc","Ue")
##        a = a.replace("\\xdf","ss")
        a = a.replace("\\t","")
        a = a.replace("\\n","  ")        
        a = a[1:]
        a = a.replace("\'","")     
        a = str(a)
        return a

    
   
    def apply(self):
        myString =  self.sonderzeichen( self.t.get(1.0, END) ).decode('iso-8859-1')
        self.result = myString.encode('iso-8859-1')


    #
    # construction hooks
    def popup(self, event): 
        self.popupMenu.post(event.x_root, event.y_root) 
   
######################################
##      mark, copy and paste
##      with widget.clipboard_append/
##      clipboard_clear/selection_get(selection="CLIPBOARD")
##      see: von Lï¿½is/Fischbeck: Python 2, S. 339-341
##	or M. Lutz: Programming Python, S. 382-385            
    def copy(self,fenster):
        win = self.t
        try:
            marked_text = win.get(SEL_FIRST, SEL_LAST)
            win.clipboard_clear()
            win.clipboard_append(marked_text)
        except TclError:
            pass
    
    def paste(self,fenster): 
        win = self.t
        try:
            text = ''
            text = win.selection_get(selection="CLIPBOARD")
            if len(text) > 0:
                win.insert(INSERT, text)
        except TclError:
            pass
                    
    def cut(self,fenster):
        win = self.t
        try:
            marked_text = win.get(SEL_FIRST, SEL_LAST)
            win.delete(SEL_FIRST, SEL_LAST)
            win.clipboard_clear()
            win.clipboard_append(marked_text)
        except TclError:
            pass


####################################################         
   
class eingabefenster(Dialog):   
    # Quelle: F. Lundh: An Introduction to Tkinter, ch. 9: Dialog Windows   
    # http://www.pythonware.com/library/tkinter/introduction/index.htm


    def __init__(self,parent,_title_ = None,_text_ = None):
        if _text_: 
            self.text = _text_
        else:
            self.text = "Ausdruck: "
        if _title_:
	 #print str(self.list)	    
            Dialog.__init__(self,parent,_title_)
        else:
            Dialog.__init__(self,parent,"Call Maxima")


    
    def body(self, master):        
        #fenstertitel = "Call Maxima"
        #self.title(fenstertitel)    
           
        self.label1 = Label(master, text=self.text)
        self.label1.grid(row=0, column=0)   
   
        self.e1 = Entry(master,width=50)   
        self.e1.grid(row=0, column=1)
        self.popupMenu = Menu(master,tearoff=0)  
        self.popupMenu.add_command(label='cut', command=(lambda arg='self.e1' : self.cut(arg) )  )
        self.popupMenu.add_command(label='copy', command=(lambda arg='self.e1' : self.copy(arg) )  )
        self.popupMenu.add_command(label='paste', command=(lambda arg='self.e1' : self.paste(arg) )  )        
        self.e1.bind("<Button-3>",self.popup)
        self.resizable('0','0')  # window should not be resizable!        
        #self.label1.bind("<Button-3>",self.popup)
        return self.e1 # initial focus
    
   
    def apply(self):   
        self.result = self.e1.get()   
    #
    # construction hooks
    def popup(self, event): 
        self.popupMenu.post(event.x_root, event.y_root) 
   

#######################################
##      mark, copy and paste
##      with widget.clipboard_append/
##      clipboard_clear/selection_get(selection="CLIPBOARD")
##      see: von Lï¿½is/Fischbeck: Python 2, S. 339-341
##	or M. Lutz: Programming Python, S. 382-385            
    def copy(self,fenster):
        win = self.e1
        try:
            marked_text = win.get()
            win.clipboard_clear()
            win.clipboard_append(marked_text)
        except TclError:
            pass
    
    def paste(self,fenster): 
        win = self.e1
        try:
            text = ''
            text = win.selection_get(selection="CLIPBOARD")
            if len(text) > 0:
                win.insert(INSERT, text)
        except TclError:
            pass
                    
    def cut(self,fenster):
        win = self.e1
        try:
            marked_text = win.get()
            win.delete(SEL_FIRST, SEL_LAST)
            win.clipboard_clear()
            win.clipboard_append(marked_text)
        except TclError:
            pass
        
####################################################
class simpleEingabe(Dialog):   
    # wird benutzt in getMatrixName 
    def __init__(self,parent,_title_ = None,_list_ = []): 
	 self.value = None
         self.list = _list_[:]
	 self.variable = StringVar(parent)	 
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_) 
         
    def body(self, master):  

        self.label2 = Label(master, text="Matrix:")
        self.label2.grid(row=0, column=0) 
        listFrame = Frame(master)
        listFrame.grid(row=0, column=1, sticky=NSEW)#side=TOP, padx=5, pady=5)

	aindex = 0
	i = 0
	for element in self.list:
		if (element == 'A'):
			aindex = i
		i = i + 1


        self.e2 = Entry(listFrame,width=6)   
        self.e2.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e2.insert(END,self.list[aindex])

        #self.variable = StringVar(master)
        self.variable.set(self.list[aindex]) # default value
        self.varOpt = apply(OptionMenu, (listFrame, self.variable) + tuple(self.list) )
        self.varOpt.pack(side=LEFT, padx=2, pady=2,fill=X)
        
        self.resizable('0','0')  # window should not be resizable!
        
        self.variable.trace('w',self.changed )
        
        return self.e2 # initial focus
   
    def apply(self):   
        #self.result = self.e1.get()
        
        self.result = self.e2.get()  #self.variable.get()

    #
    # construction hooks
    def changed(self,*args):
        # URL: http://mail.python.org/pipermail/tutor/2005-July/040018.html
        #print self.variable.get()  # das geht tatsächlich . . .
        self.e2.delete( 0,END )
        self.e2.insert( END,self.variable.get() ) # das geht !!!


####################################################        
class eingabeOptMenu(Dialog):   
    # wird benutzt in diff, stamm, kurvendisk 
    def __init__(self,parent,_title_ = None,ausdruck = None,_list_ = []): 
	 self.value = None
         self.list = _list_[:]
	 self.expr = ausdruck
	 self.variable = StringVar(parent)	 
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_) 
         
    def body(self, master):  
        self.label1 = Label(master, text="Funktion:")
        self.label1.grid(row=0, column=0) 
        self.e1 = Entry(master,width=50)   
        self.e1.grid(row=0, column=1)
        self.e1.insert(END,self.expr)

        self.label2 = Label(master, text="Variable:")
        self.label2.grid(row=0, column=2) 
        listFrame = Frame(master)
        listFrame.grid(row=0, column=3, sticky=NSEW)#side=TOP, padx=5, pady=5)

	xindex = 0
	i = 0
	for element in self.list:
		if (element == 'x'):
			xindex = i
		i = i +1


        self.e2 = Entry(listFrame,width=3)   
        self.e2.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e2.insert(END,self.list[xindex])

        #self.variable = StringVar(master)
        self.variable.set(self.list[xindex]) # default value
        self.varOpt = apply(OptionMenu, (listFrame, self.variable) + tuple(self.list) )
        self.varOpt.pack(side=LEFT, padx=2, pady=2,fill=X)
        
        self.resizable('0','0')  # window should not be resizable!
        
        self.variable.trace('w',self.changed )
        
        return self.e1 # initial focus
   
    def apply(self):   
        self.result = self.e1.get()
        
        self.value = self.e2.get()  #self.variable.get()

    #
    # construction hooks
    def changed(self,*args):
        # URL: http://mail.python.org/pipermail/tutor/2005-July/040018.html
        #print self.variable.get()  # das geht tatsächlich . . .
        self.e2.delete( 0,END )
        self.e2.insert( END,self.variable.get() ) # das geht !!!

    
class eingabeOptMenuEx(eingabeOptMenu):
    # wird benutzt in partialBruchZerlegung
    def __init__(self,parent,_title_ = None,ausdruck = None,_list_ = [],_label_ = None):
        if _label_:
            self.labelText = _label_
        #self.variable = StringVar(parent) => Variable ist schon in "eingabeOptMenu" vereinbart!!!
        eingabeOptMenu.__init__(self,parent,_title_,ausdruck,_list_) 


    def body(self, master):  
        self.label1 = Label(master, text=self.labelText)
        self.label1.grid(row=0, column=0) 
        self.e1 = Entry(master,width=50)   
        self.e1.grid(row=0, column=1)
        self.e1.insert(END,self.expr)

        self.label2 = Label(master, text="Variable:")
        self.label2.grid(row=0, column=2) 
        listFrame = Frame(master)
        listFrame.grid(row=0, column=3, sticky=NSEW)#side=TOP, padx=5, pady=5)

	xindex = 0
	i = 0
	for element in self.list:
		if (element == 'x'):
			xindex = i
		i = i + 1


        self.e2 = Entry(listFrame,width=3)   
        self.e2.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e2.insert(END,self.list[xindex])


        #self.variable = StringVar(master)
        self.variable.set(self.list[xindex]) # default value
        self.varOpt = apply(OptionMenu, (listFrame, self.variable) + tuple(self.list) )
        self.varOpt.pack(side=LEFT, padx=2, pady=2,fill=X)
        
        self.resizable('0','0')  # window should not be resizable!
        self.variable.trace('w',self.changed )
        return self.e1 # initial focus


    def changed(self,*args):
        # URL: http://mail.python.org/pipermail/tutor/2005-July/040018.html
        #print self.variable.get()  # das geht tatsächlich . . .
        self.e2.delete( 0,END )
        self.e2.insert( END,self.variable.get() ) # das geht !!!
        
##    def apply(self):   
##        self.result = self.e1.get()
##        
##        self.value = self.variable.get()
##
    
##########################################################    
class eingabe2OptMenu(Dialog):   
    # wird benutzt in limit
    def __init__(self,parent,_title_ = None,ausdruck = None,_list_ = [],_list2_ = []): 
	 self.value = None
         self.list = _list_[:]
         self.list2 = _list2_[:]         
	 self.expr = ausdruck
	 self.variable = StringVar(parent)
	 self.para = StringVar(parent)	 
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_) 
         
    def body(self, master):  
        self.label1 = Label(master, text="Funktion:")
        self.label1.grid(row=0, column=0) 
        self.e1 = Entry(master,width=50)   
        self.e1.grid(row=0, column=1)
        self.e1.insert(END,self.expr)

        self.label2 = Label(master, text="Variable: ")
        self.label2.grid(row=0, column=2,sticky=W) 
        listFrame = Frame(master)
        listFrame.grid(row=0, column=3, sticky=NSEW)#side=TOP, padx=5, pady=5)
	xindex = 0
	i = 0
	for element in self.list:
		if (element == 'x'):
			xindex = i
		i = i +1

        self.e2 = Entry(listFrame,width=3)   
        self.e2.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e2.insert(END,self.list[xindex])

	
        #self.variable = StringVar(master)
        self.variable.set(self.list[xindex]) # default value
        self.varOpt = apply(OptionMenu, (listFrame, self.variable) + tuple(self.list) )
        self.varOpt.pack(side=LEFT, padx=2, pady=2,fill=X)
        
        self.label3 = Label(master, text="Besondere Werte:")
        self.label3.grid(row=1, column=2,sticky=W) 
        listFrame = Frame(master)
        listFrame.grid(row=1, column=3, sticky=NSEW)#side=TOP, padx=5, pady=5)
        self.parameter = StringVar(master)
        self.parameter.set(self.list2[0]) # default value
        self.parOpt = apply(OptionMenu, (listFrame, self.parameter) + tuple(self.list2) )
        self.parOpt.pack(fill=X)
        
        self.label5 = Label(master, text="Stelle: ")
        self.label5.grid(row=1, column=0,sticky=W)
        self.e5 = Entry(master,width=10)   
        self.e5.grid(row=1, column=1,sticky=W)
        self.e5.insert(END,'0')
        self.resizable('0','0')  # window should not be resizable!

        self.variable.trace('w',self.changed )
        self.parameter.trace('w',self.change_e5 )
        
        return self.e1 # initial focus
   
    def apply(self):   
        self.result = self.e1.get()
        self.a5 = self.e5.get()
        self.value = self.e2.get()  #statt self.variable.get()
        


    #
    # construction hooks
    def changed(self,*args):
        # URL: http://mail.python.org/pipermail/tutor/2005-July/040018.html
        #print self.variable.get()  # das geht tatsächlich . . .
        self.e2.delete( 0,END )
        self.e2.insert( END,self.variable.get() ) # das geht !!!

    def change_e5(self,*args):
        # URL: http://mail.python.org/pipermail/tutor/2005-July/040018.html
        #print self.variable.get()  # das geht tatsächlich . . .
        self.e5.delete( 0,END )
        self.e5.insert( END,self.parameter.get() ) # das geht !!!

    
##########################################################
class eingabeDGL(Dialog):   
    # wird benutzt in solveDGL
    def __init__(self,parent,_title_ = None,ausdruck = None,_list_ = []): 
	 self.value = None
         self.list = _list_[:]
	 self.expr = ausdruck
	 self.variable = StringVar(parent)
	 self.para = StringVar(parent)	 
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_) 
         
    def body(self, master):  
        self.label1 = Label(master, text="DGL:")
        self.label1.grid(row=0, column=0, sticky=W) 
        self.e1 = Entry(master,width=50)   
        self.e1.grid(row=0, column=1, columnspan=3, sticky=NSEW)
        self.e1.insert(END,self.expr)

        self.label2 = Label(master, text="unab. Variable: ")
        self.label2.grid(row=1, column=0) 
        listFrame1 = Frame(master)
        listFrame1.grid(row=1, column=1, sticky=NSEW)#side=TOP, padx=5, pady=5)
	
	xindex = 0
	i = 0
	for element in self.list:
		if (element == 'x'):
			xindex = i
		i = i + 1

	yindex = 0
	i = 0
	for element in self.list:
		if (element == 'y'):
			yindex = i
		i = i + 1

        self.e2 = Entry(listFrame1,width=3)   
        self.e2.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e2.insert(END,self.list[xindex])
	
        #self.variable = StringVar(master)
        self.variable.set(self.list[xindex]) # default value
        self.varOpt = apply(OptionMenu, (listFrame1, self.variable) + tuple(self.list) )
        self.varOpt.pack(side=LEFT, padx=2, pady=2,fill=X)

        ###########
        
        self.label3 = Label(master, text="ab. Variable: ")
        self.label3.grid(row=1, column=2) 
        listFrame2 = Frame(master)
        listFrame2.grid(row=1, column=3, sticky=NSEW)#side=TOP, padx=5, pady=5)

        self.e3 = Entry(listFrame2,width=3)   
        self.e3.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e3.insert(END,self.list[yindex])
	        
        self.parameter = StringVar(master)
        self.parameter.set(self.list[yindex]) # default value
        self.parOpt = apply(OptionMenu, (listFrame2, self.parameter) + tuple(self.list) )
        self.parOpt.pack(side=LEFT, padx=2, pady=2,fill=X)
                
        self.resizable('0','0')  # window should not be resizable!

        self.variable.trace('w',self.changed )
        
        self.parameter.trace('w',self.change_e3 )
        
        return self.e1 # initial focus
   
    def apply(self):   
        self.result = self.e1.get()
        self.var_x = self.e2.get()  #statt self.variable.get()        
        self.var_y = self.e3.get()   #statt self.parameter.get()

    #
    # construction hooks
    def changed(self,*args):
        # URL: http://mail.python.org/pipermail/tutor/2005-July/040018.html
        self.e2.delete( 0,END )
        self.e2.insert( END,self.variable.get() ) # das geht !!!

    def change_e3(self,*args):
        self.e3.delete( 0,END )
        self.e3.insert( END,self.parameter.get() ) # das geht !!!


#####################    
class eingabeDglAwp1(Dialog):   
    # wird benutzt in dglAWP1
    def __init__(self,parent,_title_ = None,ausdruck = None,_list_ = []): 
	 self.value = None
         self.list = _list_[:]
	 self.expr = ausdruck
	 self.variable = StringVar(parent)
	 self.para = StringVar(parent)	 
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_) 
         
    def body(self, master):  
        self.label1 = Label(master, text="Lösung der DGL:")
        self.label1.grid(row=0, column=0, sticky=W) 
        self.e1 = Entry(master,width=50)   
        self.e1.grid(row=0, column=1, columnspan=3, sticky=NSEW)
        self.e1.insert(END,self.expr)

        self.label2 = Label(master, text="unab. Variable: ")
        self.label2.grid(row=1, column=0) 
        listFrame1 = Frame(master)
        listFrame1.grid(row=1, column=1, sticky=NSEW)#side=TOP, padx=5, pady=5)
	
	xindex = 0
	i = 0
	for element in self.list:
		if (element == 'x'):
			xindex = i
		i = i + 1

	yindex = 0
	i = 0
	for element in self.list:
		if (element == 'y'):
			yindex = i
		i = i + 1


        #self.variable = StringVar(master)
        self.variable.set(self.list[xindex]) # default value
        self.varOpt = apply(OptionMenu, (listFrame1, self.variable) + tuple(self.list) )
        self.varOpt.pack(side=LEFT, padx=2, pady=2,fill=X)

        self.e2 = Entry(listFrame1,width=3)   
        self.e2.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e2.insert(END,self.list[xindex])

        self.label4 = Label(listFrame1, text="=")
        self.label4.pack(side=LEFT, padx=2, pady=2,fill=X)
	
        self.e4 = Entry(listFrame1,width=3)   
        self.e4.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e4.insert(END,'0')



        ###########
        
        self.label3 = Label(master, text="ab. Variable: ")
        self.label3.grid(row=1, column=2) 
        listFrame2 = Frame(master)
        listFrame2.grid(row=1, column=3, sticky=NSEW)#side=TOP, padx=5, pady=5)

        self.parameter = StringVar(master)
        self.parameter.set(self.list[yindex]) # default value
        self.parOpt = apply(OptionMenu, (listFrame2, self.parameter) + tuple(self.list) )
        self.parOpt.pack(side=LEFT, padx=2, pady=2,fill=X)

        self.e3 = Entry(listFrame2,width=3)   
        self.e3.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e3.insert(END,self.list[yindex])


        self.label5 = Label(listFrame2, text="=")
        self.label5.pack(side=LEFT, padx=2, pady=2,fill=X)

        self.e5 = Entry(listFrame2,width=3)   
        self.e5.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e5.insert(END,'1')



                
        self.resizable('0','0')  # window should not be resizable!

        self.variable.trace('w',self.changed )
        
        self.parameter.trace('w',self.change_e3 )
        
        return self.e1 # initial focus
   
    def apply(self):   
        self.result = self.e1.get()
        self.var_x = self.e2.get()  #statt self.variable.get()
        self.val_x = self.e4.get()  #statt self.variable.get()
        
        self.var_y = self.e3.get()   #statt self.parameter.get()
        self.val_y = self.e5.get()   #statt self.parameter.get()

    #
    # construction hooks
    def changed(self,*args):
        # URL: http://mail.python.org/pipermail/tutor/2005-July/040018.html
        self.e2.delete( 0,END )
        self.e2.insert( END,self.variable.get() ) # das geht !!!

    def change_e3(self,*args):
        self.e3.delete( 0,END )
        self.e3.insert( END,self.parameter.get() ) # das geht !!!
    
#####################    
class eingabeDglAwp2(Dialog):   
    # wird benutzt in dglAWP2
    def __init__(self,parent,_title_ = None,ausdruck = None,_list_ = []): 
	 self.value = None
         self.list = _list_[:]
	 self.expr = ausdruck
	 self.variable = StringVar(parent)
	 self.para = StringVar(parent)
	 self.basis = "'diff("
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_) 
         
    def body(self, master):  
        self.label1 = Label(master, text="Lösung der DGL:")
        self.label1.grid(row=0, column=0, sticky=W) 
        self.e1 = Entry(master,width=50)   
        self.e1.grid(row=0, column=1, columnspan=3, sticky=NSEW)
        self.e1.insert(END,self.expr)

        self.label2 = Label(master, text="unab. Variable: ")
        self.label2.grid(row=1, column=0) 
        listFrame1 = Frame(master)
        listFrame1.grid(row=1, column=1, sticky=NSEW)#side=TOP, padx=5, pady=5)
	
	xindex = 0
	i = 0
	for element in self.list:
		if (element == 'x'):
			xindex = i
		i = i + 1

	yindex = 0
	i = 0
	for element in self.list:
		if (element == 'y'):
			yindex = i
		i = i + 1


        #self.variable = StringVar(master)
        self.variable.set(self.list[xindex]) # default value
        self.varOpt = apply(OptionMenu, (listFrame1, self.variable) + tuple(self.list) )
        self.varOpt.pack(side=LEFT, padx=2, pady=2,fill=X)

        self.e2 = Entry(listFrame1,width=3)   
        self.e2.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e2.insert(END,self.list[xindex])

        self.label4 = Label(listFrame1, text="=")
        self.label4.pack(side=LEFT, padx=2, pady=2,fill=X)
	
        self.e4 = Entry(listFrame1,width=3)   
        self.e4.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e4.insert(END,'0')



        ###########
        
        self.label3 = Label(master, text="ab. Variable: ")
        self.label3.grid(row=1, column=2) 
        listFrame2 = Frame(master)
        listFrame2.grid(row=1, column=3, sticky=NSEW)#side=TOP, padx=5, pady=5)

        self.parameter = StringVar(master)
        self.parameter.set(self.list[yindex]) # default value
        self.parOpt = apply(OptionMenu, (listFrame2, self.parameter) + tuple(self.list) )
        self.parOpt.pack(side=LEFT, padx=2, pady=2,fill=X)

        self.e3 = Entry(listFrame2,width=3)   
        self.e3.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e3.insert(END,self.list[yindex])


        self.label5 = Label(listFrame2, text="=")
        self.label5.pack(side=LEFT, padx=2, pady=2,fill=X)

        self.e5 = Entry(listFrame2,width=3)   
        self.e5.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e5.insert(END,'1')



        ###########
        
        self.label31 = Label(master, text="Ableitung dy/dx: ")
        self.label31.grid(row=2, column=0) 
        listFrame3 = Frame(master)
        listFrame3.grid(row=2, column=1,columnspan=3, sticky=NSEW)#side=TOP, padx=5, pady=5)


        self.e6 = Entry(listFrame3,width=11)   
        self.e6.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e6.insert(END,"'diff(y,x)")


        self.label6 = Label(listFrame3, text="=")
        self.label6.pack(side=LEFT, padx=2, pady=2,fill=X)

        self.e7 = Entry(listFrame3,width=3)   
        self.e7.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e7.insert(END,'0')



                
        self.resizable('0','0')  # window should not be resizable!

        self.variable.trace('w',self.changed )
        
        self.parameter.trace('w',self.change_e3 )
        
        return self.e1 # initial focus
   
    def apply(self):   
        self.result = self.e1.get()
        self.var_x = self.e2.get()  #statt self.variable.get()
        self.val_x = self.e4.get()  #statt self.variable.get()
        
        self.var_y = self.e3.get()   #statt self.parameter.get()
        self.val_y = self.e5.get()   #statt self.parameter.get()

        self.var_diff = self.e6.get()   #statt self.parameter.get()
        self.val_diff = self.e7.get()   #statt self.parameter.get()


    #
    # construction hooks
    def changed(self,*args):
        # URL: http://mail.python.org/pipermail/tutor/2005-July/040018.html
        self.e2.delete( 0,END )        
        self.e2.insert( END, self.variable.get() ) # das geht !!!
        
        neu = self.basis + self.variable.get() + ',' + self.parameter.get() + ')'        
        self.e6.delete( 0,END )
        self.e6.insert(END,neu)

    def change_e3(self,*args):
        self.e3.delete( 0,END )
        self.e3.insert( END, self.parameter.get() ) # das geht !!!
        
        neu = self.basis + self.variable.get() + ',' + self.parameter.get() + ')'        
        self.e6.delete( 0,END )
        self.e6.insert(END,neu)
    

#####################    
class eingabeDglRwp(Dialog):   
    # wird benutzt in dglRWP
    def __init__(self,parent,_title_ = None,ausdruck = None,_list_ = []): 
	 self.value = None
         self.list = _list_[:]
	 self.expr = ausdruck
	 self.variable = StringVar(parent)
	 self.para = StringVar(parent)	 
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_) 
         
    def body(self, master):  
        self.label1 = Label(master, text="Lösung der DGL:")
        self.label1.grid(row=0, column=0, sticky=W) 
        self.e1 = Entry(master,width=50)   
        self.e1.grid(row=0, column=1, columnspan=3, sticky=NSEW)
        self.e1.insert(END,self.expr)

        self.label2 = Label(master, text="unab. Variable: ")
        self.label2.grid(row=1, column=0, sticky=W) 
        listFrame1 = Frame(master)
        listFrame1.grid(row=1, column=1, sticky=NSEW)#side=TOP, padx=5, pady=5)
	
	xindex = 0
	i = 0
	for element in self.list:
		if (element == 'x'):
			xindex = i
		i = i + 1

	yindex = 0
	i = 0
	for element in self.list:
		if (element == 'y'):
			yindex = i
		i = i + 1


        #self.variable = StringVar(master)
        self.variable.set(self.list[xindex]) # default value
        self.varOpt = apply(OptionMenu, (listFrame1, self.variable) + tuple(self.list) )
        self.varOpt.pack(side=LEFT, padx=2, pady=2,fill=X)

        self.e2 = Entry(listFrame1,width=3)   
        self.e2.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e2.insert(END,self.list[xindex])

        self.label4 = Label(listFrame1, text="=")
        self.label4.pack(side=LEFT, padx=2, pady=2,fill=X)
	
        self.e4 = Entry(listFrame1,width=3)   
        self.e4.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e4.insert(END,'0')



        ###########
        
        self.label3 = Label(master, text="ab. Variable: ")
        self.label3.grid(row=1, column=2) 
        listFrame2 = Frame(master)
        listFrame2.grid(row=1, column=3, sticky=NSEW)#side=TOP, padx=5, pady=5)

        self.parameter = StringVar(master)
        self.parameter.set(self.list[yindex]) # default value
        self.parOpt = apply(OptionMenu, (listFrame2, self.parameter) + tuple(self.list) )
        self.parOpt.pack(side=LEFT, padx=2, pady=2,fill=X)

        self.e3 = Entry(listFrame2,width=3)   
        self.e3.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e3.insert(END,self.list[yindex])


        self.label5 = Label(listFrame2, text="=")
        self.label5.pack(side=LEFT, padx=2, pady=2,fill=X)

        self.e5 = Entry(listFrame2,width=3)   
        self.e5.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e5.insert(END,'1')
        ##########################################
        # 2. Stelle
        self.label20 = Label(master, text="Intervallende: ")
        self.label20.grid(row=2, column=0, sticky=W) 

        listFrame3 = Frame(master)
        listFrame3.grid(row=2, column=1, sticky=NSEW)#side=TOP, padx=5, pady=5)

        #self.label21 = Label(listFrame3, text="           ")
        #self.label21.pack(side=LEFT, padx=2, pady=2,fill=X)


        self.e41 = Entry(listFrame3,width=3)   
        self.e41.pack(side=RIGHT, padx=2, pady=2,fill=X)
        self.e41.insert(END,'1')

	
        self.label41 = Label(listFrame3, text="=")
        self.label41.pack(side=RIGHT, padx=2, pady=2,fill=X)
	
        self.e21 = Entry(listFrame3,width=3)   
        self.e21.pack(side=RIGHT, padx=2, pady=2,fill=X)
        self.e21.insert(END,self.list[xindex])



        ###########
        
        listFrame4 = Frame(master)
        listFrame4.grid(row=2, column=3, sticky=NSEW)#side=TOP, padx=5, pady=5)


        #self.label31 = Label(listFrame4, text="           ")
        #self.label31.pack(side=LEFT, padx=2, pady=2,fill=X)

        self.e51 = Entry(listFrame4,width=3)   
        self.e51.pack(side=RIGHT, padx=2, pady=2,fill=X)
        self.e51.insert(END,'1')                

        self.label51 = Label(listFrame4, text="=")
        self.label51.pack(side=RIGHT, padx=2, pady=2,fill=X)

        self.e31 = Entry(listFrame4,width=3)   
        self.e31.pack(side=RIGHT, padx=2, pady=2,fill=X)
        self.e31.insert(END,self.list[yindex])



                
        self.resizable('0','0')  # window should not be resizable!

        self.variable.trace('w',self.changed )
        
        self.parameter.trace('w',self.change_e3 )
        
        return self.e1 # initial focus
   
    def apply(self):   
        self.result = self.e1.get()
        self.var_x = self.e2.get()  #statt self.variable.get()
        self.val_x = self.e4.get()  #statt self.variable.get()
        self.val_x2 = self.e41.get()  #statt self.variable.get()
        
        self.var_y = self.e3.get()   #statt self.parameter.get()
        self.val_y = self.e5.get()   #statt self.parameter.get()
        self.val_y2 = self.e51.get()  #statt self.variable.get()
    #
    # construction hooks
    def changed(self,*args):
        # URL: http://mail.python.org/pipermail/tutor/2005-July/040018.html
        self.e2.delete( 0,END )
        self.e2.insert( END,self.variable.get() ) # das geht !!!
        
        self.e21.delete( 0,END )
        self.e21.insert(END,self.variable.get())
        

    def change_e3(self,*args):
        self.e3.delete( 0,END )
        self.e3.insert( END,self.parameter.get() ) # das geht !!!

        self.e31.delete( 0,END )
        self.e31.insert(END,self.parameter.get() ) 
        
##########################################################        
class eingabeOptMenuSchar(Dialog):   
    # wird benutzt in funkSchar     
    def __init__(self,parent,_title_ = None,ausdruck = None,_list_ = []): 
	 self.value = None
         self.list = _list_[:]
	 self.expr = ausdruck
	 self.variable = StringVar(parent)
	 self.para = StringVar(parent)	 
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_) 
         
    def body(self, master):  
        self.label1 = Label(master, text="Funktion:")
        self.label1.grid(row=0, column=0) 
        self.e1 = Entry(master,width=50)   
        self.e1.grid(row=0, column=1)
        self.e1.insert(END,self.expr)

        self.label2 = Label(master, text="Variable: ")
        self.label2.grid(row=0, column=2) 
        listFrame1 = Frame(master)
        listFrame1.grid(row=0, column=3, sticky=NSEW)#side=TOP, padx=5, pady=5)
	
	xindex = 0
	i = 0
	for element in self.list:
		if (element == 'x'):
			xindex = i
		i = i +1

        self.e2 = Entry(listFrame1,width=3)   
        self.e2.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e2.insert(END,self.list[xindex])
	
        #self.variable = StringVar(master)
        self.variable.set(self.list[xindex]) # default value
        self.varOpt = apply(OptionMenu, (listFrame1, self.variable) + tuple(self.list) )
        self.varOpt.pack(side=LEFT, padx=2, pady=2,fill=X)

        ###########
        
        self.label3 = Label(master, text="Parameter:")
        self.label3.grid(row=1, column=2) 
        listFrame2 = Frame(master)
        listFrame2.grid(row=1, column=3, sticky=NSEW)#side=TOP, padx=5, pady=5)

        self.e3 = Entry(listFrame2,width=3)   
        self.e3.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e3.insert(END,self.list[0])
	        
        self.parameter = StringVar(master)
        self.parameter.set(self.list[0]) # default value
        self.parOpt = apply(OptionMenu, (listFrame2, self.parameter) + tuple(self.list) )
        self.parOpt.pack(side=LEFT, padx=2, pady=2,fill=X)

        
        self.label4 = Label(master, text="Parameter-Intervall: [<Start-Wert>, . . . , <End-Wert>]")
        #self.label4.grid(row=1, column=1) # columnspan=2
        self.label4.grid(row=1, column=0, columnspan=2,sticky=W) # columnspan=2


        listFrame3 = Frame(master)
        listFrame3.grid(row=2, column=0, columnspan=3, sticky=NSEW)#side=TOP, padx=5, pady=5)
        
        self.label5 = Label(listFrame3, text="Start-Wert: ")
        self.label5.grid(row=0, column=0,sticky=W )
        self.e5 = Entry(listFrame3,width=10)   
        self.e5.grid(row=0, column=1,sticky=W, padx=8 )
        self.e5.insert(END,'1')

        self.label7 = Label(listFrame3, text="Schrittweite: ")
        self.label7.grid(row=0, column=2, sticky=W )
        self.e7 = Entry(listFrame3,width=10)   
        self.e7.grid(row=0, column=3,sticky=W, padx=8 )
        self.e7.insert(END,'1')


        
        self.label6 = Label(listFrame3, text="End-Wert: ")
        self.label6.grid(row=0, column=4,sticky=W ) 
        self.e6 = Entry(listFrame3,width=10)   
        self.e6.grid(row=0, column=5,sticky=W, padx=8 )
        self.e6.insert(END,'2')



        
        self.resizable('0','0')  # window should not be resizable!

        self.variable.trace('w',self.changed )
        
        self.parameter.trace('w',self.change_e3 )
        
        return self.e1 # initial focus
   
    def apply(self):   
        self.result = self.e1.get()
        self.a5 = self.e5.get()
        self.a6 = self.e6.get()
        self.a7 = self.e7.get()  # schrittweite
        self.value = self.e2.get()  #statt self.variable.get()
        
        self.para = self.e3.get()   #statt self.parameter.get()

    #
    # construction hooks
    def changed(self,*args):
        # URL: http://mail.python.org/pipermail/tutor/2005-July/040018.html
        self.e2.delete( 0,END )
        self.e2.insert( END,self.variable.get() ) # das geht !!!

    def change_e3(self,*args):
        self.e3.delete( 0,END )
        self.e3.insert( END,self.parameter.get() ) # das geht !!!
    
####################################################   
    
class eingabeOptMenuScharList(Dialog):   
    # wird benutzt in funkSchar 
    def __init__(self,parent,_title_ = None,ausdruck = None,_list_ = [],von_k_ = None,bis_k_ = None,schrittweite_ = None,parameter_ = None, _dici_ = {}): 
	 self.value = None
         self.list = _list_[:]
         
         self.argDici = {}
         self.len = len(_dici_)
         #print self.len
         util.kopiereDici(_dici_,self.argDici)
         
	 self.expr = ausdruck
	 self.variable = StringVar(parent)
	 self.para = StringVar(parent)
	 if von_k_:  # untere Grenze fuer Parameter k
             self._k_von = str(von_k_)
         else:
             self._k_von = '1'
	 
	 if bis_k_:  # obere Grenze fuer Parameter k
             self._k_bis = str(bis_k_)
         else:
             self._k_bis = '1'

         if schrittweite_:
             self.schrittweite_ = str(schrittweite_)
         else:
             self.schrittweite_ = '1'
             
         if parameter_:
             self.parameter_arg = str(parameter_)
         else:
             self.parameter_arg = 'k'
         #print self.parameter_arg


	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_) 
         
    def body(self, master):  
        self.label1 = Label(master, text="Funktion:")
        self.label1.grid(row=0, column=0) 
        self.e1 = Entry(master,width=50)   
        self.e1.grid(row=0, column=1)
        self.e1.insert(END,self.expr)

        self.label2 = Label(master, text="Variable: ")
        self.label2.grid(row=0, column=2) 
        listFrame1 = Frame(master)
        listFrame1.grid(row=0, column=3, sticky=NSEW)#side=TOP, padx=5, pady=5)
	
	xindex = 0
	i = 0
	for element in self.list:
		if (element == 'x'):
			xindex = i
		i = i +1
        self.e2 = Entry(listFrame1,width=3)   
        self.e2.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e2.insert(END,self.list[xindex])
	
        #self.variable = StringVar(master)
        self.variable.set(self.list[xindex]) # default value
        self.varOpt = apply(OptionMenu, (listFrame1, self.variable) + tuple(self.list) )
        self.varOpt.pack(side=LEFT, padx=2, pady=2,fill=X)

        ##############
        
        self.label3 = Label(master, text="Parameter:")
        self.label3.grid(row=1, column=2) 
        listFrame2 = Frame(master)
        listFrame2.grid(row=1, column=3, sticky=NSEW)#side=TOP, padx=5, pady=5)

        self.e3 = Entry(listFrame2,width=3)   
        self.e3.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e3.insert(END,self.parameter_arg)
        
        self.parameter = StringVar(master)
        self.parameter.set(self.parameter_arg) # default value
        self.parOpt = apply(OptionMenu, (listFrame2, self.parameter) + tuple(self.list) )
        self.parOpt.pack(side=LEFT, padx=2, pady=2,fill=X)

        
        self.label4 = Label(master, text="Parameter-Intervall: [<Start-Wert>, . . . , <End-Wert>]")
        self.label4.grid(row=1,column=0, columnspan=2,sticky=W) # columnspan=2
        
##        self.label5 = Label(master, text="Start-Wert: ")
##        self.label5.grid(row=2, column=0)
##        self.e5 = Entry(master,width=10)   
##        self.e5.grid(row=2, column=1,sticky=W)
##        self.e5.insert(END, self._k_von )
##        
##        self.label6 = Label(master, text="End-Wert: ")
##        self.label6.grid(row=2, column=2) 
##        self.e6 = Entry(master,width=10)   
##        self.e6.grid(row=2, column=3)
##        self.e6.insert(END, self._k_bis )
        
        listFrame3 = Frame(master)
        listFrame3.grid(row=2, column=0, columnspan=3, sticky=NSEW)#side=TOP, padx=5, pady=5)
        
        self.label5 = Label(listFrame3, text="Start-Wert: ")
        self.label5.grid(row=0, column=0,sticky=W )
        self.e5 = Entry(listFrame3,width=10)   
        self.e5.grid(row=0, column=1,sticky=W, padx=8 )
        self.e5.insert(END, self._k_von)

        self.label7 = Label(listFrame3, text="Schrittweite: ")
        self.label7.grid(row=0, column=2, sticky=W )
        self.e7 = Entry(listFrame3,width=10)   
        self.e7.grid(row=0, column=3,sticky=W, padx=8 )
        self.e7.insert(END, self.schrittweite_)


        
        self.label6 = Label(listFrame3, text="End-Wert: ")
        self.label6.grid(row=0, column=4,sticky=W ) 
        self.e6 = Entry(listFrame3,width=10)   
        self.e6.grid(row=0, column=5,sticky=W, padx=8 )
        self.e6.insert(END, self._k_bis)

        
        for i in range(self.len):
            #print str(self.argDici[i+1])

##            setattr(self, self.argDici[i+1][0], Label(master, text=self.argDici[i+1][1]))                        
##            getattr(self, self.argDici[i+1][0]).grid(row=self.argDici[i+1][2], column=self.argDici[i+1][3], sticky=W)
##            
##            setattr(self, self.argDici[i+1][4], Entry(master,width=10))            
##            getattr(self, self.argDici[i+1][4]).grid(row=self.argDici[i+1][6], column=self.argDici[i+1][7], sticky=W)
##            getattr(self, self.argDici[i+1][4]).insert(END,self.argDici[i+1][5])
            setattr(self, self.argDici[i+1][0], Label(listFrame3, text=self.argDici[i+1][1]))                        
            getattr(self, self.argDici[i+1][0]).grid(row=self.argDici[i+1][2], column=self.argDici[i+1][3], sticky=W)
            
            setattr(self, self.argDici[i+1][4], Entry(listFrame3,width=10))            
            getattr(self, self.argDici[i+1][4]).grid(row=self.argDici[i+1][6], column=self.argDici[i+1][7], padx=8,sticky=W)
            getattr(self, self.argDici[i+1][4]).insert(END,self.argDici[i+1][5])
            
            setattr(self, 'ret'+str(i+1), '0')
            
        self.resizable('0','0')  # window should not be resizable!

        self.variable.trace('w',self.changed )
        
        self.parameter.trace('w',self.change_e3 )
        
        return self.e1 # initial focus
   
    def apply(self):   
        self.result = self.e1.get()
        for i in range(self.len):
            setattr(self, 'ret'+str(i+1),getattr(self, self.argDici[i+1][4]).get() )
            #print str(getattr(self, 'ret'+str(i+1) ) )
        
        self.a5 = self.e5.get()
        self.a6 = self.e6.get()
        self.a7 = self.e7.get() # schrittweite
        self.value = self.e2.get()  #statt self.variable.get()
        
        self.para = self.e3.get()   #statt self.parameter.get()

    #
    # construction hooks
    def changed(self,*args):
        # URL: http://mail.python.org/pipermail/tutor/2005-July/040018.html
        self.e2.delete( 0,END )
        self.e2.insert( END,self.variable.get() ) # das geht !!!

    def change_e3(self,*args):
        self.e3.delete( 0,END )
        self.e3.insert( END,self.parameter.get() ) # das geht !!!    
    
####################################################   

class eingabeList(Dialog):   
    # wird benutzt in: solve, newton
    def __init__(self,parent,_title_ = None,ausdruck = None,label1= None,label2=None,_varlist_ = [],_dici_ = {}): 
	 self.value = None
         self.list = _varlist_[:]
         self.argDici = {}
         self.len = len(_dici_)
         util.kopiereDici(_dici_,self.argDici)
	 self.expr = ausdruck
         self.variable = StringVar(parent)
         if label1:
             self.label1_text = label1
         else:
             self.label1_text = "Funktion:"
         if label2:
             self.label2_text = label2
         else:
             self.label2_text = "Variable:"
             
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_) 
         
    def body(self, master):  
        self.label1 = Label(master, text=self.label1_text)
        self.label1.grid(row=0, column=0) 
        self.e1 = Entry(master,width=50)   
        self.e1.grid(row=0, column=1)
        self.e1.insert(END,self.expr)

        self.label2 = Label(master, text=self.label2_text)
        self.label2.grid(row=0, column=2) 
        listFrame = Frame(master)
        listFrame.grid(row=0, column=3, sticky=NSEW)#side=TOP, padx=5, pady=5)

	xindex = 0
	i = 0
	for element in self.list:
		if (element == 'x'):
			xindex = i
		i = i +1


        self.e2 = Entry(listFrame,width=3)   
        self.e2.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e2.insert(END,self.list[xindex])


        #self.variable = StringVar(master)
        self.variable.set(self.list[xindex]) # default value
        self.varOpt = apply(OptionMenu, (listFrame, self.variable) + tuple(self.list) )
        self.varOpt.pack(side=LEFT, padx=2, pady=2,fill=X)
        for i in range(self.len):
            #print str(self.argDici[i+1])

            setattr(self, self.argDici[i+1][0], Label(master, text=self.argDici[i+1][1]))                        
            getattr(self, self.argDici[i+1][0]).grid(row=i+1, column=0, sticky=W)
            
            setattr(self, self.argDici[i+1][2], Entry(master,width=10))            
            getattr(self, self.argDici[i+1][2]).grid(row=i+1, column=1, sticky=W)
            getattr(self, self.argDici[i+1][2]).insert(END,self.argDici[i+1][3])
            
            setattr(self, 'ret'+str(i+1), '0')
            
        self.resizable('0','0')  # window should not be resizable!

        self.variable.trace('w',self.changed )
        
        return self.e1 # initial focus
   
    def apply(self):   
        self.result = self.e1.get()
        for i in range(self.len):
            setattr(self, 'ret'+str(i+1),getattr(self, self.argDici[i+1][2]).get() )
            #print str(getattr(self, 'ret'+str(i+1) ) )
        self.value = self.e2.get()  # statt self.variable.get()

    #
    # construction hooks
    def changed(self,*args):
        # URL: http://mail.python.org/pipermail/tutor/2005-July/040018.html
        #print self.variable.get()  # das geht tatsächlich . . .
        self.e2.delete( 0,END )
        self.e2.insert( END,self.variable.get() ) # das geht !!!   


####################################################
class eingabeListEx(Dialog):   
    # wird benutzt in rotIntegral,integral, summe, taylor, findRoots, plot
    def __init__(self,parent,_title_ = None,ausdruck = None,_label_= None,_varlist_ = [],_dici_ = {}): 
	 self.value = None
	 self.labelText = None
         if len(_label_) > 0:             
             self.labelText = _label_
         else:
             self.labelText = "Funktion:"
	 
         self.list = _varlist_[:]
         self.argDici = {}
         self.len = len(_dici_)
         util.kopiereDici(_dici_,self.argDici)
	 self.expr = ausdruck
         self.variable = StringVar(parent)	 
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_)
         
    def body(self, master):  
        self.label1 = Label(master, text=self.labelText)
        self.label1.grid(row=0, column=0) 
        self.e1 = Entry(master,width=50)   
        self.e1.grid(row=0, column=1)
        self.e1.insert(END,self.expr)

        self.label2 = Label(master, text="Variable:")
        self.label2.grid(row=0, column=2) 
        listFrame = Frame(master)
        listFrame.grid(row=0, column=3, sticky=NSEW)#side=TOP, padx=5, pady=5)
	
	xindex = 0
	i = 0
	for element in self.list:
		if (element == 'x'):
			xindex = i
		i = i +1

        self.e2 = Entry(listFrame,width=3)   
        self.e2.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e2.insert(END,self.list[xindex])
			
	
        #self.variable = StringVar(master)
        self.variable.set(self.list[xindex]) # default value
        self.varOpt = apply(OptionMenu, (listFrame, self.variable) + tuple(self.list) )
        self.varOpt.pack(side=LEFT, padx=2, pady=2,fill=X)
        for i in range(self.len):
            #print str(self.argDici[i+1])

            setattr(self, self.argDici[i+1][0], Label(master, text=self.argDici[i+1][1]))                        
            getattr(self, self.argDici[i+1][0]).grid(row=self.argDici[i+1][2], column=self.argDici[i+1][3], sticky=W)
            
            setattr(self, self.argDici[i+1][4], Entry(master,width=10))            
            getattr(self, self.argDici[i+1][4]).grid(row=self.argDici[i+1][6], column=self.argDici[i+1][7], sticky=W)
            getattr(self, self.argDici[i+1][4]).insert(END,self.argDici[i+1][5])
            
            setattr(self, 'ret'+str(i+1), '0')
        self.resizable('0','0')  # window should not be resizable!

        self.variable.trace('w',self.changed )
        
        return self.e1 # initial focus
   
    def apply(self):   
        self.result = self.e1.get()
        for i in range(self.len):
            setattr(self, 'ret'+str(i+1),getattr(self, self.argDici[i+1][4]).get() )
            #print str(getattr(self, 'ret'+str(i+1) ) )
        self.value = self.e2.get()  # statt self.variable.get()


    #
    # construction hooks
   
    def changed(self,*args):
        # URL: http://mail.python.org/pipermail/tutor/2005-July/040018.html
        #print self.variable.get()  # das geht tatsächlich . . .
        self.e2.delete( 0,END )
        self.e2.insert( END,self.variable.get() ) # das geht !!!   

####################################################
class eingabeListEx2(Dialog):   
    # s. a.: F. Lundh: An Introduction to Tkinter, The Tkinter OptionMenu Widget   
    # http://effbot.org/tkinterbook/optionmenu.htm
    # sowie http://www.python-forum.de/viewtopic.php?p=41793&sid=ceaf21172cec8c288a1381934f322b25
    # wird benutzt in partialbruch, polyDiv
    def __init__(self,parent,_title_ = None,ausdruck = None,_varlist_ = [],_dici_ = {}): 
	 self.value = None
         self.list = _varlist_[:]
         self.argDici = {}
         self.len = len(_dici_)
         util.kopiereDici(_dici_,self.argDici)
	 self.expr = ausdruck
         self.variable = StringVar(parent)	 
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_) 
         
    def body(self, master):  
##        self.label1 = Label(master, text="Funktion:")
##        self.label1.grid(row=0, column=0) 
##        self.e1 = Entry(master,width=50)   
##        self.e1.grid(row=0, column=1)
##        self.e1.insert(END,self.expr)

        self.label2 = Label(master, text="Variable:")
        self.label2.grid(row=0, column=2) 
        listFrame = Frame(master)
        listFrame.grid(row=0, column=3, sticky=NSEW)#side=TOP, padx=5, pady=5)

	xindex = 0
	i = 0
	for element in self.list:
            if (element == 'x'):
                    xindex = i
            i = i +1

        self.e2 = Entry(listFrame,width=3)   
        self.e2.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e2.insert(END,self.list[xindex])


        #self.variable = StringVar(master)
        self.variable.set(self.list[xindex]) # default value
        self.varOpt = apply(OptionMenu, (listFrame, self.variable) + tuple(self.list) )
        self.varOpt.pack(side=LEFT, padx=2, pady=2,fill=X)
        
        for i in range(self.len):
            #print str(self.argDici[i+1])

            setattr(self, self.argDici[i+1][0], Label(master, text=self.argDici[i+1][1]))                        
            getattr(self, self.argDici[i+1][0]).grid(row=self.argDici[i+1][2], column=self.argDici[i+1][3], sticky=W)
            
            setattr(self, self.argDici[i+1][4], Entry(master,width=30))            
            getattr(self, self.argDici[i+1][4]).grid(row=self.argDici[i+1][6], column=self.argDici[i+1][7], sticky=W)
            getattr(self, self.argDici[i+1][4]).insert(END,self.argDici[i+1][5])
            setattr(self, 'ret'+str(i+1), '0')
        self.resizable('0','0')  # window should not be resizable!

        self.variable.trace('w',self.changed )
        
        return getattr(self, self.argDici[1][4]) # initial focus (Entry fuer Zaehler-Polynom)
   
    def apply(self):   
        #self.result = self.e1.get()
        for i in range(self.len):
            setattr(self, 'ret'+str(i+1),getattr(self, self.argDici[i+1][4]).get() )
            #print str(getattr(self, 'ret'+str(i+1) ) )
        self.value = self.e2.get()  #statt self.variable.get()

    #
    # construction hooks
    def changed(self,*args):
        # URL: http://mail.python.org/pipermail/tutor/2005-July/040018.html
        #print self.variable.get()  # das geht tatsächlich . . .
        self.e2.delete( 0,END )
        self.e2.insert( END,self.variable.get() ) # das geht !!!

####################################################   

class eingabeCrossProduct(Dialog):   
    # wird benutzt in: vecKreuz
    def __init__(self,parent,_title_ = None,_varlist_ = []): 
	 self.value0 = None
	 self.value1 = None
         self.list = _varlist_[:]
         self.variable1 = StringVar(parent)
         self.variable2 = StringVar(parent)
         self.label_cross = " x "
             
	 #print str(self.list)	    
         Dialog.__init__(self,parent,_title_) 
         
    def body(self, master):  
        listFrame1 = Frame(master)
        listFrame1.grid(row=0, column=0, sticky=NSEW)#side=TOP, padx=5, pady=5)


        #self.variable1 = StringVar(master)
        self.variable1.set(self.list[0]) # default value
        self.varOpt1 = apply(OptionMenu, (listFrame1, self.variable1) + tuple(self.list) )
        self.varOpt1.pack(side=LEFT, padx=2, pady=2,fill=X)

        self.e1 = Entry(listFrame1,width=6)   
        self.e1.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e1.insert(END,self.list[0])    


        self.label= Label(master, text=self.label_cross)
        self.label.grid(row=0, column=1)
        
        listFrame2 = Frame(master)
        listFrame2.grid(row=0, column=2, sticky=NSEW)#side=TOP, padx=5, pady=5)


        self.e2 = Entry(listFrame2,width=6)   
        self.e2.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e2.insert(END,self.list[1])


        #self.variable2 = StringVar(master)
        self.variable2.set(self.list[1]) # default value
        self.varOpt2 = apply(OptionMenu, (listFrame2, self.variable2) + tuple(self.list) )
        self.varOpt2.pack(side=LEFT, padx=2, pady=2,fill=X)

        self.resizable('0','0')  # window should not be resizable!

        self.variable1.trace('w',self.changed )
        self.variable2.trace('w',self.changed )
        
        return self.e1 # initial focus
   
    def apply(self):   
        self.value0 = self.e1.get()
        self.value1 = self.e2.get()  

    #
    # construction hooks
    def changed(self,*args):
        # URL: http://mail.python.org/pipermail/tutor/2005-July/040018.html        
        self.e1.delete( 0,END )
        self.e1.insert( END,self.variable1.get() ) # das geht !!!   
        
        self.e2.delete( 0,END )
        self.e2.insert( END,self.variable2.get() ) # das geht !!!   


####################################################
class eingabeZweiDPlot(Dialog):
    # wird benutzt in plot (zweiD_Plot)
    def __init__(self,parent,_title_ = None,ausdruck = None,_label_= None,_varlist_ = [],_dici_ = {},_optionDici_ = {}): 
	 self.value = None
	 self.labelText = None
	 
         self.logX = 0 # alternativ: self.logX = 1
         self.ret_logX = None
         
         self.logY = 0 # alternativ: self.logY = 1
         self.ret_logY = None
	 

         self.noKey = 0 # alternativ: self.noKey = 1
         self.ret_noKey = None

	 
	 
         if len(_label_) > 0:             
             self.labelText = _label_
         else:
             self.labelText = "Funktion:"
	 
         self.list = _varlist_[:]
         self.argDici = {}
         self.len = len(_dici_)
         util.kopiereDici(_dici_,self.argDici)
         
         self.optionDici = {}
         util.kopiereDici(_optionDici_,self.optionDici)
         
	 self.expr = ausdruck
         self.variable = StringVar(parent)                                           
         Dialog.__init__(self,parent,_title_) 
         
    def body(self, master):  
        self.label1 = Label(master, text=self.labelText)
        self.label1.grid(row=0, column=0) 
        self.e1 = Entry(master,width=50)   
        self.e1.grid(row=0, column=1)
        self.e1.insert(END,self.expr)

        self.label2 = Label(master, text="Variable:")
        self.label2.grid(row=0, column=2) 
        listFrame = Frame(master)
        listFrame.grid(row=0, column=3, sticky=NSEW)#side=TOP, padx=5, pady=5)
	
	xindex = 0
	i = 0
	for element in self.list:
		if (element == 'x'):
			xindex = i
		i = i + 1

        self.e2 = Entry(listFrame,width=3)   
        self.e2.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e2.insert(END,self.list[xindex])
			
	
        #self.variable = StringVar(master)
        self.variable.set(self.list[xindex]) # default value
        self.varOpt = apply(OptionMenu, (listFrame, self.variable) + tuple(self.list) )
        self.varOpt.pack(side=LEFT, padx=2, pady=2,fill=X)

        

        
        for i in range(self.len):
            #print str(self.argDici[i+1])

            setattr(self, self.argDici[i+1][0], Label(master, text=self.argDici[i+1][1]))                        
            getattr(self, self.argDici[i+1][0]).grid(row=self.argDici[i+1][2], column=self.argDici[i+1][3], sticky=W)
            
            setattr(self, self.argDici[i+1][4], Entry(master,width=10))            
            getattr(self, self.argDici[i+1][4]).grid(row=self.argDici[i+1][6], column=self.argDici[i+1][7], sticky=W)
            getattr(self, self.argDici[i+1][4]).insert(END,self.argDici[i+1][5])
            
            setattr(self, 'ret'+str(i+1), '0')



        zeilen_zaehler = 4

        self.label_titel = Label(master, text="Titel:")
        self.label_titel.grid(row=zeilen_zaehler, column=0, sticky=W) 
        self.e_titel = Entry(master,width=50)   
        self.e_titel.grid(row=zeilen_zaehler, column=1, sticky=W) 
        self.e_titel.insert(END, self.optionDici["Titel"] )        

        zeilen_zaehler = zeilen_zaehler + 1
        
        self.label_achsen = Label(master, text="Achsenbeschriftung")
        self.label_achsen.grid(row=zeilen_zaehler, column=0, columnspan=2, sticky=W) 

        zeilen_zaehler = zeilen_zaehler + 1

        self.label_achseX = Label(master, text="x-Achse:")
        self.label_achseX.grid(row=zeilen_zaehler, column=0, sticky=W) 
        self.e_achseX = Entry(master,width=50)   
        self.e_achseX.grid(row=zeilen_zaehler, column=1, sticky=W) 
        self.e_achseX.insert(END,self.optionDici["X_Achse"])

        zeilen_zaehler = zeilen_zaehler + 1

        self.label_achseY = Label(master, text="y-Achse:")
        self.label_achseY.grid(row=zeilen_zaehler, column=0, sticky=W) 
        self.e_achseY = Entry(master,width=50)   
        self.e_achseY.grid(row=zeilen_zaehler, column=1, sticky=W) 
        self.e_achseY.insert(END,self.optionDici["Y_Achse"])
        
        zeilen_zaehler = zeilen_zaehler + 1
            
        self.ret_noKey = IntVar(master)
        self.ret_noKey.set(self.noKey)
        Checkbutton(master,variable=self.ret_noKey).grid(row=zeilen_zaehler, column=0, sticky=E)
        self.label_noKey = Label(master, text="ohne Anzeige des Funktionsterms")
        self.label_noKey.grid(row=zeilen_zaehler, column=1, sticky=W) 

        zeilen_zaehler = zeilen_zaehler + 1

        self.label_log = Label(master, text="Logarithmisch")
        self.label_log.grid(row=zeilen_zaehler, column=0, columnspan=2, sticky=W) 


        zeilen_zaehler = zeilen_zaehler + 1
            
        self.ret_logX = IntVar(master)
        self.ret_logX.set(self.logX)
        Checkbutton(master,variable=self.ret_logX).grid(row=zeilen_zaehler, column=0, sticky=E)
        self.label_logX = Label(master, text="x-Achse")
        self.label_logX.grid(row=zeilen_zaehler, column=1, sticky=W) 

        self.ret_logY = IntVar(master)
        self.ret_logY.set(self.logY)
        Checkbutton(master,variable=self.ret_logY).grid(row=zeilen_zaehler, column=2, sticky=E) 
        self.label_logY = Label(master, text="y-Achse")
        self.label_logY.grid(row=zeilen_zaehler, column=3, sticky=W) 

            
        self.resizable('0','0')  # window should not be resizable!

        self.variable.trace('w',self.changed )
        
        return self.e1 # initial focus
   
    def apply(self):   
        self.result = self.e1.get()
        self.value =  self.e2.get() # statt self.variable.get()
        for i in range(self.len):
            setattr(self, 'ret'+str(i+1),getattr(self, self.argDici[i+1][4]).get() )
            #print str(getattr(self, 'ret'+str(i+1) ) )        
        self._ret_noKey_ = self.ret_noKey.get()
        self._ret_logX_ = self.ret_logX.get()
        self._ret_logY_ = self.ret_logY.get()
        self.ret_Title = self.e_titel.get()
        self.ret_AchseX = self.e_achseX.get()
        self.ret_AchseY = self.e_achseY.get()
        

    #
    # construction hooks
   
    def changed(self,*args):
        # URL: http://mail.python.org/pipermail/tutor/2005-July/040018.html
        #print self.variable.get()  # das geht tatsächlich . . .
        self.e2.delete( 0,END )
        self.e2.insert( END,self.variable.get() ) # das geht !!!

############################################    
class eingabeDreiDPlot(Dialog):   
    # wird benutzt in dreiD_Plot
    def __init__(self,parent,_title_ = None,ausdruck = None,_label_= None,_varlist_ = [],_dici_ = {}, _optionList_= []): 
	 self.x_value = None
	 self.y_value = None
	 self.labelText = None
	 self.optionM = None
	 
         if len(_label_) > 0:             
             self.labelText = _label_
         else:
             self.labelText = "Funktion:"
	 
         self.list = _varlist_[:]
         self.argDici = {}
         self.len = len(_dici_)
         util.kopiereDici(_dici_,self.argDici)
	 self.expr = ausdruck
         self.variableX = StringVar(parent)
         self.variableY = StringVar(parent)
         
         self.optionVar = StringVar(parent)
         
         self.optionList=_optionList_[:]         
         
         Dialog.__init__(self,parent,_title_)
         
    def body(self, master):  
        self.label1 = Label(master, text=self.labelText)
        self.label1.grid(row=0, column=0) 
        self.e1 = Entry(master,width=50)   
        self.e1.grid(row=0, column=1, columnspan=3 ) # columnspan=2
        self.e1.insert(END,self.expr)

        self.label2 = Label(master, text="x-Variable:")
        self.label2.grid(row=1, column=0, sticky=E) 
        listFrameX = Frame(master)
        listFrameX.grid(row=1, column=1, sticky=NSEW)#side=TOP, padx=5, pady=5)

        self.label3 = Label(master, text="y-Variable:")
        self.label3.grid(row=1, column=2, sticky=E) 
        listFrameY = Frame(master)
        listFrameY.grid(row=1, column=3, sticky=NSEW)#side=TOP, padx=5, pady=5)
	
	
	xindex = 0
	i = 0
	for element in self.list:
		if (element == 'x'):
			xindex = i
		i = i +1

	yindex = 0
	i = 0
	for element in self.list:
		if (element == 'y'):
			yindex = i
		i = i +1


        self.e2 = Entry(listFrameX,width=3)   
        self.e2.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e2.insert(END,self.list[xindex])
			
        self.e3 = Entry(listFrameY,width=3)   
        self.e3.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e3.insert(END,self.list[yindex])

	
        #self.variableX = StringVar(master)
        self.variableX.set(self.list[xindex]) # default value
        self.varOptX = apply(OptionMenu, (listFrameX, self.variableX) + tuple(self.list) )
        self.varOptX.pack(side=LEFT, padx=2, pady=2,fill=X)

        #self.variableY = StringVar(master)
        self.variableY.set(self.list[yindex]) # default value
        self.varOptY = apply(OptionMenu, (listFrameY, self.variableY) + tuple(self.list) )
        self.varOptY.pack(side=LEFT, padx=2, pady=2,fill=X)
        
        
        for i in range(self.len):
            #print str(self.argDici[i+1])

            setattr(self, self.argDici[i+1][0], Label(master, text=self.argDici[i+1][1]))                        
            getattr(self, self.argDici[i+1][0]).grid(row=self.argDici[i+1][2], column=self.argDici[i+1][3], sticky=E)
            
            setattr(self, self.argDici[i+1][4], Entry(master,width=10))            
            getattr(self, self.argDici[i+1][4]).grid(row=self.argDici[i+1][6], column=self.argDici[i+1][7], sticky=W)
            getattr(self, self.argDici[i+1][4]).insert(END,self.argDici[i+1][5])
            
            setattr(self, 'ret'+str(i+1), '0')


        self.label4 = Label(master, text="Optionen:")
        self.label4.grid(row=4, column=0, sticky=E) 
        listFrameOpt = Frame(master)
        listFrameOpt.grid(row=4, column=1,columnspan=3 )# sticky=NSEW)#side=TOP, padx=5, pady=5)
        
        self.e4 = Entry(listFrameOpt,width=14)   
        self.e4.pack(side=LEFT, padx=2, pady=2,fill=X)
        self.e4.insert(END,"default     ")

        #self.optionVar = StringVar(master)
        self.optionVar.set(self.optionList[0]) # default value
        self.optionMenuVar = apply(OptionMenu, (listFrameOpt, self.optionVar) + tuple(self.optionList) )
        self.optionMenuVar.pack(side=LEFT, padx=2, pady=2,fill=X)

            
        self.resizable('0','0')  # window should not be resizable!

        self.variableX.trace('w',self.changed )
        self.variableY.trace('w',self.changed )
        self.optionVar.trace('w',self.changed )
        
        return self.e1 # initial focus
   
    def apply(self):   
        self.result = self.e1.get()
        for i in range(self.len):
            setattr(self, 'ret'+str(i+1),getattr(self, self.argDici[i+1][4]).get() )
            #print str(getattr(self, 'ret'+str(i+1) ) )
        self.x_value = self.e2.get()  # statt self.variable.get()
        self.y_value = self.e3.get()  # statt self.e2.get()
        self.optionM = self.e4.get()


    #
    # construction hooks
   
    def changed(self,*args):
        # URL: http://mail.python.org/pipermail/tutor/2005-July/040018.html
        #print self.variable.get()  # das geht tatsächlich . . .
        self.e2.delete( 0,END )
        self.e2.insert( END,self.variableX.get() ) # das geht !!!
        self.e3.delete( 0,END )
        self.e3.insert( END,self.variableY.get() ) # 
        self.e4.delete( 0,END )
        self.e4.insert( END,self.optionVar.get() ) # 
        
###############################################
class kettenbruch(Dialog):      

    def __init__(self,parent,_title_ = None):
         self.wert=3.141592653589793  # 3.1416
         self.pp = None
         self.korrektur_default = 1
         self._title_ = _title_

         Dialog.__init__(self,parent,_title_) 
             
    
    def body(self, master):      
        self.title( self._title_ )

        if os.name == 'posix': 
            self.font_size = 12
            self.breite = 14
            self.font_type = 'fixed'
        else:     
            self.font_size = 10
            self.breite = 13
            self.font_type = 'ansifixed'
            
        self.korrektur = IntVar(master)
        
        Label(master, text= "Wert:        ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=0)   
        Label(master, text= "pretty print ",width=self.breite,justify=LEFT,font=(self.font_type,self.font_size,'normal'),anchor=W).grid(row=1)            
                
        self.e1 = Entry(master,width=30,font=(self.font_type,self.font_size,'normal') )   
        self.e2 = Checkbutton(master,variable=self.korrektur)        
        
        self.e1.insert(END, str(self.wert) )         
        self.korrektur.set(self.korrektur_default)        
 
        self.e1.grid(row=0, column=1)   
        self.e2.grid(row=1, column=1, sticky=W )        

        return self.e1 # initial focus   
                               
         
    def apply(self):   
        self.wert = self.e1.get() # wert
        self.pp = self.korrektur.get()
        self.result = 1

###############################################

    
###################################################
def get_Max(kopf,query,j):
   """Bestimmt die Breite (in Zeichen) der j Spalte"""
   import string
   max_length = 5 ## Anfangswert!
   laenge = len( str(kopf) ) + 4 ## kopf wird fett gedruckt => + 5!
   if laenge > max_length:
       max_length = laenge       
   for element in query:
       if string.find(str(element[j]),'\r') != -1: 
           ## '\r' gefunden: String mit new Line . . . 
           zeile = string.split(str(element[j]),'\r')
           for wort in zeile:
               laenge = len( wort )
               if laenge > max_length:
                   max_length = laenge
       else:              
           laenge = len( str(element[j]) )
       if laenge > max_length:
           max_length = laenge
   return (max_length + 1)              


def geoTable(w,  table_list=[], heading_list=[],cbFlag=1,_cancelButton_=1):
    """Erzeugt eine Tkinter-Tabelle aus heading_list + table_list"""   
    # IN:  w, query_list, heading_list
    # OUT: Table for heading_list + table_list using simpleTable
    if (len(heading_list)== 0) or (len(table_list)==0):
           return  []
    header_liste=[]    
    spalten = len(heading_list)
    liste = bearbeiteListe(table_list)
    #liste = table_list
    #print liste
    ## build header_liste, Format: ( entry, width )
    for i in range(spalten):
        element = str(heading_list[i])
        ## get the max width of the i'th column      
        br = get_Max(element,liste,i)
        br = br - 3  # neu in Vers. 0.11
        header_liste.append( (str(element), br )  )

    #################################################
    #table = tkWidgets.simpleTable(w,header_liste,liste)
    
    table = simpleTable(w,header_liste,liste,cbFlag,_cancelButton_)      
        
    if table:
	    return table.ergebnis
    else:
	    #print str("War wohl nix")
	    return []
#####################################################




def formatReal(strZahl): 
	"""
	wandelt einen Float-String mit mehr als 3 Nachkommastellen in 
	einen Float-String mit max. 2 Nachkommastellen um
	"""
	_debug_ = False
	if ("." in strZahl): # Kommazahl?
		strList = strZahl.split('.')
		if _debug_:
			print strList[1]
		if len(strList[1]) > 2:  # mehr als 2 Nachkommastellen
			try:
				fZahl = float(strZahl) # konvertieren
				neu = "%.2f" % fZahl # formatieren
				return neu # fertig
			except:
				return strZahl			
		else:
			return strZahl			
	else:
		return strZahl

def bearbeitePunkt(_punkt_):
	# IN: '([2.123,0.123,0.123])' 
	# OUT: '([2.12,0.12,0.12])' 
	punktLi = _punkt_.split(',')
	neuLi = []
	for i in range(3):		
		neueKoord = punktLi[i].lstrip()  # Leerzeichen rauswerfen
		if i == 0:
			neuLi.append(formatReal( neueKoord[2:] ) )  # '([' rauswerfen
		if i == 1:
			neuLi.append(formatReal( neueKoord) )
		if i == 2:
			neuLi.append(formatReal( neueKoord[:-2] ) )  # '])' rauswerfen
			
	# Jetzt Punkt neu aufbauen:
	retPunkt = '(['
	i = 0
	for element in  neuLi:
		if i ==2:
			retPunkt = retPunkt + element 
		else:
			retPunkt = retPunkt + element + ','
		i = i + 1
	retPunkt = retPunkt + '])'
	return retPunkt
		
def bearbeiteListe(_liste_):
	_neu_ = []
	for element in _liste_:
		neuesElement = []
		for i in range(len(element)):
			if i==3:  # Objekt-Liste, jetzt Punkte finden
				neuerKoordStr=''
				koordStr = element[3]
				koordStrLen = len(koordStr)
				#print koordStrLen
				j = 0
				while j < koordStrLen:
					#print "j: ",j
					if ((koordStr[j] == '(') and (koordStr[j+1] == '[')):
						k = j + 2
						while (k < koordStrLen):		
							#print "k: ",k
							if ((koordStr[k] == ']') and (koordStr[k+1] == ')')):
								_punkt_ = koordStr[j:(k+2)]
								#print _punkt_
								j = j + len( _punkt_ )								
								if len( _punkt_ ) >= 9:
									try:
										neuerPunkt = bearbeitePunkt(_punkt_)	
										neuerKoordStr = neuerKoordStr + neuerPunkt
									except:
										neuerKoordStr = neuerKoordStr + _punkt_
									k = koordStrLen
								else:
									k = koordStrLen
							else:
								k = k + 1
							
					else:
						neuerKoordStr = neuerKoordStr +  koordStr[j]
						j = j + 1
						#print j
					continue
				#neuesElement.append(neuerPunkt)			
				neuesElement.append(neuerKoordStr)				
			else:
				neuesElement.append(element[i])			
				
		_neu_.append(neuesElement)
	return _neu_

##########################################################
