#!/usr/bin/python
# -*- coding: utf-8 -*-
#
"""#-*- coding: cp1252 -*- """
"""
        pyMaxima.py, Version 1.80, 2016-07-13
        Walter Spiegel (walter.spiegel@web.de)
        
        pyMaxima requires:

        - Maxima, a computer algebra system
          Maxima was started by William Schelter,
          the late maintainer of Maxima
          see: http://maxima.sourceforge.net/
          Note: without Maxima, pyMaxima won't work!

        - Python 2.4 or newer
          see: http://www.python.org/download/

        - otherwise the subprocess module is necessary  
          see: http://www.lysator.liu.se/~astrand/popen5/
          Download: http://effbot.org/downloads/#subprocess
           
        - on win32/NT: Python Extensions for Windows
          see: http://sourceforge.net/projects/pywin32/


        PyGeo
        "PyGeo is a framework for the creation of dynamic geometric constructions", 
        PyGeo's creator was Arthur Siegel, see http://pygeo.sourceforge.net/


        pyMaxima uses pyGeo for the 3D-representation of the geometry objects
        as lines, planes or spheres. pyMaxima will work without pyGeo, but you
        won't see the 3D-representation of the geometry objects

        You have to install the following package, if you are using Python 2.5 to 2.7:
        http://www.wspiegel.de/pymaxima/archiv/PyGeo-1.1a_ws.tar.gz or
        http://www.wspiegel.de/pymaxima/archiv/PyGeo-1.1a_ws.zip
         
        Note: pyGeo is using additional resources: 
        VPython, see: http://www.vpython.org/ 

        Numerical Python (Numerical Python is bundled with VPython)
        see: http://sourceforge.net/projects/numpy


        Install pyMaxima

        - extract pyMaxima.zip or pyMaxima.tar.gz

        - Linux: fix the entries in <linuxInit.conf>
          note that the file "interface.lisp"
          is in the same directory as pyMaxima.py
          - call "python pyMaxima.py"
          - go Datei => Textdatei laden & ausführen
            load <example.txt>


        - win32/NT: fix the entries in <winInit.conf>
          note that the file "interface.lisp"
          is in the same directory as pyMaxima.py
          - click on pyMaxima.py 
            go Datei => Textdatei laden & ausführen
            load <example.txt>

        
        
        TO DO:
        =====
        - die beiden Methoden kebene und nebene zusammenfassen => Vers. 1.66, eventuell: Koordinatenform von Normalenform unterscheiden
        
        - Debug => History-Liste (muehsam, Problem: Ausführung von example( maxima-Befehl )
          bringt die Zeilen-Zählung durcheinander, da die per example ausgeführten Befehle
          in der Zählung nicht berücksichtigt werden (können) => dann stimmt die History-Liste
          nicht mehr mit der Zählung im Fenster überein => der Befehl wird dann nicht mehr
          gefunden . . . Einzige Lösung: Der Button >Clear all< ) => Vers. 0.78

        - Debug => plot-Test (Testen!) => Vers. 0.77, Vers. 0.79          
        
        - Tests (pyMaxima testen!)                        

        - English - Interface => mit gettext => http://docs.python.org/lib/module-gettext.html
        

        ##################################################
        Version 1.56-1.60: Maxima Package draw, erst ab Maxima 5.14!

        - self.plot => Version 1.56, 2010-04-01

        - self.dreiDPlot  => Version 1.57, 2010-04-02

        - Zeichnen der Normalverteilung !! => Version 1.58, 2010-04-07

        - Zeichnen der Binomialverteilung/Guetefunktion/Operationscharakteristik => Version 1.59, 2010-04-08

        - Funktionenschar zeichnen => Version 1.60, 2010-04-08

        - Graph zeichnen bei Kurvendiskussion => Version 1.60, 2010-04-08                       
        
        - => was fehlt: parametrische Funktionen (2d,3d)
        # # # # #  # # # # # # # 
        Package draw  => URL: http://www.telefonica.net/web2/biomates/maxima/gpdraw/index.html
        You need at least Gnuplot 4.2 (better 4.2.2) 
        => gnuplot> show version
        and Maxima 5.14 to run this package.
        Type load(draw); to make use of it.
        Dokumentation: http://maxima.sourceforge.net/docs/manual/en/maxima_48.html
        ####################################################
        Version 1.62: Umstellung der pyMaxima-Hilfe auf Wiki on a Stick (WoaS) => http://stickwiki.sourceforge.net/
             
"""
################################################
# globale Variablen setzen, Teil 1

__version__ = '1.80'  # pyMaxima-Version

pgdate = '2016-07-13'
pgname = 'pyMaxima'  

_delay_time_ = 0.2

import os

# fuer ppp/python24-Version!
_PPP_ = 0	# _PPP_- Flag: steuert die Ausführung mit dem ppp-Paket (PPP: perl, php, python)
#_winPyPPP_ = "C:" + os.sep + "ppp" + os.sep +  "py24" + os.sep + 'python.exe '
_winPyPPP_ = "L:" + os.sep + "Programme" + os.sep +  "python24" + os.sep + 'python.exe '


pyMaximaHome = os.getcwd() + os.sep
#maxima_pfad = pyMaximaHome
pfad_zu_maxima = 'G:\\Programme\\Maxima-5.11.0\\bin\\' # anpassen!!
if os.name == 'posix':
    pfad_zu_maximaDoc = '/home/walter/py_prg/pyMaxima/maxima_doc/' # anpassen!!
    pfad_zu_python = 'python '
else:
    pfad_zu_maximaDoc = 'G:\\Programme\\Maxima-5.11.0\\bin\\maxima_doc\\' # anpassen!!
    pfad_zu_python = 'C:\\Python23\\python.exe '
    if _PPP_:
	# für ppp folgende Zeile
        pfad_zu_python = _winPyPPP_  	

maximaDocHome =  os.path.join(pfad_zu_maximaDoc, 'maxima.html')


font_size_para = 10
nr_lines_main = 27 # unter Linux, win32 => nr_lines_main = 30, netbook: nr_lines_main = 24 oder 25
# >>> der folgende Parameter wird nicht mehr benutzt <<<
font_size_help = 10


###########################################################
# Import
import string 
from os import listdir, getcwd, mkdir, sep, system, chdir
from os.path import abspath,exists
from shutil import copyfile, copystat
from sys import exit
import cPickle
import subprocess
import errno
import time, math
import webbrowser
import sys,time,traceback,os,re
import Tkinter
if os.name == 'posix':
    Tkinter.wantobjects = 0
    # tkMessageBox.askyesno wrong result under Linux!
    # see http://mail.python.org/pipermail/python-bugs-list/2004-September/025172.html

from Tkinter import *  
import tkMessageBox 
from tkFileDialog import *
# try: help(tkFileDialog)
from ScrolledText import *   
import fileinput
pyMaximaInit = os.sep + 'init'
_initdir_ = os.getcwd() + pyMaximaInit
pyMaximaModule = os.sep + 'maxmod'
_subdir_ = os.getcwd() + pyMaximaModule
#print _subdir_
pygeoScripts = os.sep + 'pygeo'
_pygeodir_ = os.getcwd() + pygeoScripts
sys.path.append( os.path.abspath (os.getcwd()) )
sys.path.append( os.path.abspath ( _initdir_ ) )
sys.path.append( os.path.abspath ( _subdir_  ) )
sys.path.append( os.path.abspath ( _pygeodir_ ) )
import pymaxUtil as util
import html_tools as hts
import tkWidgets as tkw
import funcInter as maxfunc
import pymax2geo
import tkParameter, miniEditor, maxinter, tkMatrix, descartes, stochastik
import ToolTip # from IDLE
import codecs
import fileList

################################################

def showError(titel,msg):
    """show Error-Message msg"""
    main = Tk() 
    main.withdraw() 
    tkMessageBox.showerror(titel,msg) 
    main.quit()

try:
    import Popen
except:
    showError(">>> Import Error <<<",  
    "Import of Module <Popen> failed!")

# _pygeoFlag_: Flag fuer pyGeo-Import    
_pygeoFlag_ = 1
################################################
# Ini-Datei
# speichert Einstellungen
ini_dici = {}
ini_dici['delay_time'] = '0.2'
ini_dici['maxima_path'] = 'G:\\Programme\\Maxima-5.10.0\\bin\\'
if os.name == 'posix':
    ini_datei = 'linuxInit.conf'
    ini_dici['docu_path'] = '/home/walter/py_prg/pyMaxima/maxima_doc/' + 'maxima.html'
    ini_dici['py_path'] = 'python '
    ini_dici['font_size_para'] = 10           
    ini_dici['nr_lines_main'] = 27
else: # win nt
    ini_datei = 'winInit.conf'
    ini_dici['docu_path'] = 'G:\\Programme\\Maxima-5.10.0\\bin\\maxima_doc\\' + 'maxima.html'
    ini_dici['py_path'] = 'C:\\Python24\\python.exe '
    ini_dici['font_size_para'] = 10    
    ini_dici['nr_lines_main'] = 30  # Netbook: 24 oder 25
    
################################################
# globale Variablen setzen, Teil 2
_lw_arg_ = None
maxima = None
root = None #Tk()   
t2=None   
eingabe=''

pfad='' 

hilfetext=''   
#helpdatei = 'pyreadme.txt'
helpdatei = 'readme.txt'
funcDefDatei = 'func_liste.txt'

pfad = './'
#_test_ = True
_test_ = False

## für DirListe
prevUsed=fileList.DirList()
dbList_dici = {}
dbListFile = 'maxEntries.ini'
################################################
def einstellungenLaden():
    """load configuration file 'linuxInit.conf' or 'winInit.conf' """    
    global ini_dici, _delay_time_ , pfad_zu_maxima , pfad_zu_maximaDoc, maximaDocHome, pfad_zu_python, nr_lines_main
    myDici = {}
    #print pyMaximaHome
    #print _initdir_
    if os.name == 'posix':
        util.leseConfi(_initdir_,'linuxInit.conf',myDici)
    else:
        util.leseConfi(_initdir_,'winInit.conf',myDici)
    util.kopiereDici(myDici,ini_dici)
    
    #util.druckeDici(myDici)
    #util.druckeDici(ini_dici)
    
    try:
        _delay_time_ = float( myDici['delay_time'] )
    except:
        _delay_time_ = 0.2
    pfad_zu_maxima = myDici['maxima_path'] # + os.sep
    #print pfad_zu_maxima
    pfad_zu_maximaDoc = myDici['docu_path']   
    maximaDocHome =  os.path.join(pfad_zu_maximaDoc, 'maxima.html')
    pfad_zu_python = myDici['py_path']
    #print pfad_zu_python
    
    if os.name <> 'posix':
        if _lw_arg_:
            #print _lw_arg_
            pfad_zu_python = util.changeLwArg(_lw_arg_,pfad_zu_python)            
            maximaDocHome = util.changeLwArg(_lw_arg_,maximaDocHome)
            pfad_zu_maxima = util.changeLwArg(_lw_arg_,pfad_zu_maxima)
            #print pfad_zu_python,pfad_zu_maxima,maximaDocHome
        
    print pfad_zu_python
    
    font_size_para = int(myDici['font_size_para'])  # 10    
    nr_lines_main = int(myDici['nr_lines_main'])  # win nt: 30, linux: 27, netbook: 24 oder 25
    #print nr_lines_main

    #util.druckeDici(ini_dici)                

def die():
    """quit pyMaxima"""    
    if tkMessageBox.askyesno("Beenden?",pgname + " beenden?"):
        die_()


def die_():              
    try:
        maxima.endMaxima()
        time.sleep(0.2)
    except:
        pass
    dbList_speichern() 
    root.destroy()   
    sys.exit(0) 
      
       
   
def about():   
   tkMessageBox.showinfo("Info",  
     "The Maxima computer algebra system\n" +
     "was started by William Schelter, see\n" +
     "http://maxima.sourceforge.net/\n" +
     "PyGeo's creator was Arthur Siegel,\n" +
     "see http://pygeo.sourceforge.net/\n" +
     pgname + " by Walter Spiegel\n" + 
     "Version " + __version__ + ", " + pgdate)



def datei_schreiben (home,datei,text):
    """Schreibt 'text' in eine Datei
       ACHTUNG:  Die Datei wird ohne Rückfrage  überschrieben!
    """
    try:
        out_file = open( home+ os.sep + datei,"w")  # Datei öffnen
    except IOError: # EA-Fehler
        tkMessageBox.showerror("Datei-Fehler!", 'Konnte Datei \n<' + home+ os.sep + datei + '> \nnicht öffnen!')
        return 0
    else:
        out_file.write(text) # text in Datei schreiben
        out_file.close()     # Datei schliessen
        return 1
################################################
def init_dbDici(max):
    """Datenstruktur dbList_dici initialisieren"""        
    global dbList_dici
    for i in range(max):
        dbList_dici[i] = ''
    
    
def dbList_speichern():
    """DB-Liste speichern in der Datei settings.get('dblist_file')"""        
    global dbList_dici
    laenge = prevUsed.listLength()    
    init_dbDici(prevUsed.DirMax)   ## wichtig!!
    for i in range(laenge):
        dbList_dici[i] = prevUsed.DirListe[i]
        ##alternativ:
        ##dbList_dici.update({i:prevUsed.DirListe[i]})        
    try:
        dbList_datei = open(_initdir_ + os.sep + dbListFile ,'wb')
        cPickle.dump(dbList_dici,dbList_datei)
        dbList_datei.close()
    except:
        tkMessageBox.showerror('FeHlEr!', 'Speichern der ini-Datei fehlgeschlagen!' )             

def dbDici2prevUsed(max):
    """Wandelt das dbUsed_Dicitionary in eine prevUsed-Liste"""            
    tempUsed=fileList.DirList()
    tempListe = []
    for i in range(max):
        tempListe.append(dbList_dici.get(i))
    tempListe.reverse()
    for element in tempListe:
        if element <> '':
           tempUsed.addEntry(element)
    return tempUsed       

def dbList_holen(max,dbList_file=''):
    """Liest die DB-Liste aus der Datei settings.get('dblist_file')"""        
    global dbList_dici, prevUsed
    
    try:
        if dbList_file<>'':
            dbList_datei = open(dbList_file,'rb')
        else:    
            dbList_datei = open(_initdir_ + os.sep + dbListFile,'rb')
            dbList_dici = cPickle.load(dbList_datei)
        dbList_datei.close()
        prevUsed = dbDici2prevUsed(max)
    except:     ## Probleme beim Lesen, die dblist_file wird neu initialisiert
        init_dbDici(max)
        dbList_datei = open(_initdir_ + os.sep + dbListFile,'wb')
        cPickle.dump(dbList_dici,dbList_datei)
        dbList_datei.close()
        prevUsed = dbDici2prevUsed(max)        
        ##tkMessageBox.showerror('FeHlEr!', 'Lesen der ini-Datei fehlgeschlagen!' )             

################################################

##################################################

   
def ausgabe(text,fenster='t2'):   
    """Aufruf: ausgabe(text) bzw. ausgabe(text,t1)
       gibt eine Meldung im Fenster t2 aus 
    """
    global t2
    if fenster == 't2':           
        t2.insert(END, '\n' + text )   
        t2.yview(END)   
        t2.insert(END,'\n')


def setStatus(maximaOk):
    if os.name == 'posix':  
        if maximaOk:
            newStatus = "Satus:   " + '              >>> Verbunden mit Maxima! <<< '
        else:
            newStatus = "Satus:   " + '              <<< !!!NICHT VERBUNDEN!!! >>> '
    else: # win32
        if maximaOk:            
            newStatus = "Satus:   " + '              >>> Verbunden mit Maxima @ '+  pfad_zu_maxima  +  ' <<< '
        else:
            newStatus = "Satus:   " + '              <<< !!!NICHT VERBUNDEN!!! >>> '
    return newStatus


################################################


class oldPlot:
    """speichert alte 2D-Plot-Daten"""
    def __init__(self,ausdruck,value,xmin,xmax,ymin,ymax):
        self.ausdruck = ausdruck
        self.value = value
        self.xmin = xmin
        self.xmax = xmax
        self.ymin = ymin
        self.ymax = ymax

    def getPlot(self):
        """ """        
        return (self.ausdruck,self.value,self.xmin,self.xmax,self.ymin,self.ymax)

        
    def setPlot(self,ausdruck,value,xmin,xmax,ymin,ymax):
        self.ausdruck = ausdruck
        self.value = value
        self.xmin = xmin
        self.xmax = xmax
        self.ymin = ymin
        self.ymax = ymax


class oldFunkScharPlot(oldPlot):
    """speichert alte Plot-Daten ==>> Version fuer Funktionenschar"""
    def __init__(self,ausdruck,value,xmin,xmax,ymin,ymax,k_von=None,k_bis=None,schrittweite=None, parameter=None):
        oldPlot.__init__(self,ausdruck,value,xmin,xmax,ymin,ymax)
        self.k_von = k_von        
        self.k_bis = k_bis
        self.schrittweite = schrittweite
        self.parameter = parameter

    def getPlot(self):
        """mit self.k_von / self.k_bis / self.schrittweite"""
        return (self.ausdruck,self.value,self.xmin,self.xmax,self.ymin,self.ymax,self.k_von,self.k_bis, self.schrittweite, self.parameter)

        
    def setPlot(self,ausdruck,value,xmin,xmax,ymin,ymax,k_von=None,k_bis=None,schrittweite=None, parameter=None):
        oldPlot.setPlot(self,ausdruck,value,xmin,xmax,ymin,ymax)
        
        self.k_von = k_von        
        self.k_bis = k_bis
        self.schrittweite = schrittweite
        self.parameter = parameter

class dreiD_Plot(oldPlot):
    """speichert alte 3D-Plot-Daten, abgeleitet aus oldPlot"""
    def __init__(self,ausdruck,value,xmin,xmax,ymin,ymax,x_value=None,y_value=None):
        oldPlot.__init__(self,ausdruck,value,xmin,xmax,ymin,ymax)
        self.x_value = x_value        
        self.y_value = y_value

    def getPlot(self):
        """  """
        return (self.ausdruck,self.value,self.xmin,self.xmax,self.ymin,self.ymax,self.x_value,self.y_value)

        
    def setPlot(self,ausdruck,value,xmin,xmax,ymin,ymax,x_value=None,y_value=None):
        oldPlot.setPlot(self,ausdruck,value,xmin,xmax,ymin,ymax)
        self.x_value = x_value
        self.y_value = y_value
    
class zweiD_Plot(oldPlot):
    """speichert alte 2D-Plot-Daten, abgeleitet aus oldPlot"""
    def __init__(self,ausdruck,value,xmin,xmax,ymin,ymax,_title_=None,x_achse=None,y_achse=None):
        oldPlot.__init__(self,ausdruck,value,xmin,xmax,ymin,ymax)
        self.title = _title_
        self.x_achse = x_achse        
        self.y_achse = y_achse

    def getPlot(self):
        """  """
        return (self.ausdruck,self.value,self.xmin,self.xmax,self.ymin,self.ymax,self.title,self.x_achse,self.y_achse)

        
    def setPlot(self,ausdruck,value,xmin,xmax,ymin,ymax,_title_=None,x_achse=None,y_achse=None):
        oldPlot.setPlot(self,ausdruck,value,xmin,xmax,ymin,ymax)
        self.title = _title_
        self.x_achse = x_achse        
        self.y_achse = y_achse
            
################################################        
## font-type and font-size, font_size_main is global!!!
## see also tkSimpleText.py
font_size_main = 10

## use: font=gf_font,        
gf_font_art='normal'
gf_font_art2 = 'bold'
gf_font_type='courier'
gf_font_size= font_size_main  # global
gf_font_type2=gf_font_type

gf_font=(gf_font_type,gf_font_size,gf_font_art)  # normal
## gf_font2 for parameter-window
gf_font2=(gf_font_type,gf_font_size,gf_font_art2) # bold

#gf_font3=(gf_font_type,12,gf_font_art)
####################################################              
    
class MaximaMain:
    """pyMaxima-Fenster"""
    def __init__(self, parent):
        global t2, pgname, exprDici, maxima, _test_, _pygeoFlag_, pfad_zu_python, nr_lines_main
        #################
        if os.name <> 'posix':
            pfadLi =  os.path.split( pfad_zu_maxima )
            ver = string.split(pfadLi[1],"-") # ver[1] = '5.10.0'
            self.maxima_ver = ver[1]
            util.build_maxima_bat(self.maxima_ver , pfad_zu_maxima)
        #################
        self.t = parent #
        self.t.title(pgname+',  '+__version__ )    #
        self.t.withdraw()
        ##############
        self.exprDici = {}
        self.nr = 1
        self.altIn = '(%i1)'
        self.oldExpr = ''           # wg. Markierung und exprDici
        self.neuExpr = ''           # wg. Markierung und exprDici        
        self.tauscheAusdruck = 0    # wg. Markierung und exprDici        
        self.pointer = 0
        self._initdir_ = _initdir_ + os.sep
        self.home = pyMaximaHome
        self.workingDir = pyMaximaHome + os.sep + 'beispiele' # fuer loadExprTxtFileAndExec
        self.break_excute_session_file = 0
        
        
        self.maximaDocHome = None        
        self.maximaOk = 0
        self.pygeoFlag = _pygeoFlag_
        self.lispImplementationFlag = "UNKNOWN" # "GCL", "CLISP", "UNKNOWN"
        self.buildInfoMaximaVersion = None
        self.moduleDrawLoaded = 0  # Maxima Modul draw ab maxima-Version 5.14!
        # Standard-Funktionen (fuer plot-Test)
        self.standard = []
        self.getStandardFkt()
        #tkMessageBox.showinfo('Standard-Funktionen',str(self.standard))            
        self.oldPlot = oldPlot('','x','-3','3','-10','10') # ausdruck,value,xmin,xmax,ymin,ymax
        self.oldFunkScharPlot = oldFunkScharPlot('','x','-3','3','-10','10','1','2','1','k')
        self.dreiD_Plot = dreiD_Plot('','x','-5','5','-5','5','x','y') # ausdruck,value,xmin,xmax,ymin,ymax,x_value,y_value
        self.zweiD_Plot = zweiD_Plot('','x','-3','3','-10','10','','','') # ausdruck,value,xmin,xmax,ymin,ymax,title,x-Achse,y-Achse
        
        # ausdruck,value,xmin,xmax,ymin,ymax,k_von,k_bis,schrittweite,parameter
        self.plotten = 1
        if os.name == 'posix': # Argumente fuer gnuplot: linetype
            self.linetype = "set zeroaxis linetype -1; "
        else:
            self.linetype = "set zeroaxis linetype -1; "    # unter win32 geht auch linetype 8

        self.numListe = ['0','1','2','3','4','5','6','7','8','9','_']
        self.script = []
        self.geoOops = pymax2geo.geoObjects()
        maxima = maxinter.MaximaInterface( delay_time = _delay_time_, _pfad_zu_maxima_ = util.protectBlank( self._initdir_ ) ) # alt: pfad_zu_maxima
        self.maximaOk = 0
        self.maxTry = 2 # maximale Versuche fuer Connect

        self.eingabe = Tkinter.StringVar()

        self._debug_ = 0
                
        try:
            import visual
            import pygeo
            _pygeoFlag_ = 1                 
        except:
            _pygeoFlag_ = 0
            #_pygeoFlag_ = 1
        self.pygeoFlag = _pygeoFlag_
        # jetzt Menue self.pygeom aktualisieren! 

        
        self.bar = Menu(self.t)   
   
        self.filem = Menu(self.bar)   


        self.dbUsedMenu = Menu(self.bar,tearoff=0)    
        laenge = prevUsed.listLength()
        for i in range(laenge):
            argument =  prevUsed.DirListe[i]
            self.dbUsedMenu.add_command(label=argument, command=(lambda arg=argument : self.dbUsed(arg) ) )


        self.editUsedMenu = Menu(self.bar,tearoff=0)    
        laenge = prevUsed.listLength()
        for i in range(laenge):
            argument =  prevUsed.DirListe[i]
            self.editUsedMenu.add_command(label=argument, command=(lambda arg=argument : self.openWithEditor(arg) ) )


        self.filem.add_command(label="mini-Editor starten",command=self.myEditor)                
        self.filem.add_separator()
        
        self.filem.add_command(label="Export2HTML",command=self.toHTML)
        self.filem.add_separator()
        self.filem.add_command(label="Export2TEX",command=self.toTEX)
        self.filem.add_separator()                
        self.filem.add_command(label="Export2TXT",command=self.toTXT)
        self.filem.add_separator()
        self.filem.add_command(label="Export2wxMaxima",command=self.toWxMaxima)
        self.filem.add_separator()        
        self.filem.add_command(label="wxMaxima-Batchdatei", command=(lambda arg='wxMaxima=1' : self.saveExprDici2txt(arg) ) )        
        self.filem.add_separator()


        
        self.filem.add_cascade(label="Sitzungsdatei ausführen", menu=self.dbUsedMenu)          
        #self.filem.add_command(label="Sitzungsdatei ausführen", command=self.loadExprTxtFileAndExec)    
        self.filem.add_separator()
        self.filem.add_command(label="Sitzungsdatei ausführen (TEX-Mode)", command=(lambda arg='TEX_Mode=1' : self.loadExprTxtFileAndExec(arg) ))    
        self.filem.add_separator()
        self.filem.add_command(label="pp Sitzungsdatei (TEX)", command=self.loadExprTxtFileTEX_PP)    
        self.filem.add_separator()
        self.filem.add_cascade(label="Sitzungsdatei editieren", menu=self.editUsedMenu)          
        self.filem.add_separator()

        self.filem.add_command(label="Sitzung sichern", command=self.saveExprDici2txt)
        self.filem.add_separator()                 
        self.filem.add_command(label="Beenden", command=self.beenden)   


        self.fktMenu = Menu(self.bar,tearoff=0)
        self.fktMenu.add_command(label="Funktionsuntersuchung",command=self.kurvendisk )
        self.fktMenu.add_separator()
        self.fktMenu.add_command(label="Graph zeichnen",command=self.plot )
        self.fktMenu.add_separator()
        self.fktMenu.add_command(label="Graph speichern",command=(lambda arg=1 : self.plot(arg) )  )
        self.fktMenu.add_separator()
        

        self.fktScharMenu = Menu(self.bar,tearoff=0)
        self.fktScharMenu.add_command(label="Funktionenschar untersuchen",command=self.funkSchar )
        self.fktScharMenu.add_separator()
        self.fktScharMenu.add_command(label="Funktionenschar zeichnen",command=(lambda arg=1 : self.funkSchar(arg) )  )
        self.fktScharMenu.add_separator()
        self.fktScharMenu.add_command(label="Funktionenschar-Plot speichern",command=(lambda arg=2 : self.funkSchar(arg) )  )
        self.fktScharMenu.add_separator()


        self.dreiDPlotMenu = Menu(self.bar,tearoff=0)
        self.dreiDPlotMenu.add_command(label="Graph zeichnen",command=self.dreiDPlot )
        self.dreiDPlotMenu.add_separator()
        self.dreiDPlotMenu.add_command(label="Graph speichern",command=(lambda arg=1 : self.dreiDPlot(arg) )  )
        self.dreiDPlotMenu.add_separator()



        self.dglMenu = Menu(self.bar,tearoff=0)
        self.dglMenu.add_command(label="DGL 1./2. Ordnung lösen",command=self.solveDGL )
        self.dglMenu.add_separator()
        self.dglMenu.add_command(label="AWP bei DGL 1. Ordnung",command=self.dglAWP1 )
        self.dglMenu.add_separator()
        self.dglMenu.add_command(label="AWP bei DGL 2. Ordnung",command=self.dglAWP2 )
        self.dglMenu.add_separator()
        self.dglMenu.add_command(label="RWP bei DGL 2. Ordnung",command=self.dglRWP )
        self.dglMenu.add_separator()

        self.glMenu = Menu(self.bar,tearoff=0)
        self.glMenu.add_command(label="Lösen . . .",command=self.solve )
        self.glMenu.add_separator()
        self.glMenu.add_command(label="Numerisch",command=self.findRoot )
        self.glMenu.add_separator()
        self.glMenu.add_command(label="Newton",command=self.newton)  
        self.glMenu.add_separator()


        self.analysism = Menu(self.bar)
        self.analysism.add_cascade(label="Gleichungen",menu=self.glMenu)
        self.analysism.add_separator()                
        self.analysism.add_command(label="f'(x)",command=self.diff)
        self.analysism.add_separator()                                
        self.analysism.add_command(label="F(x)", command=self.stamm)
        self.analysism.add_separator()                                        
        self.analysism.add_command(label="Integral",command=self.integral)
        self.analysism.add_separator()                                        
        self.analysism.add_command(label="Rotationsintegral",command=self.rotintegral)
        self.analysism.add_separator()                                        
        self.analysism.add_cascade(label="Funktion",menu=self.fktMenu)
        self.analysism.add_separator()
        self.analysism.add_cascade(label="Funktionenschar",menu=self.fktScharMenu)
        self.analysism.add_separator()
        self.analysism.add_cascade(label="3D-Plot",menu=self.dreiDPlotMenu)
        self.analysism.add_separator()        
        self.analysism.add_command(label="Grenzwert",command=self.limit)
        self.analysism.add_separator()                                        
        self.analysism.add_command(label="Taylor-Entwicklung",command=self.taylor)
        self.analysism.add_separator()                                        
        self.analysism.add_command(label="Summe berechnen",command=self.summe)           
        self.analysism.add_separator()                                                
        self.analysism.add_command(label="Produkt berechnen",command=(lambda arg=1 : self.summe(produkt = arg) ) )           
        self.analysism.add_separator()
        self.analysism.add_command(label="Kettenbruch",command=self.kettenbruch)           
        self.analysism.add_separator()                                                
        
        self.analysism.add_command(label="Partialbruchzerlegung",command=self.partialBruch)           
        self.analysism.add_separator()                                                
        self.analysism.add_command(label="Partialbruchzerlegung %",command=self.partialBruchZerlegung)
        self.analysism.add_separator()
        self.analysism.add_cascade(label="Differenzialgleichungen",menu=self.dglMenu)
        self.analysism.add_separator()


        self.kombinatorikMenu = Menu(self.bar,tearoff=0)
        self.kombinatorikMenu.add_command(label="Ziehen mit Zurücklegen, geordnet (n^k)",command=(lambda arg=0 : self.kombinatorik(arg) ) )
        self.kombinatorikMenu.add_separator()
        self.kombinatorikMenu.add_command(label="Ziehen ohne Zurücklegen, geordnet (n!/(n-k)!) (Variationen)",command=(lambda arg=1 : self.kombinatorik(arg) ) )
        self.kombinatorikMenu.add_separator()
        self.kombinatorikMenu.add_command(label="Ziehen ohne Zurücklegen, ungeordnet (n über k) (Kombinationen)",command=(lambda arg=2 : self.kombinatorik(arg) ) )
        self.kombinatorikMenu.add_separator()
        self.kombinatorikMenu.add_command(label="Ziehen mit Zurücklegen, ungeordnet ([n+k-1] über k)",command=(lambda arg=3 : self.kombinatorik(arg) ) )
        self.kombinatorikMenu.add_separator()



        self.blrzweiSeitigMenu = Menu(self.bar,tearoff=0)
        self.blrzweiSeitigMenu.add_command(label="Zweiseitig",command=(lambda arg=0 : self.hypothesentest(arg) ) )
        self.blrzweiSeitigMenu.add_separator()
        self.blrzweiSeitigMenu.add_command(label="Linksseitig",command=(lambda arg=1 : self.hypothesentest(arg) ) )
        self.blrzweiSeitigMenu.add_separator()
        self.blrzweiSeitigMenu.add_command(label="Rechtsseitig",command=(lambda arg=2 : self.hypothesentest(arg) ) )
        self.blrzweiSeitigMenu.add_separator()
        self.blrzweiSeitigMenu.add_command(label="Alternativtest",command=(lambda arg=0 : self.alternativtest(arg) ) )
        self.blrzweiSeitigMenu.add_separator()

        self.nlrzweiSeitigMenu = Menu(self.bar,tearoff=0)
        self.nlrzweiSeitigMenu.add_command(label="Zweiseitig",command=(lambda arg=0 : self.nHypothesentest(arg) ) )
        self.nlrzweiSeitigMenu.add_separator()
        self.nlrzweiSeitigMenu.add_command(label="Linksseitig",command=(lambda arg=1 : self.nHypothesentest(arg) ) )
        self.nlrzweiSeitigMenu.add_separator()
        self.nlrzweiSeitigMenu.add_command(label="Rechtsseitig",command=(lambda arg=2 : self.nHypothesentest(arg) ) )
        self.nlrzweiSeitigMenu.add_separator()
        self.nlrzweiSeitigMenu.add_command(label="Alternativtest",command=(lambda arg=1 : self.alternativtest(arg) ) )
        self.nlrzweiSeitigMenu.add_separator()

        self.plrzweiSeitigMenu = Menu(self.bar,tearoff=0)
        self.plrzweiSeitigMenu.add_command(label="Zweiseitig",command=(lambda arg=0 : self.pHypothesentest(arg) ) )
        self.plrzweiSeitigMenu.add_separator()
        self.plrzweiSeitigMenu.add_command(label="Linksseitig",command=(lambda arg=1 : self.pHypothesentest(arg) ) )
        self.plrzweiSeitigMenu.add_separator()
        self.plrzweiSeitigMenu.add_command(label="Rechtsseitig",command=(lambda arg=2 : self.pHypothesentest(arg) ) )
        self.plrzweiSeitigMenu.add_separator()

        self.opMenu = Menu(self.bar,tearoff=0)
        self.opMenu.add_command(label="Zweiseitig",command=(lambda arg='op0' : self.guetefunktion(arg) ) )
        self.opMenu.add_separator()
        self.opMenu.add_command(label="Linksseitig",command=(lambda arg='op1' : self.guetefunktion(arg) ) )
        self.opMenu.add_separator()
        self.opMenu.add_command(label="Rechtsseitig",command=(lambda arg='op2' : self.guetefunktion(arg) ) )
        self.opMenu.add_separator()

        self.gpMenu = Menu(self.bar,tearoff=0)
        self.gpMenu.add_command(label="Zweiseitig",command=(lambda arg='gp0' : self.guetefunktion(arg) ) )
        self.gpMenu.add_separator()
        self.gpMenu.add_command(label="Linksseitig",command=(lambda arg='gp1' : self.guetefunktion(arg) ) )
        self.gpMenu.add_separator()
        self.gpMenu.add_command(label="Rechtsseitig",command=(lambda arg='gp2' : self.guetefunktion(arg) ) )
        self.gpMenu.add_separator()


        self.konfMenu = Menu(self.bar,tearoff=0)
        self.konfMenu.add_command(label="Vertrauensintervall",command=self.konfIntervall )
        self.konfMenu.add_separator()
        self.konfMenu.add_command(label="Stichprobenumfang n",command=self.stichprobenUmfang )
        self.konfMenu.add_separator()


        self.stochastikm = Menu(self.bar)
        self.stochastikm.add_cascade(label="Kombinatorik",menu=self.kombinatorikMenu)
        self.stochastikm.add_separator()

        self.stochastikm.add_command(label="Wk-Baum/bedingte Wk",command=self.wkBaum)  # ab Vers. 1.73
        self.stochastikm.add_separator()
        
        self.stochastikm.add_command(label="Binomialverteilung",command=self.binomial)
        self.stochastikm.add_separator()
        self.stochastikm.add_command(label="Summierte Binomialverteilung",command=self.sumBinomial)
        self.stochastikm.add_separator()        
        self.stochastikm.add_command(label="Normalverteilung",command=self.normalVerteilung)        
        self.stochastikm.add_separator()
        
        self.stochastikm.add_cascade(label="Signifikanztest (Binomialverteilung)",menu=self.blrzweiSeitigMenu)
        self.stochastikm.add_separator()
        self.stochastikm.add_cascade(label="Signifikanztest (Normalverteilung)",menu=self.nlrzweiSeitigMenu)
        self.stochastikm.add_separator()
        self.stochastikm.add_cascade(label="Signifikanztest (Poissonverteilung)",menu=self.plrzweiSeitigMenu)
        self.stochastikm.add_separator()

##        self.stochastikm.add_command(label="Gütefunktion g(p)",command=(lambda arg='gp' : self.guetefunktion(arg) ) )        
##        self.stochastikm.add_separator()
##        self.stochastikm.add_command(label="Operationscharakteristik O(p)",command=(lambda arg='op' : self.guetefunktion(arg) ) )        
##        self.stochastikm.add_separator()
        self.stochastikm.add_cascade(label="Operationscharakteristik O(p)", menu=self.opMenu )        
        self.stochastikm.add_separator()
        self.stochastikm.add_cascade(label="Gütefunktion g(p)", menu=self.gpMenu )        
        self.stochastikm.add_separator()

        self.stochastikm.add_cascade(label="Konfidenzintervall",menu=self.konfMenu)
        self.stochastikm.add_separator()


        self.matrixm = Menu(self.bar)
        self.matrixm.add_command(label="Matrix eingeben",command=self.mnMatrix)
        self.matrixm.add_separator()                
        self.matrixm.add_command(label="Diagonalmatrix",command=self.matDiagonal)
        self.matrixm.add_separator()
        self.matrixm.add_command(label="Einheitsmatrix",command=self.matEinheit)
        self.matrixm.add_separator()                
        self.matrixm.add_command(label="Nullmatrix",command=self.matZero)
        self.matrixm.add_separator()
        self.matrixm.add_separator()                
        self.matrixm.add_command(label="Matrix transponieren",command=(lambda arg='transpose' : self.matFunc(arg) ) )
        self.matrixm.add_separator()
        self.matrixm.add_command(label="Matrix invertieren",command=(lambda arg='invert' : self.matFunc(arg) ) )
        self.matrixm.add_separator()
        self.matrixm.add_command(label="Eigenwerte",command=(lambda arg='eigenvalues' : self.matFunc(arg) ) )
        self.matrixm.add_separator()
        self.matrixm.add_command(label="Eigenvektoren",command=(lambda arg='eigenvectors' : self.matFunc(arg) ) )
        self.matrixm.add_separator()        
        self.matrixm.add_command(label="Charakteristisches Polynom",command=self.matCharPoly)
        self.matrixm.add_separator()        
        self.matrixm.add_command(label="Determinante",command=(lambda arg='determinant' : self.matFunc(arg) ) )
        self.matrixm.add_separator()
        self.matrixm.add_command(label="Rang einer Matrix",command=(lambda arg='rank' : self.matFunc(arg) ) )
        self.matrixm.add_separator()        
        self.matrixm.add_command(label="Adjungierte einer Matrix",command=(lambda arg='adjoint' : self.matFunc(arg) ) )
        self.matrixm.add_separator()
        self.matrixm.add_separator()        
        self.matrixm.add_command(label="Lineare Gleichungssysteme",command=self.gauss)        
        self.matrixm.add_separator()

        # geometrische Objekte
        self.geoObjekteMenu = Menu(self.bar,tearoff=0)
        self.geoObjekteMenu.add_command(label="Strecke",command=self.strecke)
        self.geoObjekteMenu.add_separator()        
        self.geoObjekteMenu.add_command(label="Dreieck",command=self.dreieck)
        self.geoObjekteMenu.add_separator()
        self.geoObjekteMenu.add_command(label="Viereck",command=self.viereck)
        self.geoObjekteMenu.add_separator()
        self.geoObjekteMenu.add_command(label="Quader",command=self.quader)
        self.geoObjekteMenu.add_separator()        
        self.geoObjekteMenu.add_command(label="Tetraeder",command=self.tetraeder)
        self.geoObjekteMenu.add_separator()
        self.geoObjekteMenu.add_command(label="Pyramide",command=self.pyramide)
        self.geoObjekteMenu.add_separator()
        
        # Vektor-Eingabe => Menue
        self.vecMenu = Menu(self.bar,tearoff=0)
        self.vecMenu.add_command(label="Ortsvektor",command=self.vektor)
        self.vecMenu.add_separator()
        self.vecMenu.add_command(label="Vektor durch 2 Punkte",command=self.pVektor)
        self.vecMenu.add_separator()


        self.geom = Menu(self.bar)                
        self.geom.add_command(label="Punkt", command=self.punkt)        
        self.geom.add_separator()
        self.geom.add_cascade(label="Vektor", menu=self.vecMenu)        
        self.geom.add_separator()

        self.geom.add_cascade(label="Objekte",menu=self.geoObjekteMenu)
        self.geom.add_separator()
        
        self.geom.add_command(label="Gerade durch 2 Punkte", command=self.punktGerade)
        self.geom.add_separator()                                        
        self.geom.add_command(label="Gerade in Punkt-Richtungsform", command=self.pgerade)
        self.geom.add_separator()                                        
        
        self.geom.add_command(label="Ebene durch 3 Punkte", command=self.punktEbene)
        self.geom.add_separator()                                        
        self.geom.add_command(label="Ebene in Parameterform", command=self.pebene)
        self.geom.add_separator()                                        
        self.geom.add_command(label="Ebene in Koordinatenform", command=self.kebene)
        self.geom.add_separator()
        self.geom.add_command(label="Ebene in Normalform", command=self.nebene)
        self.geom.add_separator()        
        self.geom.add_command(label="Kugel: Mittelpunkt M, Radius r", command=self.rKugel)        
        self.geom.add_separator()
        
        self.geom.add_command(label="Kugel: Mittelpunkt M, Punkt O", command=self.kugel)
        self.geom.add_separator()                                        
        self.geom.add_command(label="Kugel durch 4 Punkte", command=self.kugel4)
        self.geom.add_separator()                                        
        self.geom.add_command(label="Kreis durch 3 Punkte", command=self.kreis3)
        self.geom.add_separator()                                        



        self.pProbeMenu = Menu(self.bar,tearoff=0)
        self.pProbeMenu.add_command(label="Punkt-Gerade",command=self.abstandPG)
        self.pProbeMenu.add_separator()
        self.pProbeMenu.add_command(label="Punkt-Ebene",command=self.abstandPE)
        self.pProbeMenu.add_separator()
        self.pProbeMenu.add_command(label="Punkt-Kugel",command=self.abstandPK)
        self.pProbeMenu.add_separator()
        self.pProbeMenu.add_command(label="Punkt-Kreis",command=self.abstandPC)
        self.pProbeMenu.add_separator()

        self.schnittm = Menu(self.bar)
        self.schnittm.add_command(label="Ebene-Ebene",command=self.schnittEE)
        self.schnittm.add_separator()
        self.schnittm.add_command(label="Gerade-Ebene",command=self.schnittGE)
        self.schnittm.add_separator()
        self.schnittm.add_command(label="Gerade-Gerade",command=self.schnittGG)
        self.schnittm.add_separator()
        self.schnittm.add_command(label="Kugel-Ebene",command=self.schnittKE)
        self.schnittm.add_separator()
        self.schnittm.add_command(label="Kugel-Gerade",command=self.schnittKG)
        self.schnittm.add_separator()
        self.schnittm.add_command(label="Kugel-Kugel",command=self.schnittKK)
        self.schnittm.add_separator()
        self.schnittm.add_command(label="Tangentialebene",command=self.tangentialebene)
        self.schnittm.add_separator()
        self.schnittm.add_command(label="Kugel aus Tangentialebene",command=self.kugelAusTangentialebene)
        self.schnittm.add_separator()        
        self.schnittm.add_command(label="Pol",command=self.pol)
        self.schnittm.add_separator()                
        self.schnittm.add_command(label="Kreis-Ebene",command=self.schnittCE)
        self.schnittm.add_separator()
        self.schnittm.add_command(label="Kreis-Gerade",command=self.schnittCG)
        self.schnittm.add_separator()
        


        self.schnittm.add_command(label="Abstand d(A,B)",command=self.abstandPP)
        self.schnittm.add_separator()
        
        self.schnittm.add_cascade(label="Punktprobe",menu=self.pProbeMenu)
        self.schnittm.add_separator()


        self.linAbMenu = Menu(self.bar,tearoff=0)
        self.linAbMenu.add_command(label="2 Vektoren",command=self.linUnab2)
        self.linAbMenu.add_separator()
        self.linAbMenu.add_command(label="3 Vektoren",command=self.linUnab3)
        self.linAbMenu.add_separator()

        self.vektorm = Menu(self.bar)
        self.vektorm.add_command(label="Betrag des Vektors",command=self.vecBetrag)
        self.vektorm.add_separator()
        self.vektorm.add_command(label="Winkel zw. 2 Vektoren",command=self.vecWinkel)
        self.vektorm.add_separator()
        self.vektorm.add_command(label="Skalarprodukt",command=self.vecSkalar)
        self.vektorm.add_separator()        
        self.vektorm.add_command(label="Kreuzprodukt",command=self.vecKreuz)
        self.vektorm.add_separator()
        self.vektorm.add_cascade(label="Lineare Unabhängigkeit",menu=self.linAbMenu)
        self.vektorm.add_separator()


        self.pygeom = Menu(self.bar)
        if self.pygeoFlag: # Teile des Menues ausblenden
            self.pygeom.add_command(label="pyGeo", command=self.pygeo)                    
            self.pygeom.add_separator()
            self.pygeom.add_command(label="Save Script as...", command=self.saveScript)                    
            self.pygeom.add_separator()            
            self.pygeom.add_command(label="Run pyGeo-Script", command=self.runScript)                    
            self.pygeom.add_separator()
            self.pygeom.add_command(label="pyGeo2maxima",command=self.pygeo2maxima)
            self.pygeom.add_separator()
            
        self.pygeom.add_command(label="EDIT pyGeo-Object ",command=self.editObjects)
        self.pygeom.add_separator()            

        self.pygeom.add_command(label="pyGeo-Object INFO",command=self.infoObjectDir)
        self.pygeom.add_separator()
        self.pygeom.add_command(label="DELETE Object(s)",command=self.deleteObjects)
        
        self.pygeom.add_separator()
        self.pygeom.add_command(label="Save pyGeo-Objects",command=self.saveObjects)            
        self.pygeom.add_separator()            
        self.pygeom.add_command(label="Get pyGeo-Objects",command=self.getObjects)
        
        self.pygeom.add_separator()            
        self.pygeom.add_command(label="Append pyGeo-Objects",command=self.saveAppendObjects)
        # >>> SAVE Append: existierende Objekte werden NICHT ueberschrieben!
       
##        self.pygeom.add_separator()            
##        self.pygeom.add_command(label="Append pyGeo-Objects",command=(lambda arg=0 : self.appendObjects( arg ) ) )
##        
##        self.pygeom.add_separator()            
##        self.pygeom.add_command(label="Append & OverWrite",command=(lambda arg=1 : self.appendObjects( arg ) ) )

        self.pygeom.add_separator()            
        self.pygeom.add_separator()            
        self.pygeom.add_command(label="DELETE Object-List",command=self.deleteObjectDir)
        self.pygeom.add_separator()
        self.pygeom.add_separator()    
                

        self.viewm = Menu(self.bar)
        self.viewm.add_command(label="Mehrzeiliger Ausdruck",command=self.mehrzeilig)           
        self.viewm.add_separator()
        self.viewm.add_separator()                                                        
        self.viewm.add_command(label="Funktionen",command=(lambda arg="functions;" : self.bearbeiteAusdruck( arg ) ) )
        self.viewm.add_separator()        
        self.viewm.add_command(label="Variable",command=(lambda arg="values;" : self.bearbeiteAusdruck( arg ) ) )
        self.viewm.add_separator()        
        self.viewm.add_command(label="Funktionen löschen",command=(lambda arg=0 : self.loescheFunktionenVar( arg ) ) )
        self.viewm.add_separator()        
        self.viewm.add_command(label="Variable löschen",command=(lambda arg=1 : self.loescheFunktionenVar( arg ) ) )
        self.viewm.add_separator()
        self.viewm.add_separator()        
        self.viewm.add_command(label="Gleitkommagenauigkeit",command=self.floatPointPrecision )
        self.viewm.add_separator()
        self.viewm.add_separator()        
        self.viewm.add_command(label="display2d", command=(lambda arg=1 : self.display(v2d = arg) ) )
        self.viewm.add_separator()        
        self.viewm.add_command(label="display1d",command=(lambda arg=0 : self.display(v2d = arg) ) )
        self.viewm.add_separator()
        self.viewm.add_separator()        
        self.viewm.add_command(label="Mit Maxima verbinden", command=self.startConnect)
        self.viewm.add_separator()        
        self.viewm.add_command(label="Verbindung beenden", command=self.stopConnect)
        self.viewm.add_separator()        

 
        self.optm = Menu(self.bar)
        self.optm.add_command(label="Einstellungen anzeigen", command=self._einstellungen)
        self.optm.add_separator()        
        self.optm.add_command(label="Einstellungen speichern",command=self._speichern)        
        self.optm.add_separator()                
        self.optm.add_command(label="*.conf-Datei lesen",command=self._laden)           
 
     
        self.helpm = Menu(self.bar)   
        self.helpm.add_command(label="Maxima-Manual", command=self.maximaManual)
        self.helpm.add_separator()
        self.helpm.add_command(label="Maxima-Info", command=self.maximaInfo)        
        self.helpm.add_separator()        
        self.helpm.add_command(label="Hilfe ", command=self.hilfe)
        self.helpm.add_separator()        
        self.helpm.add_command(label="Lizenz ", command=self.lizenz)        
        self.helpm.add_separator()                                
        self.helpm.add_command(label="Über ", command=about)
        
        #self.tipHelpm = ToolTip.ToolTip(self.helpm, " Hilfe \n Maxima-Info \n Maxima-Manual \n")           
   
        self.bar.add_cascade(label="Datei", menu=self.filem)
        self.bar.add_cascade(label="Analysis", menu=self.analysism)
        self.bar.add_cascade(label="Stochastik", menu=self.stochastikm)        

        #self.bar.add_cascade(label="Maxima", menu=self.viewm)
        self.bar.add_cascade(label="Matrix", menu=self.matrixm)        
        self.bar.add_cascade(label="Geometrie", menu=self.geom)
        self.bar.add_cascade(label="Lage", menu=self.schnittm)
        self.bar.add_cascade(label="Vektor", menu=self.vektorm)   
                
        self.bar.add_cascade(label="pyGeo", menu=self.pygeom)

        
        self.bar.add_cascade(label="Maxima", menu=self.viewm)                        
        #self.bar.add_cascade(label="Connect", menu=self.connm)        
        self.bar.add_cascade(label="Einstellungen", menu=self.optm)
        self.bar.add_cascade(label="Hilfe", menu=self.helpm)   
        self.t.config(menu=self.bar)   
   
   
        self.f = Frame(self.t,width=600)   # modified, 1.34b
        self.f.pack(expand=1, fill=BOTH)   
         
        self.midfr = Frame(self.f,relief=RIDGE,background="white")   
        self.midfr.pack(side=TOP,fill=BOTH,expand=0)   


        self.butt4=Button(self.midfr,text=">Clear all<",bd="2",command=self.clear_all)   
        self.butt4.pack(side=LEFT,fill=BOTH,expand=1)
        self.tip4 = ToolTip.ToolTip(self.butt4, "\n >Clear all<: Alle (!) Eingaben löschen! \n") 


        self.kuiviewer = PhotoImage(file=pyMaximaHome+os.sep+'images'+os.sep+'kuiviewer.gif')
        self.butt15=Button(self.midfr,image=self.kuiviewer,bd="2",command=self.loadExprTxtFileAndExec)   
        self.butt15.pack(side=LEFT,fill=BOTH,expand=1)   
        self.tip15 = ToolTip.ToolTip(self.butt15, "\n Sitzungsdatei ausführen \n")   



        self.butt14=Button(self.midfr,text="(x+1)/(x-1)",bd="2",command=self.polyDiv)   
        self.butt14.pack(side=LEFT,fill=BOTH,expand=1)
        self.tip14 = ToolTip.ToolTip(self.butt14, "\n (x+1)/(x-1)\n Polynomdivision \n")          

        self.butt5=Button(self.midfr,text="f'(x)",bd="2",command=self.diff)   
        self.butt5.pack(side=LEFT,fill=BOTH,expand=1)   
        self.tip5 = ToolTip.ToolTip(self.butt5, "\n f'(x): 1. Ableitung von f(x) \n") 


        self.butt6=Button(self.midfr,text="F(x)",bd="2",command=self.stamm)   
        self.butt6.pack(side=LEFT,fill=BOTH,expand=1)
        self.tip6 = ToolTip.ToolTip(self.butt6, "\n F(x): Stammfunktion von f(x) \n unbestimmtes Integral \n") 
        
        #self.butt11=Button(self.midfr,text="\_/",command=self.kurvendisk)   
        #self.butt11.pack(side=LEFT,fill=BOTH,expand=1)
        self.diskbild = PhotoImage(file=pyMaximaHome+os.sep+'images'+os.sep+'kurvendisk.gif')
        self.butt11=Button(self.midfr,image=self.diskbild,bd="2",command=self.kurvendisk)   
        self.butt11.pack(side=LEFT,fill=BOTH,expand=1)   
        self.tip11 = ToolTip.ToolTip(self.butt11, "\n Funktionsuntersuchung von f(x) \n")

        self.pygeobut = PhotoImage(file=pyMaximaHome+os.sep+'images'+os.sep+'pygeo.gif')        
        if self.pygeoFlag: 
            self.butt10=Button(self.midfr,image=self.pygeobut,bd="2",command=self.pygeo)   
            self.butt10.pack(side=LEFT,fill=BOTH,expand=1)
        else: # _pygeoFlag_ == 0
            self.butt10=Button(self.midfr,state=DISABLED,text="pyGeo",command=self.pygeo)   
            self.butt10.pack(side=LEFT,fill=BOTH,expand=1)
        self.tip10 = ToolTip.ToolTip(self.butt10, "\n Aufruf von pyGeo: zeichnet \n die geometrischen Objekte \n")             

        #self.butt13=Button(self.midfr,text=u"x \u2192 \u221E",font=gf_font3,command=self.limit)   # inf / minf
        self.butt13=Button(self.midfr,text=u"x --> inf",bd="2",command=self.limit)   # inf / minf        
        self.butt13.pack(side=LEFT,fill=BOTH,expand=1)
        self.tip13 = ToolTip.ToolTip(self.butt13, "\n x --> inf \n Grenzwert (Limes) \n inf: unendlich, minf: -unendlich ")


        self.plotbild = PhotoImage(file=pyMaximaHome+os.sep+'images'+os.sep+'func.gif')
        self.butt12=Button(self.midfr,image=self.plotbild,bd="2",command=self.plot)   
        self.butt12.pack(side=LEFT,fill=BOTH,expand=1)   
        self.tip12 = ToolTip.ToolTip(self.butt12, "\n Funktion f(x) plotten \n")   


        self.butt7=Button(self.midfr,text="x^5-4*x=0",bd="2",command=self.solve)   
        self.butt7.pack(side=LEFT,fill=BOTH,expand=1)
        self.tip7 = ToolTip.ToolTip(self.butt7, "\n x^5-4*x=0 \n Gleichungen lösen \n")        

        self.butt14=Button(self.midfr,text="Gauss",bd="2",command=self.gauss)
        ##self.butt14=Button(self.midfr,text="Gauss",command=self.ebene)           
        self.butt14.pack(side=LEFT,fill=BOTH,expand=1)   
        self.tip14 = ToolTip.ToolTip(self.butt14, "\n Gauss\n Lineare Gleichungssysteme lösen \n Beachte: 0<=dim<=9 \n")        



        self.maximaCon = PhotoImage(file=pyMaximaHome+os.sep+'images'+os.sep+'reddot.gif')
        self.butt25=Button(self.midfr,image=self.maximaCon,bd="2",command=self.startConnect)   
        self.butt25.pack(side=LEFT,fill=BOTH,expand=1)   
        self.tip25 = ToolTip.ToolTip(self.butt25, "\n Mit Maxima verbinden \n")   

        self.exitbut = PhotoImage(file=pyMaximaHome+os.sep+'images'+os.sep+'exit.gif')
        self.butt9=Button(self.midfr,image=self.exitbut,bd="2",command=self.beenden)   
        self.butt9.pack(side=RIGHT,fill=BOTH,expand=1)   
        self.tip9 = ToolTip.ToolTip(self.butt9, "\n pyMaxima beENDEn \n")        

         
        if os.name == 'posix':
            t2 = ScrolledText(self.f,width=80,height=nr_lines_main,relief=RAISED,font=gf_font2,bd=2,background="white")   # modified, 1.34b
            # Ubuntu: height=27
        else:     
            t2 = ScrolledText(self.f,width=80,height=nr_lines_main,relief=RAISED,font=gf_font2,bd=2,background="white")
            # Netbook: height=???
        t2.pack(fill=BOTH, expand=1)


        self.frUnten = Frame(self.f,relief=RIDGE,background="white")
        self.frUnten.pack(side=TOP,fill=BOTH,expand=0)
        
        self.downfr1li = Frame(self.frUnten,relief=RIDGE,background="white")   
        self.downfr1li.pack(side=TOP,fill=BOTH,expand=0)
        self.expr = Label(self.downfr1li, text="Ausdruck:")
        self.expr.pack(side=LEFT,fill=BOTH, expand=0)
        if os.name == 'posix':
            self.exprZeile = Entry(self.downfr1li,textvariable=self.eingabe,width=68,font=gf_font2,bd=2,background="white") # modified, 1.34b  
        else:     
            self.exprZeile = Entry(self.downfr1li,textvariable=self.eingabe,width=80,font=gf_font2,bd=2,background="white")   
        self.exprZeile.pack(side=LEFT,fill=BOTH, expand=1)   
        self.okbutt=Button(self.downfr1li,text="  OK  ",bd="2",command=self.getInput2)   
        self.okbutt.pack(side=LEFT,fill=BOTH,expand=0)
        self.tipOkButt = ToolTip.ToolTip(self.okbutt, "\n Eingabe an Maxima \n") 
        self.clearbutt=Button(self.downfr1li,text=" Clear ",bd="2",command=self.clearLabel)   
        self.clearbutt.pack(side=RIGHT,fill=BOTH,expand=0)
        self.tipClearButt = ToolTip.ToolTip(self.clearbutt, "\n Eingabe löschen \n") 
        
        self.downfr2li = Frame(self.frUnten,relief=RIDGE,background="gray")   
        self.downfr2li.pack(side=BOTTOM,fill=BOTH,expand=0)

        self.status = StringVar()
        self.status.set("Satus:   ")
        self.statusLabel=Label(self.downfr2li, anchor=W, textvariable=self.status, fg="red" )
        #self.statusLabel.text = "Satus: "
        self.statusLabel.pack(side=LEFT,fill=BOTH,expand=1)
                
        self.mehrzeilerbild = PhotoImage(file=pyMaximaHome+os.sep+'images'+os.sep+'mehrzeiler.gif')
        self.mehrzbutt=Button(self.downfr2li,image=self.mehrzeilerbild,bd="2",command=self.mehrzeilig)   
        self.mehrzbutt.pack(side=LEFT,fill=BOTH,expand=0)
        self.tipMehrz = ToolTip.ToolTip(self.mehrzbutt, "\n Mehrzeilige Eingabe \n")  
        if os.name == 'posix':
            self.commentbutt=Button(self.downfr2li,text='  "__"$  ',bd="2",command=self.kommentar)
        else:
            self.commentbutt=Button(self.downfr2li,text=' "__"$ ',bd="2",command=self.kommentar)
        self.commentbutt.pack(side=LEFT,fill=BOTH,expand=0)
        self.tipComment = ToolTip.ToolTip(self.commentbutt, "\n Maxima-Kommentar \n")
        

        self.popupMenu = Menu(self.f,tearoff=0) 
         
        self.popupMenu.add_command(label='copy', command=(lambda arg='t2' : self.gf_copy(arg) )  )
        #self.popupMenu.add_command(label='cut', command=(lambda arg='t2' : self.gf_cut(arg) )  )        
        #self.popupMenu.add_command(label='paste', command=(lambda arg='t2' : self.gf_paste(arg) )  )
        self.popupMenu.add_separator()
        self.popupMenu.add_command(label='Auswerten', command=self.evalExpr )
        self.popupMenu.add_command(label='square free', command=self.squareFree )                        
        self.popupMenu.add_command(label='Simplify', command=self.simplify )
        self.popupMenu.add_command(label='Factor', command=self.factor )
        self.popupMenu.add_command(label='Expand', command=self.expand )                
        self.popupMenu.add_separator()        
        self.popupMenu.add_command(label='alle Wurzeln', command=self.allRoots )                
        self.popupMenu.add_command(label='reelle Wurzeln', command=self.realRoots )                
        self.popupMenu.add_separator()                                
        self.popupMenu.add_command(label='cp2Input', command=(lambda arg='t2' : self.copy2Entry(arg) )  )        
        self.popupMenu.add_command(label='cp&Cut2Input', command=(lambda arg='t2' : self.cpCut2Entry(arg) )  )        
        #self.popupMenu.add_separator()        
        t2.bind("<Button-3>",self.popup)
        t2.bind("<Button-1>",self.test_mouse)        
        #t2.bind("<F3>",self.letzterBefehl)
        #t2.bind("<ButtonRelease-1>",self.markup) # klappt gerade so . . .
        #t2.bind("<Control-Button-1>",self.markup)
        #t2.bind("<Control-i>",self.getInNumber)
        t2.bind("<Escape>",self.EscapeHandler)
        if self._debug_:
            t2.bind("<Control-y>",self.showDiciInWindow_)
        
        
        
        
        self.popupMenu2 = Menu(self.f,tearoff=0)  
        self.popupMenu2.add_command(label='cut', command=(lambda arg='self.exprZeile' : self.gf_cut(arg) )  )
        self.popupMenu2.add_command(label='copy', command=(lambda arg='self.exprZeile' : self.gf_copy(arg) )  )
        self.popupMenu2.add_command(label='paste', command=(lambda arg='self.exprZeile' : self.gf_paste(arg) )  )        
        self.exprZeile.bind("<Button-3>",self.popup2)
        #self.exprZeile.bind("<F3>",self.letzterBefehl)
        self.exprZeile.bind("<Return>",self.getInput2_)
        self.exprZeile.bind('<Up>',self.prev)
        self.exprZeile.bind('<Down>',self.next)
        #self.exprZeile.bind("<Control-i>",self.getInNumber)        
        self.exprZeile.bind("<Escape>",self.EscapeHandler)
        if self._debug_:        
            self.exprZeile.bind("<Control-y>",self.showDiciInWindow_)        
        self.printExpression( self.altIn + ' ', _nl_=0 )
        self.exprZeile.focus_set()

        self.m2g = pymax2geo.max2geo()
        self.script = []

        # Maxima-Connect
        if not self.maximaOk:            
            self.maximaOk = maxima.startMaxima() # 
            time.sleep( _delay_time_ )
        if self.maximaOk:
            self.maximaCon = PhotoImage(file=pyMaximaHome+os.sep+'images'+os.sep+'greendot.gif')
            self.butt25.config(state=ACTIVE,image=self.maximaCon,bd="2")       
            self.butt25.update_idletasks()
            self.tip25 = ToolTip.ToolTip(self.butt25, "\n Mit Maxima verbunden! \n")
            self.buildInfoMaximaVersion = maxima.getMaximaVersion()
            _vers_li_ = self.buildInfoMaximaVersion.split('.')
            _sub_version_ = int(_vers_li_[1])
            if _sub_version_ >= 14:  # maxima modul draw erst ab Maxima Version 5.14!!!
                ergebnis = maxima.call("load(draw);")
                self.moduleDrawLoaded = 1
             
        #if not self.maximaOk:            
            #tkMessageBox.showerror(">>> Maxima Error <<<","Connection to Maxima \nfailed, sorry  . . .")            
                
        self.t.deiconify()  # Unhide pyMaxima
        self.t.update()     # neu
        self.t.clipboard_clear()

        
        newStatus = setStatus(self.maximaOk)        
        # prompt festsetzen
        if self.maximaOk:
            maxinter.setInOutChar(maxima) # neu in Vers. 1.01
            ##maxima.call('inchar: "%i"$')                                    
            ##maxima.call('outchar: "%o"$')
            # self.lispImplementationFlag = maxima.getLispImplementationType()
            # print "Lisp-Implementation:",self.lispImplementationFlag
            self.clear_all()
        self.status.set(newStatus)
        
        ##################
            
        try:
            import pygeo
            _pygeoFlag_ = 1                 
        except:
            _pygeoFlag_ = 0
        self.pygeoFlag = _pygeoFlag_
        if self.pygeoFlag:
            # jetzt Menue self.pygeom aktualisieren!            
            self.update_pygeom()
        #print self.pygeoFlag
        #print self.t.winfo_geometry()
        #print self.t.winfo_height()        
        
##########################################            
    #
    # construction hooks
    def popup(self, event):
        """popop-Menue im Hauptframe self.f"""
        self.popupMenu.post(event.x_root, event.y_root) 
   
    def popup2(self, event):
        """popop-Menue in der Eingabezeile self.exprZeile"""        
        self.popupMenu2.post(event.x_root, event.y_root)


    def test_mouse(self, event):
        """Teste die linke Maus-Taste und setzt die Variable
        
                self.break_excute_session_file
            
        """
        if (self.break_excute_session_file == 0):
            self.break_excute_session_file = 1
        else:
            self.break_excute_session_file = 0
        


    def showDiciInWindow_(self,event):        
        self.showDiciInWindow( self.exprDici )


    def EscapeHandler(self,event):        
        global maxima
        if self.maximaOk:
            maxima.ctrlcHandler()        
            del maxima
            maxima = None
            newStatus = "Satus:   " + '              <<< !!!NICHT VERBUNDEN!!! >>> '
            self.status.set(newStatus)
            self.maximaOk = 0
            self.maximaCon = PhotoImage(file=pyMaximaHome+os.sep+'images'+os.sep+'reddot.gif')
            self.butt25.config(state=ACTIVE,image=self.maximaCon,bd="2")       
            self.butt25.update_idletasks()
            self.tip25 = ToolTip.ToolTip(self.butt25, "\n Mit Maxima verbinden \n")
            self.moduleDrawLoaded = 0
             

            
            
        
    def getStandardFkt(self):
        """setzt die Liste der Maxima-Standardfunktionen: self.standard """
        try:
            liste = util.getFuncListe(self._initdir_,funcDefDatei) # Auslesen der Datei <func_liste.txt>
            if len(liste) > 0:
                for element in liste:
                    if element.startswith('#'): # '#' Kommentar
                        pass
                    else:
                        element = element.strip()
                        self.standard.append(element)
            else: # len(liste) == 0
                print "Lesen  der Datei <" + funcDefDatei + "> fehlgeschlagen!"
                raise            
        except: # falls Datei-Lesen nicht geht => Default-Liste
            self.standard = ['exp', 'log','acos','acosh','acot','acoth','acsc','acsch','asec','asech']
            self._st2_ = ['asin','asinh','atan','atanh','cos','cosh','cot','coth','csc','csch']
            self._st3_ = ['sec','sech','sin','sinh','tan','tanh','gamma','sqrt','abs','signum']
            self.standard.extend(self._st2_)
            self.standard.extend(self._st3_)
            self.standard.sort()
            
##############################################
    def update_pygeom(self): 
        """ !!Funktioniert!! EXPERIMENTELL!!"""
        # >>> bricht unter Python 2.6 mit einem Type-Error ab <<<<<<<<
        from sys import version_info
        if version_info < (2,6):
             self.pygeom.delete(0,END)
             if self.pygeoFlag: # Teile des Menues ausblenden
                 self.pygeom.add_command(label="pyGeo", command=self.pygeo)                    
                 self.pygeom.add_separator()
                 self.pygeom.add_command(label="Save Script as...", command=self.saveScript)                    
                 self.pygeom.add_separator()            
                 self.pygeom.add_command(label="Run pyGeo-Script", command=self.runScript)                    
                 self.pygeom.add_separator()
                 self.pygeom.add_command(label="pyGeo2maxima",command=self.pygeo2maxima)
                 self.pygeom.add_separator()


             self.pygeom.add_command(label="EDIT pyGeo-Object ",command=self.editObjects)
             self.pygeom.add_separator()            
               
             self.pygeom.add_command(label="pyGeo-Object INFO",command=self.infoObjectDir)
             self.pygeom.add_separator()
             self.pygeom.add_command(label="DELETE Object(s)",command=self.deleteObjects)
            
             self.pygeom.add_separator()
             self.pygeom.add_command(label="Save pyGeo-Objects",command=self.saveObjects)            
             self.pygeom.add_separator()            
             self.pygeom.add_command(label="Get pyGeo-Objects",command=self.getObjects)
             self.pygeom.add_separator()                   
             self.pygeom.add_command(label="Append pyGeo-Objects",command=self.saveAppendObjects)
             # SAVE Append of pyGeo-Objects
            
     ##        self.pygeom.add_separator()         
     ##        self.pygeom.add_command(label="Append pyGeo-Objects",command=(lambda arg=0 : self.appendObjects( arg ) ) )
     ##        self.pygeom.add_separator()            
     ##        self.pygeom.add_command(label="Append & OverWrite",command=(lambda arg=1 : self.appendObjects( arg ) ) )
            
             self.pygeom.add_separator()            
             self.pygeom.add_separator()            
             self.pygeom.add_command(label="DELETE Object-List",command=self.deleteObjectDir)
             self.pygeom.add_separator()
             self.pygeom.add_separator()
        
        if self.pygeoFlag: 
            self.butt10.config(state=ACTIVE,image=self.pygeobut,bd="2")               
        else: # _pygeoFlag_ == 0
            self.butt10.config(state=DISABLED,text="pyGeo")       
        self.butt10.update_idletasks()
                

##############################################

    def stopConnect(self):        
        """beendet die Verbindung zu Maxima"""
        global maxima
        if self.maximaOk:
            maxima.endMaxima()
            del maxima
            maxima = None
            newStatus = "Satus:   " + '              <<< !!!NICHT VERBUNDEN!!! >>> '
            self.status.set(newStatus)
            self.maximaOk = 0
            # Button einstellen
            self.maximaCon = PhotoImage(file=pyMaximaHome+os.sep+'images'+os.sep+'reddot.gif')
            self.butt25.config(state=ACTIVE,image=self.maximaCon,bd="2")       
            self.butt25.update_idletasks()
            self.tip25 = ToolTip.ToolTip(self.butt25, "\n Mit Maxima verbinden \n")
            self.moduleDrawLoaded = 0
             
            
            

    def startConnect(self):
        """startet die Verbindung zu Maxima"""        
        global maxima
        if not self.maximaOk:
            del maxima
            if os.name <> 'posix':
                pfadLi =  os.path.split( pfad_zu_maxima )
                ver = string.split(pfadLi[1],"-") # ver[1] = '5.10.0'
                self.maxima_ver = ver[1]
                util.build_maxima_bat(self.maxima_ver , pfad_zu_maxima)
            
            maxima = None
            maxima = maxinter.MaximaInterface( delay_time = _delay_time_, _pfad_zu_maxima_ = util.protectBlank( self._initdir_ ) ) # alt: pfad_zu_maxima
            self.maximaOk = maxima.startMaxima()
            time.sleep( _delay_time_ )
            newStatus = setStatus(self.maximaOk)                    
            self.status.set(newStatus)
            if not self.maximaOk:
                self.maximaCon = PhotoImage(file=pyMaximaHome+os.sep+'images'+os.sep+'reddot.gif')
                self.butt25.config(state=ACTIVE,image=self.maximaCon,bd="2")       
                self.butt25.update_idletasks()
                #tkMessageBox.showerror(">>> Maxima Error <<<","Connection to Maxima \nfailed, sorry  . . .")
                self.tip25 = ToolTip.ToolTip(self.butt25, "\n Mit Maxima verbinden \n")                                 
                
            else:
                self.maximaCon = PhotoImage(file=pyMaximaHome+os.sep+'images'+os.sep+'greendot.gif')
                self.butt25.config(state=ACTIVE,image=self.maximaCon,bd="2")       
                self.butt25.update_idletasks()
                self.tip25 = ToolTip.ToolTip(self.butt25, "\n Mit Maxima verbunden! \n")
                                 
                #tkMessageBox.showinfo(">>> Maxima <<<","Verbunden mit Maxima!")                
                maxinter.setInOutChar(maxima) # neu in Vers. 1.01
                ##maxima.call('inchar: "%i"$')                                    
                ##maxima.call('outchar: "%o"$')
                #tkMessageBox.showinfo(">>> Maxima <<<","Verbunden mit Maxima!")
                # self.lispImplementationFlag = maxima.getLispImplementationType()
                # der folgende Befehl wandert nach unten
                #self.clear_all() 
                self.status.set(newStatus)
                self.buildInfoMaximaVersion = maxima.getMaximaVersion()
                _vers_li_ = self.buildInfoMaximaVersion.split('.')
                _sub_version_ = int(_vers_li_[1])
                if _sub_version_ >= 14:  # maxima modul draw erst ab Maxima Version 5.14!!!
                    ergebnis = maxima.call("load(draw);")
                    self.moduleDrawLoaded = 1
                self.clear_all()
                

                                             

    def dbUsed(self, arg=None):
        """Verwaltet den Zugriff auf die Liste der letzten 15 benutzten Sitzungsdateien"""
        global prevUsed
        from os.path import abspath, exists
        debug = 0
        if debug:
            print arg
        prevUsed.swapFirst(arg)
        self.update_dbUsedMenu()        
        self.update_editUsedMenu()
        self.loadExprTxtFileAndExec(TEX_Mode=0,_sessionFile_=arg)

        
    def update_dbUsedMenu(self): 
        """ !!Funktioniert!! EXPERIMENTELL!!"""
        self.dbUsedMenu.delete(0,END)
        laenge = prevUsed.listLength()
        for i in range(laenge):
            argument =  prevUsed.DirListe[i]
            self.dbUsedMenu.add_command(label=argument, command=(lambda arg=argument:self.dbUsed(arg) ))
            
    def update_editUsedMenu(self):
        """ !!Funktioniert!! EXPERIMENTELL!!"""
        self.editUsedMenu.delete(0,END)
        laenge = prevUsed.listLength()
        for i in range(laenge):
            argument =  prevUsed.DirListe[i]
            self.editUsedMenu.add_command(label=argument, command=(lambda arg=argument:self.openWithEditor(arg) ))



    def markup(self, event):
        #self.feedback( str( t2.winfo_height() ) )
        xy =  "@0,%d" % (event.y)        
        transferY = t2.index(xy)
        liste = transferY.split('.')
        nr = str(liste[0])
        lineStart = str(nr) + '.0'
        lineEnd = str(nr) + '.end'
        zeile = t2.get(lineStart,lineEnd)
        zeile = string.strip(zeile)
        #print zeile
        (inhalt,start,ende) = self.scanZeile(zeile)
        if len(inhalt) > 0:
            von = str(nr) + '.' + str(start)
            bis = str(nr) + '.' + str(ende)
            t2.tag_remove(SEL,'1.0', END)  # alte Markierungen entfernen!
            t2.tag_delete(SEL)
            t2.tag_add(SEL,von,bis)        
            #t2.update_idletasks()        


    def scanZeile(self,zeile):
        import string
        start = 0
        ende = 0
        maxZeile = len(zeile)
        startLeer = 0               
        i = 0
        liste = zeile.split('%')
        if len(liste) > 1:            
            if (liste[1][0] != 'i'):            
                return ('', start, ende)
            else:
                for i in range(maxZeile):
                    if (zeile[i]== ')') and (zeile[i-1] in ['0','1','2','3','4','5','6','7','8','9']): # ????
                        startLeer = i+1
                        break
                inhalt = zeile[startLeer:maxZeile]
                inhalt2 = string.strip(inhalt,' ')
                inhalt3 = string.strip(inhalt2,'\t')
                expr = string.strip(inhalt3,' ')
                k = inhalt.find(expr)
                start = startLeer  + k
                ende = maxZeile
                if (zeile[maxZeile-1] == ';'):
                    ende = ende - 1
                return (expr, start, ende)
        else:
            return ('', start, ende)




    def mark(self,von,bis):
        t2.tag_remove(SEL,'1.0', END)  # alte Markierungen entfernen!
        t2.tag_add(SEL,von,bis)        
        #t2.update_idletasks()        
        
        
   
    def feedback(self,nachricht,nl=1):
        """ gibt eine <nachricht> im Hauptfenster aus
        """                
        pl = END
        if nl:
            t2.insert(END,str(nachricht)+'\n')
        else:
            t2.insert(END,str(nachricht) )
        t2.see(END)   
        #t2.focus_set() # funktioniert!


    def cl_unten(self):
        """>>> Nicht aufrufen, da sonst im Textfenster keine Expressions mehr vorhanden!!!!"""
        global t2
        t2.delete(1.0, END)
        t2.update_idletasks()

    def hilfe(self):
        """ laedt das Hilfe-Wiki in den Web-Browser
        """        
        pfad_ = abspath( self.home )
        os.chdir(pfad_)
        hilfeWiki = pfad_ + os.sep + 'docu' + os.sep + "pymaxima_wiki.htm"

        try: 
            webbrowser.open(hilfeWiki)
        except:
            tkMessageBox.showinfo("Browser-Fehler", "Konnte Hilfe-Wiki im Browser \n nicht anzeigen!")
        

    def lizenz(self):
        """ laedt die Lizenz-Datei in den Web-Browser
        """
        pfad_ = abspath( self.home )
        os.chdir(pfad_)
        htmlLizenz = pfad_ + os.sep + 'docu' + os.sep + "license.html"

        try: 
            webbrowser.open(htmlLizenz)
        except:
            tkMessageBox.showinfo("Browser-Fehler", "Konnte Lizenz im Browser \n nicht anzeigen!")



    def maximaManual(self):
        """ laedt das Maxima-Manual im Default-Browser
            Achtung: maximaDocHome ist global!"""
        try:
            if not exists(maximaDocHome):                            
                    raise # => wir probieren http://maxima.sourceforge.net/docs/manual/en/maxima.html
            else:
                self.maximaDocHome =  maximaDocHome 
                    
        except:
            #maximaDocHome = "http://maxima.sourceforge.net/"
            # http://maxima.sourceforge.net/docs/manual/en/maxima.html
            self.maximaDocHome = "http://maxima.sourceforge.net/docs/manual/en/maxima.html"
        #print self.maximaDocHome     
        try: 
            webbrowser.open(self.maximaDocHome)
        except:
            tkMessageBox.showinfo("Browser-Fehler", "Konnte Maxima-Manual im \nBrowser nicht anzeigen!")


    def beenden(self):   
        die()


    def delExpr(self):
        win = t2
        #self.feedback( str(SEL_FIRST)  + str(SEL_LAST) )
        von = t2.index(SEL_FIRST)
        bis = t2.index(SEL_LAST)
        #self.feedback( str(von) + '  ' + str(bis) )
        mini1 = von.split('.')
        nr1 = int(mini1[0])
        mini2 = bis.split('.')
        nr2 = int(mini2[0])
        if (nr1 != nr2):
            return
        else: # nr == nr2!
            #self.feedback( str(self.exprDici[nr][0])  )
            self.modDici(self.exprDici,nr1)
            self.showDici(self.exprDici,_clear_=1)
            self.pointer = len(self.exprDici)

    def prev(self, event):
        _debug_ = 0

        if self._debug_:
            _debug2_ = 1
        else:
            _debug2_ = 0            
        
	#maxDici = len(self.exprDici)
	#print maxDici
	
        if self.pointer > 0:
            if self.pointer > 1:
                self.pointer = self.pointer - 1
            else:   # self.pointer = 1, also von vorne
                self.pointer = len(self.exprDici)            
            #if (self.exprDici[self.pointer][1]  == 'O'):
            #    self.pointer = self.pointer - 1
            
            try:  # KeyError
                # self.pointer+1 => alter self.pointer
                expr = self.exprDici[self.pointer+1][0]
                ret = self.searchInNr(self.pointer+1)
                
                if _debug_:
                    self.feedback( expr + '   ' + str(self.pointer+1) )
                    self.feedback( ret )
                    self.exprZeile.focus_set()                
                    #return
                if _debug2_:
                    print "Methode prev: ",                
                    print ret,
                    print ", AUSDRUCK: " + str(expr)
                    
                if ret:    
                    try:    
                        self.markExpr(ret,expr)
                    except:
                        pass
                    self.exprZeile.delete(0,END)
                    self.exprZeile.insert(INSERT, expr)
                    self.exprZeile.focus_set()
                else:
                    self.exprZeile.delete(0,END)
                    self.exprZeile.focus_set()
            except:
                pass
            
  
	    
            
    def next(self, event):
        _debug_ = 0
        if self._debug_:
            _debug2_ = 1
        else:
            _debug2_ = 0            
        

        maxDici = len(self.exprDici)
	#print maxDici
        if self.pointer <= maxDici:
            if (self.pointer < maxDici):                
                self.pointer = self.pointer + 1
            else: # self.pointer = maxDici, also von vorne
                self.pointer = 1
            #if (self.exprDici[self.pointer][1]  == 'O'):
            #    self.pointer = self.pointer + 1                

            try:    
                expr = self.exprDici[self.pointer+1][0]
                ret = self.searchInNr(self.pointer+1)

                if _debug_:            
                    self.feedback( expr + '   ' + str(self.pointer+1) )
                    self.feedback( ret  )                
                    self.exprZeile.focus_set()
                    #return
                if _debug2_:
                    print "Methode next: ",
                    print ret,
                    print ", AUSDRUCK: " + str(expr)
                    
                if ret:    
                    try:    
                        self.markExpr(ret,expr)
                        #print 'ok'
                    except:
                        pass
                    self.exprZeile.delete(0,END)
                    self.exprZeile.insert(INSERT, expr)
                    self.exprZeile.focus_set()
                else:
                    self.exprZeile.delete(0,END)
                    self.exprZeile.focus_set() 
            except: # KeyError
                pass

            
    def searchInNr(self, nr):
        if self._debug_:
            _debug_ = 1
        else:
            _debug_ = 0            

        
        magic = '(%o'
        if  (string.find(self.exprDici[nr][1],'(%i') != -1):
            return '(%i' + str(nr-1) + ')'
        zeile = self.exprDici[nr][0]
        liste = zeile.split()
        if _debug_:
            print "Methode searchInNr: ",            
            print str(liste)
        for element in liste:
            if (string.find(element,magic) != -1):
                li2 = element.split(')')
                buildNr = li2[0].strip()
                buildNr = buildNr[3:] # zahl ohne '(%o'
                inNr = '(%i' + buildNr + ')'
                if _debug_:
                    print "searchInNr, gefunden: " + str(inNr)
                return inNr            

                    
                    
    def markExpr(self,inNrStr,expr):
        #get('1.0',END): holt den Text aus dem Textfenster ab der Position 1.0 bis zum ENDe,
        #die Schreibweise '1.0' steht fuer die 1. Zeile und die 0. Spalte.
        #Beispiel 1: textfenster.get('1.0',END) (holt den ganzen Text!)
        #Beispiel 2: textfenster.get('1.0','1.end') (holt 1. Zeile)
        #self.feedback('Nr: ' + str(nr) )
        if self._debug_:
            _debug_ = 1
        else:
            _debug_ = 0
            
        if _debug_:
            print "Methode markExpr: ",
            print expr
            print inNrStr
        expr = expr.strip()
        gefunden = 0
        start = '1.0'
        while not gefunden: # notwendig, da Hilfe-Texte auch den Prompt '(%i7)' enthalten!
            pos = t2.search(inNrStr, start, stopindex=END)
            nrLi = pos.split('.')
            if _debug_:
                print pos
            text = ''
            try:
                text = t2.get(pos,nrLi[0]+'.end')
                if _debug_:
                    print text
            except:
                pass
            if len(text) > 0:
                 #ret = test[len(inNrStr): ]
                 #ret = ret.strip()
                 #print ret
                 zeile = pos.split('.')[0]
                 nextLine = int(zeile) + 1
                 start = str(nextLine)+ '.0'
                 if _debug_:
                     print start
                 pos = string.find(text,expr)
                 if _debug_:
                     print pos
                 if (pos != -1):
                     gefunden = 1
            else: # len(text) == 0
                gefunden = 1

        von = nrLi[0] + '.' + str(pos)
        bis = nrLi[0] + '.end'
        #print von,bis
        t2.tag_remove(SEL,'1.0', END)  # alte Markierungen entfernen!         
        t2.tag_add(SEL,von,bis)         
        t2.see(bis) # markierten Ausdruck sichtbar machen
        t2.update_idletasks()
        t2.focus_set()
         

            
    def getInput(self):
        gefunden = 0
        eingabe = tkw.eingabefenster(root,"Wert-Eingabe","Wert: ")
        if eingabe.result <> None:
            ausdruck = self.addStrichpunkt(eingabe.result)
            if len(ausdruck) > 0:                
                #self.feedback(ausdruck)
                (text,self.nextIn,gefunden)  = maxima.call(ausdruck)
                #print text
                if (gefunden != 0):
                    self.ausdruckAusgeben(ausdruck,gefunden,text)
                return gefunden
            else:
                return 0
        else:
            return 0

    def getInput2_(self,event):
        self.getInput2()
    
    def getInput2(self):
        eingabe = self.eingabe.get() #self.exprZeile.get()
        if eingabe <> None:
            #ausdruck = str(eingabe)
            try:
                eingabe_iso = eingabe.decode('iso-8859-1')
            except:
                eingabe_iso = eingabe
            ausdruck = self.addStrichpunkt( eingabe_iso.encode('iso-8859-1') )
            if len(ausdruck) > 0:
                #self.feedback(ausdruck)
                ausdruck = self.addStrichpunkt(ausdruck)
                self.callMaxima(expression=ausdruck)
                #self.showInput(ausdruck)
                self.exprZeile.focus_set() 

    def showInput(self,ergebnis):
        global lineNr
        if len(ergebnis) > 0:
            self.nr = self.nr + 1
            if self.nr < 10:
                ausdruck = '#' + str(self.nr) +  ' ' +  ' ' + str(ergebnis) 
            else:
                ausdruck = '#' + str(self.nr) +  ' ' + str(ergebnis) 
            self.feedback(ausdruck)

    def letzterBefehl(self,event):
        try:
            marked_text = t2.get(SEL_FIRST, SEL_LAST)
            #self.feedback(marked_text)
            self.exprZeile.insert(INSERT, marked_text)
        except:
            pass

    def clearLabel(self):
            #newStatus = "Satus:   "
            #self.status.set(newStatus)
            self.exprZeile.delete(0,END) # 0.41
        
        

    def exprTest(self,expr):
        # Gibt eine Liste zurück: macht aus x:=2;3*x^2 ['x:=2','3*x^2']
        # vorläufig!!!
        import os, string
        _debug_ = 0
        
        neueListe = []
        treffer = 0
        gefunden = 0
        start = 0
        dollar = 0
        noexpr = 0
        expr = string.strip(expr,' ') # keine Leerzeichen am Anfang/Ende!
        maxLen = len(expr)
        j = 0
        while j < maxLen:
            if _debug_:
                print maxLen,j,start,expr[j]
            if (expr[j] == '"'): # Trennzeichen '"'
                for i in range(j+1,maxLen):
                    if (expr[i] == '"'): #  gefunden: ';"' oder '; "'
                            treffer = 0
                            dollar = 0
                            j = i + 1 # !!!!
                            break
            if _debug_:
                print maxLen,j,start,expr[j]
                
            if (expr[j] == ';') or (expr[j] == '$'): # Trennzeichen ';' oder "gnuplot;"?
                if (expr[j] == '$'):
                    dollar = 1
                
                if (j == maxLen-1) and (gefunden==0): # am Ende des Eingabestrings EINZIGES ';' gefunden
                    newExpr = expr
                    if _debug_:
                        print newExpr
                    neueListe.append(newExpr)
                    return neueListe # und Liste retour
                elif (j < maxLen-1):   # im Eingabestring ';' gefunden => Test auf ';"'
                    for i in range(j+1,maxLen):
                        if expr[i].isalnum():  # nach ';' kommt eine Zahl oder ein Buchstabe => nächster Ausdruck
                            treffer = 1
                            gefunden = 1
                            break
                        
                elif (j == maxLen-1):  # am Ende des Eingabestrings ';' gefunden
                    treffer = 1
                if treffer:
                    if _debug_:
                        print "Treffer: " + expr[start:j]
                    if dollar:
                        neueListe.append(expr[start:j]+'$')
                    else:
                        neueListe.append(expr[start:j]+';')
                    start = j + 1
                    treffer = 0
                    dollar = 0
            j = j + 1
        return neueListe                    


    def printExpression(self,expression,_nl_=1):        
        #zeile = str(expression)
        try:
            zeile = unicode(expression) # neu: 2007-01-14
        except:
            zeile = str(expression)
        self.feedback(zeile, nl=_nl_)
        t2.update_idletasks()
        self.exprZeile.focus_set()
        self.exprZeile.update_idletasks()

        
        
    def clear_all(self):
        self.bearbeiteAusdruck('kill(all);')
        self.cl_unten()
        self.exprDici.clear()
        util.kopiereDici({},self.exprDici)
        #self.exprDici = {}
        self.nr = 1
        self.altIn = '(%i1)'
        self.oldExpr = ''
        self.tauscheAusdruck = 0        
        self.printExpression( self.altIn + ' ', _nl_=0 )
        self.exprZeile.focus_set()        
        self.pointer = 0
        self.standard = []
        self.getStandardFkt()
        newStatus = setStatus(self.maximaOk)                
        self.status.set(newStatus)
        


    def getExpr(self):
        expr = ''
        win = self.exprZeile  
        try:
            #expr = t2.get(SEL_FIRST, SEL_LAST)
            expr = self.t.selection_get(selection="CLIPBOARD")
        except:
            expr = win.get()
        expr = expr.lower()    
        return expr

    def getVar(self,expr):
        """zieht eine Liste von Variablen aus dem Ausdruck <expr>"""
        liste = ['x']
        i = 0
        maxi = len(expr)
        if maxi > 1:
            for i in range(maxi):
                if (i == 0) and expr[i].isalpha() and not expr[i+1].isalpha():
                    liste.append(expr[i])
                elif (i == maxi-1) and expr[i].isalpha() and not expr[i-1].isalpha():
                    if not(expr[i] in liste):
                        liste.append(expr[i])
                else:
                    if expr[i].isalpha() and not expr[i-1].isalpha() and not expr[i+1].isalpha():
                        if not(expr[i] in liste):
                            liste.append(expr[i])
            if len(liste) < 1:
                return liste
            else:
                return liste
        elif (maxi == 1):
            if expr[0].isalpha():
                return [expr[0]]
            else:
                return liste
        else:
            return liste


                

    def stripStrichpunkt(self,arg):
            _arg_ = string.strip(arg)
            maxi = len(_arg_)
            if maxi > 0:
                if _arg_[maxi-1] == ';':
                    return _arg_[:-1]
                elif _arg_[maxi-1] == ',':
                    return _arg_[:-1]		
                else:
                    return arg
            else:
                return arg

    def addStrichpunkt(self,arg):
            _arg_ = string.strip(arg)
            maxi = len(_arg_)
            #print _arg_
            if maxi > 0:
                if (_arg_[maxi-1] != ';') and (_arg_[maxi-1] != '$'): # kein ';' am Ende UND kein '$' am Ende
                    return _arg_ + ';'
                else:
                    return arg
            else:
                return arg


###################################################
    def prolog(self):
        expr = ''
        varLi = []
        expr = self.getExpr()
        varLi = self.getVar(expr)
        varLi.sort()
        return (expr,varLi)        
        

    def display(self,v2d=1):
        if v2d:
            self.bearbeiteAusdruck('display2d:true$')
        else: # v2d == 0
            self.bearbeiteAusdruck('display2d:false$')


    def printMessage(self,nachricht):
        if self.maximaOk:
            (text,self.nextIn,gefunden)  = maxima.call('"' + nachricht  + '"$')
            self.ausdruckAusgeben('"' + nachricht  + '"$',gefunden,text)


    def maximaInfo(self):
        _vers_li_ = self.buildInfoMaximaVersion.split('.')
        _sub_version_ = int(_vers_li_[1])
        if _sub_version_ >= 27:  # maxima liefert ab Version 5.27 bei "build_info()$" keine Ausgabe mehr . . . 
            self.bearbeiteAusdruck('build_info();')
        else:
            self.bearbeiteAusdruck('build_info()$')

            
    def simplify(self):
        self.bearbeiteAusdruck('ratsimp(%);')

    def factor(self):
        self.bearbeiteAusdruck('factor(%);')


    def expand(self):
        self.bearbeiteAusdruck('expand(%);')


    def squareFree(self):
        self.bearbeiteAusdruck('sqfr(%);')

        
    def evalExpr(self):
        self.bearbeiteAusdruck('ev(%,numer);')

    def allRoots(self):
        self.bearbeiteAusdruck('allroots(%);')

    def realRoots(self):
        self.bearbeiteAusdruck('realroots(%);')



    def getValues(self,_maxLaenge_=4):
        """gibt eine Liste von aktuellen Variablen zurück
        _maxLaenge_ : legt max. Namenslänge der Variablen fest  <== !!!
        """
        maxLength = _maxLaenge_   
        if self.maximaOk:
            varLi = []
            (text,self.nextIn,gefunden)  = maxima.call('values;')            
            self.ausdruckAusgeben('values;',gefunden,text)
            pos = text.find('[')
            last = len(text)
            valStr = text[pos+1:].strip()
            valStr = valStr[:-1]
            if len(valStr) > 0:
                    valListe = valStr.split(',')
            else:
                    valListe = []
            if len(valListe) > 0:
                for element in valListe:
                    val = element.strip()
                    if len( val ) < maxLength:
                        varLi.append( val )
            else: # keine Variablen => return ["%th(2)"]  (nicht "%" , da zuvor Aufruf von "value()" )
                varLi.append( "%th(2)" )
            return varLi
                



#################################

    def runScript(self,default=False):
        """Startet pyGeo-Script"""
        tmpGeoFile = "_temp_.py"        
        datei = _pygeodir_  + os.sep + tmpGeoFile
        self.bearbeiteAusdruck('"Starte PyGeo"$')
        if default:
            #pyGeoFile =  self.home + "pygeo" + os.sep + tmpGeoFile
            pyGeoFile = _pygeodir_  + os.sep + tmpGeoFile
        else:
            pyGeoFile = askopenfilename(filetypes=[("Python", "*.py"),("Alle Dateien","*.*")],initialdir=_pygeodir_)                        
        if exists(pyGeoFile):                
            _path2python_  = pfad_zu_python # pfad_zu_python is global
            
            if len(_path2python_) > 0: 
                if os.name == 'posix':
                    self.t.iconify()  # hide pyMaxima
                    batch_String =  _path2python_ + " " + pyGeoFile
                else: # win nt
                    # Achtung: Aufruf von python bzw. pyGeoFile unter Windows wg. Leerzeichen maskieren!
                    #batch_String = 'start "' + _path2python_ + '"  "' +  pyGeoFile + '"'
                    batch_String = 'start ' + util.protectBlank(_path2python_) + ' '  +  util.protectBlank(pyGeoFile)
                #print batch_String
                try:
                    system( batch_String )
                    #
                except:
                    pass                    
                self.t.deiconify()
                self.t.update()        
    
    def saveScript(self):
        """speichert aktuelles Skript: self.script """
        if len(self.script) > 0:        
            scriptname = asksaveasfilename(filetypes=[("Python", "*.py"),("Alle Dateien","*.*")],initialdir=_pygeodir_)
            if scriptname:
                if (not self.test_extension(scriptname,".py")):
                    scriptname = scriptname + ".py"                
                geoScript = pymax2geo.generateScript(scriptList=self.script)
                try:      
                    out_file = open(scriptname,"w")
                    out_file.write(geoScript) # text in Datei schreiben
                    out_file.close()     # Datei schliessen
                except:
                    pass
    
    def saveObjects(self):
        """speichert die aktuellen pyGeo-Objekte aus self.geoOops.geoDir"""
        if len(self.geoOops.geoDir) > 0:        
            geoObjectsFile = asksaveasfilename(filetypes=[("pyGeo Objects", "*.pgo"),("Alle Dateien","*.*")],initialdir=_pygeodir_)
            if geoObjectsFile:
                if (not self.test_extension(geoObjectsFile,".pgo")):
                    geoObjectsFile = geoObjectsFile + ".pgo"                
                pymax2geo.saveGeoOops(geoObjectsFile,self.geoOops.geoDir)

    def infoObjectDir(self):
        """gibt Info zum  pyGeo-Objekte aus dem Objekte-Directory self.geoOops.geoDir
        """
        if self.maximaOk:
            if len(self.geoOops.geoDir) > 0:
                liste = []
                geoKeys = []
                liste,geoKeys = self.geoOops.getSimpleDir()
                # In "geoKeys" steht der Schluessel (= Name) des pyGeo-Objektes
                # In der Liste "liste" steht die Beschreibung der pyGeo-Objekte
                # Beispiel-Liste
                # [
                ##
                ##['0', 'D1', 'Dreieck', '([-3,0,0]), ([0,-3,0]), ([0,0,-3])'], 
                ##['0', 'Py1', 'Pyramide', '([1,0,0]), ([0,1,0]), ([1,2,0]), ([2,1,0]), ([1,1,3])'], 
                ##['0', 'Qu1', 'Quader', '([1.0,0.0,0.0]), ([1.0,1.0,0.0]), ([0.0,1.0,0.0]), ([0.0,0.0,0.0]), ([1.0,0.0,1.0]), ([1.0,1.0,1.0]), ([0.0,1.0,1.0]), ([0.0,0.0,1.0])'], 
                ##['0', 'T1', 'Tetraeder', '([1,0,0]), ([0,1,0]), ([0,0,1]), ([1.732,1.732,1.732])'], 
                ##['0', 'V2', 'Viereck', '([0,0,0]), ([0,5,0]), ([0,5,5]), ([0,0,5])']
                ##
                ##]
                # Aufbau der Liste 'liste':
                # - Elemente der Liste sind Listen
                # - Jedes Element (= Objekt-Liste) beschreibt ein pyGeo-Objekt
                #
                # Aufbau der Element-Liste (= pyGeo-Objekt-Liste):
                # (1) cb-Flag = Check-Button-Flag, falls Objekt ausgewaehlt wurde
                # (2) Name des pyGeo-Objektes => die Namen stehen als Schluessel auch in der Liste "geoKeys"
                # (3) Typ des pyGeo-Objektes => 'Dreieck', 'Viereck' oder 'Quader' usw.
                # (4) Tupel mit den entsprechenden Punkten, die Laenge dieses Tupels ist variabel
                # (4.1) Jeder Punkt ist wiederum eine Liste mit drei Elementen (= Koordinaten/Zahlen): '([x,y,z])'
                # Anmerkung zu (4): Das Tupel (von Punkten) wird als String abgespeichert                

                headers = ['Info','Name','Typ','Beschreibung']
                
                ergebnis = tkw.geoTable(root,liste,headers)  # Objekte anzeigen
                # print liste
                
                # Beispiel: ergebnis == [1, 1, 0, 1, 0]
                try:
                    (anzahl,ort) = util.ergebnisTest(ergebnis)
                    if (anzahl == 1):
                        # hole Name, Typ & Maxima-Repraesentation => abhaengig vom Typ Info-Methode aufrufen
                        maximaName = self.geoOops.getGeoName(geoKeys[ort]) # Objekt-Name
                        #print maximaName
                        geoTyp = self.geoOops.getGeoType(geoKeys[ort]) # Objekt-Typ
                        #print geoTyp
                        maxRep = self.geoOops.getMaximaEintrag(geoKeys[ort])
                        #print maxRep
                        for element in maxRep:
                            #print element
                            self.bearbeiteAusdruck( element )                                            
                        if geoTyp == "Ebene":
                            #print maxRep
                            if len(maxRep) < 6:
                                self.ebenenInfo(maximaName,"nebene")
                            else:                                
                                self.ebenenInfo(maximaName,"pebene")
                        elif geoTyp == "Kugel":
                            self.kugelInfo(maximaName)
                        elif geoTyp == "Gerade":
                            self.geradenInfo(maximaName)
                        elif geoTyp == "Punkt":
                            pScript = self.geoOops.getScriptEintrag(maximaName)                            
                            pVec = self.geoOops.script2Intern(pScript)[0]
                            self.punktInfo(pVec,maximaName)
                        elif geoTyp == "Kreis":
                            self.kreisInfo(maximaName)
                        elif geoTyp == "Strecke":                            
                            self.streckenInfo(maximaName)                            
                        elif geoTyp == "Dreieck":
                            self.dreieckInfo(maximaName)
                        elif geoTyp == "Viereck":
                            self.viereckInfo(maximaName)
                        elif geoTyp == "Quader":
                            self.quaderInfo(maximaName)                            
                        elif geoTyp == "Tetraeder":
                            self.tetraederInfo(maximaName)
                        elif geoTyp == "Pyramide":
                            self.pyramidenInfo(maximaName)
                            
                        elif geoTyp == "Vektor":                            
                            if len(maxRep) > 1:
                                vecLi1  = maxRep[1].split("[")
                                vecLi2 = vecLi1[1].split("]")
                                vec = vecLi2[0].split(",")
                                self.vektorInfo(vec,maximaName)                                                            
                            
                    else:
                        print "Too much . . ."
                except:
                    pass
            
    def ebenenInfo(self,e_name = "E1",ebenenTyp="pebene"):
        self._ebene_(e_name,ebenenTyp)

            
    def geradenInfo(self,g_name = "g1"):
        self._gerade_(g_name)

    def punktInfo(self,pVec,p_name = "A"):
        self._betrag_(pVec,p_name)

    def kugelInfo(self,k_name = "K1"):
        self._kugel_(k_name)

    def streckenInfo(self,s_name = "S1"):
        #print "call streckenInfo"
        pScript = self.geoOops.getScriptEintrag(s_name)
        punktListe = pScript[:-1] # hole nur Punkte
        zweiPuList = []
        for element in punktListe:
            elLi = []
            elLi.append(element) # script2Intern erwartet eine Liste                           
            pVec = self.geoOops.script2Intern(elLi,1)[0]
            #print pVec
            zweiPuList.append(pVec) # zweiPuList aufbauen
        #print zweiPuList
        ret = descartes.strecke(s_name,zweiPuList[0],zweiPuList[1])
        #print ret       
        self.bearbeiteAusdruck('"Laenge l der Strecke ' + s_name +  ': l =' + ret[1] + '"$')            
               

    def dreieckInfo(self,d_name = "D1"):
        #print "Dreieck"
        pScript = self.geoOops.getScriptEintrag(d_name)
        punktListe = pScript[:-3] # hole nur Punkte
        dreiPuList = []
        for element in punktListe:
            elLi = []
            elLi.append(element) # script2Intern erwartet eine Liste                           
            pVec = self.geoOops.script2Intern(elLi,1)[0]
            dreiPuList.append(pVec) # dreiPuList aufbauen
        ret = descartes.dreieck(d_name,dreiPuList[0],dreiPuList[1],dreiPuList[2])
        for i in range(0,len(ret)-1,2): # Dreieck-Daten ausgeben
            self.bearbeiteAusdruck('"' +  str(ret[i]) + str(ret[i+1]) + '"$')
        self.bearbeiteAusdruck( str(ret[-1]) + ';' )  # Schwerpunkt am Ende
        #
        eNF = descartes.dreieckEbeneNF(dreiPuList[0],dreiPuList[1],dreiPuList[2])  # eNF[0] == nvec, eNF[1] = _d_
        if eNF:
            self.bearbeiteAusdruck('"Koordinatenform der Dreiecks-Ebene E_' + d_name + '"$')
            self.bearbeiteAusdruck('"' + descartes.koordinatenform(eNF[0],eNF[1]) + '"$')
            #print descartes.koordinatenform(eNF[0],eNF[1])
        

    def tetraederInfo(self,t_name = "T1"):
        pScript = self.geoOops.getScriptEintrag(t_name)
        punktListe = pScript[:-7] # hole nur Punkte
        vierPuList = []
        for element in punktListe:
            elLi = []
            elLi.append(element) # script2Intern erwartet eine Liste                           
            pVec = self.geoOops.script2Intern(elLi,1)[0]
            pVec = descartes.vec2float(pVec)
            vierPuList.append(pVec) # dreiPuList aufbauen
        #print vierPuList    
        eNF = descartes.dreieckEbeneNF(vierPuList[0],vierPuList[1],vierPuList[2])  # eNF[0] == nvec, eNF[1] = _d_
        #print eNF
        if (descartes.punktProbeE(eNF[0],eNF[1],vierPuList[3]) == 0): 
            area1 = descartes.dreieckFlaeche(vierPuList[0],vierPuList[1],vierPuList[2])
            area2 = descartes.dreieckFlaeche(vierPuList[0],vierPuList[1],vierPuList[3])
            area3 = descartes.dreieckFlaeche(vierPuList[0],vierPuList[2],vierPuList[3])
            area4 = descartes.dreieckFlaeche(vierPuList[1],vierPuList[2],vierPuList[3])        
            area = area1 + area2 + area3 + area4
            area_str = "%16.12f" % area
            self.bearbeiteAusdruck('"Oberflaeche O=' + area_str + '"$')

            vol = descartes.tetraederVol(vierPuList[0],vierPuList[1],vierPuList[2],vierPuList[3])
            self.bearbeiteAusdruck('"Volumen V=' + vol + '"$')
        else:
            print "Hhmm . . . \nKein Tetraeder, da die vier Punkte in einer Ebene liegen!"
            


    def quaderInfo(self,v_name = "Q1"):        
        pScript = self.geoOops.getScriptEintrag(v_name)
        #print pScript
        punktListe = pScript[:-13] # hole nur Punkte
        #print punktListe

        vierPuList = []
        for element in punktListe:
            elLi = []
            elLi.append(element) # script2Intern erwartet eine Liste                           
            pVec = self.geoOops.script2Intern(elLi,1)[0]
            vierPuList.append(pVec) # dreiPuList aufbauen
        #print str(vierPuList)
            
        area01 = descartes.viereckFlaeche(vierPuList[0],vierPuList[1],vierPuList[2],vierPuList[3])
        self.bearbeiteAusdruck('"Fläche A= ' + str(area01) + '"$')
        area02 = descartes.viereckFlaeche(vierPuList[0],vierPuList[1],vierPuList[4],vierPuList[5])
        self.bearbeiteAusdruck('"Fläche B= ' + str(area02) + '"$')
        area03 = descartes.viereckFlaeche(vierPuList[1],vierPuList[2],vierPuList[5],vierPuList[6])
        self.bearbeiteAusdruck('"Fläche C= ' + str(area03) + '"$')
        oberfl = 2 * area01 + 2 * area02 + 2 * area03
        self.bearbeiteAusdruck('"Oberfläche O= ' + str(oberfl) + '"$')              
##        
        a = descartes.abstandPP(vierPuList[0],vierPuList[1])
        self.bearbeiteAusdruck('"Seite a= ' + str(a) + '"$')   
        b = descartes.abstandPP(vierPuList[1],vierPuList[2])
        self.bearbeiteAusdruck('"Seite b= ' + str(b) + '"$')   
        h = descartes.abstandPP(vierPuList[0],vierPuList[4])
        self.bearbeiteAusdruck('"Höhe h= ' + str(h) + '"$')   
        volumen = float(a) * float(b) * float(h)
        self.bearbeiteAusdruck('"Volumen V= ' + str(volumen) + '"$')                
        

    def viereckInfo(self,v_name = "V1"):        
        pScript = self.geoOops.getScriptEintrag(v_name)
        punktListe = pScript[:-5] # hole nur Punkte
        vierPuList = []
        for element in punktListe:
            elLi = []
            elLi.append(element) # script2Intern erwartet eine Liste                           
            pVec = self.geoOops.script2Intern(elLi,1)[0]
            vierPuList.append(pVec) # dreiPuList aufbauen
        #print str(vierPuList)
            
        area = descartes.viereckFlaeche(vierPuList[0],vierPuList[1],vierPuList[2],vierPuList[3])
        self.bearbeiteAusdruck('"Fläche A= ' + str(area) + '"$')
        
        a = descartes.abstandPP(vierPuList[0],vierPuList[1])
        b = descartes.abstandPP(vierPuList[1],vierPuList[2])
        c = descartes.abstandPP(vierPuList[2],vierPuList[3])
        d = descartes.abstandPP(vierPuList[3],vierPuList[0])
        
        umfang = float(a)+float(b)+float(c)+float(d)
        self.bearbeiteAusdruck('"Umfang U= ' + str(umfang) + '"$')
##        self.bearbeiteAusdruck('"Seite a=' + str(a) + '"$')
##        self.bearbeiteAusdruck('"Seite b=' + str(b) + '"$')
##        self.bearbeiteAusdruck('"Seite c=' + str(c) + '"$')
##        self.bearbeiteAusdruck('"Seite d=' + str(d) + '"$')
        self.bearbeiteAusdruck('"Seite a=' + str(a) + ', Seite b=' + str(b) + ', Seite c=' + str(c) + ', Seite d=' + str(d) + '"$')

        
        eNF = descartes.dreieckEbeneNF(vierPuList[0],vierPuList[1],vierPuList[2])  # eNF[0] == nvec, eNF[1] = _d_
        if eNF:
            self.bearbeiteAusdruck('"Koordinatenform der Vierecks-Ebene E_' + v_name + '"$')
            self.bearbeiteAusdruck('"' + descartes.koordinatenform(eNF[0],eNF[1]) + '"$')
            #print descartes.koordinatenform(eNF[0],eNF[1])
        

    def pyramidenInfo(self,t_name = "Py1"):
        pScript = self.geoOops.getScriptEintrag(t_name)
        punktListe = pScript[:-9] # hole nur Punkte
        #print punktListe
        
        vierPuList = []
        for element in punktListe:
            elLi = []
            elLi.append(element) # script2Intern erwartet eine Liste                           
            pVec = self.geoOops.script2Intern(elLi,1)[0]
            pVec = descartes.vec2float(pVec)
            vierPuList.append(pVec) # dreiPuList aufbauen
            
        #print vierPuList
        eNF = descartes.dreieckEbeneNF(vierPuList[0],vierPuList[1],vierPuList[2])  # eNF[0] == nvec, eNF[1] = _d_
        #print eNF
        if (descartes.punktProbeE(eNF[0],eNF[1],vierPuList[4]) == 0): 
            area1 = descartes.dreieckFlaeche(vierPuList[0],vierPuList[1],vierPuList[4])
            area2 = descartes.dreieckFlaeche(vierPuList[1],vierPuList[2],vierPuList[4])                    
            area3 = descartes.dreieckFlaeche(vierPuList[2],vierPuList[3],vierPuList[4])
            area4 = descartes.dreieckFlaeche(vierPuList[3],vierPuList[0],vierPuList[4])
            
            #print area3

            grund = descartes.viereckFlaeche(vierPuList[0],vierPuList[1],vierPuList[2],vierPuList[3])
            
            area = area1 + area2 + area3 + area4 + grund
            area_str = "%16.12f" % area
            self.bearbeiteAusdruck('"Oberfläche O=' + area_str + '"$')

            vol = descartes.pyramidenVol(vierPuList[0],vierPuList[1],vierPuList[2],vierPuList[3],vierPuList[4])
            # letztes Argument: Spitze der Pyramide!
            self.bearbeiteAusdruck('"Volumen V=' + vol + '"$')
            hoehe = descartes.pyramidenHoehe(vierPuList[0],vierPuList[1],vierPuList[2],vierPuList[3],vierPuList[4])
            # letztes Argument: Spitze der Pyramide!
            self.bearbeiteAusdruck('"Höhe H=' + hoehe + '"$')
            grund_str = "%16.12f" % grund            
            self.bearbeiteAusdruck('"Grundfläche G=' + grund_str + '"$')
        else:
            print "Hhmm . . . \nKeine Pyramide, da die fünf Punkte in einer Ebene liegen!"        
        



    def kreisInfo(self,k_name = "k1"):
        """ToDo ==>> kreisInfo: Umfang, Flaeche, Mittelpunkt, radius """
        pScript = self.geoOops.getScriptEintrag(k_name)
        #print pScript
        pRep = self.geoOops.script2Intern(pScript,3)
        #print pRep
        #eNF = descartes.dreieckEbeneNF(pRep[0],pRep[1],pRep[2])  # eNF[0] == nvec, eNF[1] = _d_
        eNF = descartes.ebeneNF(pRep)        
        self.bearbeiteAusdruck('"Koordinatenform der Kreis-Ebene E_' + k_name + '"$')
        self.bearbeiteAusdruck('"' + descartes.koordinatenform(eNF[0],eNF[1]) + '"$')
        
        erg = descartes.kreis(k_name,pRep[0],pRep[1],pRep[2])
        mittelpunkt = erg[0]
        rad = erg[1]
        _umfang_ = 2 * float(rad) * math.pi
        umfang = "%16.12f" % _umfang_    
        self.bearbeiteAusdruck('"Umfang U des Kreises ' + k_name + ': U = ' + umfang + '"$')
        _flaeche_ = float(rad) * float(rad) * math.pi
        flaeche = "%16.12f" % _flaeche_            
        self.bearbeiteAusdruck('"Fläche A des Kreises ' + k_name + ': A = ' + flaeche + '"$')

        


    def vektorInfo(self,vec,v_name = "v1"):
        ergebnis = descartes.betrag(vec)        
        self.bearbeiteAusdruck('"Länge des Vektors ' +  v_name  + '"$')
        #self.bearbeiteAusdruck(str(ergebnis)  + ' $')
        self.bearbeiteAusdruck('sqrt(' + v_name + " . " + v_name  + ');')
        self.bearbeiteAusdruck('ev(% ,numer );')
        #self._vektor_(v_name)        


    def nameInObjectDir(self,name):
        if len(self.geoOops.geoDir) > 0:        
            nameList = self.geoOops.getAllObjectNameList()
            #print "Hier nameInObjectDir . . . "
            if name in nameList:
                #print "Problem"
                return 1
            else:
                return 0

    def deleteObjects(self):
        """löscht pyGeo-Objekte aus dem Objekte-Directory self.geoOops.geoDir"""
        if len(self.geoOops.geoDir) > 0:
            liste = []
            geoKeys = []
            liste,geoKeys = self.geoOops.getSimpleDir() 

            headers = ['Delete','Name','Typ','Beschreibung']
            
            ergebnis = tkw.geoTable(root,liste,headers)  # Objekte anzeigen
            for j in range(len(ergebnis)):  
                if ergebnis[j]==1:
                    self.geoOops.delGeoDirEntry(geoKeys[j]) # Objekt löschen
            

    def deleteObjectDir(self):
        """löscht das komplette Objekte-Directory self.geoOops.geoDir (!)"""
        # was fehlt: clear u. s. w., eventuell nur fuer pyGeo Objekte . . . 
        if len(self.geoOops.geoDir) > 0:
            self.geoOops.clearGeoDir() # self.geoOops.geoDir löschen!
            

    def getObjects(self):
        """liest eine *.pgo (=pyGeo-Objects)-Datei und kopiert sie in self.geoOops.geoDir"""
        # was fehlt: Uebernahme der Objekte in das Main-Fenster; eventuell: clear u. s. w.
        tmpDici = {}
        geoObjectsFile = askopenfilename(filetypes=[("pyGeo Objects", "*.pgo"),("Alle Dateien","*.*")],initialdir=_pygeodir_)
        if geoObjectsFile:
            tmpDici = pymax2geo.getGeoOops(geoObjectsFile) # lesen,
            #print str(tmpDici)
            util.kopiereDici(tmpDici,self.geoOops.geoDir) # kopieren und
            # anzeigen:
            liste = []
            geoKeys = []
            liste,geoKeys = self.geoOops.getSimpleDir() 
            headers = ['_','Name','Typ','Beschreibung']
            cbFlag = 0
            _cancelButton_ = 0
            tkw.geoTable(root,liste,headers[1:],cbFlag,_cancelButton_)  # Objekte anzeigen ohne Checkbuttons (das geht . . .)


##    def appendObjects(self,_OverWrite_):
##        """liest eine *.pgo (=pyGeo-Objects)-Datei und kopiert sie in self.geoOops.geoDir"""
##        # was fehlt: Uebernahme der Objekte in das Main-Fenster; eventuell: clear u. s. w.
##        tmpDici = {}
##        geoObjectsFile = askopenfilename(filetypes=[("pyGeo Objects", "*.pgo"),("Alle Dateien","*.*")],initialdir=_pygeodir_)
##        if geoObjectsFile:
##            tmpDici = pymax2geo.getGeoOops(geoObjectsFile) # lesen,
##            #print str(tmpDici)
##            if (_OverWrite_ == 1):
##                util.appendDici(tmpDici,self.geoOops.geoDir,0) # kopieren und
##            else:
##                util.appendDici(tmpDici,self.geoOops.geoDir,1) # kopieren und
##            # anzeigen:
##            liste = []
##            geoKeys = []
##            liste,geoKeys = self.geoOops.getSimpleDir() 
##            headers = ['_','Name','Typ','Beschreibung']
##            cbFlag = 0
##            _cancelButton_ = 0
##            tkw.geoTable(root,liste,headers[1:],cbFlag,_cancelButton_)  # Objekte anzeigen ohne Checkbuttons (das geht . . .)



    def saveAppendObjects(self):
        """liest eine *.pgo (=pyGeo-Objects)-Datei und kopiert sie in self.geoOops.geoDir"""
        # was fehlt: Uebernahme der Objekte in das Main-Fenster; eventuell: clear u. s. w.
        appendDici = {}
        geoObjectsFile = askopenfilename(filetypes=[("pyGeo Objects", "*.pgo"),("Alle Dateien","*.*")],initialdir=_pygeodir_)
        if geoObjectsFile:
            appendDici = pymax2geo.getGeoOops(geoObjectsFile) # lesen,
            #print str(tmpDici)
            
            self.geoOops.saveAppendGeoDir(appendDici) # anhaengen und
            
            # anzeigen:
            liste = []
            geoKeys = []
            liste,geoKeys = self.geoOops.getSimpleDir() 
            headers = ['_','Name','Typ','Beschreibung']
            cbFlag = 0
            _cancelButton_ = 0
            tkw.geoTable(root,liste,headers[1:],cbFlag,_cancelButton_)  


    def editObjects(self):
        """ediitieren von pyGeo-Objekten (aus dem Objekte-Directory self.geoOops.geoDir)"""        
        if len(self.geoOops.geoDir) > 0:
            liste = []
            geoKeys = []
            liste,geoKeys = self.geoOops.getSimpleDir()
            #print "Liste"
            #print liste
            #print "geoKeys"
            #print geoKeys
            # In "geoKeys" steht der Schluessel (= Name) des pyGeo-Objektes
            # In der Liste "liste" steht die Beschreibung der pyGeo-Objekte
            # Beispiel-Liste
            # [
            ##
            ##['0', 'D1', 'Dreieck', '([-3,0,0]), ([0,-3,0]), ([0,0,-3])'], 
            ##['0', 'Py1', 'Pyramide', '([1,0,0]), ([0,1,0]), ([1,2,0]), ([2,1,0]), ([1,1,3])'], 
            ##['0', 'Qu1', 'Quader', '([1.0,0.0,0.0]), ([1.0,1.0,0.0]), ([0.0,1.0,0.0]), ([0.0,0.0,0.0]), ([1.0,0.0,1.0]), ([1.0,1.0,1.0]), ([0.0,1.0,1.0]), ([0.0,0.0,1.0])'], 
            ##['0', 'T1', 'Tetraeder', '([1,0,0]), ([0,1,0]), ([0,0,1]), ([1.732,1.732,1.732])'], 
            ##['0', 'V2', 'Viereck', '([0,0,0]), ([0,5,0]), ([0,5,5]), ([0,0,5])']
            ##['0', 'S1', 'Strecke', '([0.0,0.0,0.0]), ([1.0,2.0,3.0])']
            ##
            ##]
            # Aufbau der Liste 'liste':
            # - Elemente der Liste sind Listen
            # - Jedes Element (= Objekt-Liste) beschreibt ein pyGeo-Objekt
            #
            # Aufbau der Element-Liste (= pyGeo-Objekt-Liste):
            # (1) cb-Flag = Check-Button-Flag, falls Objekt ausgewaehlt wurde
            # (2) Name des pyGeo-Objektes => die Namen stehen als Schluessel auch in der Liste "geoKeys"
            # (3) Typ des pyGeo-Objektes => 'Dreieck', 'Viereck' oder 'Quader' usw.
            # (4) Tupel mit den entsprechenden Punkten, die Laenge dieses Tupels ist variabel
            # (4.1) Jeder Punkt ist wiederum eine Liste mit drei Elementen (= Koordinaten/Zahlen): '([x,y,z])'
            # Anmerkung zu (4): Das Tupel (von Punkten) wird als String abgespeichert

            headers = ['Edit','Name','Typ','Beschreibung']
            
            ergebnis = tkw.geoTable(root,liste,headers)  # Objekte anzeigen
            
            try:
                (anzahl,ort) = util.ergebnisTest(ergebnis)
                #print anzahl, ort
                if (anzahl == 1):
                    # hole Name, Typ & Maxima-Repraesentation => abhaengig vom Typ Info-Methode aufrufen
                    maximaName = self.geoOops.getGeoName(geoKeys[ort]) # Objekt-Name
                    geoTyp = self.geoOops.getGeoType(geoKeys[ort]) # Objekt-Typ
                    maxRep = self.geoOops.getMaximaEintrag(geoKeys[ort])

                    if geoTyp == "Vektor":
                        v_eintrag = self.geoOops.geoDir[maximaName]
                        if len(v_eintrag) > self.geoOops._maxPosten_: # Kriterium fuer Vektor durch zwei Punkte
                            _vIntern_ = v_eintrag[-1]
                            _vIntern_ = _vIntern_[1:]
                            #print _vIntern_
                            self.pVektor(edit=1,vec_name=maximaName,_vector_= _vIntern_)                            
                        elif len(maxRep) > 1:   # Ortsvektor
                            vecLi1  = maxRep[1].split("[")
                            vecLi2 = vecLi1[1].split("]")
                            vec = vecLi2[0].split(",")
                            self.vektor(edit=1,vec_name=maximaName,_vector_=vec)
                            
                    elif geoTyp == "Ebene":
                        pScript = self.geoOops.getScriptEintrag(maximaName)
                        pRep = self.geoOops.script2Intern(pScript,3)
                        if len(maxRep) < 6: # Ebene in Normalform
                            _normalformFlag_ = False # Voreinstellung: Koordinatenform
                            e_eintrag = self.geoOops.geoDir[maximaName]
                            
                            if len(e_eintrag) > self.geoOops._maxPosten_:
                                _eInt_ = e_eintrag[-1]
                                # Ebene in Normalenform oder in Koordinatenform
                                _ebenentyp_ = _eInt_[0][0]
                                if (string.find(_ebenentyp_,"nebene",0) >= 0):
                                    _normalformFlag_ = True
                                    
                                if len(_eInt_) == 2: # color-mode
                                    #print _eInt_
                                    _eIntern_ = _eInt_[0][1:]
                                    #print _eIntern_  # nvec, d                            
                                    nvec = _eIntern_[0]
                                    _d_ = _eIntern_[1]
                                    color = _eInt_[1][1]
                                    #print color
                                else:
                                    _eIntern_ = _eInt_[1:]
                                    nvec = _eIntern_[0]
                                    _d_ = _eIntern_[1]            
                                    color = 0
                                    
                            else: # "alte" Vorgehensweise 
                                _eIntern_ = descartes.ebeneNF(pRep)
                                nvec = _eIntern_[0]
                                _d_ = _eIntern_[1]
                                #print _eIntern_
                                color = 0
                                
                            if _normalformFlag_:    
                                self.nebene(edit=1,e_name=maximaName,_nvec_= nvec,_dn_=_d_,_color_=color )
                            else:
                                self.kebene(edit=1,e_name=maximaName,_nvec_= nvec,_dn_=_d_,_color_=color )    
                        else:
                            e_eintrag = self.geoOops.geoDir[maximaName]
                            if len(e_eintrag) > self.geoOops._maxPosten_: # Kriterium fuer Ebene in Parameterform
                                if (e_eintrag[-1][0][0]) == "pebene":
                                    _eInt_ = e_eintrag[-1]
                                    if len(_eInt_) == 2: # color-mode                                
                                        _eIntern_ = _eInt_[0][1:]
                                        color = _eInt_[1][1]                                    
                                    else: 
                                        _eIntern_ = _eInt_[1:]
                                        color = 0                                    
                                    #print _eIntern_  # aufpunkt, uvec, vvec
                                    self.pebene(edit=1,e_name=maximaName,_paramListe_= _eIntern_,_color_=color)
                                    
                                else: #Ebene aus drei Punkten A, B, C ==>> (e_eintrag[-1][0][0]) == "ebene" !!!
                                    _eInt_ = e_eintrag[-1]
                                    if len(_eInt_) == 2: # color-mode                                
                                        color = _eInt_[1][1]                                    
                                    else:
                                        color = 0                                    
                                    self.punktEbene(edit=1,e_name=maximaName,_puListe_= pRep,_color_=color)
                                    
                            else: # Ebene aus drei Punkten A, B, C
                                # in pRep stehen drei Punkte der Ebene                                
                                color = 0
                                self.punktEbene(edit=1,e_name=maximaName,_puListe_= pRep,_color_=color)
                        
                            
                            
                    elif geoTyp == "Kugel":
                        k_eintrag = self.geoOops.geoDir[maximaName]
                        #print k_eintrag
                        color = 0
                        if len(k_eintrag) > self.geoOops._maxPosten_: # Kriterium fuer Kugel aus Mittel-und Oberflächenpunkt
                            _kInt_ = k_eintrag[-1]
                            if len(_kInt_) == 2: # => neu, ab Vers. 1.33 wg. Farbverwaltung
                                # Kuerzel: 'rkm', 'kmo', 'k4p'
                                color = _kInt_[1][1]
                                _kIntern_ = _kInt_[0][1:]
                                _ObjectType_ = _kInt_[0][0] # _ObjectType_ aus {'rkm', 'kmo', 'k4p'}
                                if _ObjectType_ == "rkm":
                                    mittelpunkt = _kIntern_[0]
                                    rad = _kIntern_[1]
                                    self.rKugel(edit=1,k_name=maximaName,_m_punkt_=mittelpunkt,_rad_=rad,_color_=color)
                                elif _ObjectType_ == "kmo":
                                    self.kugel(edit=1,k_name=maximaName,_paramListe_= _kIntern_,_color_=color)
                                elif _ObjectType_ == "k4p":
                                    self.kugel4(edit=1,k_name=maximaName,_paramListe_= _kIntern_,_color_=color)
                                else:
                                    pass # da kann was nicht stimmen . . .
                            else:
                                _kIntern_ = _kInt_[1:]
                                #print _kIntern_  # Mittelpunkt, Oberflächenpunkt                            
                                if len(_kIntern_) <= 2: # Mittelpunkt, Oberflächenpunkt
                                    self.kugel(edit=1,k_name=maximaName,_paramListe_= _kIntern_,_color_=color)
                                else: # Kugel durch 4 Punkte
                                    self.kugel4(edit=1,k_name=maximaName,_paramListe_= _kIntern_,_color_=color)
                        else:                        
                            pScript = self.geoOops.getScriptEintrag(maximaName)
                            pRep = self.geoOops.script2Intern(pScript,2)
                            mittelpunkt = pRep[0]
                            rad = descartes.getRadius(mittelpunkt,pRep[1])
                            self.rKugel(edit=1,k_name=maximaName,_m_punkt_=mittelpunkt,_rad_=rad,_color_=color)
                        
                    elif geoTyp == "Gerade":
                        g_eintrag = self.geoOops.geoDir[maximaName]
                        if len(g_eintrag) > self.geoOops._maxPosten_: # Kriterium fuer Gerade durch zwei Punkte
                            if (g_eintrag[-1][0][0]) == "pgerade":
                                _gInt_ = g_eintrag[-1]
                                if len(_gInt_) == 2: # color-mode                                
                                    _gIntern_ = _gInt_[0][1:]
                                    color = _gInt_[1][1]                                    
                                else: # kann eigentlich nicht vorkommen . . .
                                    _gIntern_ = _gInt_[1:]
                                    color = 0
                                aufpunkt = _gIntern_[0]
                                uvec = _gIntern_[1]
                                #print aufpunkt,uvec
                                self.pgerade(edit=1,g_name=maximaName,_aufpunkt_=aufpunkt,_rvec_=uvec ,_color_=color)
                                
                            else: # Gerade durch zwei Punkte
                                #print g_eintrag
                                _gInt_ = g_eintrag[-1]
                                if len(_gInt_) == 2: # color-mode                                
                                    _gIntern_ = _gInt_[0][1:]
                                    color = _gInt_[1][1]
                                else: 
                                    _gIntern_ = _gInt_[1:]
                                    color = 0
                                #print _gIntern_
                                #print color
                                self.punktGerade(edit=1,g_name=maximaName,_paramListe_= _gIntern_ ,_color_=color)                            
                        else: # alte Version (vor 1.27)
                            #self.geoOops.getObject(maximaName)
                            pScript = self.geoOops.getScriptEintrag(maximaName)
                            pRep = self.geoOops.script2Intern(pScript,2)                                
                            aufpunkt = pRep[0]
                            uvec = descartes.punkt2Gerade(pRep)[1]
                            #print aufpunkt,uvec
                            color = 0
                            self.pgerade(edit=1,g_name=maximaName,_aufpunkt_=aufpunkt,_rvec_=uvec ,_color_=color)
                        
                    elif geoTyp == "Punkt":
                        p_eintrag = self.geoOops.geoDir[maximaName]
                        if len(p_eintrag) > self.geoOops._maxPosten_: # neu: ab Vers. 1.27 wg. color
                            _pInt_ = p_eintrag[-1]
                            color = _pInt_[1][1]
                        else: # alte Version vor 1.27
                            color = 0
                        pScript = self.geoOops.getScriptEintrag(maximaName)
                        #print pScript
                        pVec = self.geoOops.script2Intern(pScript)[0]
                        self.punkt(edit=1,p_name=maximaName,_punkt_=pVec,_color_=color)

                    elif geoTyp == "Strecke":
                        d_eintrag = self.geoOops.geoDir[maximaName]
                        if len(d_eintrag) > self.geoOops._maxPosten_: # neu: ab Vers. 1.36 wg. color
                            _dInt_ = d_eintrag[-1]
                            color = _dInt_[1][1]
                        else: # alte Version vor 1.36
                            color = 0
                        pScript = self.geoOops.getScriptEintrag(maximaName)
                        #print pScript
                        pRep = self.geoOops.script2Intern(pScript,2)
                        #print "pRep"
                        #print pRep
                        #print pScript                        
                        punktListe = pScript[:-1] # hole nur Punkte
                        #print punktListe

                        zweiPuList = []
                        for element in punktListe:
                            elLi = []
                            elLi.append(element) # script2Intern erwartet eine Liste                           
                            pVec = self.geoOops.script2Intern(elLi,1)[0]
                            #print pVec
                            zweiPuList.append(pVec) # zweiPuList aufbauen
                        #print "zweiPuList"
                        #print zweiPuList
                        _objectRep_ = (zweiPuList[0],zweiPuList[1])
                        #print _objectRep_
                        #print color
                        #print maximaName
                        self.strecke(edit=1,d_name=maximaName,_puListe_=zweiPuList,_color_=color)                        
 

                    elif geoTyp == "Dreieck":
                        d_eintrag = self.geoOops.geoDir[maximaName]
                        if len(d_eintrag) > self.geoOops._maxPosten_: # neu: ab Vers. 1.36 wg. color
                            _dInt_ = d_eintrag[-1]
                            color = _dInt_[1][1]
                        else: # alte Version vor 1.36
                            color = 0
                        pScript = self.geoOops.getScriptEintrag(maximaName)
                        punktListe = pScript[:-3] # hole nur Punkte
                        dreiPuList = []
                        for element in punktListe:
                            elLi = []
                            elLi.append(element) # script2Intern erwartet eine Liste                           
                            pVec = self.geoOops.script2Intern(elLi,1)[0]
                            #print pVec
                            dreiPuList.append(pVec) # dreiPuList aufbauen
                        #print dreiPuList                        
                        self.dreieck(edit=1,d_name=maximaName,_puListe_=dreiPuList,_color_=color)

                    elif geoTyp == "Tetraeder":
                        t_eintrag = self.geoOops.geoDir[maximaName]
                        if len(t_eintrag) > self.geoOops._maxPosten_: # neu: ab Vers. 1.37 wg. color
                            _tInt_ = t_eintrag[-1]
                            color = _tInt_[1][1]
                        else: # alte Version vor 1.37
                            color = 0                        
                        pScript = self.geoOops.getScriptEintrag(maximaName)
                        #print pScript
                        #print
                        punktListe = pScript[:-7] # hole nur Punkte
                        vierPuList = []
                        #print punktListe
                        #print
                        for element in punktListe:
                            elLi = []
                            elLi.append(element) # script2Intern erwartet eine Liste                           
                            pVec = self.geoOops.script2Intern(elLi,1)[0]
                            #print pVec
                            vierPuList.append(pVec) # vierPuList aufbauen
                        #print    
                        #print vierPuList
                        #print
                        self.tetraeder(edit=1,t_name=maximaName,_puListe_=vierPuList,_color_=color)

######################  to do!!!!
                    elif geoTyp == "Quader":
                        q_eintrag = self.geoOops.geoDir[maximaName]
                        if len(q_eintrag) > self.geoOops._maxPosten_: # neu: ab Vers. 1.37 wg. color
                            _qInt_ = q_eintrag[-1]
                            color = _qInt_[1][1]
                        else: # alte Version vor 1.37
                            color = 0                        
                        
                        pScript = self.geoOops.getScriptEintrag(maximaName)
                        #print pScript
                        #print
                        punktListe = pScript[:-16] # hole nur Punkte
                        vierPuList = []
                        #print punktListe
                        #print
                        for element in punktListe:
                            elLi = []
                            elLi.append(element) # script2Intern erwartet eine Liste                           
                            pVec = self.geoOops.script2Intern(elLi,1)[0]
                            #print pVec
                            vierPuList.append(pVec) # dreiPuList aufbauen
                        #print vierPuList
                        self.quader(edit=1,q_name=maximaName,_puListe_=vierPuList,_color_=color)                        
######################  to do!!!!


                    elif geoTyp == "Viereck":
                        v_eintrag = self.geoOops.geoDir[maximaName]
                        if len(v_eintrag) > self.geoOops._maxPosten_: # neu: ab Vers. 1.37 wg. color
                            _vInt_ = v_eintrag[-1]
                            color = _vInt_[1][1]
                        else: # alte Version vor 1.37
                            color = 0                        
                        
                        pScript = self.geoOops.getScriptEintrag(maximaName)
                        #print pScript
                        #print
                        punktListe = pScript[:-4] # hole nur Punkte
                        vierPuList = []
                        #print punktListe
                        #print
                        for element in punktListe:
                            elLi = []
                            elLi.append(element) # script2Intern erwartet eine Liste                           
                            pVec = self.geoOops.script2Intern(elLi,1)[0]
                            #print pVec
                            vierPuList.append(pVec) # dreiPuList aufbauen
                        self.viereck(edit=1,v_name=maximaName,_puListe_=vierPuList,_color_=color)

                    elif geoTyp == "Kreis":
                        k_eintrag = self.geoOops.geoDir[maximaName]
                        #print k_eintrag
                        if len(k_eintrag) > self.geoOops._maxPosten_:
                            #print k_eintrag
                            _kInt_ = k_eintrag[-1]
                            #print _kInt_
                            if len(_kInt_) <= 2:                                
                                _kIntern_ = _kInt_[0][1:]
                                color = _kInt_[1][1]
                                #print _kIntern_  # Punkt A, B, C
                            else:
                                _kIntern_ = _kInt_[1:]
                                color = 0
                            #print _kIntern_  # Punkt A, B, C                                
                            self.kreis3(edit=1,k_name=maximaName,_paramListe_= _kIntern_,_color_=color)
                            
                        else: # Schnittkreis => wird nicht editiert/modifiziert . . .
                            pass


                    elif geoTyp == "Pyramide":
                        p_eintrag = self.geoOops.geoDir[maximaName]
                        if len(p_eintrag) > self.geoOops._maxPosten_: # neu: ab Vers. 1.37 wg. color
                            _pInt_ = p_eintrag[-1]
                            color = _pInt_[1][1]
                        else: # alte Version vor 1.37
                            color = 0                        
                        
                        pScript = self.geoOops.getScriptEintrag(maximaName)
                        #print pScript
                        #print 
                        punktListe = pScript[:-9] # hole nur Punkte
                        vierPuList = []
                        #print punktListe
                        #print
                        for element in punktListe:
                            elLi = []
                            elLi.append(element) # script2Intern erwartet eine Liste                           
                            pVec = self.geoOops.script2Intern(elLi,1)[0]
                            #print pVec
                            vierPuList.append(pVec) # dreiPuList aufbauen
                        self.pyramide(edit=1,p_name=maximaName,_puListe_=vierPuList,_color_=color)
                        
                else:
                    print "Too much . . ."
                                
            except:
                pass





    def pygeo2maxima(self,default=False):
        """Konvertierung pyGeo => maxima"""
        if self.maximaOk:
            import pymax2geo
            tmpGeoFile = "_temp_.py"
            if default:
                #pyGeoFile =  self.home + "pygeo" + os.sep + tmpGeoFile
                pyGeoFile = _pygeodir_  + os.sep + tmpGeoFile
            else:
                pyGeoFile = askopenfilename(filetypes=[("Python", "*.py"),("Alle Dateien","*.*")],initialdir=_pygeodir_)                        
            
            ##### Test: Script lesen und konvertieren in pyMaxima-Datei => funktioniert
            try:
                in_file = open(pyGeoFile,"r")
                newScript = in_file.readlines()     # Script lesen
                in_file.close()                     # Datei schliessen
            except:
                pass
            testli =  pymax2geo.script2maxima(scriptList=newScript)
            util.saveList2txt(self.home,testli)
            for element in testli:
                self.bearbeiteAusdruck(element)
                    



    def pygeo(self,default=True):
        """erzeugt und startet pyGeo-Skript"""
        import pymax2geo
        tempListe = []
        scripting = []
        self.script = []
        tempListe,scripting = self.geoOops.getGeoDir() 
        # Vektoren aus liste entfernen
        liste = []
        for element in tempListe:
            if element[2] == "Vektor": # Vektoren werden von pyGeo nicht gezeichnet!!!
                pass # print "Treffer"
            else:
                liste.append(element)
                
        headers = ['Select','Name','Typ','Beschreibung']
	
        ergebnis = tkw.geoTable(root,liste,headers)  # Objekte anzeigen

        for j in range(len(ergebnis)):  # script wird aufgebaut
            if ergebnis[j]==1:
                self.script.append(scripting[j])
		#print scripting[j]
	#print str(self.script)
        if len(self.script) > 0:
            geoScript = pymax2geo.generateScript(scriptList=self.script)
            #print geoScript
            
            tmpGeoFile = "_temp_.py"    # Script wird in die Datei "_temp_.py" geschrieben
            datei = _pygeodir_  + os.sep + tmpGeoFile                
            
            try:      
                out_file = open(datei,"w")
                out_file.write(geoScript) # text in Datei schreiben
                out_file.close()     # Datei schliessen
            except:
                return
            else:
                self.runScript(True)
#######################
    def kommentar(self):
        """loescht Funktionen/Variablen in einer Maxima-Sitzung
           Parameter: funktionen
        """
        if self.maximaOk:
            eingabe = tkw.Kommentar(root,"Maxima-Kommentar","Maxima-Kommentar: ","")     
            if eingabe.result <> None:
                # Fenster-Werte übernehmen
                maximaComment = str(eingabe.a1)
                maximaComment = '"' + maximaComment + '"$'
                self.bearbeiteAusdruck( maximaComment )

#######################                
    def loescheFunktionenVar(self,arg):
        """loescht Funktionen/Variablen in einer Maxima-Sitzung
           Parameter: funktionen
        """
        if self.maximaOk:
            argListe = ["Funktionen","Variable"]

            eingabe = tkw.Einzeiler(root,"Lösche " + argListe[arg], argListe[arg]+": ","all")     
            if eingabe.result <> None:
                # Fenster-Werte übernehmen
                try:
                    funcListe = str(eingabe.a1)
                    if arg == 0: # Funktionen
                        self.bearbeiteAusdruck( 'remfunction(' + funcListe + ');' )
                    else:
                        self.bearbeiteAusdruck( 'remvalue(' + funcListe + ');' )
                except:
                    tkMessageBox.showinfo("Parameter-Fehler!",  
                         "Fehler bei der Parametereingabe")

    def floatPointPrecision(self):
        """stellt Gleitkommagenauigkeit ein
        """
        if self.maximaOk:
            eingabe = tkw.Einzeiler(root,"Gleitkommagenauigkeit", "Gleitkommagenauigkeit: ","16")     
            if eingabe.result <> None:
                # Fenster-Werte übernehmen
                try:
                    fpp = int(eingabe.a1)
                    self.bearbeiteAusdruck( 'fpprec: ' + str(fpp) + ';' )
                except:
                    tkMessageBox.showinfo("Parameter-Fehler!",  
                         "Fehler bei der Parametereingabe")
              
######################################
    def wkBaum(self):
        """bestimmt Wahrscheinlichkeitsbaum/Vierfeldertafel/bedingte Wk
           Parameter: viele . . .
        """
        if self.maximaOk:
            retListe = maxfunc.wkBaum(root,self.moduleDrawLoaded)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )

                    
    def binomial(self):
        """bestimmt summierte Binomialverteilung
           Parameter: n, p 
        """
        if self.maximaOk:
            retListe = maxfunc.binomial(root,self.moduleDrawLoaded)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )


    def sumBinomial(self):
        """bestimmt summierte Binomialverteilung
           Parameter: n, p, von li=k1 bis re=k2 summiert
        """
        if self.maximaOk:
            retListe = maxfunc.sumBinomial(root)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )


#######################
    def normalVerteilung(self):
        """zeichnet & integriert Normalverteilung
           Parameter: li=x1, re=x2 
        """
        if self.maximaOk:
            retListe = maxfunc.normalVerteilung(root,self.moduleDrawLoaded)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )
        
########################################
    def konfIntervall(self):
        """bestimmt das Vertrauens/Konfidenzintervall
           Parameter: n, gamma, Stichprobenwert X (mit 0 <= X <= n)
        """
        if self.maximaOk:
            retListe = maxfunc.konfIntervall(root)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )


    def stichprobenUmfang(self):
        """bestimmt den StichprobenUmfang n
        Parameter: gamma, Intervalllänge d (0<= d <=1)
        """
        if self.maximaOk:
            retListe = maxfunc.stichprobenUmfang(root)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )



########################
            
    def kombinatorik(self,arg):
        """bestimmt Anzahlen
           Parameter: Anzahl der Kugeln in der Urne n, gezogen werden: k
        """
        if self.maximaOk:
            retListe = maxfunc.kombinatorik(root,arg)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )                
###############
    def guetefunktion(self,arg):
        """Bestimmt die Gütefunktion g(p) bzw. die
           Operationscharakteristik O(p) zu einem
           zweisetigen Signifikanztest einer binomial-
           verteilten Zufallsvariablen.
           Parameter: n, alpha, Nullhypothese h0         
        """
        if self.maximaOk:
            #print arg
            retListe = maxfunc.gueteFunktion(root,arg,self.moduleDrawLoaded)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )
                    
###############
    def alternativtest(self,arg):
        """Alternativtest mit Hilfe der (summierten) Binomialverteilung
           Parameter: n, alpha, Nullhypothese H0, Alternativhypothese H1
           arg == 0  <==> Binomialverteilung
           arg == 1  <==> Normalverteilung           
        """
        if self.maximaOk:
            retListe = maxfunc.alternativtest(root,arg)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )
                    
            
###############
    def hypothesentest(self,arg):
        """Hypothesentest mit Hilfe der (summierten) Binomialverteilung
           Parameter: n, alpha, Nullhypothese h0
           arg == 0 <==> zweiseitig
           arg == 1 <==> linksseitig
           arg == 2 <==> rechtsseitig           
        """
        if self.maximaOk:
            retListe = maxfunc.hypothesentest(root,arg)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )
                    

#######################################################        
    def pHypothesentest(self,arg):
        """Hypothesentest mit Hilfe der Poissonverteilung
           Parameter: n, alpha, Nullhypothese h0
           arg == 0 <==> zweiseitig
           arg == 1 <==> linksseitig
           arg == 2 <==> rechtsseitig           
        """
        if self.maximaOk:
            retListe = maxfunc.pHypothesentest(root,arg)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )


#######################################################
    def nHypothesentest(self,arg):
        """Hypothesentest mit Hilfe der Normalverteilung
           Parameter: n, alpha, Nullhypothese h0, Korrekturglied (0 oder 1)
           arg == 0 <==> zweiseitig
           arg == 1 <==> linksseitig
           arg == 2 <==> rechtsseitig           
        """
        if self.maximaOk:
            retListe = maxfunc.nHypothesentest(root,arg)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )

######################################
    def dreiDPlot(self,speichern = 0):
        if self.maximaOk:
            if speichern:
                (expr,value,xmin,xmax,ymin,ymax,x_value,y_value) = self.dreiD_Plot.getPlot()
                varLi = []
                varLi.append(x_value)
                varLi.append(y_value)
            else:
                (expr,varLi) = self.prolog()
                (expr,value,xmin,xmax,ymin,ymax,x_value,y_value) = self.dreiD_Plot.getPlot()
                varLi.append('y')
                
            optionList = []    
            if self.moduleDrawLoaded:
                optionList.append("default      ")
                optionList.append("enhanced3d   ")
                optionList.append("+surface_hide")                
                optionList.append("gray         ")
                optionList.append("gray: +hide  ")
            else:
                optionList.append("default     ")
                optionList.append("set hidden3d")
                optionList.append("gnuplot_pm3d")
            
            optionDir = {}
            if self.moduleDrawLoaded:
                optionDir["default      "] = ''
                optionDir["enhanced3d   "] = 'enhanced3d = true,'
                optionDir["+surface_hide"] = 'enhanced3d = true,surface_hide = true,'
                optionDir["gray         "] = 'palette = gray,enhanced3d = true,'
                optionDir["gray: +hide  "] = 'palette = gray,enhanced3d = true,surface_hide = true,'  
            else:
                optionDir["default     "] = ''
                optionDir["set hidden3d"] = '[gnuplot_preamble,"set hidden3d"]'
                optionDir["gnuplot_pm3d"] = '[gnuplot_pm3d,true]'
            #print optionDir
            
            
            mydici = {}

            mydici[1] = ('lab1',"x-min:       ",'2','0','ein1',xmin,'2','1')
            mydici[2] = ('lab2',"x-max:       ",'2','2','ein2',xmax,'2','3')
            mydici[3] = ('lab3',"y-min:       ",'3','0','ein3',ymin,'3','1')
            mydici[4] = ('lab4',"y-max:       ",'3','2','ein4',ymax,'3','3')            
            
            if speichern:
                eingabe = tkw.eingabeDreiDPlot(root,"3D-Plot: Graph speichern",expr,"Funktion:",varLi,mydici,optionList)            
            else:
                eingabe = tkw.eingabeDreiDPlot(root,"3D-Plot",expr,"Funktion:",varLi,mydici,optionList)
            if eingabe.result <> None:
                ausdruck = string.strip(str(eingabe.result))
                #print ausdruck
                if len(ausdruck) > 0:
                    ausdruck = self.stripStrichpunkt(ausdruck)
                    # print ausdruck
                    x_value = str(eingabe.x_value)
                    y_value = str(eingabe.y_value)
                    
                    # print x_value,y_value
                    xmin = str(eingabe.ret1)
                    xmax = str(eingabe.ret2)
                    ymin = str(eingabe.ret3)
                    ymax = str(eingabe.ret4)

                    _option_ = str(eingabe.optionM)
                    if _option_ not in optionList:
                        _option_ = optionList[0]
                    selOption = optionDir[_option_]
                    
                    if speichern:
                        self.dreiD_Plot.setPlot(ausdruck,value,xmin,xmax,ymin,ymax,x_value,y_value)
                        #pfad_ = abspath( self.home )
                        #os.chdir(pfad_)                    
                        oname = asksaveasfilename(filetypes=[("Encapsulated PostScript", "*.eps"),("Portable Network Graphics", "*.png")],initialdir=self.workingDir)
                        if oname:
                            postscript = 0
                            if (string.find(oname,".png") == -1) and (string.find(oname,".eps") == -1):
                                oname = oname + ".png"
                                postscript = 0
                            elif (self.test_extension(oname,".eps")):                            
                                postscript = 1
                            else:
                                postscript = 0
                                
                            #print oname
                            if self.moduleDrawLoaded:  #draw3d
                                oname_base = os.path.splitext(oname)
                                #print oname_base[0]
                                if postscript:
                                    ret = 'draw3d(' + selOption + 'explicit(' + ausdruck + ',' +  value+','+xmin+','+xmax+ ',' + y_value + ','+ ymin+','+ ymax+'), terminal=eps,' + 'file_name = "' + oname_base[0] + '")$'
                                else:
                                    ret = 'draw3d(' + selOption + 'explicit(' + ausdruck + ',' +  value+','+xmin+','+xmax+',' + y_value + ','+ ymin+','+ ymax+'), terminal=png,' + 'file_name = "' + oname_base[0] + '")$'
                                #print ret
                            else: # plot3d                                                                
                                if len(selOption) > 1:    
                                    ret = 'plot3d(' + ausdruck + ',['+ x_value +','+xmin+','+xmax+'],['+ y_value + ','+ymin+','+ymax +'],' + selOption 
                                else:
                                    ret = 'plot3d(' + ausdruck + ',['+ x_value +','+xmin+','+xmax+'],['+ y_value + ','+ymin+','+ymax +']'
                                if postscript:                                
                                    ret = ret + ',[gnuplot_term, ps],[gnuplot_out_file,"'
                                else:
                                    ret = ret + ',[gnuplot_term, png],[gnuplot_out_file,"'                            
                                ret = ret + str(oname) + '"])$'
                            self.plotten = 0
                            self.bearbeiteAusdruck(ret,var=x_value)
                            #print ret
                    else:
                        self.dreiD_Plot.setPlot(ausdruck,value,xmin,xmax,ymin,ymax,x_value,y_value)
                        if self.moduleDrawLoaded:
                            if len(selOption) > 1:
                                ret = 'draw3d(' + selOption + 'explicit(' + ausdruck + ',' + x_value + ','+xmin+','+xmax + ',' + y_value + ','+ ymin+','+ ymax+') )$'
                            else:                                                            
                                ret = 'draw3d(' + 'explicit(' + ausdruck + ',' + x_value + ','+xmin+','+xmax + ',' + y_value + ','+ ymin+','+ ymax+')' + ')$'
                        else:
                            if len(selOption) > 1:
                                ret = 'plot3d(' + ausdruck + ',['+ x_value+','+xmin+','+xmax+'],['+ y_value + ','+ymin+','+ymax+'],' + selOption + ')$'
                            else: # Default!
                                ret = 'plot3d(' + ausdruck + ',['+ x_value+','+xmin+','+xmax+'],['+ y_value + ','+ymin+','+ymax+'])$'
                        self.plotten = 1
                        self.bearbeiteAusdruck(ret,var=x_value)

#####################################
                    
    def plot(self,speichern = 0):
        if self.maximaOk:
            

            if speichern:
                (expr,value,xmin,xmax,ymin,ymax,set_Title,set_AchseX,set_AchseY) = self.zweiD_Plot.getPlot()
                varLi = []
                varLi.append(value)
            else:
                (expr,varLi) = self.prolog()
                (expr,value,xmin,xmax,ymin,ymax,set_Title,set_AchseX,set_AchseY) = self.zweiD_Plot.getPlot()
            
            mydici = {}

            mydici[1] = ('lab1',"x-min:",'1','0','ein1',xmin,'1','1')
            mydici[2] = ('lab2',"x-max:",'1','2','ein2',xmax,'1','3')
            mydici[3] = ('lab3',"y-min:",'2','0','ein3',ymin,'2','1')
            mydici[4] = ('lab4',"y-max:",'2','2','ein4',ymax,'2','3')

            moreOptions = {}
            moreOptions["Titel"] = set_Title
            moreOptions["X_Achse"] = set_AchseX
            moreOptions["Y_Achse"] = set_AchseY

           
            
##            if speichern:
##                eingabe = tkw.eingabeListEx(root,"gnuPlot: Graph speichern",expr,"Funktion:",varLi,mydici)            
##            else:
##                eingabe = tkw.eingabeListEx(root,"gnuPlot",expr,"Funktion:",varLi,mydici)
            if speichern:
                eingabe = tkw.eingabeZweiDPlot(root,"gnuPlot: Graph speichern",expr,"Funktion:",varLi,mydici,moreOptions) 
            else:
                eingabe = tkw.eingabeZweiDPlot(root,"gnuPlot",expr,"Funktion:",varLi,mydici,moreOptions) 
                
                
            if eingabe.result <> None:
                klammerTest = False
                ausdruck = string.strip(str(eingabe.result))
                if len(ausdruck) > 0:
                    ausdruck = self.stripStrichpunkt(ausdruck)
                    if self.moduleDrawLoaded:  #draw2d
                        fLen = len(ausdruck)                        
                        oldAusdruck = ausdruck
                        for i in range(fLen):
                            if ausdruck[i] == '(':
                                klammerTest = True
                                _aus_ = "_f_ : " + ausdruck + ";"  # lokalen Namen "_f_"  <== Trick!!!
                                self.bearbeiteAusdruck(_aus_,var=value)
                                #ausdruckListe = self.funcParser(ausdruck)
                                ausdruck = "_f_" #
                                #print ausdruck, ausdruckListe
                                break
                        
                        
                        #print self.funcParser(ausdruck)                    
                    value = str(eingabe.value)
                    xmin = str(eingabe.ret1)
                    xmax = str(eingabe.ret2)
                    ymin = str(eingabe.ret3)
                    ymax = str(eingabe.ret4)

                    if len(eingabe.ret_Title) > 0:
                        if self.moduleDrawLoaded:
                            set_Title = 'title = "' +  util.ucodefilter_( eingabe.ret_Title, org = 1 ) + '",'
                        else:
                            set_Title = "set title '" + util.ucodefilter_( eingabe.ret_Title, org = 1 )  + "'; "
                    else:
                        set_Title = ""

                    if len(eingabe.ret_AchseX) > 0:
                        if self.moduleDrawLoaded:
                            set_AchseX = 'xlabel = "'  + util.ucodefilter_( eingabe.ret_AchseX, org = 1 ) + '",' 
                        else:
                            set_AchseX = "set xlabel '" + util.ucodefilter_( eingabe.ret_AchseX, org = 1 ) + "'; "
                    else:
                        set_AchseX = ""

                    if len(eingabe.ret_AchseY) > 0:
                        if self.moduleDrawLoaded:
                            set_AchseY = 'ylabel = "'  + util.ucodefilter_( eingabe.ret_AchseY, org = 1 ) + '",' 
                        else:
                            set_AchseY = "set ylabel '" + util.ucodefilter_( eingabe.ret_AchseY, org = 1 ) + "'; "
                    else:
                        set_AchseY = ""


                    if eingabe._ret_noKey_:
                        if self.moduleDrawLoaded:
                            set_noKey = ""
                        else:
                            set_noKey = "set nokey; "
                    else:
                        if self.moduleDrawLoaded:
                            set_noKey = 'key = "' + ausdruck + '",'
                        else:
                            set_noKey = ""


                    if eingabe._ret_logX_:
                        if self.moduleDrawLoaded:
                            set_logX = 'logx=true,'
                        else:    
                            set_logX = "set log x; "
                    else:
                        set_logX = ""

                    if eingabe._ret_logY_:
                        if self.moduleDrawLoaded:
                            set_logY = 'logy=true,'
                        else:
                            set_logY = "set log y; "
                    else:
                        set_logY = ""

                        
                    if self.moduleDrawLoaded:
                        set_arg = ',' +  set_Title + 'xaxis = true,' + 'yaxis = true,' + set_logX + set_logY + set_AchseX  + set_AchseY + 'grid = true'
                        set_pre_arg = set_noKey 
                    else:
                        set_arg = self.linetype + set_Title + set_AchseX + set_AchseY + set_noKey + set_logX + set_logY 
                    
                    if speichern: # save the plot
                        if klammerTest:                            
                            self.zweiD_Plot.setPlot(oldAusdruck,value,xmin,xmax,ymin,ymax,eingabe.ret_Title,eingabe.ret_AchseX,eingabe.ret_AchseY)
                        else:
                            self.zweiD_Plot.setPlot(ausdruck,value,xmin,xmax,ymin,ymax,eingabe.ret_Title,eingabe.ret_AchseX,eingabe.ret_AchseY)
                        #pfad_ = abspath( self.home )                        
                        #os.chdir(pfad_)                    
                        oname = asksaveasfilename(filetypes=[("Portable Network Graphics", "*.png"),("Encapsulated PostScript", "*.eps")],initialdir=self.workingDir)
                        if oname:
                            postscript = 0
                            if (string.find(oname,".png") == -1) and (string.find(oname,".eps") == -1):
                                oname = oname + ".png"
                                postscript = 0
                            elif (self.test_extension(oname,".eps")):                            
                                postscript = 1
                            else:
                                postscript = 0
                                
                            #print oname
                            if self.moduleDrawLoaded:  #draw2d
                                oname_base = os.path.splitext(oname)
                                #print oname_base[0]
                                if klammerTest:
                                    set_pre_arg = ''  # dann steht in set_pre_arg nur "_f_"
                                if postscript:
                                    ret = 'draw2d(' + set_pre_arg + 'explicit(' + ausdruck + ',' +  value+','+xmin+','+xmax+')' + ', terminal=eps,' + 'file_name = "' + oname_base[0] + '"' + set_arg + ')$'
                                else:
                                    ret = 'draw2d(' + set_pre_arg + 'explicit(' + ausdruck + ',' +  value+','+xmin+','+xmax+')' + ', terminal=png,' + 'file_name = "' + oname_base[0] + '"' + set_arg + ')$'
                                #print ret
                            else: # plot2d                                
                                ret = 'plot2d([' + ausdruck  + '],['+ value+','+xmin+','+xmax+'],[y,'+ymin+','+ymax
                                if postscript:
                                    ret = ret + '],[gnuplot_preamble,"set grid;' + set_arg + '"],[gnuplot_term, ps],[gnuplot_out_file,"'
                                else:
                                    ret = ret + '],[gnuplot_preamble,"set grid;' + set_arg + '"],[gnuplot_term, png],[gnuplot_out_file,"'                            
                                ret = ret + str(oname) + '"])$'
                            self.plotten = 0
                            self.bearbeiteAusdruck(ret,var=value)
                                
                    else: # do not  save => plot function
                        if klammerTest:
                            self.zweiD_Plot.setPlot(oldAusdruck,value,xmin,xmax,ymin,ymax,eingabe.ret_Title,eingabe.ret_AchseX,eingabe.ret_AchseY)
                        else:
                            self.zweiD_Plot.setPlot(ausdruck,value,xmin,xmax,ymin,ymax,eingabe.ret_Title,eingabe.ret_AchseX,eingabe.ret_AchseY)
                        if self.moduleDrawLoaded:
                            if klammerTest:
                                ret = 'draw2d(explicit(' + ausdruck + ',' +  value+','+xmin+','+xmax+')'+ set_arg + ')$'
                            else:
                                ret = 'draw2d(' + set_pre_arg + 'explicit(' + ausdruck + ',' +  value+','+xmin+','+xmax+')'+ set_arg + ')$'
                            #print ret
                        else:
                            ret = 'plot2d([' + ausdruck  + '],['+ value+','+xmin+','+xmax+'],[y,'+ymin+','+ymax+'],[gnuplot_preamble,"set grid; ' + set_arg + '"])$'
                        self.plotten = 1
                        self.bearbeiteAusdruck(ret,var=value)
			


                

    def taylor(self):
        if self.maximaOk:
            (expr,varLi) = self.prolog()
            (ret,value) = maxfunc.taylor(root,expr,varLi)
            if len(ret) > 0:
                self.bearbeiteAusdruck(ret,var=value)


    def summe(self,produkt=0):
        if self.maximaOk:
            (expr,varLi) = self.prolog()
            (ret,value) = maxfunc.summe(root,expr,varLi,produkt)
            if len(ret) > 0:
                self.bearbeiteAusdruck(ret,var=value)


    def limit(self):
        if self.maximaOk:
            (expr,varLi) = self.prolog()        
            (ret,value) = maxfunc.limit(root,expr,varLi)
            if len(ret) > 0:
                self.bearbeiteAusdruck(ret,var=value)


    def polyDiv(self):
        if self.maximaOk:
            (expr,varLi) = self.prolog()
            (ret,value) = maxfunc.polyDiv(root,expr,varLi)
            if len(ret) > 0:
                self.bearbeiteAusdruck(ret,var=value)	


    def kettenbruch(self):
        if self.maximaOk:                    
            retLi = maxfunc.kettenbruch(root)
            if len(retLi) > 0:
                for element in retLi:
                    self.bearbeiteAusdruck( element )


    def partialBruchZerlegung(self):
        if self.maximaOk:        
            (expr,varLi) = self.prolog()
            (ret,value) = maxfunc.partialBruchZerlegung(root,expr,varLi)
            if len(ret) > 0:
                self.bearbeiteAusdruck(ret,var=value)
        

    def partialBruch(self):
        if self.maximaOk:        
            (expr,varLi) = self.prolog()
            (ret,value) = maxfunc.partialBruch(root,expr,varLi)
            if len(ret) > 0:
                self.bearbeiteAusdruck(ret,var=value)
                

    def solve(self):
        if self.maximaOk:
            (expr,varLi) = self.prolog()
            (ret,value) = maxfunc.solve(root,expr,varLi)
            if len(ret) > 0:
                self.bearbeiteAusdruck(ret,var=value)
                self.bearbeiteAusdruck('ev(%,numer);',var=value)  	

    def solveDGL(self):
        if self.maximaOk:
            (expr,varLi) = self.prolog()
            dglSolved = maxfunc.solveDGL(root,expr,varLi)
            if len( dglSolved ) > 0:
                #print ret
                self.bearbeiteAusdruck( dglSolved )                

    def dglAWP1(self):
        if self.maximaOk:
            (expr,varLi) = self.prolog()
            dglAwp = maxfunc.dglAWP1(root,expr,varLi)
            if len( dglAwp ) > 0:
                #print ret
                self.bearbeiteAusdruck( dglAwp )                


    def dglAWP2(self):
        if self.maximaOk:
            (expr,varLi) = self.prolog()
            dglAwp = maxfunc.dglAWP2(root,expr,varLi)
            if len( dglAwp ) > 0:
                #print ret
                self.bearbeiteAusdruck( dglAwp )                


    def dglRWP(self):
        if self.maximaOk:
            (expr,varLi) = self.prolog()
            dglRwp = maxfunc.dglRWP(root,expr,varLi)
            if len( dglRwp ) > 0:
                #print ret
                self.bearbeiteAusdruck( dglRwp )                



    def gauss(self):
        if self.maximaOk:
            self.matrixEingabe()

    def schnittKG(self):
        """Schnitt Kugel-Gerade"""
        if self.maximaOk:
            retListe = maxfunc.schnittKG(root,self.geoOops,self.m2g)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )                                
                            

    def schnittKE(self):
        """Schnitt Kugel-Ebene"""                        
        if self.maximaOk:
            retListe = maxfunc.schnittKE(root,self.geoOops,self.m2g)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )



    def schnittKK(self):
        """Schnitt Kugel-Kugel"""
        if self.maximaOk:
            retListe = maxfunc.schnittKK(root,self.geoOops,self.m2g)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )
                                        

    def schnittCG(self):
        """Schnitt Kreis-Gerade (C: Circle)"""
        if self.maximaOk:
            retListe = maxfunc.schnittCG(root,self.geoOops,self.m2g)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )                                
                            

    def schnittCE(self):
        """Schnitt Kreis-Ebene (C: Circle)"""                        
        if self.maximaOk:
            retListe = maxfunc.schnittCE(root,self.geoOops,self.m2g)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )



    def abstandPP(self):
        """Abstand Punkt-Punkt"""
        self.abstandPG("Punkt")


    def abstandPE(self):
        """Abstand Punkt-Ebene"""
        self.abstandPG("Ebene")

    def abstandPK(self):
        """Abstand Punkt-Kugel"""
        self.abstandPG("Kugel")


    def abstandPC(self):
        """Abstand Punkt-Kreis (C: Circle)"""
        self.abstandPG("Kreis")

    
    def abstandPG(self,geoObjekt="Gerade"):
        """Abstand Punkt-Gerade/Ebene/Kugel/Kreis (je nach Argument)"""                
        if self.maximaOk:
            ret = maxfunc.abstand(root,self.geoOops,geoObjekt)
            #print ret
            self.bearbeiteAusdruck( ret )

    def tangentialebene(self):
        """konstruiert Tangentialebene aus Kugelgleichung und Punkt auf Oberflaeche"""                
        if self.maximaOk:
            retListe = maxfunc.tangentialebene(root,self.geoOops,self.m2g)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )

    def kugelAusTangentialebene(self):
        """konstruiert Kugel aus Mittelpunkt und Tangentialebene"""                
        if self.maximaOk:
            retListe = maxfunc.kugelAusTangentialebene(root,self.geoOops,self.m2g)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )

    def pol(self):
        """konstruiert zu einer Polarebene p und einer Kugelgleichung den Pol P"""                
        if self.maximaOk:
            retListe = maxfunc.pol(root,self.geoOops,self.m2g)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )




    def schnittGE(self):
        """Lagebeziehungen zwischen Ebene und Gerade"""                
        if self.maximaOk:
            retListe = maxfunc.schnittGE(root,self.geoOops,self.m2g)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )
                


    def schnittGG(self):
        """Lagebeziehungen zwischen Geraden"""                
        if self.maximaOk:
            retListe = maxfunc.schnittGG(root,self.geoOops,self.m2g)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )
                

                    
    def schnittEE(self):
        """Lagebeziehungen zwischen Ebenen"""                
        if self.maximaOk:
            retListe = maxfunc.schnittEE(root,self.geoOops,self.m2g)
            if retListe > 0:
                for element in retListe:
                    self.bearbeiteAusdruck( element )

    def getVectors(self,anzahl):
        """Holt anzahl Vektoren aus der pyGeo-ObjectList,
        Retour: Liste der Namen, Liste der Vektoren"""
        vecListe = []
        liste = []
        selectVec = []
        
        vecListe = self.geoOops.getObjectList("Vektor")
        for element in vecListe:
            liste.append(self.geoOops.getGeoElement(element) )
        # in liste stehen jetzt alle Vektoren
        headers = ['Select','Name','Typ','Beschreibung']	
        ergebnis = tkw.geoTable(root,liste,headers)  # Objekte anzeigen

        for j in range(len(ergebnis)):  # Vektoren auswaehlen
            if ergebnis[j]== 1:
                selectVec.append(vecListe[j])
        if len(selectVec) != anzahl:
            return ([],[])
        else: # Vektor ausgewaehlt => Betrag
            vectorListe = [] # hier werden die Vektoren gesammelt
            for vec in selectVec:
                #print vec => vec ist der Name des Vectors
                vecRep = []
                vecRep = self.geoOops.getMaximaEintrag(vec)
                #print vecRep                             
                if len(vecRep) > 1:
                    vecLi1  = vecRep[1].split("[")
                    vecLi2 = vecLi1[1].split("]")
                    vector = vecLi2[0].split(",")
                    vectorListe.append(vector)
            return (selectVec,vectorListe)
                        
        

    def vecBetrag(self):
        """Betrag/Laenge eines Vektors"""                
        if len(self.geoOops.geoDir) > 0:
            (vecName,vectorListe) = self.getVectors(1)
            print vectorListe
            if len(vectorListe) > 0: 
                ergebnis = descartes.betrag(vectorListe[0])        
                self.bearbeiteAusdruck('"Länge l des Vektors ' +  vecName[0]  + ': l = ' + str(ergebnis) +  '"$')
            else:
                print "Mist . . ."

    def vecWinkel(self):
        "Bestimmt den Winkel zwischen zwei Vektoren"        
        if len(self.geoOops.geoDir) > 0:
            (vecName,vectorListe) = self.getVectors(2)
            if len(vectorListe) > 0: 
                ergebnis = descartes.winkel(vectorListe[0],vectorListe[1])
                if ergebnis:
                    self.bearbeiteAusdruck('"Winkel zwischen Vektor ' +  vecName[0]  + ' und Vektor '  +  vecName[1]  + ': alpha = '+ str(ergebnis) +  'ﾰ "$')
                else: # ergebnis = None
                    self.bearbeiteAusdruck('"Winkel zwischen Vektor ' +  vecName[0]  + ' und Vektor '  +  vecName[1]  + ': alpha = 0ﾰ "$')                        
            else:
                print "Mist . . ."

    def vecSkalar(self):
        "Bestimmt das Skalarprodukt zweier Vektoren"        
        if len(self.geoOops.geoDir) > 0:
            (vecName,vectorListe) = self.getVectors(2)
            if len(vectorListe) > 0: 
                ergebnis = descartes.skalarProdukt(vectorListe[0],vectorListe[1])        
                self.bearbeiteAusdruck('"Skalarprodukt zwischen den Vektoren : <' +  vecName[0]  + ','  +  vecName[1]  + '> = '+ str(ergebnis) +  '"$')
            else:
                print "Mist . . ."

    def vecKreuz(self):
        "Bestimmt das Kreuzprodukt zweier Vektoren => muehsam"
        if len(self.geoOops.geoDir) > 0:
            (vecName,vectorListe) = self.getVectors(2)
            if len(vectorListe) > 0:
                ergCP = tkw.eingabeCrossProduct(root,"Kreuzprodukt",_varlist_ = vecName)
                if (ergCP.value0 == vecName[0]) and (ergCP.value1 == vecName[1]):
                    ergebnis = descartes.crossProdukt(vectorListe[0],vectorListe[1])
                elif (ergCP.value0 == vecName[1]) and (ergCP.value1 == vecName[0]):
                    ergebnis = descartes.crossProdukt(vectorListe[1],vectorListe[0])                                   
                elif (ergCP.value0 == vecName[0]) and (ergCP.value1 == vecName[0]):
                    ergebnis = descartes.crossProdukt(vectorListe[0],vectorListe[0])
                elif (ergCP.value0 == vecName[1]) and (ergCP.value1 == vecName[1]):
                    ergebnis = descartes.crossProdukt(vectorListe[1],vectorListe[1])

                
                if len(ergebnis) > 0:
                    vec = "matrix (["
                    cp_geo = []
                    i = 1
                    for koord in ergebnis:
                        if (i!=3):
                            vec =  vec + str(koord) + ','
                            cp_geo.append(koord)
                        else:
                            vec =  vec + str(koord) + '])'
                            cp_geo.append(koord)
                        i = i + 1
                    cp_ = 'transpose(' + vec + ')'
                    
                    if (ergCP.value0 == vecName[0]) and (ergCP.value1 == vecName[1]):
                        cp_name = vecName[0]  + '_'  +  vecName[1]                        
                    elif (ergCP.value0 == vecName[1]) and (ergCP.value1 == vecName[0]):
                        cp_name = vecName[1]  + '_'  +  vecName[0]
                    elif (ergCP.value0 == vecName[0]) and (ergCP.value1 == vecName[0]):
                        cp_name = 'nullVec'
                    else:
                        cp_name = 'nullVec'
                        
                    script = []
                    script.append( self.m2g.richtVec(cp_geo,cp_,cp_name) )
                    #print script
                    liste = self.geoOops.buildListEntry(cp_name,"Vektor",vec, script )
                    self.geoOops.addGeoDirEntry(cp_name,liste)
                    
                    if (ergCP.value0 == vecName[0]) and (ergCP.value1 == vecName[1]):
                        self.bearbeiteAusdruck('"Kreuzprodukt zwischen den Vektoren: ' +  vecName[0]  + ' x '  +  vecName[1]  + ' "$')
                    elif (ergCP.value0 == vecName[1]) and (ergCP.value1 == vecName[0]):
                        self.bearbeiteAusdruck('"Kreuzprodukt zwischen den Vektoren: ' +  vecName[1]  + ' x '  +  vecName[0]  + ' "$')
                    elif (ergCP.value0 == vecName[0]) and (ergCP.value1 == vecName[0]):
                        self.bearbeiteAusdruck('"Kreuzprodukt zwischen den Vektoren: ' +  vecName[0]  + ' x '  +  vecName[0]  + ' "$')                        
                    else:
                        self.bearbeiteAusdruck('"Kreuzprodukt zwischen den Vektoren: ' +  vecName[1]  + ' x '  +  vecName[1]  + ' "$')
                        
                    self.bearbeiteAusdruck(cp_name  + ' : ' + cp_ + ';')
            else:
                print "Mist . . ."


    def linUnab2(self):
        "Test auf lineare Unabhängigkeit zweier Vektoren"        
        if len(self.geoOops.geoDir) > 0:
            (vecName,vectorListe) = self.getVectors(2)
            if len(vectorListe) > 0: 
                ergebnis = descartes.linearAbhaengig(vectorListe[0],vectorListe[1])
                if (ergebnis == 1):
                    self.bearbeiteAusdruck('"Die beiden Vektoren ' +  vecName[0]  + ' und '  +  vecName[1]  + ' sind linear abhängig!"$')
                else:
                    self.bearbeiteAusdruck('"Die beiden Vektoren ' +  vecName[0]  + ' und '  +  vecName[1]  + ' sind linear unabhängig!"$')                    
            else:                
                print "Mist . . ."

    def linUnab3(self):
        "Test auf lineare Unabhängigkeit dreier Vektoren"
        if len(self.geoOops.geoDir) > 0:
            (vecName,vectorListe) = self.getVectors(3)
            if len(vectorListe) > 0: 
                mat = descartes.vecToMatrix(vectorListe[0],vectorListe[1],vectorListe[2])
                erg = descartes.det3(mat)
                if descartes.nullTest(erg):
                    self.bearbeiteAusdruck('"Die drei Vektoren ' +  vecName[0]  + ', ' +  vecName[1]  + ' und '  +  vecName[2]  + ' sind linear abhängig!"$')
                else:
                    self.bearbeiteAusdruck('"Die drei Vektoren ' +  vecName[0]  + ', ' +  vecName[1]  + ' und '  +  vecName[2]  + ' sind linear unabhängig!"$')                                    
            else:                
                print "Mist . . ."


    def overwriteGeoDirEntry(self,edit,geoName,neuName):
        if edit:            
            if (geoName == neuName):
                _typ_ = self.geoOops.getGeoType(neuName)
                _message_ = _typ_ +  " " +  neuName + " überschreiben?"
                _title_ = _typ_ + " -> " + neuName
                
                ergebnis = tkMessageBox.askokcancel(_title_,_message_, default=tkMessageBox.CANCEL ) 
                if ergebnis:
                    self.geoOops.delGeoDirEntry(neuName)
                    return 1                    
                else:
                    return 0
            else:
                if self.nameInObjectDir(neuName):
                    _typ_ = self.geoOops.getGeoType(neuName)
                    _message_ = _typ_ +  " " +  neuName + " überschreiben?"
                    _title_ = _typ_ + " -> " + neuName
                    
                    ergebnis = tkMessageBox.askokcancel(_title_,_message_, default=tkMessageBox.CANCEL ) 
                    if ergebnis:
                        self.geoOops.delGeoDirEntry(neuName)
                        return 1
                    else:
                        return 0
                else:
                    return 1
                
        else:
            if self.nameInObjectDir(neuName):
                _typ_ = self.geoOops.getGeoType(neuName)
                _message_ = _typ_ +  " " +  neuName + " überschreiben?"
                _title_ = _typ_ + " -> " + neuName
                
                ergebnis = tkMessageBox.askokcancel(_title_,_message_, default=tkMessageBox.CANCEL ) 
                if ergebnis:
                    self.geoOops.delGeoDirEntry(neuName)
                    return 1
                else:
                    return 0
            else:
                return 1

                    
    def vektor(self,edit=0,vec_name=None,_vector_=None):
        """Eingabe => Ortsvektor"""
        if self.maximaOk:
            dim = 3
            _overWrite_ = 1
            if edit:
                geoName = vec_name
                koeff_ = tkMatrix.Vektor(root,geoName,dimension=dim,_koeff_=_vector_)                
            else:                
                geoName = self.geoOops.generateObjectName("v")            
                koeff_ = tkMatrix.Vektor(root,geoName,dimension=dim)
            
            #print str(koeff_.mtx) # in koeff_.mtx[0] ist der Vektor
            #print koeff_.name

            if koeff_.result <> None:
                _overWrite_ = self.overwriteGeoDirEntry(edit,geoName,koeff_.name)
                            
                if _overWrite_:
                    a_punkt = "matrix (["
                    a_geo = []
                    a_name = koeff_.name
                    if len(a_name) < 1:
                        a_name = geoName
                    for j in range(3): # Spalte der Matrix
                        if (j < 2):
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + ','
                            a_geo.append(koeff_.mtx[0][j])
                        else:
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + '])'
                            a_geo.append(koeff_.mtx[0][j])
                    #self.script = []
                    #self.script.append( self.m2g.punkt(a_geo,a_punkt,a_name) )
                    
                    ap_ = 'transpose(' + a_punkt +')'
                    
                    script = []
                    script.append( self.m2g.richtVec(a_geo,ap_,a_name) )
                    #print script
                    liste = self.geoOops.buildListEntry(a_name,"Vektor",a_punkt, script )
                    self.geoOops.addGeoDirEntry(a_name,liste)
                    #geoOops.getObject(a_name)
                    
                    ############################
                    self.bearbeiteAusdruck('"Vektor ' + a_name + '"$')
                    self.bearbeiteAusdruck(a_name + ': ' + ap_ +  ';')



    def pVektor(self,edit=0,vec_name=None,_vector_=None):
        """Eingabe => Vektor zwischen zwei Punkten A, B"""
        if self.maximaOk:
            dim = 2
            _overWrite_ = 1
            titel = "Vektor zwischen zwei Punkten A, B"
            if edit:
                geoName = vec_name
                koeff_ = tkMatrix.VecPunkte(root,geoName,_title_= titel,dimension=dim,_parameter_ = _vector_)                            
            else:
                geoName = self.geoOops.generateObjectName("v") 
                koeff_ = tkMatrix.VecPunkte(root,geoName,_title_= titel,dimension=dim)            
            

            if koeff_.result <> None:
                _overWrite_ = self.overwriteGeoDirEntry(edit,geoName,koeff_.name)
                            
                if _overWrite_:                   
                    u_name = koeff_.name                    
                    if len(u_name) < 1:
                        u_name = geoName
                    
                    a_punkt = "matrix (["
                    a_geo = []
                    b_geo = []                
                    b_punkt = "matrix (["
                    for j in range(dim+1): # Spalte der Matrix
                        a_geo.append(koeff_.mtx[0][j])
                        b_geo.append(koeff_.mtx[1][j])                                            
                        if (j < dim):
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + ','
                            b_punkt =  b_punkt +  koeff_.mtx[1][j] + ','
                        else:
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + '])'
                            b_punkt =  b_punkt +  koeff_.mtx[1][j] + '])'

                    ap_ = 'transpose(' + a_punkt +')'
                    bp_ = 'transpose(' + b_punkt +')'
                    
                    uvec_geo = descartes.genRichtVec(a_geo,b_geo)
                    #u_vect_ = bp_ + " - " +  ap_
                    u_vect_ = descartes.punkt2maxima(uvec_geo)
                    
                    
                    script = []
                    #script.append( self.m2g.punkt(a_geo,ap_,"p1"+u_name) )
                    #script.append( self.m2g.punkt(b_geo,bp_,"p2"+u_name) )                    
                    script.append( self.m2g.richtVec(uvec_geo,u_vect_,u_name) )

                    _objectRep_ = ["vec2p",a_geo,b_geo] #
                    maximaObjectRep = b_punkt + " - " + a_punkt
                    maximaObjectRep = uvec_geo
                    liste = self.geoOops.buildListEntry(u_name,"Vektor",maximaObjectRep, script, _objectRep_ )
                    self.geoOops.addGeoDirEntry(u_name,liste)
                    #geoOops.getObject(u_name)
                    
                    ############################
                    self.bearbeiteAusdruck('"Vektor ' + u_name + '"$')
                    #self.bearbeiteAusdruck(u_name + ': ' + u_vect_ +  ';')
                    self.bearbeiteAusdruck(u_name + ': ' + bp_ + " - "  + ap_ +  ';')


    def punkt(self,edit=0,p_name=None,_punkt_=None, _color_=0):
        if self.maximaOk:
            dim = 1
            _overWrite_ = 1
            if edit:
                geoName = p_name
                koeff_ = tkMatrix.Punkte(root,geoName,self.m2g.colList,dimension=dim,_koeff_=_punkt_,colorIndex=_color_)                                
            else:
                geoName = self.geoOops.generateObjectName("P")            
                koeff_ = tkMatrix.Punkte(root,geoName,self.m2g.colList,dimension=dim)
            #print str(koeff_.mtx[0]) # in koeff_.mtx[0] ist der Punkt
            if koeff_.result <> None:
                _overWrite_ = self.overwriteGeoDirEntry(edit,geoName,koeff_.name)
                            
                if _overWrite_:                
                    a_punkt = "matrix (["
                    a_geo = []
                    a_name = koeff_.name
                    if len(a_name) < 1:
                        a_name = geoName
                    for j in range(3): # Spalte der Matrix
                        if (j < 2):
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + ','
                            a_geo.append(koeff_.mtx[0][j])
                        else:
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + '])'
                            a_geo.append(koeff_.mtx[0][j])
                    #self.script = []
                    #self.script.append( self.m2g.punkt(a_geo,a_punkt,a_name) )
                    
                    ap_ = 'transpose(' + a_punkt +')'
                    
                    _colorIndex_ = self.m2g.colList.index(koeff_.color)
                    script = []
                    script.append( self.m2g.punkt(a_geo,ap_,a_name,None,_colorIndex_) )

                    _objectRep_ = [("punkt",),("colorIndex",_colorIndex_)]
                    liste = self.geoOops.buildListEntry(a_name,"Punkt",a_punkt, script, _objectRep_  )
                    self.geoOops.addGeoDirEntry(a_name,liste)
                    #geoOops.getObject(a_name)
                    
                    ############################
                    self.bearbeiteAusdruck('"Punkt ' + a_name + '"$')
                    self.bearbeiteAusdruck(a_name + ': ' + ap_ +  ';')


    def kugel(self,edit=0,k_name=None,_paramListe_=None, _color_=0):
        """Kugel durch Mittelpunkt M und Punkt O auf Oberfläche"""
        if self.maximaOk:
            varListe = ['x','y','z']
            _overWrite_ = 1
            dim = 2
            if edit:                
                geoName = k_name                                   
                koeff_ = tkMatrix.kugel(root,geoName,self.m2g.colList,dimension=dim,_parameter_ = _paramListe_,colorIndex=_color_)            
            else:
                geoName = self.geoOops.generateObjectName("K")                                    
                koeff_ = tkMatrix.kugel(root,geoName,self.m2g.colList,dimension=dim)            
            if koeff_.result <> None:
                _overWrite_ = self.overwriteGeoDirEntry(edit,geoName,koeff_.name)
                            
                if _overWrite_:                                
                    k_name = koeff_.name
                    #print g_name
                    x_vect = "matrix (["
                    m_punkt = "matrix (["
                    m_geo = []
                    o_geo = []                
                    o_punkt = "matrix (["
                    for j in range(dim+1): # Spalte der Matrix
                        m_geo.append(koeff_.mtx[0][j])
                        o_geo.append(koeff_.mtx[1][j])                                            
                        if (j < dim):
                            m_punkt =  m_punkt +  koeff_.mtx[0][j] + ','
                            o_punkt =  o_punkt +  koeff_.mtx[1][j] + ','
                            x_vect =  x_vect +  varListe[j] + ','
                        else:
                            m_punkt =  m_punkt +  koeff_.mtx[0][j] + '])'
                            o_punkt =  o_punkt +  koeff_.mtx[1][j] + '])'
                            x_vect =  x_vect +  varListe[j] + '])'
                    #print a_punkt
                    #self.script = []
                    
                    #print b_punkt
                    mp_ = 'transpose(' + m_punkt +')'
                    op_ = 'transpose(' + o_punkt +')'
                    x_vect_ = 'transpose(' + x_vect +')'

                    radius_geo = descartes.genRichtVec(m_geo,o_geo)
                    rad = descartes.betrag(radius_geo)
                    
                    _colorIndex_ = self.m2g.colList.index(koeff_.color)
                    maxima_rep = '(' + x_vect_ + ' - ' + mp_ + ').('+  x_vect_ + ' - ' + mp_ + ')  = ' + str(rad) + '^2'
                    script = []
                    script.append( self.m2g.punkt(m_geo,mp_,k_name+"m") )
                    script.append( self.m2g.punkt(o_geo,op_,k_name+"o") )
                    script.append( self.m2g.kugel(k_name+"m",k_name+"o",maxima_rep,k_name,_colorIndex_) )

                    _objectRep_ = [("kmo",m_geo,o_geo),("colorIndex",_colorIndex_)] #

                    maximaObjectRep = '(' + x_vect + ' - ' + m_punkt + ')^2 = ' + str(rad) + '^2'
                    liste = self.geoOops.buildListEntry(k_name,"Kugel",maximaObjectRep, script, _objectRep_ )
                    self.geoOops.addGeoDirEntry(k_name,liste)
                    self.bearbeiteAusdruck('"Kugel mit Mittelpunkt M und Oberflächenpunkt O"$')
                    self.bearbeiteAusdruck('M : ' + mp_ +  ';')
                    self.bearbeiteAusdruck('O : ' + op_ +  ';')
                    self.bearbeiteAusdruck('xvec : ' + x_vect_ +  ';')                 
                    self.bearbeiteAusdruck('"Kugelgleichung"$')
                    rad_ = "%16.12f" % rad                
                    self.bearbeiteAusdruck('(xvec - M).(xvec - M) = ' + rad_  +  '^2;')                





    def kugel4(self,edit=0,k_name=None,_paramListe_=None, _color_=0):
        """Kugel durch 4 Punkte auf Oberfläche"""
        if self.maximaOk:
            varListe = ['x','y','z']
            _overWrite_ = 1
            dim = 4
            titel = "Kugel durch 4 Punkte"            
            if edit:                
                geoName = k_name                                   
                koeff_ = tkMatrix.Kugel4P(root,geoName,self.m2g.colList, _title_= titel, dimension=dim,_punkte_ = _paramListe_, colorIndex=_color_)            
            else:
                geoName = self.geoOops.generateObjectName("K")                                    
                koeff_ = tkMatrix.Kugel4P(root,geoName,self.m2g.colList,_title_= titel,dimension=dim)                
            if koeff_.result <> None:
                _overWrite_ = self.overwriteGeoDirEntry(edit,geoName,koeff_.name)
                            
                if _overWrite_:                                
                    #print g_name
                    k_name = koeff_.name
                    x_vect = "matrix (["
                    a_punkt = "matrix (["
                    b_punkt = "matrix (["
                    c_punkt = "matrix (["
                    d_punkt = "matrix (["                    
                    a_geo = []
                    b_geo = []
                    c_geo = []
                    d_geo = []                
                    

                    for j in range(3): # Spalte der Matrix
                        a_geo.append(koeff_.mtx[0][j])
                        b_geo.append(koeff_.mtx[1][j])
                        c_geo.append(koeff_.mtx[2][j])
                        d_geo.append(koeff_.mtx[3][j])                                            
                        
                        if (j < 2):
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + ','
                            b_punkt =  b_punkt +  koeff_.mtx[1][j] + ','
                            c_punkt =  c_punkt +  koeff_.mtx[2][j] + ','
                            d_punkt =  d_punkt +  koeff_.mtx[3][j] + ','
                            x_vect =  x_vect +  varListe[j] + ','                            
                            
                        else:
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + '])'
                            b_punkt =  b_punkt +  koeff_.mtx[1][j] + '])'
                            c_punkt =  c_punkt +  koeff_.mtx[2][j] + '])'
                            d_punkt =  d_punkt +  koeff_.mtx[3][j] + '])'
                            x_vect =  x_vect +  varListe[j] + '])'

                    
                    #self.script = []
                                        
                    ap_ = 'transpose(' + a_punkt +')'
                    bp_ = 'transpose(' + b_punkt +')'
                    cp_ = 'transpose(' + c_punkt +')'
                    dp_ = 'transpose(' + d_punkt +')'
                    x_vect_ = 'transpose(' + x_vect +')'

                    _ebene_ = descartes.ebeneNF([a_geo,b_geo,c_geo])    
                    _erg_ = descartes.punktProbeE(_ebene_[0],_ebene_[1],d_geo)  #
                    if _erg_:
                        tkMessageBox.showerror("Eingabe-Fehler!", 'Hhmm . . . \ndie vier Punkte liegen in einer Ebene!')
                    else:
                        _colorIndex_ = self.m2g.colList.index(koeff_.color)
                        (m_geo,radius)  = descartes.kugel4P(a_geo,b_geo,c_geo,d_geo)
                        #print radius
                        mp_ =  descartes.punkt2maxima(m_geo)
                        m_punkt = descartes.punkt2maxima(m_geo,_t_=0)
                        

                        maxima_rep = " " #
                        maxima_rep = '(' + x_vect_ + ' - ' + mp_ + ').('+  x_vect_ + ' - ' + mp_ + ')  = ' + str(radius) + '^2'
                        #print maxima_rep
                        
##                        script = []
##                        script.append( self.m2g.punkt(a_geo,ap_,k_name+"p1") )
##                        script.append( self.m2g.punkt(b_geo,bp_,k_name+"p2") )
##                        script.append( self.m2g.punkt(c_geo,cp_,k_name+"p3") )
##                        script.append( self.m2g.punkt(d_geo,dp_,k_name+"p4") )
                        
                        #print m_geo
                        #print descartes.vec2str(m_geo)
                        script = []
                        script.append( self.m2g.punkt( descartes.vec2str(m_geo) ,mp_,k_name+"m") ) # Mittelpunkt
                        script.append( self.m2g.punkt(a_geo,ap_,k_name+"o") ) # Punkt auf Oberflaeche
                        script.append( self.m2g.kugel(k_name+"m",k_name+"o",maxima_rep,k_name,_colorIndex_) )
                                                
                        _objectRep_ = [("k4p",a_geo,b_geo,c_geo,d_geo),("colorIndex",_colorIndex_) ] #

                        maximaObjectRep = '(' + x_vect + ' - ' + str(m_geo) + ')^2 = ' + str(radius) + '^2'                        
                        liste = self.geoOops.buildListEntry(k_name,"Kugel",maximaObjectRep, script, _objectRep_ )
                        self.geoOops.addGeoDirEntry(k_name,liste)
                        
                        ############################
                        self.bearbeiteAusdruck('"Kugel durch vier Punkte A, B, C, D"$')
                        self.bearbeiteAusdruck(k_name + '_A : ' + ap_ +  ';')
                        self.bearbeiteAusdruck(k_name + '_B : ' + bp_ +  ';')
                        self.bearbeiteAusdruck(k_name + '_C : ' + cp_ +  ';')
                        self.bearbeiteAusdruck(k_name + '_D : ' + dp_ +  ';')
                        self.bearbeiteAusdruck('M : ' + mp_ +  ';')
                        self.bearbeiteAusdruck('xvec : ' + x_vect_ +  ';')                 
                        self.bearbeiteAusdruck('"Kugelgleichung"$')                               
                        self.bearbeiteAusdruck('(xvec - M).(xvec - M) = ' + str(radius)  +  '^2;')








    def kreis3(self,edit=0,k_name=None,_paramListe_=None, _color_=0):
        """Kreis durch 3 Punkte"""
        if self.maximaOk:
            varListe = ['x','y','z']
            _overWrite_ = 1
            dim = 3
            titel = "Kreis durch 3 Punkte"            
            if edit:                
                geoName = k_name                                   
                koeff_ = tkMatrix.Dreieck(root,geoName,self.m2g.colList,_title_= titel,dimension=dim,_punkte_ = _paramListe_,colorIndex=_color_)            
            else:
                geoName = self.geoOops.generateObjectName("K")                                    
                koeff_ = tkMatrix.Dreieck(root,geoName,self.m2g.colList,_title_= titel,dimension=dim)                
            if koeff_.result <> None:
                _overWrite_ = self.overwriteGeoDirEntry(edit,geoName,koeff_.name)
                
                #print koeff_.color
                
                if _overWrite_:                                
                    #print k_name
                    k_name = koeff_.name
                    x_vect = "matrix (["
                    a_punkt = "matrix (["
                    b_punkt = "matrix (["
                    c_punkt = "matrix (["

                    a_geo = []
                    b_geo = []
                    c_geo = []

                    

                    for j in range(3): # Spalte der Matrix
                        a_geo.append(koeff_.mtx[0][j])
                        b_geo.append(koeff_.mtx[1][j])
                        c_geo.append(koeff_.mtx[2][j])

                        
                        if (j < 2):
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + ','
                            b_punkt =  b_punkt +  koeff_.mtx[1][j] + ','
                            c_punkt =  c_punkt +  koeff_.mtx[2][j] + ','
                            x_vect =  x_vect +  varListe[j] + ','                            
                            
                        else:
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + '])'
                            b_punkt =  b_punkt +  koeff_.mtx[1][j] + '])'
                            c_punkt =  c_punkt +  koeff_.mtx[2][j] + '])'

                            x_vect =  x_vect +  varListe[j] + '])'

                    
                    #self.script = []
                                        
                    ap_ = 'transpose(' + a_punkt +')'
                    bp_ = 'transpose(' + b_punkt +')'
                    cp_ = 'transpose(' + c_punkt +')'

                    x_vect_ = 'transpose(' + x_vect +')'
                
                    
                    
                    # Test: liegen die drei Punkte auf einer Geraden?
                    _gerade_ = descartes.punkt2Gerade([a_geo,b_geo])    
                    _erg_ = descartes.punktProbeG(_gerade_[0],_gerade_[1],c_geo)
                    if _erg_:
                        tkMessageBox.showerror("Eingabe-Fehler!", 'Hhmm . . . \ndie drei Punkte liegen auf einer Geraden!')
                    else:                    
                        maxima_rep = " " #
                        
                        _colorIndex_ = self.m2g.colList.index(koeff_.color)
                        _objectRep_ = [("k3p",a_geo,b_geo,c_geo),("colorIndex",_colorIndex_)] #
                        
                        #print _objectRep_                                                

                        erg = descartes.kreis(k_name,a_geo,b_geo,c_geo)
                        m_geo =  erg[0]
                        radius = erg[1]
                        mp_ =  descartes.punkt2maxima(m_geo)
                        m_punkt = descartes.punkt2maxima(m_geo,_t_=0)
                        
##                        print m_geo
##                        print mp_
##                        print m_punkt
                        
                        maxima_rep = '(' + x_vect_ + ' - ' + mp_ + ').('+  x_vect_ + ' - ' + mp_ + ')  = ' + str(radius) + '^2'                    
                    
                        script = []

                        script.append( self.m2g.punkt(a_geo,ap_,k_name+"p1") )
                        script.append( self.m2g.punkt(b_geo,bp_,k_name+"p2") )
                        script.append( self.m2g.punkt(c_geo,cp_,k_name+"p3") )
                        script.append( self.m2g.kreis(k_name+"p1",k_name+"p2",k_name+"p3",maxima_rep,k_name,_colorIndex_)  )                        


                        maximaObjectRep = '(' + x_vect + ' - ' + str(m_geo) + ')^2 = ' + str(radius) + '^2'                        
                        liste = self.geoOops.buildListEntry(k_name,"Kreis",maximaObjectRep, script, _objectRep_ )
                        self.geoOops.addGeoDirEntry(k_name,liste)
                        
                        ############################
                        self.bearbeiteAusdruck('"Kreis durch drei Punkte A, B, C"$')
                        self.bearbeiteAusdruck(k_name + '_A : ' + ap_ +  ';')
                        self.bearbeiteAusdruck(k_name + '_B : ' + bp_ +  ';')
                        self.bearbeiteAusdruck(k_name + '_C : ' + cp_ +  ';')
                        self.bearbeiteAusdruck('M : ' + mp_ +  ';')
                        self.bearbeiteAusdruck('xvec : ' + x_vect_ +  ';')                 
                        self.bearbeiteAusdruck('"Kreisgleichung"$')                               
                        self.bearbeiteAusdruck('(xvec - M).(xvec - M) = ' + str(radius)  +  '^2;')
                        eNF = descartes.dreieckEbeneNF(a_geo,b_geo,c_geo)  # eNF[0] == nvec, eNF[1] = _d_
                        if eNF:
                            self.bearbeiteAusdruck('"Koordinatenform der Kreis-Ebene E_' + k_name + '"$')
                            self.bearbeiteAusdruck('"' + descartes.koordinatenform(eNF[0],eNF[1]) + '"$')
                        




    def rKugel(self,edit=0,k_name=None,_m_punkt_=None,_rad_=None, _color_=0):
        """Kugel mit Mittelpunkt M und Radius r"""
        if self.maximaOk:
            _overWrite_ = 1
            varListe = ['x','y','z']
            dim = 2
            if edit:
                geoName = k_name
                #print str(geoName),str(_m_punkt_),str(_rad_)
                koeff_ = tkMatrix.rKugel(root,geoName,self.m2g.colList,dimension=dim,_mittelp_ = _m_punkt_, _radius_ = _rad_, colorIndex=_color_)
            else:
                geoName = self.geoOops.generateObjectName("K")  
                koeff_ = tkMatrix.rKugel(root,geoName,self.m2g.colList,dimension=dim)
                
            if koeff_.result <> None:
                _overWrite_ = self.overwriteGeoDirEntry(edit,geoName,koeff_.name)
                            
                if _overWrite_:                                
                    
                    k_name = koeff_.name
                    radius = koeff_.radius

                    x_vect = "matrix (["
                    m_punkt = "matrix (["
                    m_geo = []
                    for j in range(dim+1): # Spalte der Matrix
                        m_geo.append(koeff_.mtx[0][j])
                        if (j < dim):
                            m_punkt =  m_punkt +  koeff_.mtx[0][j] + ','
                            x_vect =  x_vect +  varListe[j] + ','
                        else:
                            m_punkt =  m_punkt +  koeff_.mtx[0][j] + '])'
                            x_vect =  x_vect +  varListe[j] + '])'

                    o_geo = descartes.genOberflaechenpunkt(m_geo,radius)
                    o_punkt = descartes.punkt2maxima(o_geo)
                    op_ = 'transpose(' + o_punkt +')'
                    mp_ = 'transpose(' + m_punkt +')'
                    x_vect_ = 'transpose(' + x_vect +')'

                    _colorIndex_ = self.m2g.colList.index(koeff_.color)
                    maxima_rep = '(' + x_vect_ + ' - ' + mp_ + ').('+  x_vect_ + ' - ' + mp_ + ')  = ' + radius + '^2'
                    script = []
                    script.append( self.m2g.punkt(m_geo,mp_,k_name+"m") )
                    script.append( self.m2g.punkt(o_geo,op_,k_name+"o") )
                    script.append( self.m2g.kugel(k_name+"m",k_name+"o",maxima_rep,k_name,_colorIndex_) )
                    
                    _objectRep_ = [("rkm", m_geo, float(radius) ),("colorIndex",_colorIndex_)] # rkm: radius Kugel Mittelpunkt
                    
                    maximaObjectRep = '(' + x_vect + ' - ' + m_punkt + ')^2 = ' + radius + '^2'
                    liste = self.geoOops.buildListEntry(k_name,"Kugel",maximaObjectRep, script,_objectRep_ )
                    self.geoOops.addGeoDirEntry(k_name,liste)
                    self.bearbeiteAusdruck('"Kugel mit Mittelpunkt M und Radius r"$')
                    self.bearbeiteAusdruck('M : ' + mp_ +  ';')
                    self.bearbeiteAusdruck('xvec : ' + x_vect_ +  ';')                 
                    self.bearbeiteAusdruck('"Kugelgleichung"$')                               
                    self.bearbeiteAusdruck('(xvec - M).(xvec - M) = ' + radius  +  '^2;')



    def _kugel_(self,k_i = "K1"):
        """ Bestimmt Abstand d(Kugel K_i, Ursprung O)"""        
        nullPkt = ['0', '0', '0']
        pScript = self.geoOops.getScriptEintrag(k_i)
        pRep = self.geoOops.script2Intern(pScript,2)
        mittelpunkt = pRep[0]
        rad = descartes.getRadius(mittelpunkt,pRep[1])
        _volumen_ = float(4.0/3.0) * float(rad) * float(rad) * float(rad) * math.pi
        #print _volumen_
        self.bearbeiteAusdruck('"Volumen V der Kugel ' + k_i + ': V = ' + str(_volumen_) + '"$')
        
        (quadAbstand,abstand) = descartes.abstandKP(mittelpunkt,rad,nullPkt)        
        if descartes.nullTest( quadAbstand ):
            self.bearbeiteAusdruck('"Ursprung liegt auf der Kugel ' + k_i  + '"$')
        elif ( quadAbstand > 0):
            self.bearbeiteAusdruck('"Ursprung liegt außerhalb der Kugel ' + k_i  + ': d = ' + abstand +  '"$')
        else: # (quadAbstand < 0):
            self.bearbeiteAusdruck('"Ursprung liegt innerhalb der Kugel ' + k_i + ': d = ' + abstand +  '"$')
        
                


    def punktGerade(self,edit=0,g_name=None,_paramListe_= None, _color_=0):
        """Gerade durch zwei Punkte"""
        if self.maximaOk:
            _overWrite_ = 1
            varListe = ['x','y','z']
            dim = 2
            titel = "Gerade durch zwei Punkte A, B"
            if edit:
                geoName = g_name
                koeff_ = tkMatrix.gPunkte(root,geoName,self.m2g.colList,_title_= titel,dimension=dim,_parameter_ = _paramListe_ ,colorIndex=_color_)                            
            else:
                geoName = self.geoOops.generateObjectName("g") 
                koeff_ = tkMatrix.gPunkte(root,geoName,self.m2g.colList,_title_= titel,dimension=dim)            
            if koeff_.result <> None:
                _overWrite_ = self.overwriteGeoDirEntry(edit,geoName,koeff_.name)
                            
                if _overWrite_:                                
                    
                    g_name = koeff_.name
                    #print g_name
                    x_vect = "matrix (["
                    a_punkt = "matrix (["
                    a_geo = []
                    b_geo = []                
                    b_punkt = "matrix (["
                    for j in range(dim+1): # Spalte der Matrix
                        a_geo.append(koeff_.mtx[0][j])
                        b_geo.append(koeff_.mtx[1][j])                                            
                        if (j < dim):
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + ','
                            b_punkt =  b_punkt +  koeff_.mtx[1][j] + ','
                            x_vect =  x_vect +  varListe[j] + ','
                        else:
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + '])'
                            b_punkt =  b_punkt +  koeff_.mtx[1][j] + '])'
                            x_vect =  x_vect +  varListe[j] + '])'
                    #print a_punkt
                    #self.script = []
                    
                    #print b_punkt
                    ap_ = 'transpose(' + a_punkt +')'
                    bp_ = 'transpose(' + b_punkt +')'
                    x_vect_ = 'transpose(' + x_vect +')'
                    
                    uvec_geo = descartes.genRichtVec(a_geo,b_geo)
                    u_vect_ = bp_ + " - " +  ap_

                    
                    _colorIndex_ = self.m2g.colList.index(koeff_.color)
                    maxima_rep = "stuetz"+g_name +  " + r * uvec"+g_name
                    script = []
                    script.append( self.m2g.punkt(a_geo,ap_,"stuetz"+g_name) )
                    script.append( self.m2g.punkt(b_geo,bp_,g_name+"p2") )
                    script.append( self.m2g.richtVec(uvec_geo,u_vect_,"uvec"+g_name) )
                    
                    script.append( self.m2g.gerade(g_name+"p2","stuetz"+ g_name,maxima_rep,g_name,_colorIndex_) )

                    # so bestimmt man Spurpunkte
                    uvec_geo = descartes.genRichtVec(a_geo,b_geo)
                    #print uvec_geo                
    ##                spur = descartes.spurpunkte(a_geo,uvec_geo)
    ##                ebene = ["yz","xz","xy"]
    ##                i = 0                
    ##                for punkt in spur:
    ##                    if len(punkt) > 0:                        
    ##                        maxima_rep = descartes.punkt2maxima(punkt)                              
    ##                        script.append( self.m2g.punkt(punkt,maxima_rep,g_name+"sp_" + ebene[i],ebene[i]) )
    ##                    i = i + 1

                    _objectRep_ = [("g2p",a_geo,b_geo),("colorIndex",_colorIndex_)] #
                    maximaObjectRep = a_punkt + " + r * (" + b_punkt + " - " + a_punkt + ")" 
                    liste = self.geoOops.buildListEntry(g_name,"Gerade",maximaObjectRep, script, _objectRep_ )
                    self.geoOops.addGeoDirEntry(g_name,liste)
                    #geoOops.getObject(g_name)
                    
                    
                    ############################
                    self.bearbeiteAusdruck('"Zwei Punkte A, B der Gerade g"$')
                    self.bearbeiteAusdruck('A : ' + ap_ +  ';')
                    self.bearbeiteAusdruck('B : ' + bp_ +  ';')
                    self.bearbeiteAusdruck('"Aufpunkt stuetzvec"$')
                    self.bearbeiteAusdruck('stuetz' +  g_name  +' : ' + ap_ +  ';')                
                    self.bearbeiteAusdruck('"Richtungsvektor uvec"$')
                    self.bearbeiteAusdruck('uvec' +  g_name  +' : B - A;')
                    self.bearbeiteAusdruck('"x-Vektor xvec"$')
                    self.bearbeiteAusdruck('xvec : ' + x_vect_ +  ';')                
                    #self._gerade_(g_name)


                
    def pgerade(self,edit=0,g_name=None,_aufpunkt_=None,_rvec_=None, _color_=0):
        #self.pgerade(edit=1,g_name=maximaName,_auf_punkt_=aufpunkt,_rvec_=uvec)
        """Gerade in Parameterform"""
        _test_ = 0
        if self.maximaOk:            
            varListe = ['x','y','z']
            _overWrite_ = 1
            dim = 3
            if edit:
                #print g_name,_aufpunkt_,_rvec_
                geoName = g_name
                koeff_ = tkMatrix.Gerade(root,geoName,self.m2g.colList,dimension=dim,_stuetzvec_=_aufpunkt_,_richtvec_=_rvec_ ,colorIndex=_color_)                
            else:
                geoName = self.geoOops.generateObjectName("g") 
                koeff_ = tkMatrix.Gerade(root,geoName,self.m2g.colList,dimension=dim)
            if koeff_.result <> None:
                _overWrite_ = self.overwriteGeoDirEntry(edit,geoName,koeff_.name)
                            
                if _overWrite_:                                
                    
                    g_name = koeff_.name
                    #print g_name
                    aufpunkt = "matrix (["
                    x_vect = "matrix (["
                    u_vect = "matrix (["
                    auf_geo = []
                    uvec_geo = []
                    for i in range(dim): # Zeile der Matrix
                        x_vect =  x_vect +  varListe[i] 
                        aufpunkt = aufpunkt + koeff_.mtx[i][0]
                        u_vect =  u_vect +  koeff_.mtx[i][1]
                        auf_geo.append(koeff_.mtx[i][0])                       
                        uvec_geo.append(koeff_.mtx[i][1])   

                        if (i < dim-1):
                            x_vect =  x_vect +  ','
                            aufpunkt = aufpunkt + ','
                            u_vect =  u_vect +  ','
                            
                        else:
                            x_vect =  x_vect +  '])'                       
                            aufpunkt = aufpunkt +  '])'
                            u_vect =  u_vect +  '])'
                                                              
                    aufpunkt_ = 'transpose(' + aufpunkt +')'
                    x_vect_ = 'transpose(' + x_vect +')'
                    u_vect_ = 'transpose(' + u_vect +')'
                    #parameterformGerade(self,g1,p1,maxima,name="g1"):
                    #maxima_rep = aufpunkt_ + " + r * (" + u_vect_ + ")"

                    
                    _colorIndex_ = self.m2g.colList.index(koeff_.color)
                    maxima_rep = "stuetz"+g_name + " + r * uvec"  + g_name
                    script = []
                    script.append( self.m2g.punkt(auf_geo,aufpunkt_,"stuetz"+g_name) )
                    script.append( self.m2g.richtVec(uvec_geo,u_vect_,"uvec"+g_name) )
                    script.append( self.m2g.parameterformGerade(uvec_geo,auf_geo,maxima_rep,g_name,_colorIndex_) )

                    _objectRep_ = [("pgerade",auf_geo,uvec_geo),("colorIndex",_colorIndex_)]
                    maximaObjectRep = aufpunkt + " + r * (" + u_vect + ")"

                    liste = self.geoOops.buildListEntry(g_name,"Gerade",maximaObjectRep, script, _objectRep_  )
                    self.geoOops.addGeoDirEntry(g_name,liste)
                    
                    # so bestimmt man Spurpunkte
    ##                spur = descartes.spurpunkte(auf_geo,uvec_geo)
    ##                ebene = ["yz","xz","xy"]
    ##                i = 0                
    ##                for punkt in spur:
    ##                    if len(punkt) > 0:                        
    ##                        maxima_rep = descartes.punkt2maxima(punkt)                              
    ##                        script.append( self.m2g.punkt(punkt,maxima_rep,g_name+"sp_" + ebene[i],ebene[i]) )
    ##                        #print punkt
    ##                    i = i + 1

                    ############################

                    self.bearbeiteAusdruck('"x-Vektor xvec"$')
                    self.bearbeiteAusdruck('xvec : ' + x_vect_ +  ';')
                    self.bearbeiteAusdruck('"Aufpunkt stuetzvec"$')
                    self.bearbeiteAusdruck('stuetz' +  g_name  +' : ' + aufpunkt_ +  ';')                
                    self.bearbeiteAusdruck('"Richtungsvektor uvec"$')
                    self.bearbeiteAusdruck('uvec' +  g_name  + ' : ' + u_vect_ +  ';')
                    self.bearbeiteAusdruck('"Geradengleichung g"$')
                    self.bearbeiteAusdruck('display(stuetz'+g_name  +' +  r * uvec'+g_name  + ')$')
                    self.bearbeiteAusdruck(g_name + ' : stuetz'+g_name  +' +  r * uvec'+g_name + '$')
                    
                    #self._gerade_(g_name)
                


    def _gerade_(self,g_i = "g1"):
        # Stuetzvektor: stuetz-g_i, Richtungsvektor: uvec-g_i
        #self.bearbeiteAusdruck('"Geradengleichung g"$')
        self.bearbeiteAusdruck('display(stuetz' +  g_i  +' +  r * uvec' +  g_i  +')$')
        #self.bearbeiteAusdruck(g_i + ' : stuetz' +  g_i  +' +  r * uvec' +  g_i  +'$')
        spur = 'spurpunkte(g,stuetzvec,uvec) := block(ebene : ["yz","xz","xy"],for i: 1 thru 3 do(ergebnis : solve(g[i,1] = 0,r),'
        spur = spur + 'if not atom(ergebnis)  then (if (length(ergebnis) > 0) then'
        spur = spur + '(stmp : stuetzvec +  args(part(ergebnis,1))[2] * uvec,'
        spur = spur + 'print("Schnittpunkt mit der",ebene[i],"- Koordinatenebene:",transpose(stmp)))'
        spur = spur + 'else (print("Kein Schnittpunkt mit der",ebene[i],"- Koordinatenebene!")))'
        spur = spur + 'else (print("Schnittpunkt mit der",ebene[i],"- Koordinatenebene:",transpose(stuetzvec)) )))$'
        self.bearbeiteAusdruck(spur)
        self.bearbeiteAusdruck('"Spurpunkte"$')
        self.bearbeiteAusdruck('spurpunkte(' + g_i +', stuetz' +  g_i  +', uvec' +  g_i   + ')$')
        nullPkt = ['0', '0', '0']
        pScript = self.geoOops.getScriptEintrag(g_i)        
        pRep = self.geoOops.script2Intern(pScript,2)
        aufpunkt = pRep[0]
        uvec = descartes.punkt2Gerade(pRep)[1]
        abstand = descartes.abstandGP(aufpunkt,uvec,nullPkt)
        self.bearbeiteAusdruck('"Abstand der Geraden ' + g_i + ' vom Ursprung: d = ' + abstand +  '"$')
        
        


    def _betrag_(self,pVec,p_i = "A"):
        ergebnis = descartes.betrag(pVec)        
        self.bearbeiteAusdruck('"Abstand d_' +  p_i  + '(Ursprung O, Punkt ' +  p_i  +  ')"$')
        self.bearbeiteAusdruck('d_'+ p_i  +' : ' + str(ergebnis)  + ' $')      

    def strecke(self,edit=0,d_name=None,_puListe_= None, _color_=0):
        """Strecke durch zwei Punkte"""
        _test_ = 0       
        if self.maximaOk:
            varListe = ['x','y','z']
            _overWrite_ = 1
            dim = 3
            titel = "Strecke von A nach B"
            if edit:
                #print "Argumente"
                #print _puListe_
                #print d_name
                #print _color_
                geoName = d_name
                #print geoName
                koeff_ = tkMatrix.gPunkte(root,geoName,self.m2g.colList, _title_ = titel,dimension=dim,_parameter_ = _puListe_ , colorIndex=_color_)
            else:
                geoName = self.geoOops.generateObjectName("S") 
                koeff_ = tkMatrix.gPunkte(root,geoName,self.m2g.colList, _title_ = titel,dimension=dim)
                
            if koeff_.result <> None:
                _overWrite_ = self.overwriteGeoDirEntry(edit,geoName,koeff_.name)
                            
                if _overWrite_:                                
                    
                    d_name = koeff_.name
                    #print d_name
                    a_geo = []
                    b_geo = []                
                    
                    
                    a_punkt = "matrix (["
                    b_punkt = "matrix (["
                                    
                    for j in range(dim): # Spalte der Matrix
                        a_geo.append(koeff_.mtx[0][j])
                        b_geo.append(koeff_.mtx[1][j])
                        		    

                        if (j < dim-1):
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + ','
                            b_punkt =  b_punkt +  koeff_.mtx[1][j] + ','
                            

                        else:
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + '])'
                            b_punkt =  b_punkt +  koeff_.mtx[1][j] + '])'
                            

                            
                            
                    ap_ = 'transpose(' + a_punkt +')'
                    bp_ = 'transpose(' + b_punkt +')'
                    

                    #maxima_rep = ap_ + " + r * (" + bp_ + " - " + ap_ + ") + s * (" + cp_ + " - " + ap_ + ")"
                    # Test: liegen die drei Punkte auf einer Geraden?
                    _strecke_ = descartes.punkt2Gerade([a_geo,b_geo])
                    _erg_ = descartes.punktProbeP(_strecke_[0],_strecke_[1])
                    
                    if _erg_:
                        tkMessageBox.showerror("Eingabe-Fehler!", 'Hhmm . . . \ndie zwei Punkte sind identisch!')
                    else:
                        _colorIndex_ = self.m2g.colList.index(koeff_.color)
                        
                        maxima_rep = " " #
                        script = []
                        script.append( self.m2g.punkt(a_geo,ap_,d_name+"p1") )
                        script.append( self.m2g.punkt(b_geo,bp_,d_name+"p2") )
                        
                        script.append( self.m2g.strecke(a_geo,b_geo,maxima_rep,d_name,_colorIndex_) )

                        _objectRep_ = [("strecke",a_geo,b_geo),("colorIndex",_colorIndex_)]
                        #print _objectRep_
                        maximaObjectRep = a_punkt + ", " +  b_punkt 
                        liste = self.geoOops.buildListEntry(d_name,"Strecke",maximaObjectRep, script, _objectRep_  )
                        self.geoOops.addGeoDirEntry(d_name,liste)
                        
                        ############################
                        self.bearbeiteAusdruck('"Strecke von A nach B"$')
                        self.bearbeiteAusdruck(d_name + '_A : ' + ap_ +  ';')
                        self.bearbeiteAusdruck(d_name + '_B : ' + bp_ +  ';')
                        
                    
                else:
                    print "abgebrochen . . ."

                    

    def dreieck(self,edit=0,d_name=None,_puListe_= None, _color_=0):
        """Dreieck durch drei Punkte"""
        _test_ = 0       
        if self.maximaOk:
            varListe = ['x','y','z']
            _overWrite_ = 1
            dim = 3
            titel = "Dreieck durch Punkte A, B, C"
            if edit:
                #print str(_puListe_)
                #print d_name
                geoName = d_name
                koeff_ = tkMatrix.Dreieck(root,geoName,self.m2g.colList, _title_ = titel,dimension=dim,_punkte_ = _puListe_,colorIndex=_color_)
            else:
                geoName = self.geoOops.generateObjectName("D") 
                koeff_ = tkMatrix.Dreieck(root,geoName,self.m2g.colList, _title_ = titel,dimension=dim)            
            if koeff_.result <> None:
                _overWrite_ = self.overwriteGeoDirEntry(edit,geoName,koeff_.name)
                            
                if _overWrite_:                                
                    
                    d_name = koeff_.name
                    #print e_name
                    a_geo = []
                    b_geo = []                
                    c_geo = []
                    
                    a_punkt = "matrix (["
                    b_punkt = "matrix (["
                    c_punkt = "matrix (["                
                    for j in range(dim): # Spalte der Matrix
                        a_geo.append(koeff_.mtx[0][j])
                        b_geo.append(koeff_.mtx[1][j])
                        c_geo.append(koeff_.mtx[2][j])		    

                        if (j < dim-1):
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + ','
                            b_punkt =  b_punkt +  koeff_.mtx[1][j] + ','
                            c_punkt =  c_punkt +  koeff_.mtx[2][j] + ','

                        else:
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + '])'
                            b_punkt =  b_punkt +  koeff_.mtx[1][j] + '])'
                            c_punkt =  c_punkt +  koeff_.mtx[2][j] + '])'

                            
                            
                    ap_ = 'transpose(' + a_punkt +')'
                    bp_ = 'transpose(' + b_punkt +')'
                    cp_ = 'transpose(' + c_punkt +')'

                    #maxima_rep = ap_ + " + r * (" + bp_ + " - " + ap_ + ") + s * (" + cp_ + " - " + ap_ + ")"
                    # Test: liegen die drei Punkte auf einer Geraden?
                    _gerade_ = descartes.punkt2Gerade([a_geo,b_geo])    
                    _erg_ = descartes.punktProbeG(_gerade_[0],_gerade_[1],c_geo)
                    if _erg_:
                        tkMessageBox.showerror("Eingabe-Fehler!", 'Hhmm . . . \ndie drei Punkte liegen auf einer Geraden!')
                    else:
                        _colorIndex_ = self.m2g.colList.index(koeff_.color)
                        
                        maxima_rep = " " #
                        script = []
                        script.append( self.m2g.punkt(a_geo,ap_,d_name+"p1") )
                        script.append( self.m2g.punkt(b_geo,bp_,d_name+"p2") )
                        script.append( self.m2g.punkt(c_geo,cp_,d_name+"p3") )
                        script.append( self.m2g.dreieck(a_geo,b_geo,c_geo,maxima_rep,d_name,_colorIndex_) )

                        _objectRep_ = [("dreieck",a_geo,b_geo,c_geo),("colorIndex",_colorIndex_)]
                        maximaObjectRep = a_punkt + ", " +  b_punkt + ", " + c_punkt 
                        liste = self.geoOops.buildListEntry(d_name,"Dreieck",maximaObjectRep, script, _objectRep_  )
                        self.geoOops.addGeoDirEntry(d_name,liste)
                        
                        ############################
                        self.bearbeiteAusdruck('"Dreieck durch die Punkte A, B, C"$')
                        self.bearbeiteAusdruck(d_name + '_A : ' + ap_ +  ';')
                        self.bearbeiteAusdruck(d_name + '_B : ' + bp_ +  ';')
                        self.bearbeiteAusdruck(d_name + '_C : ' + cp_ +  ';')
                    
                else:
                    print "abgebrochen . . ."
                                    


    def tetraeder(self,edit=0,t_name=None,_puListe_= None, _color_=0):
        """Tetraeder ist festgelegt durch 4 Punkte"""
        _test_ = 0
        #print "Hier tetraeder . . ."
        if self.maximaOk:
            varListe = ['x','y','z']
            _overWrite_ = 1
            dim = 4
            titel = "Tetraeder"
            if edit:
                #print str(_puListe_)
                #print t_name
                geoName = t_name
                koeff_ = tkMatrix.tPunkte(root,geoName,self.m2g.colList,_title_= titel,dimension=dim,_punkte_ = _puListe_,colorIndex=_color_)
            else:
                geoName = self.geoOops.generateObjectName("T") 
                koeff_ = tkMatrix.tPunkte(root,geoName,self.m2g.colList,_title_= titel,dimension=dim)


            if koeff_.result <> None:
                _overWrite_ = self.overwriteGeoDirEntry(edit,geoName,koeff_.name)
                            
                if _overWrite_:                                
                    
                    t_name = koeff_.name
                    a_punkt = "matrix (["
                    b_punkt = "matrix (["
                    c_punkt = "matrix (["
                    d_punkt = "matrix (["                    
                    a_geo = []
                    b_geo = []
                    c_geo = []
                    d_geo = []                
                    

                    for j in range(3): # Spalte der Matrix
                        a_geo.append(koeff_.mtx[0][j])
                        b_geo.append(koeff_.mtx[1][j])
                        c_geo.append(koeff_.mtx[2][j])
                        d_geo.append(koeff_.mtx[3][j])                                            
                        
                        if (j < 2):
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + ','
                            b_punkt =  b_punkt +  koeff_.mtx[1][j] + ','
                            c_punkt =  c_punkt +  koeff_.mtx[2][j] + ','
                            d_punkt =  d_punkt +  koeff_.mtx[3][j] + ','
                            
                        else:
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + '])'
                            b_punkt =  b_punkt +  koeff_.mtx[1][j] + '])'
                            c_punkt =  c_punkt +  koeff_.mtx[2][j] + '])'
                            d_punkt =  d_punkt +  koeff_.mtx[3][j] + '])'                            

                    #print a_punkt
                    #self.script = []
                                        
                    ap_ = 'transpose(' + a_punkt +')'
                    bp_ = 'transpose(' + b_punkt +')'
                    cp_ = 'transpose(' + c_punkt +')'
                    dp_ = 'transpose(' + d_punkt +')'                    
                    
                    #print ap_, bp_, cp_, dp_,                                    

                    #maxima_rep = ap_ + " + r * (" + bp_ + " - " + ap_ + ") + s * (" + cp_ + " - " + ap_ + ")"
                    # Test: liegen die vier Punkte in einer Ebene?
                    _ebene_ = descartes.ebeneNF([a_geo,b_geo,c_geo])    
                    _erg_ = descartes.punktProbeE(_ebene_[0],_ebene_[1],d_geo)  #
                    if _erg_:
                        tkMessageBox.showerror("Eingabe-Fehler!", 'Hhmm . . . \ndie vier Punkte liegen in einer Ebene!')
                    else:                                                            
                        maxima_rep = " " #
                        _colorIndex_ = self.m2g.colList.index(koeff_.color)
                        script = []
                        script.append( self.m2g.punkt(a_geo,ap_,t_name+"p1") )
                        script.append( self.m2g.punkt(b_geo,bp_,t_name+"p2") )
                        script.append( self.m2g.punkt(c_geo,cp_,t_name+"p3") )
                        script.append( self.m2g.punkt(d_geo,dp_,t_name+"p4") )
                        
                        script.append( self.m2g.tetraeder(a_geo,b_geo,c_geo,d_geo,maxima_rep,t_name,_colorIndex_) )
                        
                        _objectRep_ = [("tetraeder",a_geo,b_geo,c_geo,d_geo),("colorIndex",_colorIndex_)]
                        
                        maximaObjectRep = a_punkt + ", " +  b_punkt + ", " + c_punkt + ", " +  d_punkt
                        liste = self.geoOops.buildListEntry(t_name,"Tetraeder",maximaObjectRep, script, _objectRep_  )
                        self.geoOops.addGeoDirEntry(t_name,liste)
                        
                        ############################
                        self.bearbeiteAusdruck('"Tetraeder (Punkte A, B, C, D)"$')
                        self.bearbeiteAusdruck(t_name + '_A : ' + ap_ +  ';')
                        self.bearbeiteAusdruck(t_name + '_B : ' + bp_ +  ';')
                        self.bearbeiteAusdruck(t_name + '_C : ' + cp_ +  ';')
                        self.bearbeiteAusdruck(t_name + '_D : ' + dp_ +  ';')
                    
                else:
                    print "abgebrochen . . ."
                                    
    def viereck(self,edit=0,v_name=None,_puListe_= None, _color_=0):
        """Viereck ist festgelegt durch 4 Punkte"""
        _test_ = 0
        #print "Hier tetraeder . . ."
        if self.maximaOk:
            varListe = ['x','y','z']
            _overWrite_ = 1
            dim = 4
            titel = "Viereck"
            if edit:
                #print str(_puListe_)
                #print t_name
                geoName = v_name
                koeff_ = tkMatrix.tPunkte(root,geoName,self.m2g.colList, _title_= titel,dimension=dim,_punkte_ = _puListe_,colorIndex=_color_)
            else:
                geoName = self.geoOops.generateObjectName("V")
                koeff_ = tkMatrix.tPunkte(root,geoName,self.m2g.colList,_title_= titel,dimension=dim)


            if koeff_.result <> None:
                _overWrite_ = self.overwriteGeoDirEntry(edit,geoName,koeff_.name)
                            
                if _overWrite_:                                
                    
                    v_name = koeff_.name
                    a_punkt = "matrix (["
                    b_punkt = "matrix (["
                    c_punkt = "matrix (["
                    d_punkt = "matrix (["                    
                    a_geo = []
                    b_geo = []
                    c_geo = []
                    d_geo = []                
                    

                    for j in range(3): # Spalte der Matrix
                        a_geo.append(koeff_.mtx[0][j])
                        b_geo.append(koeff_.mtx[1][j])
                        c_geo.append(koeff_.mtx[2][j])
                        d_geo.append(koeff_.mtx[3][j])                                            
                        
                        if (j < 2):
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + ','
                            b_punkt =  b_punkt +  koeff_.mtx[1][j] + ','
                            c_punkt =  c_punkt +  koeff_.mtx[2][j] + ','
                            d_punkt =  d_punkt +  koeff_.mtx[3][j] + ','
                            
                        else:
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + '])'
                            b_punkt =  b_punkt +  koeff_.mtx[1][j] + '])'
                            c_punkt =  c_punkt +  koeff_.mtx[2][j] + '])'
                            d_punkt =  d_punkt +  koeff_.mtx[3][j] + '])'                            

                    #print a_punkt
                    #self.script = []
                                        
                    ap_ = 'transpose(' + a_punkt +')'
                    bp_ = 'transpose(' + b_punkt +')'
                    cp_ = 'transpose(' + c_punkt +')'
                    dp_ = 'transpose(' + d_punkt +')'                    
                    
                    #print ap_, bp_, cp_, dp_,                                    

                    #maxima_rep = ap_ + " + r * (" + bp_ + " - " + ap_ + ") + s * (" + cp_ + " - " + ap_ + ")"
                    # Test: liegen die vier Punkte in einer Ebene?
                    _ebene_ = descartes.ebeneNF([a_geo,b_geo,c_geo])    
                    _erg_ = descartes.punktProbeE(_ebene_[0],_ebene_[1],d_geo)  #
                    if not _erg_:
                        tkMessageBox.showerror("Eingabe-Fehler!", 'Hhmm . . . \ndie vier Punkte liegen NICHT in einer Ebene!')
                    else:
                        maxima_rep = " " #
                        _colorIndex_ = self.m2g.colList.index(koeff_.color)
                        script = []
                        script.append( self.m2g.punkt(a_geo,ap_,v_name+"p1") )
                        script.append( self.m2g.punkt(b_geo,bp_,v_name+"p2") )
                        script.append( self.m2g.punkt(c_geo,cp_,v_name+"p3") )
                        script.append( self.m2g.punkt(d_geo,dp_,v_name+"p4") )
                        
                        script.append( self.m2g.viereck(a_geo,b_geo,c_geo,d_geo,maxima_rep,v_name,_colorIndex_) )
                        
                        _objectRep_ = [("viereck",a_geo,b_geo,c_geo,d_geo),("colorIndex",_colorIndex_)]
                        
                        maximaObjectRep = a_punkt + ", " +  b_punkt + ", " + c_punkt + ", " +  d_punkt
                        liste = self.geoOops.buildListEntry(v_name,"Viereck",maximaObjectRep, script, _objectRep_  )
                        self.geoOops.addGeoDirEntry(v_name,liste)
                        
                        ############################
                        self.bearbeiteAusdruck('"Viereck (Punkte A, B, C, D)"$')
                        self.bearbeiteAusdruck(v_name + '_A : ' + ap_ +  ';')
                        self.bearbeiteAusdruck(v_name + '_B : ' + bp_ +  ';')
                        self.bearbeiteAusdruck(v_name + '_C : ' + cp_ +  ';')
                        self.bearbeiteAusdruck(v_name + '_D : ' + dp_ +  ';')
                    
                else:
                    print "abgebrochen . . ."

    def quader(self,edit=0,q_name=None,_puListe_= None, _color_=0):
        """Quader ist festgelegt durch 5 Punkte"""
        _test_ = 0
        #print "Hier tetraeder . . ."
        if self.maximaOk:
            varListe = ['x','y','z']
            _overWrite_ = 1
            dim = 5
            titel = "Quader"
            if edit:
                #print str(_puListe_)
                #print q_name
                geoName = q_name
                koeff_ = tkMatrix.pyramide(root,geoName,self.m2g.colList,_title_= titel,dimension=dim,_punkte_ = _puListe_,colorIndex=_color_)
            else:
                geoName = self.geoOops.generateObjectName("Qu")
                koeff_ = tkMatrix.pyramide(root,geoName,self.m2g.colList,_title_= titel,dimension=dim)
                #koeff_ = tkMatrix.tPunkte(root,geoName,_title_= titel,dimension=dim)


            if koeff_.result <> None:
                _overWrite_ = self.overwriteGeoDirEntry(edit,geoName,koeff_.name)
                            
                if _overWrite_:                                
                    
                    q_name = koeff_.name

                    a_punkt = "matrix (["
                    b_punkt = "matrix (["
                    c_punkt = "matrix (["
                    d_punkt = "matrix (["
                    e_punkt = "matrix (["
                    f_punkt = "matrix (["
                    g_punkt = "matrix (["
                    h_punkt = "matrix (["
                   
                    a_geo = []
                    b_geo = []
                    c_geo = []
                    d_geo = []
                    e_geo = []
                    f_geo = []
                    g_geo = []
                    h_geo = []
                    

                    for j in range(3): # Spalte der Matrix
                        a_geo.append(koeff_.mtx[0][j])
                        b_geo.append(koeff_.mtx[1][j])
                        c_geo.append(koeff_.mtx[2][j])
                        d_geo.append(koeff_.mtx[3][j])
                        e_geo.append(koeff_.mtx[4][j])                                                                    
                        
                        if (j < 2):
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + ','
                            b_punkt =  b_punkt +  koeff_.mtx[1][j] + ','
                            c_punkt =  c_punkt +  koeff_.mtx[2][j] + ','
                            d_punkt =  d_punkt +  koeff_.mtx[3][j] + ','
                            e_punkt =  e_punkt +  koeff_.mtx[4][j] + ','
                            f_punkt =  f_punkt +  koeff_.mtx[1][j] + ','  # Punkt B
                            g_punkt =  g_punkt +  koeff_.mtx[2][j] + ','  # Punkt C
                            h_punkt =  h_punkt +  koeff_.mtx[3][j] + ','  # Punkt D                            
                            
                        else:
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + '])'
                            b_punkt =  b_punkt +  koeff_.mtx[1][j] + '])'
                            c_punkt =  c_punkt +  koeff_.mtx[2][j] + '])'
                            d_punkt =  d_punkt +  koeff_.mtx[3][j] + '])'
                            e_punkt =  e_punkt +  koeff_.mtx[4][j] + '])'                                                        
                            f_punkt =  f_punkt +  koeff_.mtx[4][j] + '])' # Punkt E (!)
                            g_punkt =  g_punkt +  koeff_.mtx[4][j] + '])' # Punkt E (!)
                            h_punkt =  h_punkt +  koeff_.mtx[4][j] + '])' # Punkt E (!)

                    #print a_punkt
                    #self.script = []
                    # 3 Punkte des Quaders basteln       
                    for j in range(2): # Spalte der Matrix
                        f_geo.append(b_geo[j])
                        g_geo.append(c_geo[j])
                        h_geo.append(d_geo[j])
                    f_geo.append(e_geo[2])
                    g_geo.append(e_geo[2])
                    h_geo.append(e_geo[2])
                    

                                        
                    ap_ = 'transpose(' + a_punkt +')'
                    bp_ = 'transpose(' + b_punkt +')'
                    cp_ = 'transpose(' + c_punkt +')'
                    dp_ = 'transpose(' + d_punkt +')'
                    ep_ = 'transpose(' + e_punkt +')'                                        
                    fp_ = 'transpose(' + f_punkt +')'
                    gp_ = 'transpose(' + g_punkt +')'
                    hp_ = 'transpose(' + h_punkt +')'
                    
                    #print ap_, bp_, cp_, dp_, ep_,
                    #print e_geo, f_geo, g_geo, h_geo

                    #maxima_rep = ap_ + " + r * (" + bp_ + " - " + ap_ + ") + s * (" + cp_ + " - " + ap_ + ")"
                    _ebene_ = descartes.ebeneNF([a_geo,b_geo,c_geo])                    
                    _erg1_ = descartes.punktProbeE(_ebene_[0],_ebene_[1],d_geo)  #
                    _erg2_ = descartes.punktProbeE(_ebene_[0],_ebene_[1],e_geo)  #
                    if not _erg1_:
                        tkMessageBox.showerror("Eingabe-Fehler!", 'Hhmm . . . \ndie vier Punkte der Grundfläche \nliegen NICHT in einer Ebene!')                    
                    elif _erg2_:
                        tkMessageBox.showerror("Eingabe-Fehler!", 'Hhmm . . . \nder Punkt E liegt in der Grundfläche!')
                    else:
                        _colorIndex_ = self.m2g.colList.index(koeff_.color)
                        maxima_rep = " " #
                        script = []
                        script.append( self.m2g.punkt(a_geo,ap_,q_name+"p1") )
                        script.append( self.m2g.punkt(b_geo,bp_,q_name+"p2") )
                        script.append( self.m2g.punkt(c_geo,cp_,q_name+"p3") )
                        script.append( self.m2g.punkt(d_geo,dp_,q_name+"p4") )
                        script.append( self.m2g.punkt(e_geo,ep_,q_name+"p5") )                    
                        script.append( self.m2g.punkt(f_geo,fp_,q_name+"p6") )
                        script.append( self.m2g.punkt(g_geo,gp_,q_name+"p7") )
                        script.append( self.m2g.punkt(h_geo,hp_,q_name+"p8") )                    
                        script.append( self.m2g.quader(a_geo,b_geo,c_geo,d_geo,e_geo,f_geo,g_geo,h_geo,maxima_rep,q_name,_colorIndex_) )
                        
                        _objectRep_ = [("quader",a_geo,b_geo,c_geo,d_geo,e_geo,f_geo,g_geo,h_geo),("colorIndex",_colorIndex_)]
                        
                        maximaObjectRep = a_punkt + ", " +  b_punkt + ", " + c_punkt + ", " +  d_punkt + ", " +  e_punkt + ", " +  f_punkt + ", " +  g_punkt + ", " + h_punkt
                        liste = self.geoOops.buildListEntry(q_name,"Quader",maximaObjectRep, script, _objectRep_  )
                        self.geoOops.addGeoDirEntry(q_name,liste)
                        
                        ############################
                        self.bearbeiteAusdruck('"Quader (Punkte A, B, C, D, E)"$')
                        self.bearbeiteAusdruck(q_name + '_A : ' + ap_ +  ';')
                        self.bearbeiteAusdruck(q_name + '_B : ' + bp_ +  ';')
                        self.bearbeiteAusdruck(q_name + '_C : ' + cp_ +  ';')
                        self.bearbeiteAusdruck(q_name + '_D : ' + dp_ +  ';')
                        self.bearbeiteAusdruck(q_name + '_E : ' + ep_ +  ';')
                        self.bearbeiteAusdruck(q_name + '_F : ' + fp_ +  ';')
                        self.bearbeiteAusdruck(q_name + '_G : ' + gp_ +  ';')
                        self.bearbeiteAusdruck(q_name + '_H : ' + hp_ +  ';')
                    
                else:
                    print "abgebrochen . . ."




    def pyramide(self,edit=0,p_name=None,_puListe_= None, _color_=0):
        """Pyramide ist festgelegt durch 5 Punkte"""
        _test_ = 0        
        if self.maximaOk:
            varListe = ['x','y','z']
            _overWrite_ = 1
            dim = 5
            titel = "Pyramide"
            if edit:
                #print p_name
                #print str(_puListe_)                
                geoName = p_name
                #koeff_ = tkMatrix.tPunkte(root,geoName,_title_= titel,dimension=dim,_punkte_ = _puListe_)
                koeff_ = tkMatrix.pyramide(root,geoName,self.m2g.colList,_title_= titel,dimension=dim,_punkte_ = _puListe_,colorIndex=_color_)
            else:
                geoName = self.geoOops.generateObjectName("Py")
                koeff_ = tkMatrix.pyramide(root,geoName,self.m2g.colList,_title_= titel,dimension=dim)
                #koeff_ = tkMatrix.tPunkte(root,geoName,_title_= titel,dimension=dim)


            if koeff_.result <> None:
                _overWrite_ = self.overwriteGeoDirEntry(edit,geoName,koeff_.name)
                            
                if _overWrite_:                                
                    
                    p_name = koeff_.name
                    a_punkt = "matrix (["
                    b_punkt = "matrix (["
                    c_punkt = "matrix (["
                    d_punkt = "matrix (["
                    s_punkt = "matrix (["                                        
                    a_geo = []
                    b_geo = []
                    c_geo = []
                    d_geo = []
                    s_geo = []                                    
                    

                    for j in range(3): # Spalte der Matrix
                        a_geo.append(koeff_.mtx[0][j])
                        b_geo.append(koeff_.mtx[1][j])
                        c_geo.append(koeff_.mtx[2][j])
                        d_geo.append(koeff_.mtx[3][j])
                        s_geo.append(koeff_.mtx[4][j])                                                                    
                        
                        if (j < 2):
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + ','
                            b_punkt =  b_punkt +  koeff_.mtx[1][j] + ','
                            c_punkt =  c_punkt +  koeff_.mtx[2][j] + ','
                            d_punkt =  d_punkt +  koeff_.mtx[3][j] + ','
                            s_punkt =  s_punkt +  koeff_.mtx[4][j] + ','                            
                            
                        else:
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + '])'
                            b_punkt =  b_punkt +  koeff_.mtx[1][j] + '])'
                            c_punkt =  c_punkt +  koeff_.mtx[2][j] + '])'
                            d_punkt =  d_punkt +  koeff_.mtx[3][j] + '])'
                            s_punkt =  s_punkt +  koeff_.mtx[4][j] + '])'                                                        

                    #print a_punkt
                    #self.script = []
                                        
                    ap_ = 'transpose(' + a_punkt +')'
                    bp_ = 'transpose(' + b_punkt +')'
                    cp_ = 'transpose(' + c_punkt +')'
                    dp_ = 'transpose(' + d_punkt +')'
                    sp_ = 'transpose(' + s_punkt +')'                                        
                    
                    #print ap_, bp_, cp_, dp_, sp_,                                    

                    #maxima_rep = ap_ + " + r * (" + bp_ + " - " + ap_ + ") + s * (" + cp_ + " - " + ap_ + ")"
                    _ebene_ = descartes.ebeneNF([a_geo,b_geo,c_geo])                    
                    _erg1_ = descartes.punktProbeE(_ebene_[0],_ebene_[1],d_geo)  #
                    _erg2_ = descartes.punktProbeE(_ebene_[0],_ebene_[1],s_geo)  #
                    if not _erg1_:
                        tkMessageBox.showerror("Eingabe-Fehler!", 'Hhmm . . . \ndie vier Punkte der Grundfläche \nliegen NICHT in einer Ebene!')                    
                    elif _erg2_:
                        tkMessageBox.showerror("Eingabe-Fehler!", 'Hhmm . . . \ndie Spitze liegt in der Grundfläche!')
                    else:
                        _colorIndex_ = self.m2g.colList.index(koeff_.color)
                        maxima_rep = " " #
                        script = []
                        script.append( self.m2g.punkt(a_geo,ap_,p_name+"p1") )
                        script.append( self.m2g.punkt(b_geo,bp_,p_name+"p2") )
                        script.append( self.m2g.punkt(c_geo,cp_,p_name+"p3") )
                        script.append( self.m2g.punkt(d_geo,dp_,p_name+"p4") )
                        script.append( self.m2g.punkt(s_geo,sp_,p_name+"s1") )                    
                        
                        script.append( self.m2g.pyramide(a_geo,b_geo,c_geo,d_geo,s_geo,maxima_rep,p_name,_colorIndex_) )
                        
                        _objectRep_ = [("pyramide",a_geo,b_geo,c_geo,d_geo,s_geo),("colorIndex",_colorIndex_)]
                        
                        maximaObjectRep = a_punkt + ", " +  b_punkt + ", " + c_punkt + ", " +  d_punkt + ", " +  s_punkt
                        liste = self.geoOops.buildListEntry(p_name,"Pyramide",maximaObjectRep, script, _objectRep_  )
                        self.geoOops.addGeoDirEntry(p_name,liste)
                        
                        ############################
                        self.bearbeiteAusdruck('"Pyramide (Punkte A, B, C, D, S)"$')
                        self.bearbeiteAusdruck(p_name + '_A : ' + ap_ +  ';')
                        self.bearbeiteAusdruck(p_name + '_B : ' + bp_ +  ';')
                        self.bearbeiteAusdruck(p_name + '_C : ' + cp_ +  ';')
                        self.bearbeiteAusdruck(p_name + '_D : ' + dp_ +  ';')
                        self.bearbeiteAusdruck(p_name + '_S : ' + sp_ +  ';')
                    
                else:
                    print "abgebrochen . . ."





    def punktEbene(self,edit=0,e_name=None,_puListe_= None, _color_=0):
        """Ebene durch drei Punkte"""
        _test_ = 0       
        if self.maximaOk:
            varListe = ['x','y','z']
            _overWrite_ = 1
            dim = 3
            titel = "Ebene durch drei Punkte A, B, C"
            if edit:
                #print str(_puListe_)                
                geoName = e_name
                koeff_ = tkMatrix.ePunkte(root,geoName,self.m2g.colList,_title_ = titel,dimension=dim,_punkte_ = _puListe_,colorIndex=_color_)
            else:
                geoName = self.geoOops.generateObjectName("E") 
                koeff_ = tkMatrix.ePunkte(root,geoName,self.m2g.colList,_title_ = titel,dimension=dim)            
            if koeff_.result <> None:
                _overWrite_ = self.overwriteGeoDirEntry(edit,geoName,koeff_.name)
                            
                if _overWrite_:                                
                    
                    e_name = koeff_.name
                    #print e_name
                    a_geo = []
                    b_geo = []                
                    c_geo = []
                    x_vect = "matrix (["
                    a_punkt = "matrix (["
                    b_punkt = "matrix (["
                    c_punkt = "matrix (["                
                    for j in range(dim): # Spalte der Matrix
                        a_geo.append(koeff_.mtx[0][j])
                        b_geo.append(koeff_.mtx[1][j])
                        c_geo.append(koeff_.mtx[2][j])		    

                        if (j < dim-1):
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + ','
                            b_punkt =  b_punkt +  koeff_.mtx[1][j] + ','
                            c_punkt =  c_punkt +  koeff_.mtx[2][j] + ','
                            x_vect =  x_vect +  varListe[j] + ','
                        else:
                            a_punkt =  a_punkt +  koeff_.mtx[0][j] + '])'
                            b_punkt =  b_punkt +  koeff_.mtx[1][j] + '])'
                            c_punkt =  c_punkt +  koeff_.mtx[2][j] + '])'
                            x_vect =  x_vect +  varListe[j] + '])'
                            
                            
                    ap_ = 'transpose(' + a_punkt +')'
                    bp_ = 'transpose(' + b_punkt +')'
                    cp_ = 'transpose(' + c_punkt +')'
                    x_vect_ = 'transpose(' + x_vect +')'
                    #maxima_rep = ap_ + " + r * (" + bp_ + " - " + ap_ + ") + s * (" + cp_ + " - " + ap_ + ")"
                    uvec_geo = descartes.genRichtVec(a_geo,b_geo)
                    u_vect_ = bp_ + " - " +  ap_
                    vvec_geo = descartes.genRichtVec(a_geo,c_geo)
                    v_vect_ = cp_ + " - " +  ap_

                    _colorIndex_ = self.m2g.colList.index(koeff_.color)
                    maxima_rep = "stuetz"+e_name+ " + r * uvec"+e_name + " + s * vvec"+e_name
                    script = []
                    script.append( self.m2g.punkt(a_geo,ap_,"stuetz"+e_name) )
                    script.append( self.m2g.punkt(b_geo,bp_,e_name+"p2") )
                    script.append( self.m2g.punkt(c_geo,cp_,e_name+"p3") )
                    script.append( self.m2g.richtVec(uvec_geo,u_vect_,"uvec"+e_name) )
                    script.append( self.m2g.richtVec(vvec_geo,v_vect_,"vvec"+e_name) )                
                    script.append( self.m2g.ebene("stuetz"+e_name,e_name+"p2",e_name+"p3",maxima_rep,e_name,_colorIndex_) )


                    _objectRep_ = [("ebene",),("colorIndex",_colorIndex_)]
                    maximaObjectRep = a_punkt + " + r * (" + b_punkt + " - " + a_punkt + ") + s * (" + c_punkt + " - " + a_punkt + ")"
                    liste = self.geoOops.buildListEntry(e_name,"Ebene",maximaObjectRep, script, _objectRep_ )
                    self.geoOops.addGeoDirEntry(e_name,liste)

                    
                    ############################
                    self.bearbeiteAusdruck('"Drei Punkte A, B, C der Ebene"$')
                    self.bearbeiteAusdruck('A : ' + ap_ +  ';')
                    self.bearbeiteAusdruck('B : ' + bp_ +  ';')
                    self.bearbeiteAusdruck('C : ' + cp_ +  ';')                
                    self.bearbeiteAusdruck('"Aufpunkt stuetzvec"$')
                    self.bearbeiteAusdruck('stuetz' + e_name  + ' : ' + ap_ +  ';')                
                    self.bearbeiteAusdruck('"Richtungsvektoren uvec und vvec"$')
                    self.bearbeiteAusdruck('uvec' + e_name  + ' : B - A;')
                    self.bearbeiteAusdruck('vvec' + e_name  + ' : C - A;')
                    self.bearbeiteAusdruck('"Ebenengleichung"$')
                    self.bearbeiteAusdruck('display(stuetz' +  e_name  + ' +  r * uvec' +  e_name  + ' +  s * vvec' +  e_name  +')$')
                    
                    #self.bearbeiteAusdruck('"x-Vektor xvec"$')
                    #self.bearbeiteAusdruck('xvec : ' + x_vect_ +  ';')                
                    #self._ebene_(e_name,ebene="pebene")
                else:
                    print "abgebrochen . . ."
                    
                                    
    def kebene(self,edit=0,e_name=None,_nvec_= None,_dn_= None, _color_=0):
        """Ebene in Koordinatenform"""
        self._nebene_(edit,e_name,_nvec_,_dn_, _color_,_nebenenFlag_=False)
        

    def nebene(self,edit=0,e_name=None,_nvec_= None,_dn_= None, _color_=0):
        """Ebene in Normalenform"""
        self._nebene_(edit,e_name,_nvec_,_dn_, _color_,_nebenenFlag_=True)                


    def _nebene_(self,edit=0,e_name=None,_nvec_= None,_dn_= None, _color_=0,_nebenenFlag_=True):
        """Ebene in Normalenform"""
        if self.maximaOk:
            varListe = ['x','y','z']
            _overWrite_ = 1
            dim = 3
            if _nebenenFlag_: # Ebene in Normalform
                if edit:
                    geoName = e_name
                    koeff_ = tkMatrix.Normalform(root,geoName,self.m2g.colList,dimension=dim,nvec=_nvec_, _d_ = _dn_,colorIndex=_color_)                
                else:
                    geoName = self.geoOops.generateObjectName("E") 
                    koeff_ = tkMatrix.Normalform(root,geoName,self.m2g.colList,dimension=dim)
            else: # Ebene in Koordinatenform
                if edit:
                    geoName = e_name
                    koeff_ = tkMatrix.Koordinatenform(root,geoName,self.m2g.colList,dimension=dim,nvec=_nvec_, _d_ = _dn_,colorIndex=_color_)                
                else:
                    geoName = self.geoOops.generateObjectName("E") 
                    koeff_ = tkMatrix.Koordinatenform(root,geoName,self.m2g.colList,dimension=dim)
            ####################################    
            if koeff_.result <> None:
                _overWrite_ = self.overwriteGeoDirEntry(edit,geoName,koeff_.name)
                            
                if _overWrite_:                                
                    
                    e_name = koeff_.name
                    x_vect = "matrix (["
                    n_vect = "matrix (["
                    n_geo = []
                    for i in range(dim): # Spalte der Matrix
                        n_geo.append(koeff_.mtx[i][0])
                        if (i < dim-1):
                            n_vect =  n_vect +  koeff_.mtx[i][0] + ','
                            x_vect =  x_vect +  varListe[i] + ','
                        else:
                            n_vect =  n_vect +  koeff_.mtx[i][0] + '])'
                            x_vect =  x_vect +  varListe[i] + '])'
                    d_ = "d" + e_name + " : " + koeff_.mtx[dim][0] + "$"

                    # für pyGeo Richtungsvektoren und Aufpunkt erzeuegn                
                    d_geo = koeff_.mtx[dim][0]
                    s_x = descartes.spurpunkt(n_geo,d_geo,1) # Spurpunkt auf x-Achse
                    s_y = descartes.spurpunkt(n_geo,d_geo,2) # Spurpunkt auf y-Achse
                    s_z = descartes.spurpunkt(n_geo,d_geo,3) # Spurpunkt auf z-Achse
                    uvec_geo = descartes.rvec1(n_geo)
                    vvec_geo = descartes.rvec2(n_geo)
                    auf_geo = descartes.generateAufpunkt(s_x,s_y,s_z)
                    
                    x_vect_ = 'transpose(' + x_vect +')'
                    n_vect_ = 'transpose(' + n_vect +')'


                    _colorIndex_ = self.m2g.colList.index(koeff_.color)
                    maxima_rep = x_vect_ + " . nvec"+e_name + " - d" + e_name + " = 0"
                    script = []
                    script.append( self.m2g.richtVec(n_geo,n_vect_,"nvec"+e_name) )		
                    script.append( self.m2g.normalformEbene(uvec_geo,vvec_geo,auf_geo,d_geo,maxima_rep,e_name,_colorIndex_) )
                    
                    if _nebenenFlag_: # Ebene in Normalform
                        _objectRep_ = [("nebene",n_geo,d_geo),("colorIndex",_colorIndex_)]
                        # In maximaObjectRep wird die Darstellung der Normalform aufgebaut
                        maximaObjectRep = x_vect + " . " + n_vect + " - " + d_geo + " = 0"
                    else: # Ebene in Koordinatenform
                        _objectRep_ = [("kebene",n_geo,d_geo),("colorIndex",_colorIndex_)]
                        # In maximaObjectRep wird die Darstellung der Koordinatenform aufgebaut
                        maximaObjectRep = n_geo[0] + " * " + varListe[0] + " + "
                        maximaObjectRep = maximaObjectRep + n_geo[1] + " * " + varListe[1]  + " + "
                        maximaObjectRep = maximaObjectRep + n_geo[2] + " * " + varListe[2]  + " = " + d_geo

                    
                    liste = self.geoOops.buildListEntry(e_name,"Ebene",maximaObjectRep, script, _objectRep_ )
                    self.geoOops.addGeoDirEntry(e_name,liste)
                                        
                    self.bearbeiteAusdruck('"x-Vektor xvec"$')
                    self.bearbeiteAusdruck('xvec : ' + x_vect_ +  ';')
                    self.bearbeiteAusdruck('"Normalenvektor"$')
                    self.bearbeiteAusdruck('nvec' + e_name  + ' : ' + n_vect_ +  ';')
                    self.bearbeiteAusdruck('"Abstand d' + e_name  + '"$')
                    self.bearbeiteAusdruck(d_)
                    self.bearbeiteAusdruck('koordinatenform(x,n,d) := (print( x . n  = d) )$')
                    self.bearbeiteAusdruck('"Koordinatenform"$')                
                    self.bearbeiteAusdruck('koordinatenform(xvec,nvec'+e_name  + ',d'+e_name +')$')
                                    
                    #self._ebene_(e_name,ebene="nebene")
                else:
                    print "abgebrochen . . ."


                


    def pebene(self, edit=0,e_name=None,_paramListe_= None, _color_=0):
        """Ebene in Parameterform"""                
        if self.maximaOk:
            varListe = ['x','y','z']
            _overWrite_ = 1
            dim = 3
            if edit:
                geoName = e_name
                #print _paramListe_
                koeff_ = tkMatrix.Ebene(root,geoName,self.m2g.colList,dimension=dim,_parameter_ = _paramListe_,colorIndex=_color_)
                
            else:
                geoName = self.geoOops.generateObjectName("E")             
                koeff_ = tkMatrix.Ebene(root,geoName,self.m2g.colList,dimension=dim)
            if koeff_.result <> None:
                _overWrite_ = self.overwriteGeoDirEntry(edit,geoName,koeff_.name)
                            
                if _overWrite_:                                
                    
                    e_name = koeff_.name                
                    aufpunkt = "matrix (["
                    x_vect = "matrix (["
                    u_vect = "matrix (["
                    v_vect = "matrix (["
                    auf_geo = []
                    uvec_geo = []
                    vvec_geo = []                
                    for i in range(dim): # Zeile der Matrix
                        x_vect =  x_vect +  varListe[i] 
                        aufpunkt = aufpunkt + koeff_.mtx[i][0]
                        u_vect =  u_vect +  koeff_.mtx[i][1]
                        v_vect =  v_vect +  koeff_.mtx[i][2]
                        auf_geo.append(koeff_.mtx[i][0])                       
                        uvec_geo.append(koeff_.mtx[i][1])   
                        vvec_geo.append(koeff_.mtx[i][2])   

                        if (i < dim-1):
                            x_vect =  x_vect +  ','
                            aufpunkt = aufpunkt + ','
                            u_vect =  u_vect +  ','
                            v_vect =  v_vect +  ','                        
                        else:
                            x_vect =  x_vect +  '])'                       
                            aufpunkt = aufpunkt +  '])'
                            u_vect =  u_vect +  '])'
                            v_vect =  v_vect +  '])'                                               
                    aufpunkt_ = 'transpose(' + aufpunkt +')'
                    x_vect_ = 'transpose(' + x_vect +')'
                    u_vect_ = 'transpose(' + u_vect +')'
                    v_vect_ = 'transpose(' + v_vect +')'
                    ############################
                    _colorIndex_ = self.m2g.colList.index(koeff_.color)
                    maxima_rep = "stuetz"+e_name+ " + r * uvec"+e_name + " + s * vvec"+e_name 
                    script = []
                    script.append( self.m2g.punkt(auf_geo,aufpunkt_,"stuetz"+e_name) )
                    
                    script.append( self.m2g.richtVec(uvec_geo,u_vect_,"uvec"+e_name) )
                    script.append( self.m2g.richtVec(vvec_geo,v_vect_,"vvec"+e_name) )                
                    
                    script.append( self.m2g.parameterformEbene(uvec_geo,vvec_geo,auf_geo,maxima_rep,e_name,_colorIndex_) )
                    
                    _objectRep_ = [("pebene",auf_geo,uvec_geo,vvec_geo),("colorIndex",_colorIndex_)]
                    maximaObjectRep = aufpunkt + " + r * (" + u_vect + ")" + " + s * (" + v_vect + ")"
                    liste = self.geoOops.buildListEntry(e_name,"Ebene",maximaObjectRep, script, _objectRep_ )
                    self.geoOops.addGeoDirEntry(e_name,liste)
                    
                    # so bestimmt man einen Normalenvektor/eine Koordinatenform
                    nvec_geo = descartes.crossProdukt(uvec_geo,vvec_geo)
                    _d_ = descartes.skalarProdukt(auf_geo,nvec_geo)
                    #print descartes.koordinatenform(nvec_geo, _d_)


                    
                    #self.bearbeiteAusdruck('"x-Vektor xvec"$')
                    #self.bearbeiteAusdruck('xvec : ' + x_vect_ +  ';')
                    self.bearbeiteAusdruck('"Aufpunkt stuetz' + e_name  + '"$')
                    self.bearbeiteAusdruck('stuetz' + e_name  + ' : ' + aufpunkt_ +  ';')                
                    self.bearbeiteAusdruck('"Richtungsvektoren uvec und vvec"$')
                    self.bearbeiteAusdruck('uvec' + e_name  + ' : ' + u_vect_ +  ';')
                    self.bearbeiteAusdruck('vvec' + e_name  + ' : ' + v_vect_ +  ';')
                    self.bearbeiteAusdruck('"Ebenengleichung"$')
                    self.bearbeiteAusdruck('display(stuetz' +  e_name  + ' +  r * uvec' +  e_name  + ' +  s * vvec' +  e_name  +')$')
                    
                    #self._ebene_(e_name,ebene="pebene")
                else:
                    print "abgebrochen . . ."


                





    def _ebene_(self,e_i = "E1",ebene="nebene"):
        """ Gemeinsam fuer alle drei Ebenen Prozeduren """
        x_vect = "matrix ([x,y,z])"
        x_vect_ = 'transpose(' + x_vect +')'
        self.bearbeiteAusdruck('"x-Vektor xvec"$')
        self.bearbeiteAusdruck('xvec : ' + x_vect_ +  ';')
        
        if (ebene != "nebene"):
            #self.bearbeiteAusdruck('"Ebenengleichung"$')
            #self.bearbeiteAusdruck('display(stuetz' +  e_i  + ' +  r * uvec' +  e_i  + ' +  s * vvec' +  e_i  +')$')
            ####                
            self.bearbeiteAusdruck(e_i  +' : addcol(matrix([]),xvec - stuetz' +  e_i  +',uvec' +  e_i  +',vvec' +  e_i  +')$')
            self.bearbeiteAusdruck('det' +  e_i  +' : determinant( ' +  e_i  +' );')
            self.bearbeiteAusdruck('"Normalenvektor nvec' +  e_i  +'"$')
            self.bearbeiteAusdruck('nvec' +  e_i  +' : transpose(coefmatrix([det' +  e_i  +'],[x,y,z]));')
            self.bearbeiteAusdruck('d' +  e_i  +' : (-1) *determinant( col(augcoefmatrix([det' +  e_i  +'],[x,y,z]),4) );')
        self.bearbeiteAusdruck('koordinatenform(x,n,d) := (print( x . n  = d) )$')
        self.bearbeiteAusdruck('"Koordinatenform"$')                
        self.bearbeiteAusdruck('koordinatenform(xvec,nvec' +  e_i  +',d' +  e_i  +')$')
        self.bearbeiteAusdruck('"Normalform"$')
        self.bearbeiteAusdruck('display(xvec . nvec' +  e_i  +')$')
        self.bearbeiteAusdruck('xvec . nvec' +  e_i  +'  - d' +  e_i  +' = 0;')
        ####
        self.bearbeiteAusdruck('s1 : transpose(matrix ([x,0,0]))$')
        self.bearbeiteAusdruck('s2 : transpose(matrix ([0,y,0]))$')
        self.bearbeiteAusdruck('s3 : transpose(matrix ([0,0,z]))$')
        spurpkt = 'spurpunkt(stmp,nvec,d,i) := (ergebnis : solve(stmp . nvec  = d,stmp[i,1]),if not atom(ergebnis) '
        spurpkt = spurpkt + ' then (if (length(ergebnis) > 0) then (stmp[i,1] : part(first(ergebnis),2),print(stmp) ) else (print("Kein Spurpunkt auf der ",stmp[i,1],"-Achse"))) '
        spurpkt = spurpkt + ' else (print(transpose(matrix ([0,0,0])))) )$'
        #spurpkt = 'spurpunkt(s,nvec,d,i) := (ergebnis : solve(s . nvec  = d,s[i,1]),s[i,1] : part(first(ergebnis),2), print(s) )$'
        # Loesungsmenge: solve => all, [], x = 2 (= 1 Loesung)
        self.bearbeiteAusdruck(spurpkt)
        self.bearbeiteAusdruck('"Spurpunkte"$')
        self.bearbeiteAusdruck('"x-Achse"$')                                       
        self.bearbeiteAusdruck('spurpunkt(s1,nvec' +  e_i  +',d' +  e_i  +',1)$')
        self.bearbeiteAusdruck('"y-Achse"$')                                                       
        self.bearbeiteAusdruck('spurpunkt(s2,nvec' +  e_i  +',d' +  e_i  +',2)$')
        self.bearbeiteAusdruck('"z-Achse"$')                                                       
        self.bearbeiteAusdruck('spurpunkt(s3,nvec' +  e_i  +',d' +  e_i  +',3)$')                
        nullPkt = ['0', '0', '0']
        pScript = self.geoOops.getScriptEintrag(e_i)
        pRep = self.geoOops.script2Intern(pScript,3)
        #print pScript
        #print pRep
        eIntern = descartes.ebeneNF(pRep)
        #print eIntern
        nvec = eIntern[0]
        _d_ = eIntern[1]
        abstand = descartes.abstandEP(nvec,_d_,nullPkt)        
        self.bearbeiteAusdruck('"Abstand der Ebene ' + e_i + ' vom Ursprung: d = ' + abstand +  '"$')

                                    
    def matrixDimMN(self,holeName=0):
        """holt die Dimension m x n der Matrix
           holeName == 1: fuer matZero: Name + Dim
           holeName == 0: fuer mnMatrix: Dim
        """
        if self.maximaOk:        
            ok = 1
            zeile = 0
            spalte = 0
            _objectName_ = ""
            
            if holeName:
                dim_ = tkMatrix.DimMNfensterName(root)
            else:
                dim_ = tkMatrix.DimMNfenster(root)
            if dim_.result <> None:
                # Fenster-Werte übernehmen
            
                try:
                    zeile = int(dim_.a1)
                    if not util.intervall(zeile,0,15):
                        ok = 0
                        tkMessageBox.showinfo("Eingabe-Fehler!",  
                             "Fehler bei der Eingabe der Dimension\nBeachte: 0 <= m <= 15")
                
                    spalte = int(dim_.a2)
                    if not util.intervall(spalte,0,15):
                        ok = 0
                        tkMessageBox.showinfo("Eingabe-Fehler!",  
                             "Fehler bei der Eingabe der Dimension\nBeachte: 0 <= m <= 15")
                        
                    if holeName:
                        _objectName_ = dim_.aName
                        return (ok,zeile,spalte,_objectName_ )
                    else:
                        return (ok,zeile,spalte )            
                    #print zeile,spalte
                except:
                    tkMessageBox.showinfo("Parameter-Fehler!",  
                         "Fehler bei der Parametereingabe")
                    if holeName:
                        return (0,0,0,0) # 0 == not ok
                    else:    
                        return (0,0,0) # 0 == not ok              
            else:
                if holeName:
                    return (0,0,0,0) # 0 == not ok
                else:    
                    return (0,0,0) # 0 == not ok


    def mnMatrix(self):
        if self.maximaOk:        
            (ok,zeile,spalte)= self.matrixDimMN()
            if ok:
                if (zeile > 0) or (spalte > 0):
                    (ret,value) = maxfunc.mnMatrix(root,zeile,spalte)
                    if len(ret) > 0:
                        #print ret
                        self.bearbeiteAusdruck(ret) # das geht wirklich!

    def matFunc(self,arg):
        """über <arg> wird die entsprechende Matrix-Operation selektiert """
        if self.maximaOk:
            varLi = self.getValues()
            if len(varLi) > 0:
                (ret,value) = maxfunc.getMatrixName(root,varLi)
            
            if len(ret) > 0:
                self.bearbeiteAusdruck( arg + "(" + str(ret) + ");")

    def matCharPoly(self):                        
        if self.maximaOk:
            varLi = self.getValues()
            (ret,value) = maxfunc.getMatrixName(root,varLi)
            
            if len(ret) > 0:
                self.bearbeiteAusdruck("charpoly(" + str(ret) + ",x);")
                self.bearbeiteAusdruck("expand(%);") 


    def matDiagonal(self):                        
        if self.maximaOk:
            eingabe = tkMatrix.dreiZeiler(root,"D1","Diagonalmatrix","Zeilen-/Spaltenrang n:","Wert in der Diagonalen n:")
            if eingabe.result <> None:
                dim = string.strip(str(eingabe.a1))
                wert = string.strip(str(eingabe.a2))
                matrixName = string.strip( str(eingabe.aName) )
                if (int(dim) > 0):
                    self.bearbeiteAusdruck(matrixName + ": diagmatrix(" + str(dim) + "," + str(wert)  + ");")

    def matZero(self):                        
        if self.maximaOk:
            mitName = 1
            (ok,zeile,spalte,_objectName_)= self.matrixDimMN( mitName )
            if ok:
                if (zeile > 0) or (spalte > 0):
                    self.bearbeiteAusdruck(_objectName_  + ": zeromatrix(" + str(zeile) + "," + str(spalte)  + ");")


    def matEinheit(self):                        
        if self.maximaOk:
            eingabe = tkMatrix.dreiZeiler(root,"E1","Einheitsmatrix","Zeilen-/Spaltenrang n:","")
            if eingabe.result <> None:
                dim = string.strip(str(eingabe.a1))
                matrixName = string.strip( str(eingabe.aName) )
                if (int(dim) > 0):
                    self.bearbeiteAusdruck(matrixName + ": ident(" + str(dim) + ");")        



                
    def holeDimension(self,_title_,dimMax = 9):
        """holt die Dimension des Gleichungssystems        
        """
        if self.maximaOk:        
            ok = 1
            dimension = 0            
            dim_ = tkMatrix.Dimfenster(root,_title_)     
            if dim_.result <> None:
                # Fenster-Werte übernehmen
                try:
                    dimension = int(dim_.a1)                
                    if not util.intervall(dimension,0,dimMax):
                        ok = 0
                        tkMessageBox.showinfo("Eingabe-Fehler!",  
                             "Fehler bei der Eingabe der Dimension\nBeachte: 0 <= dim <= " + str(dimMax) )
                    return (ok,dimension)            
                except:
                    tkMessageBox.showinfo("Parameter-Fehler!",  
                         "Fehler bei der Parametereingabe")
                    return (0,0) # 0 == not ok
            else:
                return (0,0) # 0 == not ok


        
    def matrixEingabe(self):
        if self.maximaOk:        
            (ok,dim)= self.holeDimension("Anzahl der Gleichungen/Variablen")
            if ok:
                if (dim > 0):
                    (ret,value) = maxfunc.matrixEingabe(root,dim)
                    if len(ret) > 0:
                        self.bearbeiteAusdruck(ret) # das geht wirklich!                                                                


    def newton(self):
        if self.maximaOk:            
            (expr,varLi) = self.prolog()
            (ret,value) = maxfunc.newton(root,expr,varLi)	
            if len(ret) > 0:                
                    self.bearbeiteAusdruck('load("mnewton");',var=value)
                    self.bearbeiteAusdruck(ret,var=value)
                    #self.bearbeiteAusdruck('Simplify(%);',var=value)

                
    def funkSchar(self,plot=0):
        """ behandelt Funktionenscharen:
            plot = 0 => Funktionenschar untersuchen
            plot = 1 => Funktionenschar zeichnen
            plot = 2 => Funktionenschar-Plot speichern
        """
        if self.maximaOk:
            
            ret = ''
            _k_von = None
            _k_bis = None
            _schrittweite = None
            _parameter = None
            
            if (plot == 0): # plot == 0
                (expr,varLi) = self.prolog()
            else: # plot == 1 / 2
                (expr,varLi) = self.prolog()
                (expr,value,xmin,xmax,ymin,ymax,_k_von,_k_bis, _schrittweite, _parameter) = self.oldFunkScharPlot.getPlot()
                if not value in varLi:
                    varLi.append(value)
                
            if not 'k' in varLi:
                varLi.append('k')
                
            varLi.sort()
            
            if not _k_von:
                _k_von = '1'
                
            if not _k_bis:
                _k_bis = '1'

            if not _schrittweite:
                _schrittweite = '1'
                
            if not _parameter:
                _parameter = 'k'


            mydici = {}
            if (plot == 0): # Funktionsuntersuchung
                mydici[1] = ('lab1',"x-min:",'4','0','ein1',"-3",'4','1')
                mydici[2] = ('lab2',"x-max:",'4','2','ein2',"3",'4','3')
                mydici[3] = ('lab3',"y-min:",'5','0','ein3',"-10",'5','1')
                mydici[4] = ('lab4',"y-max:",'5','2','ein4',"10",'5','3')
                
            else: # plot == 1/2
                mydici[1] = ('lab1',"x-min:",'4','0','ein1',xmin,'4','1')
                mydici[2] = ('lab2',"x-max:",'4','4','ein2',xmax,'4','5')
                mydici[3] = ('lab3',"y-min:",'5','0','ein3',ymin,'5','1')
                mydici[4] = ('lab4',"y-max:",'5','4','ein4',ymax,'5','5')           

            
            if (plot == 1):
                eingabe = tkw.eingabeOptMenuScharList(root,"Funktionenschar plotten",expr,varLi,_k_von,_k_bis,_schrittweite,_parameter,mydici)                
            elif (plot == 2):
                eingabe = tkw.eingabeOptMenuScharList(root,"Funktionenschar: Graph speichern",expr,varLi,_k_von,_k_bis,_schrittweite,_parameter,mydici)                                                            
            else:
                eingabe = tkw.eingabeOptMenuSchar(root,"Funktionenschar",expr,varLi)
                
            von = 1; bis = 5        
            if eingabe.result <> None:        
                ausdruck = string.strip( str(eingabe.result) )
                if len(ausdruck) > 0:
                    ausdruck = self.stripStrichpunkt(ausdruck)
                    von = float(eingabe.a5)
                    bis = float(eingabe.a6) 
                    schrittweite = float(eingabe.a7)
                    #print schrittweite
                    if (bis <= von):
                        bis = von + 1
                    value = str(eingabe.value)
                    parameter = str(eingabe.para)
                    if (value == parameter):
                        tkMessageBox.showinfo(">>> Problem <<<", "Variable <" + value + "> und \nParameter <" + parameter + "> \nsind identisch!" )
                        return
                    if (int(plot) > 0):
                        xmin = str(eingabe.ret1)
                        xmax = str(eingabe.ret2)
                        ymin = str(eingabe.ret3)
                        ymax = str(eingabe.ret4)                    
                    else:
                        self.printMessage("Funktionenschar untersuchen")
                        self.funkUntersuchung(ausdruck,value)
                    funcListe = ''
                    drawListe = [] # for maxima-modul draw
                    j = von
                    while (j <= bis):
                        neuExpr = string.replace(ausdruck,parameter,str(j))
                        #print neuExpr
                        funcListe = funcListe + ',' + neuExpr
                        if (int(plot) > 0):
                            drawExpr = "gr2d(explicit(" + neuExpr + "," + value + "," + xmin + "," + xmax+ "), yrange = [" + ymin + "," + ymax+ "],xaxis = true,yaxis = true,grid = true )"
                        else: # Funktionenschar untersuchen
                            drawExpr = "gr2d(explicit(" + neuExpr + "," + value + ",-4,4), yrange = [-10, 10],xaxis = true,yaxis = true,grid = true )"
                        drawListe.append(drawExpr)
                        j = j + schrittweite
                    funcListe = funcListe[1:] # funcListe ohne ',' am Anfang!   
                    #print str(funcListe)

                    if self.moduleDrawLoaded:                
                        gifanim_path = self.workingDir.replace('\\', '/') # wichtig, sonst kommt draw mit dem Pfad nicht klar!
                        # print gifanim_path
                        ret_draw = 'draw(terminal = animated_gif,delay = 70, file_name = "'+ gifanim_path + 'gifanim", pic_width  = 400, pic_height = 400,'
                        for element in drawListe:
                            ret_draw = ret_draw + element + ',' 
                        ret_draw = ret_draw[:-1] 
                        ret_draw = ret_draw + ')$'
                        # print ret_draw                        
                    

                    if (int(plot) == 1):
                        self.oldFunkScharPlot.setPlot(ausdruck,value,xmin,xmax,ymin,ymax,von,bis,schrittweite,parameter)
                        if self.moduleDrawLoaded:
                            ret = ret_draw
                        else:    
                            ret = 'plot2d([' + funcListe  + '],['+ value+','+xmin+','+xmax+'],[y,'+ymin+','+ymax+'],[gnuplot_preamble,"set grid; ' + self.linetype + '"]);'
                    elif (int(plot) == 2):
                        #pfad_ = abspath( self.home )
                        #os.chdir(pfad_)
                        if self.moduleDrawLoaded:
                                ret = ret_draw
                        else:                                        
                            oname = asksaveasfilename(filetypes=[("Portable Network Graphics", "*.png"),("Encapsulated PostScript", "*.eps")],initialdir=self.workingDir)
                            if oname:
                                postscript = 0 #default: png
                                if (string.find(oname,".png") == -1) and (string.find(oname,".eps") == -1):
                                    oname = oname + ".png"
                                    postscript = 0
                                elif (self.test_extension(oname,".eps")):                            
                                    postscript = 1
                                else:
                                    postscript = 0
                                    
                                #print oname
                                ret = 'plot2d([' + funcListe  + '],['+ value+','+xmin+','+xmax+'],[y,'+ymin+','+ymax
                                if postscript:
                                    ret = ret + '],[gnuplot_preamble,"set grid;' + self.linetype + '"],[gnuplot_term, ps],[gnuplot_out_file,"'
                                else:
                                    ret = ret + '],[gnuplot_preamble,"set grid;' + self.linetype + '"],[gnuplot_term, png],[gnuplot_out_file,"'                            
                                ret = ret + str(oname) + '"])$'
                        self.plotten = 0
                            #print ret                        
                    else: # (int(plot) == 0) => Funktionenschar untersuchen
                        #self.oldFunkScharPlot.setPlot(ausdruck,value,xmin,xmax,ymin,ymax,von,bis)  # gibt einen Fehler, da xmin,xmax u. s. w. nicht gesetzt!
                        if self.moduleDrawLoaded:
                            ret = ret_draw
                        else:    
                            ret = 'plot2d([' + funcListe  + '],[' + value + ',-4,4],[y,-10,10],[gnuplot_preamble,"set grid; ' + self.linetype + '"]);'                        
                    #print plot
                    self.bearbeiteAusdruck(ret,var=value)
                    if self.moduleDrawLoaded:
                        maxfunc.funkSchar2HTML(root,self.workingDir,ausdruck,value,parameter,von,bis)            

            
    def kurvendisk(self):
        if self.maximaOk:
            (expr,varLi) = self.prolog()        
            eingabe = tkw.eingabeOptMenu(root,"Funktionsuntersuchung",expr,varLi)
            if eingabe.result <> None:
                ausdruck = string.strip( str(eingabe.result) )
                if len(ausdruck) > 0:
                    ausdruck = self.stripStrichpunkt(ausdruck)
                    value = str(eingabe.value)                                
                    # alternativ: '\A[ ]*[a-z]+\_?[a-z0-9]*[ ]*\([ ]*[a-z][ ]*\)[ ]*\Z'  f_5(x), f5(x), f(x)                        
                    reFunc = re.compile(r'\A[ ]*[a-z]+\_?[a-z0-9]*[ ]*\([ ]*[a-z0-9\^]+[ ]*\)[ ]*\Z', re.M)
                    result = reFunc.search(ausdruck)
                    self.printMessage("Funktionsuntersuchung")
                    if result != None:  # Funktion als Parameter: f(x)              
                        (text,self.nextIn,gefunden)  = maxima.call('disp(' + ausdruck  + ',display2d:false);')
                        self.ausdruckAusgeben('disp(' + ausdruck  + ',display2d:false);',gefunden,text)
                        pos = text.find('false')
                        function = text[:pos].strip()
                        # wichtig: display2d auf true setzen!
                        (text,self.nextIn,gefunden)  = maxima.call('display2d:true$')
                        self.ausdruckAusgeben('display2d:true;',gefunden,text)
                        ausdruck = function # wichtig!!!!
                        
                    self.funkUntersuchung(ausdruck,value)
                    f_x = 'f(' + value + ')'
                    if self.moduleDrawLoaded:
                            #plot = 'draw2d(key="f\('+  value +'\)",explicit(' + f_x + ',' +  value + ',-4,4), xaxis = true,yaxis = true,grid = true);'
                            plot = 'draw2d(key="Graph G_f",explicit(' + f_x + ',' +  value + ',-4,4), xaxis = true,yaxis = true,grid = true);'                        
                    else:
                            plot = 'plot2d([' + f_x  + '],[' + value + ',-4,4],[y,-10,10],[gnuplot_preamble,"set grid; ' + self.linetype + '"]);'
                    self.bearbeiteAusdruck(plot,var=value)



            
    def funkUntersuchung(self,ausdruck,value):
        if self.maximaOk:
            self.t.update()  # neu 0.30
            f_x = 'f(' + value + ')'
            d_x = 'diff('
            fdef = f_x + ' := ' + ausdruck + ';'
            self.bearbeiteAusdruck(fdef,var=value)

            try:  # Schnittpunkt mit y-Achse
                f_null = 'f(0);'
                self.printMessage("Schnittpunkt mit y-Achse")
                self.bearbeiteAusdruck(f_null,var=value)
            except:
                pass

                    
            try:  # Nullstellen (Schnittpunkt mit x-Achse)
                null = 'solve(' +  ausdruck + '= 0, ' + value + ');'
                self.printMessage("Nullstellen")
                self.bearbeiteAusdruck(null,var=value)
                self.bearbeiteAusdruck('ev(%,numer);',var=value)  
            except:
                pass

            # 1. Ableitung
            f1 =d_x + f_x + ',' +  value + ') ;'
            self.printMessage("1. Ableitung")
            self.bearbeiteAusdruck(f1,var=value)
            #self.simplify()
            simple = 'sqfr(%);'
            self.bearbeiteAusdruck(simple,var=value)
            
            try:  # Extremstellen
                extrem = 'solve( %th(2) = 0,' + value + ');'
                self.printMessage("Mögliche Extremstellen")            
                self.bearbeiteAusdruck(extrem,var=value)
                self.bearbeiteAusdruck('ev(%,numer);',var=value)  
            except:
                pass

            # 2. Ableitung            
            f2 = d_x + f_x + ',' +  value + ',2) ;'
            self.printMessage("2. Ableitung")
            self.bearbeiteAusdruck(f2,var=value)
            simple = 'sqfr(%);'
            self.bearbeiteAusdruck(simple,var=value)        
            
            try:   # Wendestellen
                wende = 'solve( %th(2) = 0,' + value + ');'
                self.printMessage("Mögliche Wendestellen")                        
                self.bearbeiteAusdruck(wende,var=value)
                self.bearbeiteAusdruck('ev(%,numer);',var=value)  
            except:
                pass

            # 3. Ableitung            
            f3 = d_x + f_x + ',' +  value + ',3) ;'
            self.printMessage("3. Ableitung")
            self.bearbeiteAusdruck(f3,var=value)
            simple = 'sqfr(%);'
            self.bearbeiteAusdruck(simple,var=value)

            # Stammfunktion
            stamm = 'integrate('+ f_x + ', ' + value + ');'
            self.printMessage("Stammfunktion")                            
            self.bearbeiteAusdruck(stamm,var=value)
            simple = 'sqfr(%);'
            self.bearbeiteAusdruck(simple,var=value)
	    


    def diff(self):
        """alle anderen Funktionen analog anpassen!"""
        if self.maximaOk:        
            (expr,varLi) = self.prolog()
            (ret,value) = maxfunc.diff(root,expr,varLi)	
            if len(ret) > 0:                
                    self.bearbeiteAusdruck(ret,var=value)




    def rotintegral(self):
        if self.maximaOk:
            (expr,varLi) = self.prolog()
            (ret,value) = maxfunc.rotintegral(root,expr,varLi)	
            if len(ret) > 0:                
                    self.bearbeiteAusdruck(ret,var=value)
                    self.bearbeiteAusdruck('ev(%,numer);',var=value)	

    def findRoot(self):
        if self.maximaOk:
            (expr,varLi) = self.prolog()
            (ret,value) = maxfunc.findRoot(root,expr,varLi)	
            if len(ret) > 0:
                #print ret
                self.bearbeiteAusdruck(ret,var=value)




    def integral(self):
        if self.maximaOk:            
            (expr,varLi) = self.prolog()
            (ret,value) = maxfunc.integral(root,expr,varLi)	
            if len(ret) > 0:                
                    self.bearbeiteAusdruck(ret,var=value)
                    self.bearbeiteAusdruck('ev(%,numer);',var=value)	
	

        
    def stamm(self):
        if self.maximaOk:        
            (expr,varLi) = self.prolog()
            (ret,value) = maxfunc.stamm(root,expr,varLi)	
            if len(ret) > 0:                
                    self.bearbeiteAusdruck(ret,var=value)
	

                
                
    def mehrzeilig(self):
        _debug_ = 0
        if self.maximaOk:
            (ret,ausdruck) = maxfunc.mehrzeilig(root)
            if len(ret) > 0:                
                if _debug_:
                    print ret
                    
                try:
    ####                self.bearbeiteAusdruck(ausdruck)
                    (text,self.nextIn,gefunden) = maxima.call(ret)
                    ausdruck = string.replace(ausdruck,',  ',',\n ')                      
                    self.ausdruckAusgeben(ausdruck,gefunden,text)
                except:
                    pass
            


##################################################################
    def getNr(self,nr):
        return '(%i' + str(nr)+ ') '

    def getInNumber(self,event):
        (text,self.nextIn,gefunden)  = maxima.call("labels$")
        self.ausdruckAusgeben("",gefunden,text)
        

    def plotTest(self,arg,var):
        """plotTest: Test auf definierte oder Standard-Funktion        
           mit Hilfe reguläerer Ausdrücke
           Hintergrund: gnuplot mag keine undefinierten Funktionen
        """
        _debug_ = 0
        if _debug_:
            print "Hier plotTest"
            print var
        funcliste = []
        defListe = []
        funcdefListe = []
        funcdef = 0
        if (string.find(arg,'plot2d')  != -1) or (string.find(arg,'draw2d')  != -1): # plot gefunden : plot2d
                rest = arg[6:].strip()
                if rest[0] != '(': # da stimmt was nicht . . .
                    return 0
                else: # zuerst: Liste der Funktionen aufbauen
                    if rest[1] == '[': # [x^2]
                        func = rest[2:]
                        next = string.find(func,']')
                        func = func[:next]
                        funcliste = func.split(',')
                        if _debug_:
                            print funcliste
                        #return 1
                        
                    else:  # x^2  (ohne [] )
                        next = string.find(rest,',') # suche Argumenttrennzeichen ','
                        func = rest[1:next]
                        funcliste.append(func)
                        if _debug_:
                            print funcliste
                        #return 1
                    for element in funcliste:
                        if _debug_:
                            print element                        
                        #^[ ]*[a-z]+[ ]*\([ ]*[a-z][ ]*\)[ ]*$  reg. expression for Function: f(x)
                        #oder \A[ ]*[a-z]+[ ]*\([ ]*[a-z][ ]*\)[ ]*\Z  reg. expression for Function
                        # alternativ: '\A[ ]*[a-z]+\_?[a-z0-9]*[ ]*\([ ]*[a-z][ ]*\)[ ]*\Z'  f_5(x), f5(x), f(x)                        
                        reFunc = re.compile(r'[ ]*[a-z]+\_?[a-z0-9]*[ ]*\([ ]*\%?[a-z0-9/*+-\^]+[ ]*\)[ ]*', re.M)
                        result = reFunc.search(element)
                        if result == None:
                            funcdef = 0
                        else:
                            funcdef = 1 # Funktion gefunden!
                            break
                    if _debug_:    
                        print 'funcdef: ' + str(funcdef)
                    if funcdef: # selbst erstellte Funktion behandeln
                        (text,self.nextIn,gefunden)  = maxima.call('functions;')
                        self.ausdruckAusgeben('functions;',gefunden,text)
                        pos = text.find('[')
                        last = len(text)
                        definiert = text[pos+1:].strip()
                        definiert = definiert[:-1]
                        if len(definiert) > 0:
                            defListe = definiert.split(',')
                        else:
                            defListe = []
                        if len(defListe) > 0:
                            if _debug_:
                                print "defListe: ",str(defListe)
                            for element in defListe:
                                neuel = element.strip('\n \t') # wichtig wg. solchen Defs: ' \n\t\t\t\t       formel(x'
                                if _debug_:
                                    print "neuel: ",neuel
                                reFuncName = re.compile(r'\A[ ]*[a-z%]+\_?[a-z0-9]*', re.M)
                                try:
                                    result = reFuncName.search(neuel)
                                    if _debug_:
                                        print result.end()
                                    pos = string.find(neuel,'(')                                
                                    name = neuel[:result.end()].strip()
                                    if _debug_:   
                                        print name, neuel, neuel[:result.end()]
                                    if not name in self.standard: # in Liste aufnehmen                                   
                                        self.standard.append(name)
                                except:
                                    print "Fehler im reg. Ausdruck (Methode plotTest, pyMaxima.py)"
                        if _debug_:            
                            print 'selbstdefinierte Fkt.: ' + str(defListe)
                            print self.standard
                        ret = 1
                        zaehler = 1
                        for element in funcliste:
                            if _debug_:
                                print ">>>> funcliste[",zaehler,"]",element,
                                zaehler = zaehler + 1
                            if (string.find(element,'(') != -1):   # Test: Ist eine Funktion definiert???? Suche Klammer!
                                element = element.strip()
                                #print element
                                testExpr = string.replace(element,' ','')
                                if _debug_:
                                    print "#### TEST-Expr: ",testExpr
                                if string.find(testExpr,'(') != 0: # Test: Klammer am Anfang => Keine Funktion!!!!
                                    #if (string.find(testExpr,'(') == 0):   # Test: Ist eine Funktion definiert???? Suche Klammer!    
                                    liste = self.funcParser(testExpr,_debug_)
                                    if len(liste) > 0:
                                        for name in liste:
                                            if _debug_:
                                                print ' -> ',name                                
                                            if name in self.standard: # Funktion definiert
                                                pass
                                            else:  # nichtdefiniterte Funktion => Fehler!
                                                if _debug_:
                                                    print ' nichtdefiniterte Funktion!! '
                                                
                                                ret = 0
                                                break
                                    else: # liste = [] => Fehler!
                                        if _debug_:
                                            print ' liste leer: == [] => Problem '
                                        
                                        ret = 0 
                        if _debug_:            
                            print 'return: ' + str(ret)                                    
                        return ret
                    else: # keine Funktion definiert, alles ok
                        return 1        
        else:  # kein plot2d/plot3d!
            return 1

    def funcParser(self,expr,_debug_=0):        
        #self.numListe = ['0','1','2','3','4','5','6','7','8','9','_']
        if _debug_:
            print "\nHier funcParser"
        varLi = []        
        fLen = len(expr)
        ret = []
        offen = 0
        if fLen < 1: # kann nicht sein!
            return ret
        elif fLen == 1: # kann auch nicht sein!
            return ret
        else:
            i = 0
            while i < fLen: # ermittle Variable(n)
                if (expr[i]=='('):
                    offen = offen + 1
                elif (expr[i]==')'):
                    offen = offen - 1
                elif (offen > 0): # Klammer offen => Variable als Argument
                    j = i
                    var = ''
                    while expr[j].isalpha():
                        var = var + expr[j]
                        j = j + 1
                        if j >= fLen:
                            break # kann nicht sein!!!!
                    if len(var) > 0:   # ohne '' 
                        if not (var in varLi):
                            if not (var in self.standard): # neu, 2008-01-08
                                varLi.append(var)
                
                    i = j # Vorsicht!    
                i = i + 1
            if _debug_:	
                print "\n\nAusdruck: " +  expr + ", Var: " +  str(varLi)    
            tmp = ''
            j = 0
            for i in range(fLen):
                if (expr[i].isalpha() and (j==0)): # ermittle Funktionsnamen
                    tmp = tmp + expr[i]
                    j = j + 1
                elif (expr[i].isalpha() or (expr[i] in self.numListe) ) and (j > 0):
                    tmp = tmp + expr[i]
                    j = j + 1                    
                else: # expr[i] not in alpha
                    if len(tmp) > 0:
                        if _debug_:
                            print "\n Untersuche: " + tmp
                        if not (tmp in varLi): # nur Funktionsnamen
                            if not (tmp in ret):                                                                 
                                ret.append(tmp)                    
                    tmp = ''
                    j = 0
            return ret
        
            
            
        
    def bearbeiteAusdruck(self,expr,var=None,_debug_=0):
        #_debug_=1
        gefunden = 0
        text = ''
        if var==None:
            var = ''
        #self.altIn
        if self.maximaOk:	    
            if self.plotTest(expr,var):
                if (string.find(expr,'plot')  != -1)  or (string.find(expr,'draw')  != -1):	# neu 0.28
                    if (os.name != 'posix'):
			#time.sleep(_delay_time_)
                        if  self.plotten:
                            self.t.withdraw()  # Hide pyMaxima
                        else:
                            self.t.update()
                #expr = util.ucodefilter_(expr)      # experimentell: Vers. 1.46      
                (text,self.nextIn,gefunden)  = maxima.call(expr)
                if (string.find(expr,'plot')  != -1)   or (string.find(expr,'draw')  != -1):	# neu 0.28
                    if (os.name != 'posix'):                    
                        #time.sleep(_delay_time_)
                        if  self.plotten:
                            self.t.deiconify()  # Unhide pyMaxima
                            self.t.update()  # neu 0.30
                        else: # plot speichern
                            self.plotten = 1  # wichtig!!                          
                self.ausdruckAusgeben(expr,gefunden,text)
            else:
                expr = util.ucodefilter_(expr)      # experimentell: Vers. 1.46      
                newStatus = "Satus:      " + 'Fehler im Ausdruck: <' + str(expr) + '>'
                self.status.set(newStatus)                
        return gefunden


    def ausdruckAusgeben(self,expr,gefunden,text,_debug_=0):
        if self._debug_:
            _debug2_ = 1
        else:
            _debug2_ = 0        
        #_debug2_ = 1
        
        counter = 1        
        magic = '(%i'
        magic2 = '("%i'
        muell1 = "Ignoring trailing garbage in input."
        muell2 = "Incorrect syntax"
        if (string.find(text,muell1) != -1) or (string.find(text,muell2) != -1):
            counter = 0
            if _debug2_:
                print ">>> Muell-Treffer <<<: ",
                print "text: ",text
                print            
        
        if (gefunden != 0):
            if (string.find(self.nextIn,magic) != -1):                
                
                if counter:
                    self.nr = self.nr + 1
                if self.tauscheAusdruck:
                    if len(self.oldExpr) > 0:
                        self.exprDici[self.nr] = (self.oldExpr,self.nextIn) # Flag 'I': Input/Eingabe
                    #self.exprDici[self.nr] = (expr,self.nextIn) # Flag 'I': Input/Eingabe
                else:
                    if counter:
                        self.printExpression( expr + '\n' , _nl_=0)
                        if len(expr) > 0:
                            self.exprDici[self.nr] = (expr,self.nextIn) # Flag 'I': Input/Eingabe
                
                self.pointer = len(self.exprDici)
                
                self.tauscheAusdruck = 0
                
                util.debugAusdruckAusgeben(_debug2_,expr,gefunden,text,self.nextIn,self.tauscheAusdruck,self.oldExpr,counter)
                
            else: # Anfrage (z. B. '? integ') gelesen, alten Ausdurck benutzen
                self.printExpression( expr + '\n' , _nl_=0)
                
                self.tauscheAusdruck = 1
                #print expr
                util.debugAusdruckAusgeben(_debug2_,expr,gefunden,text,self.nextIn,self.tauscheAusdruck,self.oldExpr,counter)
                
        else: # not gefunden
            util.debugAusdruckAusgeben(_debug2_,expr,gefunden,text,self.nextIn,self.tauscheAusdruck,self.oldExpr,counter)
            
        if len(text) > 0:            
            if _debug_:
                self.feedback( unicode(text) )
            self.printExpression( text + '\n' + self.nextIn + ' ', _nl_=0 )
            
##            if (gefunden == 2):
##                    newStatus = "Satus:   " + '  Bitte Wert eingeben!'
##                    self.status.set(newStatus)
##                    #t2.wait_variable(self.eingabe)
##            elif (gefunden == 1):
##                    self.clearLabel()                    
##            else: # gefunden = 0

        else: # len(text) == 0! nr >self.nextIn< ausgeben! neu => Vers. 1.04
            self.printExpression( self.nextIn + '\n', _nl_=0 )

            
        if (gefunden == 0):
            expr = util.ucodefilter_(expr)      # experimentell: Vers. 1.46      
            newStatus = "Satus:   " + 'Fehler im Ausdruck: <' + str(expr) + '>'
            self.status.set(newStatus)
        t2.update_idletasks() # neu 0.28
        if not self.tauscheAusdruck:  ################
            if counter:
                self.oldExpr = expr
                self.neuExpr = ''
        else:
            if counter:
                self.oldExpr = self.neuExpr
                
        return gefunden
        


    def callMaxima(self,expression="",_debug_= 0):        
        #import time
        if self._debug_:
            _debug_ = 1
        else:
            _debug_ = 0
            
        #_debug_ = 1
            
        exprListe = []
        if self.maximaOk:
            if len(expression) > 0:
                exprListe = self.exprTest(expression)
                #self.feedback(str(exprListe) )            
                for element in exprListe:
                    if len(element) > 0:
                        self.neuExpr = element.strip() # neu
                        if _debug_:
                            print "callMaxima: >" + str(self.neuExpr) + "<"
                        self.bearbeiteAusdruck( self.neuExpr )
                        #time.sleep(0.2)
                        t2.update_idletasks()
                        


    def maximaTest(self,nr=9):
        _debug_ = 0
        #pass
        test0 = '2+7;'
        test1 = 'diff(x*(x^2+100)^(-0.5),x);'
        test2 = 'm:2;3*m^2;'
        test3 = 'taylor(exp(x),x,0,3); '	
        test4 = '23!;diff(a^4+3*a^3-(1/a),a);diff(exp(x^2),x);diff(sin(x) * cos(x),x);integrate( x^5,x);integrate((x+3*x^2)^4,x);taylor(exp(x),x,0,4);m:2;3*m^2;solve(x^2-4*x-2=0,x);solve(c+x*y=z,x);2^4*4^2;'
        test5 = 'f(x):=0.5*x^2;plot2d([f(x)],[x,-3,3],[y,-10,10],[gnuplot_preamble,"set grid;"]);'
        test6 = 'f(z):=sin(z) * cos(z);plot2d([f(z)],[z,-4,4],[y,-10,10],[gnuplot_preamble,"set grid;"]);'
        test7 = '23!;diff(a^4+3*a^3-(1/a),a);diff(exp(x^2),x);diff(sin(x) * cos(x),x);integrate( x^5,x);integrate((x+3*x^2)^4,x);taylor(exp(x),x,0,4);m:2;3*m^2;3b-6;2^4*4^2;solve(x^2-4*x-2=0,x);solve(c+x*y=z,x);f(x):=0.5*x^2;plot2d([f(x)],[x,-4,4],[y,-10,10],[gnuplot_preamble,"set grid;"]);'		
        test8 = 'linsolve([3*x -y =1, 3*y +z =2, x + y - z =0], [x,y,z]);'
        test9 = 'plot2d([g(x)],[x,-3,3],[y,-10,10],[gnuplot_preamble,"set grid;"]);'

  
        if (nr == 0):
            expression = test0
        elif (nr == 1):
            expression = test1
        elif (nr == 2):
            expression = test2
        elif (nr == 3):
            expression = test3
        elif (nr == 4):
            expression = test4
        elif (nr == 5):
            expression = test5
        elif (nr == 6):
            expression = test6
        elif (nr == 7):
            expression = test7
        elif (nr == 8):
            expression = test8
        elif (nr == 9):
            expression = test9
            
        else: # Default: alles!!
            expression = test0
        if self.maximaOk:    
            if len(expression) > 0:
                exprListe = self.exprTest(expression)
                for element in exprListe:
                    if len(element) > 0:
                        #print element
                        self.bearbeiteAusdruck(element)
                        #time.sleep(0.5)
                t2.update_idletasks()            
	



#######################################
##      mark, copy and paste
##      with widget.clipboard_append/
##      clipboard_clear/selection_get(selection="CLIPBOARD")
##      see: von Löwis/Fischbeck: Python 2, S. 339-341
##	or M. Lutz: Programming Python, S. 382-385            
    def gf_copy(self,fenster):
        if (fenster == "t2"):
            win = t2
        else:
            win = self.exprZeile
        try:
            if (fenster == "t2"):
                marked_text = win.get(SEL_FIRST, SEL_LAST)
            else:
                marked_text = win.get()
            win.clipboard_clear()
            win.clipboard_append(marked_text)
        except TclError:
            pass


    def copy2Entry(self,fenster):
        win = t2
        try:
            marked_text = win.get(SEL_FIRST, SEL_LAST)
            if len(str(marked_text)) > 0:
                self.exprZeile.insert(INSERT, marked_text)
                self.exprZeile.focus_set()
        except TclError:
            pass


    def cpCut2Entry(self,fenster):
        win = t2
        try:
            marked_text = win.get(SEL_FIRST, SEL_LAST)
            if len(str(marked_text)) > 0:
                self.exprZeile.delete(0,END)
                self.exprZeile.insert(INSERT, marked_text)
                self.exprZeile.focus_set()
        except TclError:
            pass

    
    def gf_paste(self,fenster): 
        if (fenster == "t2"):
            win = t2
        else:
            win = self.exprZeile
        try:
            text = ''
            text = win.selection_get(selection="CLIPBOARD")
            if len(text) > 0:
                win.insert(INSERT, text)
##            if (fenster != 'self.exprZeile'):
##                self.set_cursor()            
        except TclError:
            pass
                    
    def gf_cut(self,fenster):
        if (fenster == "t2"):
            win = t2
        else:
            win = self.exprZeile
        try:
            if (fenster == "t2"):
                marked_text = win.get(SEL_FIRST, SEL_LAST)
            else:
                marked_text = win.get()
            #self.feedback(marked_text)
            win.delete(SEL_FIRST, SEL_LAST)
            win.clipboard_clear()
            win.clipboard_append(marked_text)
        except TclError:
            pass
        
#######################################


    def openfile(self):
        """Datei wird im Textfenster angezeigt"""
        pfad_ = abspath( pfad )
        os.chdir(pfad_) 
        dateitext = ''
        oname = askopenfilename(filetypes=[("Texte", "*.txt"),("Ini-Dateien","*.ini"),("Alle Dateien","*.*")])
        ##print oname
        if oname:
            offene_dateiname = oname            
            for line in fileinput.input(oname):   
                dateitext = dateitext + line   
            textfenster = tkw.hilfefenster(root,dateitext,str(oname) )

    def showDiciInWindow(self,myDici):
        """zeigt das Dici im Fenster an
        """
        maxDici = len(myDici)
	#print maxDici
        dicitext = ''
        if maxDici > 0:
            nr = 0
            bereich = myDici.iterkeys()
            for j in bereich:
                nr = nr + 1
                #if( myDici[j][1] == 'I'):
                dicitext = dicitext  +  string.strip(str(myDici[j][0])) + '\n'                    
##                    if (nr < 10):
##                        dicitext = dicitext  + '#' + str(nr) + ' ' +  ' ' +   str(myDici[j][0]) + '\n'
##                    else:
##                        dicitext = dicitext  + '#' + str(nr) + ' ' +   str(myDici[j][0]) + '\n'

            myButton = "Schliessen"    
            myTitle = 'Show Dicitionary'
            miniEditor.Editor(root,title=myTitle, button_message=myButton, TEXT=dicitext, resize=1,loadfile=None )            
            #textfenster = tkw.hilfefenster(root,dicitext,'Show Dicitionary')

    def showDici(self,myDici,_clear_=0):
        """zeigt das Dici im Fenster an,
        _clear_==1: Fenster wird geleert UND self.nr =0 !!!
        """
        maxDici = len(myDici)
        if maxDici > 0:
            if _clear_:
                self.nr = 1
                self.cl_unten()
            bereich = myDici.iterkeys()
            for j in bereich:
                #self.nr = self.nr + 1
                #if( myDici[j][1] == 'I'):
                self.printExpression(myDici[j][0])
                #else:
                #    self.printExpression(self.nr,myDici[j][0],'     \t')
                    
    def modDici(self,myDici,nr):
        maxDici = len(myDici)
        if maxDici > 0 and nr <= maxDici:
            bereich = range(nr,maxDici)#  
            for j in bereich:
                myDici[j] = myDici[j+1]
            del myDici[maxDici]    
            #self.feedback( str(myDici) )
        


    def saveExprDici2txt(self,wxmaxima=0):
        """Sichern einer Expression-Dici als *.txt-datei"""
        debug = 0
        pfad_ = abspath( self.home )
        os.chdir(pfad_)
        temp = {}
        if wxmaxima:
            oname = asksaveasfilename(filetypes=[("wxMaxima", "*.mac"),("Alle Dateien","*.*")],initialdir=self.workingDir)
        else:
            oname = asksaveasfilename(filetypes=[("Texte", "*.txt"),("Alle Dateien","*.*")],initialdir=self.workingDir)
        if oname:
            
            #self.feedback(str(oname))
            if wxmaxima:
                if (not self.test_extension(oname,".mac")):
                    oname = oname + ".mac"                
            else:
                if (not self.test_extension(oname,".txt")):
                    oname = oname + ".txt"
		    
            kopf = "#\n# generated by pyMaxima\n"
            zeit = "# @ " +  time.strftime("%A %B %d, %Y %I:%M %p",time.localtime(time.time())) + "\n#\n"		    
            try:
                exprDatei = open(oname,'w')
                bereich = self.exprDici.iterkeys()
                seq = []
                if not wxmaxima:
                    seq.append(kopf)
                    seq.append(zeit)
                for j in bereich:
                    if debug:
                        print self.exprDici[j][0]
                    
                    if (string.find(self.exprDici[j][0],'functions') == -1): # Keine Funktionentests
                        if (string.find(self.exprDici[j][0],'? ') == -1): # Keine Anfragen!
                            seq.append(self.exprDici[j][0]+'\n')

                #exprDatei.writelines(seq)
                data = string.join(seq,'')
                if debug:
                    print data
                    print
                _data_ = util.ucodefilter_(data) 
                if debug:
                    print _data_
                    print
                #print repr(data_unicode)
                #print
                #self.feedback( data_unicode )
                exprDatei.write( _data_ )                                
                
                exprDatei.close()
                self.workingDir = os.path.abspath(os.path.dirname(oname)) + os.sep
            except IOError:
                print 'EA-Fehler: Sichern fehlgeschlagen!'


    def sonderzeichen(self,eing):
        # vgl. auch class exprFenster
        # Modifiziert: 2008-10-16
        _debug_ = 0
        
        org_length = len(eing)        
        if _debug_:
            print "Original >>> ", eing
            
        a = repr(eing)
        
        if _debug_:        
            print "repr >>> ",a            
            
        a = a.replace("\\xe4","ä")
        a = a.replace("\\xfc","ü")
        a = a.replace("\\xf6","ö")
        a = a.replace("\\xd6","Ö")
        a = a.replace("\\x84","Ä")
        a = a.replace("\\xc4","Ä")
        a = a.replace("\\x9c","Ü")
        a = a.replace("\\xdc","Ü")
        a = a.replace("\\xdf","ß")
        a = a.replace("\\t","")
        a = a.replace("\\n","  ")
        a = a.replace("\\'","'") # neu => Vers. 1.46        

        
        if _debug_:        
            print "nach replace >>> ",a
            
        diff = len(a) - org_length
        if _debug_:
            print "diff = ",diff

        a = a[1:-1] # ohne erstes Zeichen & ohne letztes Zeichen
            
        if _debug_:        
            print "nach a[1:-1] >>> ",a
            #self.feedback( "nach a[1:-1] >>> " + a)
            
        
        #Der folgende Befehl NICHT, da dann auch "'" in der Mitte rausfliegen!
        # a = a.replace("\'","")
                
        a = str(a)
        
        if _debug_:        
            print "str(a) >>> ",a
            #self.feedback( "str(a) >>> " + a)

        return a


    def loadExprTxtFileTEX_PP(self):
        """generate TEX-file from pyMaxima-Session-file
        Two Steps:
        (1) Sitzungsdatei => tex( ___ )$
        (2) eingabe => verbatim ( ___)
        Die Ausgabe ist wg. des tex()-Befehl schon in Latex . . ."""
        # Zeitstempel
        _date_ = time.strftime("%A %B %d, %Y %I:%M %p",time.localtime(time.time())) 
        
        kopf = r"""% Generated by pyMaxima
% on """
        kopf2=r"""\documentclass[a4paper,ngerman,10pt]{article}
% landscape = DIN A 4 quer         
%\documentclass[a4paper,landscape,ngerman,10pt]{article}
\usepackage{ngerman}
\usepackage{graphicx}
\usepackage[latin1]{inputenc}
\usepackage{babel}
\usepackage{float}
% landscape = DIN A 4 quer
%\usepackage[landscape]{geometry}
\usepackage{listings}
\usepackage{geometry}
\geometry{a4paper,left=25mm,right=25mm, top=25mm, bottom=2cm}
%%%%%%%%%%%%%%
\begin{document}
"""
        
        # includegraphics auskommentiert!
        _graphic_  = r"""\begin{center}""" + "\n"
        _graphic_ = _graphic_ + r"""%\includegraphics[scale=0.6]{test.png}""" + "\n"
        _graphic_ = _graphic_ + r"""\end{center}""" + "\n"
        
        tail = r"""\end{document}"""
        
        _ergebnis_ = self.loadExprTxtFileAndExec(TEX_Mode=1)
        if (_ergebnis_ == 1):            
            tex_start = "(%i"
            tex_end = ")$\n"
            sname = asksaveasfilename(filetypes=[('TEX-Datei', "*.tex")],initialdir=self.workingDir)
            if len(sname) < 1:
                return
            else:
                if ( (not self.test_extension(sname,".tex"))):
                    sname = sname + ".tex"
                
                data = t2.get("1.0",END)
                _data_ = util.ucodefilter_(data,0)            
                sitzungsListe = string.split(_data_,tex_start)
                
                tex_in = []  # sammelt Eingaben
                tex_out = [] # sammelt Ausgaben
                
                for element in sitzungsListe: # Sitzung abarbeiten
                    if len(element) > 0:
                        element = tex_start + element
                        #print element
                        miniListe = string.split( element,tex_end )
                        #print miniListe
                        if len(miniListe) > 1 and len(miniListe) < 3:
                            tex_in.append( util.tex_verbatim( miniListe[0] + tex_end ) )
                            tex_out.append( miniListe[1] )
                
                anzahl = len(tex_in)
                ret_data = kopf + _date_ + "\n" + kopf2 # TEX-Datei aufbauen
                ret_data = ret_data + util.tex_subsection(r"pyMaxima-Sitzung ({\today})") 
                for i in range(anzahl):
                    ret_data = ret_data + tex_in[i] + tex_out[i] # Sitzung einarbeiten
                ret_data = ret_data + _graphic_ + tail 
                #print ret_data
                
                ofp = open(sname,"w")
                ofp.write(ret_data) # und schreiben
                ofp.flush() 
                ofp.close()
                self.workingDir = os.path.abspath(os.path.dirname(sname)) + os.sep

    def loadExprTxtFileAndExec(self,TEX_Mode=0,_sessionFile_=None):
        """Laden und Ausführen einer Expression-Text-Datei: *.txt
        TEX-Mode=0: normal abarbeiten
        TEX-Mode=1: tex(_expr_)$
        _sessionFile_=None : call askopenfilename
        _sessionFile_=path_file : oname = path_file
        """
        #pfad_ = abspath( self.home )
        #os.chdir(pfad_)
        if self.maximaOk:
            magic = "generated by pyMaxima"
            ok = 0
            _debug_ = 0
            _debug2_ = 0

            if _sessionFile_:
                oname = _sessionFile_
            else:
                oname = askopenfilename(filetypes=[("Texte", "*.txt"),("Alle Dateien","*.*")],initialdir=self.workingDir)
            
            ##print oname
            if oname:
                #self.feedback(str(oname))
                try:
                    exprDatei = open(oname,'r')
                    exprListe = exprDatei.readlines()
                    exprDatei.close()
                    self.workingDir = os.path.abspath(os.path.dirname(oname)) + os.sep
                    if (TEX_Mode==0) and (_sessionFile_==None):
                        #print "hier loadExprTxtFileAndExec: " +  oname  # !!!!!!!!!!!!!!!!!!!!!!!!
                        prevUsed.addEntry(oname)
                        prevUsed.swapFirst(oname)
                        self.update_dbUsedMenu()
                        self.update_editUsedMenu()


                    #t2.update_idletasks() 
                    time.sleep(_delay_time_) # neu 0.28
                    self.t.update()  # neu 0.28
                    if _debug_:
                        print exprListe
                    maxi = len(exprListe)
                    i = 0
                    self.break_excute_session_file = 0
                    
                    while i < maxi:
                        
                        _expr_ = self.sonderzeichen(  exprListe[i].strip() ).decode('iso-8859-1')
                        if _debug2_:
                            print _expr_
                        #_expr_ = exprListe[i].strip()
                        _expr_ = _expr_.encode('iso-8859-1')
                        if _debug2_:
                            print _expr_
                        if len(_expr_) > 0:
                            if _expr_.startswith('#'):  # Kommentar ueberlesen
                                if (string.find(_expr_,magic) != -1):
                                    ok = 1
                            elif (string.find(_expr_,'(%i') != -1):
                                # keine Sitzungsdatei!!!
                                #raise IOError
                                tkMessageBox.showerror("Datei-Fehler!", 'Die Datei \n<' + oname + '>\n ist keine pyMaxima-Sitzungsdatei!')
                                return
    ##                        elif (string.find(_expr_,'(%o') != -1):
    ##                            tkMessageBox.showerror("Datei-Fehler!", 'Die Datei \n<' + oname + '>\n ist keine pyMaxima-Sitzungsdatei!')
    ##                            return                            
                            else:
                                if ok:
                                    if (string.find(_expr_,';') != -1) or (string.find(_expr_,'$') != -1):
                                        if TEX_Mode:
                                            _expr_ = util.expr2TEX( _expr_)
                                        
                                        ret = self.bearbeiteAusdruck(_expr_)
                                        self.t.update()
                                        if (ret == 2):
                                            while (ret == 2): 
                                                ret = self.getInput()
                                                #self.t.update()
                                                t2.update_idletasks()
                                    else: # ';' fehlt => Ausdruck aufbauen!
                                        j = i
                                        buildexpr = ''
                                        found = 0
                                        while (j < maxi and not found):
                                           _expr_ = exprListe[j].strip()
                                           if _expr_.startswith('#'):  # Kommentar ueberlesen
                                               j = j + 1
                                           elif (string.find(_expr_,'(%i') != -1):
                                                tkMessageBox.showerror("Datei-Fehler!", 'Die Datei \n<' + oname + '>\n ist keine pyMaxima-Sitzungsdatei!')
                                                return 0
                                           else:
                                               if (string.find(_expr_,';') != -1):
                                                   found = 1
                                               elif (string.find(_expr_,'$') != -1):
                                                   found = 1
                                               buildexpr = buildexpr + _expr_
                                               j = j + 1
                                        i = j - 1 # Vorsicht!!!
                                        if _debug_:
                                            print buildexpr
                                            
                                        if TEX_Mode:
                                            buildexpr = util.expr2TEX( buildexpr )
                                            
                                        ret = self.bearbeiteAusdruck(buildexpr)
                                        self.t.update()
                                        if (ret == 2):
                                            while (ret == 2): 
                                                ret = self.getInput()
                                                #self.t.update()
                                                t2.update_idletasks()
                                           
                                        
                                else:
                                    tkMessageBox.showerror("Datei-Fehler!", 'Die Datei \n<' + oname + '>\n ist keine pyMaxima-Sitzungsdatei!')
                                    return 0
                        i = i + 1
                        if self.break_excute_session_file:
                            ret = tkMessageBox.askokcancel(title="Ausführung fortsetzen?", message="Ausführung der Sitzungsdatei fortsetzen?")
                            if ret: # Rückfrage vor Beenden
                                self.break_excute_session_file = 0                                
                            else:
                                break
                        # Ende while-Schleife
                    return 1  # >>> alles ok  <<<

                except IOError:
                    print 'EA-Fehler: Konnte Datei nicht öffnen!'
                    return 0
            else: # no oname
                return 0
        else: # self.maximaOk == False 
            return 0

    def holeVerzeichnisname(self):           
        eingabe = askdirectory(initialdir=self.home) # aus dem Modul 
        if eingabe <> '': 
            return os.path.abspath (eingabe)
        else:
            return None

    def myEditor(self):
        _debug_ = 0

        myButton = "Schliessen"
        
        dat_name = askopenfilename(filetypes=[('Text-Dateien', "*.txt"),('HTML-Dateien', "*.html"),('Python-Dateien', "*.py"),('Alle Dateien',"*")],initialdir=self.workingDir) 
        if _debug_:
            print dat_name
        if dat_name:
            myTitle = "Datei: " + str(dat_name)
            workDir = os.path.dirname( dat_name )
            miniEditor.Editor(root,title=myTitle, button_message=myButton, TEXT=None, resize=1,loadfile=dat_name, pfad = workDir, module = pyMaximaModule )
            self.workingDir = os.path.abspath(os.path.dirname(dat_name)) + os.sep

    def openWithEditor(self,dat_name):
        _debug_ = 0

        myButton = "Schliessen"
        
        #dat_name = prevUsed.getFirst()
        if _debug_:
            print dat_name
        
        if len(dat_name) > 0:
            myTitle = "Datei: " + str(dat_name)
            workDir = os.path.dirname( dat_name )
            miniEditor.Editor(root,title=myTitle, button_message=myButton, TEXT=None, resize=1,loadfile=dat_name, pfad = workDir, module = pyMaximaModule )
            self.workingDir = os.path.abspath(os.path.dirname(dat_name)) + os.sep



            
    def toHTML(self):
        """generate html-file from pyMaxima-Session"""
        mitBild = 0
        plotDateiname = None
        htmlfenster = tkParameter.htmlFenster(root,pgname,self.home,self.workingDir)
        #sname = asksaveasfilename(filetypes=[('HTML', "*.html *.htm")],initialdir=pfad) 

        if htmlfenster.result <> None:
            #print sname
            #plotDateiname = os.path.basename(sname)
            sname = htmlfenster.a1
            if len(sname) < 1:
                return
            else:
                #print sname
                if (not htmlfenster.holeName): # Dateiname wurde NICHT ueber "asksaveasfilename" geholt
                    if ( (not self.test_extension(sname,".html")) and (not self.test_extension(sname,".htm")) ):
                        testname = sname + ".html"
                    else:
                        testname = sname
                    
                    if os.path.exists(testname): # nachfragen, ob ueberschreiben!!
                        ret = tkMessageBox.askokcancel(title="Datei überschreiben?", message="Die Datei \n<" + testname + ">\ngibts schon, Datei überschreiben?")
                        if not ret: # Datei NICHT ueberschreiben, Vorgang abbrechen!!!
                            return
                    sname = testname
                
            if htmlfenster.a3:                
                plotDateiname = os.path.basename(htmlfenster.a2)
                if plotDateiname:
                    mitBild = 1
            else:
                plotDateiname = os.path.splitext(os.path.basename(sname))[0]
                plotDateiname = plotDateiname + '.png'
                
            #print plotDateiname
            #data = t2.get("1.0",END)
            _data_ = t2.get("1.0",END)            
            data = hts.ucodefilter_(_data_)
            #print data
            #self.feedback( data)
            dname = "pyMaxima-Sitzung"
            zeit =  hts.ucodefilter_( util._DatumZeit_() )
            kopf1 = hts.h_(dname,3,align='center') + '\n'
            kopf2 =  hts.h_(zeit,3,align='center') + '\n'
            arialKopf = hts.font_(kopf1 +  kopf2) + '\n'
            kopf = hts.hr_(80) + '\n' + arialKopf +  hts.hr_(80) + '\n'
            #print kopf            
            #plot = hts.img_('plot.png','Funktionen-Plot')
            plot = hts.img_(plotDateiname ,'Funktionen-Plot')
            plotc = hts.center_ ( plot )
            
            if mitBild:
                rest = hts.b_( hts.pre_(data) )  + plotc
            else:
                kommentar = hts.comment_( plotc )
                rest = hts.b_( hts.pre_(data) )  + kommentar
                
            #print rest
            textfarbe = '#000000'
            bg = '#efefef'
            html_session = hts.html_('pyMaxima-Session',kopf + rest,textfarbe,bg,generated_by="pyMaxima")
            #print html_session
            if ( (not self.test_extension(sname,".html")) and (not self.test_extension(sname,".htm")) ):
                    sname = sname + ".html"
            #ofp = codecs.open(sname,"w", "iso-8859-1")                
            ofp = open(sname,"w") 
            ofp.write(html_session) 
            ofp.flush() 
            ofp.close()
            self.workingDir = os.path.abspath(os.path.dirname(sname)) + os.sep



    def toTEX(self):
        """generate TEX-file from pyMaxima-Session"""
        # Zeitstempel
        _date_ = time.strftime("%A %B %d, %Y %I:%M %p",time.localtime(time.time())) 

        # TEX-Einstellungen
        kopf = r"""% Generated by pyMaxima
% on """
        kopf2=r"""\documentclass[a4paper,ngerman,10pt]{article}
% landscape = DIN A 4 quer         
%\documentclass[a4paper,landscape,ngerman,10pt]{article}
\usepackage{ngerman}
\usepackage{graphicx}
\usepackage[latin1]{inputenc}
\usepackage{babel}
\usepackage{float}
% landscape = DIN A 4 quer
%\usepackage[landscape]{geometry}
\usepackage{listings}
\usepackage{geometry}
\geometry{a4paper,left=25mm,right=25mm, top=25mm, bottom=2cm}
%%%%%%%%%%%%%%
\begin{document}
"""
        
        tail = r"""\end{document}"""

        mitBild = 0
        plotDateiname = None

        texFenster = tkParameter.texFenster(root,pgname,self.home,self.workingDir)

        if texFenster.result <> None:
            sname = texFenster.a1
            if len(sname) < 1:
                return
            else:
                if (not texFenster.holeName): # Dateiname wurde NICHT ueber "asksaveasfilename" geholt
                                 
                    if ( (not self.test_extension(sname,".tex"))):
                        testname = sname + ".tex"
                    else:
                        testname = sname
                    
                    if os.path.exists(testname): # nachfragen, ob ueberschreiben!!
                        ret = tkMessageBox.askokcancel(title="Datei überschreiben?", message="Die Datei \n<" + testname + ">\ngibts schon, Datei überschreiben?")
                        if not ret: # Datei NICHT ueberschreiben, Vorgang abbrechen!!!
                            return
                    sname = testname
                        
            if texFenster.a3:                
                plotDateiname = os.path.basename(texFenster.a2)
                if plotDateiname:
                    mitBild = 1
                _graphic_ = r"""%\includegraphics[scale=0.6]{test.png}"""
            else:
                plotDateiname = os.path.splitext(os.path.basename(sname))[0]
                plotDateiname = plotDateiname + '.png'        	

            # TEX-Datei "basteln"
            _data_ = kopf + _date_ + "\n"
            _data_ = _data_ + kopf2
            _data_ = _data_ + util.tex_subsection(r"pyMaxima-Sitzung ({\today})")
            win_data = t2.get("1.0",END)
            
            _data_ = _data_ + util.tex_lstlisting(util.ucodefilter_(win_data,0))
            
            #_data_ = _data_ + endListing  + "\n"
            
            if mitBild:
                _graphic_ = r"""\begin{center}""" + "\n"
                _graphic_ = _graphic_ + r"""\includegraphics[scale=0.6]{""" + plotDateiname + "}" + "\n"
                _graphic_ = _graphic_ + r"""\end{center}""" + "\n"
                _data_ = _data_ + _graphic_ + "\n"
            else:
                kommentar = r"""\begin{center}""" + "\n"
                kommentar = kommentar + r"""%\includegraphics[scale=0.6]{test.png}""" + "\n"
                kommentar = kommentar + r"""\end{center}""" + "\n"
                _data_ = _data_ + kommentar 
            

            _data_ = _data_ +  tail + "\n"
            
            # TEX-Datei schreiben
            ofp = open(sname,"w")
            ofp.write(_data_) 
            ofp.flush() 
            ofp.close()
            self.workingDir = os.path.abspath(os.path.dirname(sname)) + os.sep


    def toTXT(self):
        """generate txt-file from pyMaxima-Session"""        
        sname = asksaveasfilename(filetypes=[('Text', "*.txt")],initialdir=self.workingDir) 
	
        if sname:
            if (not self.test_extension(sname,".txt")):
                sname = sname + ".txt"

            data = t2.get("1.0",END)
            _data_ = util.ucodefilter_(data,0)
            #self.feedback( data)
            ofp = open(sname,"w")
            #ofp = codecs.open(sname,"w", "utf-8") 
            ofp.write(_data_) 
            ofp.flush() 
            ofp.close()
            self.workingDir = os.path.abspath(os.path.dirname(sname)) + os.sep

            
    def toWxMaxima(self):
        """ speichert laufende Sitzung als wxMaxima-Sitzung ab (nur Eingaben!)
        """
        kopf = '/* [wxMaxima batch file version 1] [ DO NOT EDIT BY HAND! ]*/'
        start = '/* [wxMaxima: input   start ] */'
        end = '/* [wxMaxima: input   end   ] */'
        ende = "/* Maxima can't load/batch files which end with a comment! */"
        ende = ende + '\n' + '"Created with pyMaxima"$' + '\n'
        
        pfad_ = abspath( self.home )
        os.chdir(pfad_)
        oname = asksaveasfilename(filetypes=[("wxMaxima-Session", "*.wxm"),("Alle Dateien","*.*")],initialdir=self.workingDir)
        if oname:
            #self.feedback(str(oname))            
            if (not self.test_extension(oname,".wxm")):
                    oname = oname + ".wxm"		    
            try:
                exprDatei = open(oname,'w')
                #exprDatei = codecs.open(oname,"w", "iso-8859-1") #"utf-8") 
                bereich = self.exprDici.iterkeys()
                seq = []
                seq.append(kopf+'\n\n\n')
                #seq.append(zeit)
                for j in bereich:
                    if (string.find(self.exprDici[j][0],'functions') == -1):
                        seq.append(start + '\n')
                        seq.append(self.exprDici[j][0]+'\n')
                        seq.append(end + '\n\n')
                seq.append(ende)
                data = string.join(seq,'')
                _data_ = util.ucodefilter_(data)
                #print repr(data_unicode)
                #print
                #self.feedback( data_unicode )
                exprDatei.write( _data_ )                                
                #exprDatei.writelines(seq)
                exprDatei.close()
                self.workingDir = os.path.abspath(os.path.dirname(oname)) + os.sep
            except IOError:
                print 'EA-Fehler: Sichern fehlgeschlagen!'



    def _laden(self): 
        global ini_dici, _delay_time_ , pfad_zu_maxima , pfad_zu_maximaDoc, maximaDocHome, pfad_zu_python, nr_lines_main
        sname = askopenfilename(title=u'ini-Datei w\xe4hlen',filetypes=[("Konfigurations-Dateien","*.conf")],initialdir=_initdir_)
        if sname <> '':
            home = os.path.dirname(sname)
            datei = os.path.basename(sname)
            myDici = {}
            util.leseConfi(home,datei,myDici)
            util.kopiereDici(myDici,ini_dici)
            _delay_time_ = float( myDici['delay_time'] )
            pfad_zu_maxima = myDici['maxima_path'] # + os.sep
            pfad_zu_maximaDoc = myDici['docu_path']
            pfad_zu_python = myDici['py_path']
            #font_size_main = int(myDici['font_size_main'])  # 10    
            nr_lines_main = int(myDici['nr_lines_main'])  # 27
            maximaDocHome =  os.path.join(pfad_zu_maximaDoc, 'maxima.html')
            util.druckeDici(ini_dici)
            self.t.update()  # neu 0.28
            try:
                self.startConnect()
            except:
                pass
                    

    def _speichern(self):
        global ini_dici
        text = ''
        text = util.iniDici2Text(ini_dici)
        if os.name == 'posix':
            datei_schreiben(_initdir_,'linuxInit.conf',text)
        else:
            datei_schreiben(_initdir_,'winInit.conf',text)
        

    def _einstellungen(self):
        global ini_dici, _delay_time_ , pfad_zu_maxima , pfad_zu_maximaDoc, maximaDocHome, pfad_zu_python, _pygeoFlag_, nr_lines_main, font_size_para
        debug = 0
        changed = 0
        b1 = ini_dici['delay_time']  
        b2 = ini_dici['maxima_path'] 
        b3 = ini_dici['docu_path']
        b4 = ini_dici['py_path']
        b5 = ini_dici['font_size_para']
        b6 = ini_dici['nr_lines_main']        

        parameter = tkParameter.parameterfenster(root,pgname,pyMaximaHome,font_size_para,ini_dici)     
        if parameter.result <> None:
            
            ini_dici['delay_time'] = parameter.a1
            if b1 <> parameter.a1:
                changed=1
                _delay_time_ = float( ini_dici['delay_time']  )
                
            ini_dici['maxima_path'] = parameter.a2
            if b2 <> parameter.a2:
                changed=1
                pfad_zu_maxima = ini_dici['maxima_path']  # + os.sep
                
            ini_dici['docu_path'] = parameter.a3            
            if b3 <> parameter.a3:
                changed=1
                pfad_zu_maximaDoc = ini_dici['docu_path']
                maximaDocHome =  os.path.join(pfad_zu_maximaDoc, 'maxima.html')    

            ini_dici['py_path'] = parameter.a4
            if b4 <> parameter.a4:
                changed=1
                pfad_zu_python = ini_dici['py_path']
                try:
                    import pygeo
                    _pygeoFlag_ = 1                    
                except:
                    _pygeoFlag_ = 0
                    
                self.pygeoFlag = _pygeoFlag_
                if self.pygeoFlag:
                    # jetzt Menue self.pygeom aktualisieren!
                    self.update_pygeom()  


            ini_dici['font_size_para'] = parameter.a5           
            if b5 <> parameter.a5:
                changed=1
                font_size_para = ini_dici['font_size_para']


            ini_dici['nr_lines_main'] = parameter.a6           
            if b6 <> parameter.a6:
                changed=1
                nr_lines_main = ini_dici['nr_lines_main']

            
            if changed:
                antwort = tkMessageBox.askyesno('Speichern?','Neue Einstellungen speichern?')
                if antwort:
                    self._speichern()
            if debug:               
               print '\n\nHier Methode _test'
               util.druckeDici(ini_dici)
        

    def test_extension(self,Dateiname,Erweiterung):
        Dateiname = string.lower(Dateiname)
        if ( string.rfind(Dateiname,Erweiterung) == -1 ):
            return 0
        else:  ## Erweiterung in Dateiname gefunden
            return 1
        
#########################################   
# main   
if __name__ == "__main__":    
    #  Test-Mode: set _test_ True
    #_test_ = True
    _test_ = False
    
    
    pfad = os.getcwd()  
    pfad = os.path.abspath (os.getcwd()) + os.sep
    
    ## print pgname + ': ' + pfad
    
    if os.name <> 'posix': # win nt!
        if len(sys.argv) > 1:
            try:
                _lw_arg_ = sys.argv[1]
                #print _lw_arg_
            except:
                pass
            
    ## import pymaxUtil as util
    ## util.test()
    einstellungenLaden()
    dbList_holen(prevUsed.DirMax)
    root = Tk()
    #
    # root.tk.eval('source {init/gtklook.tcl}')
    # root.tk.eval('gtk_look')
    if os.name == 'posix': # in Linux: gtkLook!
        try:
            root.tk.eval('source {init/gtklook.tcl}')
            root.tk.eval('gtk_look')
        except:
            pass

    root.protocol('WM_DELETE_WINDOW',die)
    root.geometry('800x600+0+0')
    app = MaximaMain(root)
    
    root.mainloop() # los geht's






